#if 1

#pragma once
#include "../ai_core/dTrees/dTreeCommons.hpp"


namespace DTree
{

	struct TestExpander{float operator()(float val){return val;}};

	class AIUnitAdapter;

	template<class RefObj_type, class Adapter_type>
		class C45Generator;


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		C45
	**	\brief		Implements the C45 D-Tree module. Currently only supports AIUnits
	**				 but in the future it will use compile-time polymorphism (ie 
	**				 templitized class) to support ADNs aswell.
	**	\author		Nate Cleveland
	**	\date		11/14/2002 1:47:45 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	class TestC45
	{

	public:
		//!Short hand typedef for the type of Node used by this class.
		typedef Node<float, bool, TestExpander> TestNode;


		//a example object for this hackish test file
		struct TestExample
		{
			bool				m_result;
			std::vector<float>	m_caseVec;
		};

		//vector to store examples in.
		std::vector<TestExample>	m_exampleVec;


		class AIUnitAdapter
		{
		public:
			//!All typedefs needed to fulfill the C45Generator requirments.
			typedef	TestC45						myRefClass;
			typedef bool						Class_type;
			typedef float						Case_type;
			typedef TestExample					Example_type;
			typedef TestNode					Node_type;

			//!Methods to fulfill the C45Generator requirments.
			static float		GetCaseValue(const Case_type& Case){return Case;}
			static Class_type	GetExampleResult(Example_type ex){return ex.m_result;}
			static Node_type&	GetTreeRootRef(myRefClass& myRef){return *(myRef.m_root);}
			static Node_type*	GetTreeRootPtr(myRefClass& myRef){return myRef.m_root;}
			static Node_type**	GetTreeRootPtrPtr(myRefClass& myRef){return &(myRef.m_root);}
			static Node_type*&	GetTreeRootPtrRef(myRefClass& myRef){return myRef.m_root;}
			static std::vector<Example_type>&	GetExampleVec(myRefClass &myRef){return myRef.m_exampleVec;}
			static const std::vector<Case_type>&GetCaseVec(const Example_type& ex){return ex.m_caseVec;}
		};

		//! Allow our generator class complete access to our data.
		friend AIUnitAdapter;	

		
		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Basic Ctor for C45 class.
		**	\return		none
		**	\exception	none
		**	\par		DESCRIPTION:
		**				 Sets members to zero. Nothing else.	
		**	\author		Nate Cleveland
		**	\date		11/14/2002 1:49:12 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		TestC45() throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Defualt dtor for C45 Tree.
		**	\exception	none
		**	\par		DESCRIPTION:
		**				 Frees the tree sturcture and clears out all examples.
		**	\author		Nate Cleveland
		**	\date		11/18/2002 3:12:24 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		~TestC45() throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Creates a deep copy of this object.
		**	\param		toCopyFrom		Reference to the object that we're to copying from.
		**	\exception	none
		**	\par		DESCRIPTION:
		**				 Creates a C45 object that is a copy of the given class. Calls the
		**				  parent's copy ctor.
		**	\author		Nate Cleveland
		**	\date		11/22/2002 3:57:52 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		TestC45(const TestC45& toCopyFrom) throw();
		
		
		//saves a tree to file.
		bool save(const char *pathName = "dTree.acu") throw();
		
		//loads a tree from file.
		bool load(const char *pathName = "dTree.acu") throw();

		//examines current situation and produces a result. case vector must remain constant
		// during the learning process or else badness happens. 
		bool Think(std::vector<float> caseVec);

		// uses the c45 generator to reshape the tree structure.
		void Learn(void);

		void TestC45::GiveExample(std::vector<float> caseVec, bool result);
		
		
	protected:
		TestNode*						m_root;			//!< Pntr to the tree root.
	};//end TestC45 class.
}//namespace
#endif