////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_Attribute.h
**	\brief		The attribute struct
**	\author		John Corpening
**	\date		10/18/2002 1:12:57 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace AI_System
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		AI_Attribute
**	\brief		Attributes to be used by learning techniques
**	\author		John Corpening
**	\date		10/18/2002 1:13:22 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
struct AI_Attribute
{
	Attribute_ID	ID;		//!< Identifier for this attribute
	float			value;	//!< Value returned from Data Filter
	

	// Added by nate so he could use some algorithmal techniques. [11/14/2002]
	float GetValue() const throw(){return value;}
};

struct AI_ActionTag
{
	AI_ID			unit_ID;
	Decision_ID		decisionID;
	float			fitness;
	bool			completed;
	bool operator < (const AI_ActionTag& T) const throw()
	{
		//compare the unit ID's
		if(unit_ID < T.unit_ID)
			return true;
		
		//compare the decision ID's
		if(decisionID < T.decisionID)
			return true;

		return false;
	}
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		AI_Action
**	\brief		
**	\author		John Corpening
**	\date		10/18/2002 1:26:15 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
struct AI_Action
{
public:
	Action_ID		ID;
	float			value;
	AI_ActionTag	tag;		
	Object_ID		objectID;	//!< Optional ID of the currently evaluated object
};

struct AI_ExampleUnit
{
public:
	bool						result;		//!< Whether the result was positive or not
	float						fitness;	//!< The resulting fitness
	std::vector<AI_Attribute>	attributes;	//!< The attributes to use in the training example
};

struct AI_ExampleADN
{
public:
	bool						result;		//!< Whether the result was positive or not
	float						fitness;	//!< The resulting fitness
	AI_ID						ID;			//!< Used by ADN: The AI_UNIT that was chosen
	std::vector<AI_Attribute>	attributes;	//!< The attribute-value pairs for this example
};

}//End namespace AI_System