////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_DataFilterBase.cpp
**	\brief		This is the implementation file for AI_DataFilterBase.h
**	\author		John Corpening
**	\date		10/26/2002 1:43:56 PM
*/
////////////////////////////////////////////////////////////////////////////////

#include"AI_System.h"
using namespace AI_System;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	AI_DataFilter::AI_DataFilter
/*!	\brief		Constructor gets access to the MessageHandler
**	\exception	none
**	\author		John Corpening
**	\date		10/18/2002 4:28:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_DataFilter::AI_DataFilter()
{
	MH = MsgHandler::MessageHandler::Instance();
	if(!MH)
	{
		throw(Nexception("Null MessageHandler", __FILE__,__LINE__,__DATE__,__TIME__));
	}

	dataProcessed = false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	AI_DataFilter::getAttribute
/*!	\brief		returns the attribute calculated by the DataFilter
**	\return		an AI_Attribute to be used by the AI Units
**	\exception	none
**	\author		John Corpening
**	\date		10/18/2002 4:14:38 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_Attribute AI_DataFilter::getAttribute()
{
	//Check if the data filter has already made its calculations
	if(!dataProcessed)
	{
		//if not then process the data
		attribute.value = processData();
		dataProcessed = true;
		
		//Return a copy of the attribute
		//ensure that the ID is filled out
		attribute.ID = ID;
	}
	return attribute;
}