////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_Helpers.h
**	\brief		Helper functions for the AI System
**	\author		John Corpening
**	\date		11/22/2002 11:49:17 AM
**	\par		DESCRIPTION:
**				File includes functions for:
**				-Getting random AI_Units
**				-Getting random AI_Units given an action type
**				-copying an AI_Unit
**				-Getting a random ADN given an action ID
**	\par		REMARKS:
**				This file was created to try to buffer AI System entities
**				from implementation details.
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace AI_System
{

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getRandomAIUnit
/*!	\brief		creates and returns a random AIUnit
**	\return		pointer to an AI_Unit
**	\exception	none
**	\author		John Corpening
**	\date		11/7/2002 7:47:12 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_Unit* getRandomAIUnit() throw();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getRanomAIUnit
/*!	\brief		creates a random AI Unit given an Action Tpe
**	\param		actionType an Action_ID
**	\return		a pointer to the newly created AI_Unit
**	\exception	none
**	\author		John Corpening
**	\date		11/22/2002 11:56:24 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_Unit* getRandomAIUnit(Action_ID actionType) throw();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	copyAIUnit
/*!	\brief		creates a copy of an AI Unit
**	\param		unit, the AI Unit to copy
**	\return		pointer to the newly created AI Unit
**	\exception	none
**	\author		John Corpening
**	\date		11/22/2002 11:56:28 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_Unit* copyAIUnit(AI_Unit* unit) throw();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getRandomADN
/*!	\brief		creates and returns a random Action Decision Node
**	\param		ID Action ID of the ADN to create
**	\return		pointer to an ADN
**	\exception	none
**	\author		John Corpening
**	\date		11/7/2002 7:47:49 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_ADN* getRandomADN(Action_ID ID) throw();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getRandomFF_ID
/*!	\brief		used to get a random ID for a fitness function
**	\return		a fitness function ID
**	\exception	none
**	\author		John Corpening
**	\date		10/26/2002 12:30:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
FF_ID getRandomFF_ID();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getFitnessFunction
/*!	\brief		given an ID, creates a fitness function a returns a pointer to it
**	\param		id the ID of the fitness function to create
**	\return		a pointer to the newly created fitness function
**	\exception	none
**	\author		John Corpening
**	\date		10/26/2002 12:30:08 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_FitnessFunction* getFitnessFunction(FF_ID id);

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getRandomHG_ID
/*!	\brief		used to get a random ID for a fitness function
**	\return		a fitness function ID
**	\exception	none
**	\author		John Corpening
**	\date		10/26/2002 12:30:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
HG_ID getRandomHG_ID();

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	getHigherGoal
/*!	\brief		given an ID, creates a fitness function a returns a pointer to it
**	\param		id the ID of the fitness function to create
**	\return		a pointer to the newly created fitness function
**	\exception	none
**	\author		John Corpening
**	\date		10/26/2002 12:30:08 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
AI_HigherGoal* getHigherGoal(HG_ID id);

const char* getActionString(Action_ID action);

const char* getDataFilterString(DF_ID df_id);

const char* getFitnessFunctionString(FF_ID ff_id);

const char* getHigherGoalString(HG_ID hg_id);

}