////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_Keys.h
**	\brief		various ID primitives for the AI_System
**	\author		John Corpening
**	\date		10/28/2002 2:54:32 PM
**	\par		DESCRIPTION:
**				-AI_ID
**					- ID for the AI_Units
**					- Not unique across AI_Modules
**				-Attribute_ID
**					- ID for the Attributes
**					- Unique
**				-Object_ID
**					- typedef'ed from MsgHandler::Key
**				-Action_ID
**					- ID for the actions an object must interpret
**					- High two bytes are for Action Class
**				-DF_ID
**					- ID for Data Filters
**				-FF_ID
**					- ID for Fitness Functions
**				-HG_ID
**					- ID for Higher Goals
*/
////////////////////////////////////////////////////////////////////////////////


#pragma once


#include "../messageHandler/MessageHandler.h"


namespace AI_System
{
/*	typedef union
	{
		struct 
		{
			unsigned short Module;
			unsigned short Unit;
		};
		unsigned int	ID;
	} AI_ID;
*/
	typedef unsigned int AI_ID;
	typedef unsigned int Decision_ID;
	typedef unsigned int Attribute_ID;

	typedef MsgHandler::Key Object_ID;

	enum Action_ID
	{
		AC_TEST = 0xFFFFFFFF,
		AC_NULL = 0,
		//Action classes bits 15 - 31
		AC_MASK		= 0xFFFF0000, 
		AC_MOVE		= (1 << 16),
		AC_MOUTH	= (1 << 17),
		AC_MARKER	= (1 << 18),
		//Action IDs any number up to an unsigned short
		//Movement actions
		MOVETO			= AC_MOVE | 1,
		FLEE			= AC_MOVE | 2,
		FOLLOW			= AC_MOVE | 3,
		MOVEWITH		= AC_MOVE | 4,
		ROAM			= AC_MOVE | 5,
		ATTACK			= AC_MOVE | 6,
		GOTO_POS_MARKER = AC_MOVE | 7,
		FLEE_NEG_MARKER = AC_MOVE | 8,
		IGNORE_MARKERS	= AC_MOVE | 9,
		HEED_MARKERS	= AC_MOVE | 10,
		//Mouth actions

		EAT				= AC_MOUTH | 2,
		PICKUP			= AC_MOUTH | 3,
		DROP			= AC_MOUTH | 4,
		//Location actions
		DROP_POS_MARKER	= AC_MARKER | 1,
		DROP_NEG_MARKER = AC_MARKER | 2,
		EVALUATEMARKER	= AC_MARKER | 3,
	};// Action_ID;

	typedef enum
	{	
//		DF_TEST_A	= -3,
//		DF_TEST_B	= -2,
//		DF_INVALID	= -1, 
		DF_NULL,

		//! Fish (not AI) Attributes DataFilter IDs (matching filters are declared in
		//!  AI_DataFilters_Attributes.h
		DF_METABOLISM,
		DF_ARMOR,
		DF_EYES,
		DF_SIZE,
		DF_TEETH_BALEEN,
		DF_REACTIONTIME,
		DF_MAXSPEED,
		DF_BECUM,
		DF_LIFESPAN,
		DF_ATTACKSTRENGTH,
		DF_SUCKABILITY,
		DF_ENERGYLEVEL_OBJECT,
		DF_ENERGYLEVEL_TARGET,
		DF_STOMACH_OBJECT,
		DF_STOMACH_TARGET,
		DF_HEALTH_OBJECT,
		DF_HEALTH_TARGET,
		DF_MATINGINCLINE_OBJECT,
		DF_MATINGINCLINE_TARGET,
		DF_HEALTH_NORMALIZED_OBJECT,
		DF_HEALTH_NORMALIZED_TARGET,
		DF_STOMACH_NORMALIZED_OBJECT,
		DF_STOMACH_NORMALIZED_TARGET,
		DF_ENERGY_NORMALIZED_OBJECT,
		DF_ENERGY_NORMALIZED_TARGET,


		//! Physical DataFilter IDs (matching filters are declared in 
		//!  AI_DataFilters_Other.h).
		DF_AGE,			
		DF_DISTANCE,
		DF_OBJECTSPEED,		//!Speed of the object we're evaluating.
		DF_TARGETSPEED,		//!Speed of the FishBody.
		DF_OBJECTTYPE,

		
		//! Environment DataFilter ID (matching filters are declared in
		//!  AI_DataFilters_Other.h.
		DF_PLANKTONDENSITY_OBJECT,
		DF_PLANKTONDENSITY_TARGET,



		//! Statistical DataFilter IDs (matching filters are declared in
		//!  AI_DataFilters_Stats.h.
		DF_SPEED_RATE_OBJECT,
		DF_SPEED_RATE_TARGET,
		DF_SPEED_COUNT_OBJECT,
		DF_SPEED_COUNT_TARGET,

		DF_DISTANCE_RATE_OBJECT,
		DF_DISTANCE_RATE_TARGET,
		DF_DISTANCE_COUNT_OBJECT,
		DF_DISTANCE_COUNT_TARGET,

		DF_HEALTH_RATE_OBJECT,
		DF_HEALTH_RATE_TARGET,
		DF_HEALTH_COUNT_OBJECT,
		DF_HEALTH_COUNT_TARGET,

		DF_INCLINE_RATE_OBJECT,
		DF_INCLINE_RATE_TARGET,
		DF_INCLINE_COUNT_OBJECT,
		DF_INCLINE_COUNT_TARGET,

		DF_STOMACH_RATE_OBJECT,
		DF_STOMACH_RATE_TARGET,
		DF_STOMACH_COUNT_OBJECT,
		DF_STOMACH_COUNT_TARGET,
		
		DF_ENERGY_RATE_OBJECT,
		DF_ENERGY_RATE_TARGET,
		DF_ENERGY_COUNT_OBJECT,
		DF_ENERGY_COUNT_TARGET,

		DF_KILLS_RATE_OBJECT,
		DF_KILLS_RATE_TARGET,
		DF_KILLS_COUNT_OBJECT,
		DF_KILLS_COUNT_TARGET,

		DF_OFFSPRING_RATE_OBJECT,
		DF_OFFSPRING_RATE_TARGET,
		DF_OFFSPRING_COUNT_OBJECT,
		DF_OFFSPRING_COUNT_TARGET,

		DF_BEEN_ATTACKED_RATE_OBJECT,
		DF_BEEN_ATTACKED_RATE_TARGET,
		DF_BEEN_ATTACKED_COUNT_OBJECT,
		DF_BEEN_ATTACKED_COUNT_TARGET,

		DF_HAS_ATTACKED_RATE_OBJECT,
		DF_HAS_ATTACKED_RATE_TARGET,
		DF_HAS_ATTACKED_COUNT_OBJECT,
		DF_HAS_ATTACKED_COUNT_TARGET,

		DF_PLANKTON_RATE_OBJECT,
		DF_PLANKTON_RATE_TARGET,
		DF_PLANKTON_COUNT_OBJECT,
		DF_PLANKTON_COUNT_TARGET,

		DF_CHUM_RATE_OBJECT,
		DF_CHUM_RATE_TARGET,
		DF_CHUM_COUNT_OBJECT,
		DF_CHUM_COUNT_TARGET,

		DF_TOTAL_FOOD_RATE_OBJECT,
		DF_TOTAL_FOOD_RATE_TARGET,
		DF_TOTAL_FOOD_COUNT_OBJECT,
		DF_TOTAL_FOOD_COUNT_TARGET,

		//The last two are for ease of processing
		//Do not add anything past them
		DF_LAST,
		DF_MAX		= DF_LAST - 1,
	} DF_ID;

	typedef enum
	{
//		FF_TEST_AND	= -5,
//		FF_TEST_OR	= -4,
//		FF_TEST_XOR	= -3,
//		FF_INVALID	= -1,
		FF_NULL,

		FF_MAX_HEALTH,
		FF_MAX_STOMACH,
		FF_MAX_SPEED,
		FF_MAX_ENERGY,
		FF_MAX_SPEED_ENERGY,
		FF_MAX_STOMACH_HEALTH,
		FF_MAX_INCLINE,
		FF_MAX_PLANKTON,
		FF_MAX_DISTANCE,
		FF_MAX_CHUM,
		FF_MAX_PLANKTON_DENSITY,

		FF_MIN_BEEN_ATTACKED,
		FF_MIN_DISTANCE,


		//The last two are for ease of processing
		//Do not add anything past them
		FF_LAST,
		FF_MAX		= FF_LAST - 1
	} FF_ID;

	typedef enum
	{
//		HG_TEST_AND = -5,
//		HG_TEST_OR	= -4,
//		HG_TEST_XOR = -3,
//		HG_INVALID = -1,
		HG_NULL,
		HG_KILLS,
		HG_NOMAD,
		HG_DUKE_WELLINGTON,		//randomly yells 1 or 0.
		HG_RANDOM,				//assigns total random (ranged) values to everything
		HG_GLUTTON,
		HG_PLANKTON_DENSITY,
		HG_BEST_CASE,
		HG_WORST_CASE,
		HG_AVERAGE_CASE,
		HG_HERBIVORE,			//hates eating chum, attacking. loves plankton
		HG_CARNIVORE,

		//The last two are for ease of processing
		//Do not add anything past them
		HG_LAST,
		HG_MAX		= HG_LAST - 1
	}HG_ID;

	typedef enum
	{
//		TT_TEST = -1,	//!< Test
		TT_NULL,	//!< Invalid
		TT_NBC,		//!< Naive Bayes Classifiers
		TT_BPNN,	//!< Backward Propagation Neural Network
		TT_C45,		//!< C4.5 Decision Trees

		//The last two are for ease of processing
		//Do not add anything past them
//		TT_LAST,	//
//		TT_MAX		= TT_LAST - 1
	} TT_ID;
}