////////////////////////////////////////////////////////////////////////////////
/*!	\file		BayesClassifierFileMementoOpt.cpp
**	\brief		Implementation
**	\author		Nathan Frost
**	\date		2/28/2003 7:15:21 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)

#include "BayesClassifier.h"
#include "BayesClassifierFileMemento.h"
using namespace Bayes;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif




////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	operator<< for BCFMRep
/*!	\brief		Overloaded insertion operator for inserting a BCFMRep into an ostream
**	\param		std::ostream &Os:	to insert the data
	\param		TheBCFMRep:			to get the data from
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		3/1/2003 1:07:17 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
std::ostream &operator<<(std::ostream &Os, 
						 const Bayes::BayesClassifierFileMemento::BCFMRep &TheBCFMRep)
{
	Os.write(reinterpret_cast<const char *>(&TheBCFMRep), sizeof(TheBCFMRep));

	return Os;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	operator>> for BCFMRep
/*!	\brief		Overloaded extraction operator for extracting a BCFMRep from an istream
**	\param		std::istream &Is:	to extract the data from
	\param		TheBCFMRep:			to put the data into
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		3/1/2003 1:07:17 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
std::istream &operator>>(std::istream &Is, Bayes::BayesClassifierFileMemento::BCFMRep &TheBCFMRep)
{
	Is.read(reinterpret_cast<char *>(&TheBCFMRep), sizeof(TheBCFMRep));

	return Is;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	PushBack
/*!	\brief		Pushes back a BCFMRep, and increments the total number of BCFMReps that have been
				pushed back
**	\param		numBCFMReps:	The total number of BCFMReps that have been pushed back so far
	\param		bcfmRep:		To push back
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		3/1/2003 2:04:13 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void BayesClassifierFileMemento::PushBack(int &numBCFMReps, BCFMRep &bcfmRep)
{
	++numBCFMReps;
	rep.push_back(bcfmRep);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	MakeFileMemento
/*!	\brief		*this BayesClassifierFileMemento stores TheBayesClassifier's state
**	\param		TheBayesClassifier: to save
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		2/28/2003 9:06:52 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void BayesClassifierFileMemento::MakeFileMemento(const BayesClassifier &TheBayesClassifier)
{
	rep.clear();
	BCFMRep bcfmRep;

	int numBCFMReps = 0;

	//record the total number of training examples
	bcfmRep.theUnsignedInt = TheBayesClassifier.numTrainingExamples;
	PushBack(numBCFMReps, bcfmRep);
	
	//record the number of Choices
	bcfmRep.theUnsignedInt = TheBayesClassifier.choice.size();
	PushBack(numBCFMReps, bcfmRep);

	/* Record all Choice information */
	for(std::vector<BayesClassifier::Choice>::const_iterator it = TheBayesClassifier.choice.begin(); 
		it != TheBayesClassifier.choice.end(); ++it)
	{
		//record this Choice's ID
		bcfmRep.theUnsignedInt = it->id;
		PushBack(numBCFMReps, bcfmRep);

		//record the number of times this Choice has occurred
		bcfmRep.theUnsignedInt = it->numOccurrenceChoice;
		PushBack(numBCFMReps, bcfmRep);

		//record the number of DataFilterKinds in this Choice
		bcfmRep.theUnsignedInt = it->dataFilterKind.size();
		PushBack(numBCFMReps, bcfmRep);

		/* Record all DataFilterKind information */
		for(std::vector<BayesClassifier::DataFilterKind>::const_iterator jt = it->dataFilterKind.begin();
			jt != it->dataFilterKind.end(); ++jt)
		{
			//record this DataFilterKind's ID
			bcfmRep.theUnsignedInt = jt->id;
			PushBack(numBCFMReps, bcfmRep);

			//record the number of Categories in this DataFilterKind
			bcfmRep.theUnsignedInt = jt->category.size();
			PushBack(numBCFMReps, bcfmRep);

			/* Record all Category information */
			for(std::vector<BayesClassifier::Category>::const_iterator kt = jt->category.begin(); 
				kt != jt->category.end(); ++kt)
			{
				//record the lower boundary of the Category
				bcfmRep.theFloat = kt->lowerBound;
				PushBack(numBCFMReps, bcfmRep);

				//record the upper boundary of the Category
				bcfmRep.theFloat = kt->upperBound;
				PushBack(numBCFMReps, bcfmRep);

				//record the number of times this Category has occurred
				bcfmRep.theUnsignedInt = kt->numOccurrences;
				PushBack(numBCFMReps, bcfmRep);
			}
		}
	}

	//record the total number of BCFMReps that comprised the BayesClassifier we just stored
	bcfmRep.theUnsignedInt = numBCFMReps;
	rep.push_front(bcfmRep);

	//BEG_HACK
	BayesClassifier bc = Dereference();
	static int integer;
	char dummy;
	std::string string("BayesClassifier ");
	string += itoa(integer, &dummy, 10);
	string += ".txt";
	std::ofstream os(string.c_str());
	bc.DebugOut(os);
	++integer;
	//END_HACK
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Write
/*!	\brief		Outputs *this BayesClassifierFileMemento's internal state data to an ostream.
**	\param		OutputStream: to output the BayesClassifierFileMemento's internal state data to
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		2/28/2003 9:37:39 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void BayesClassifierFileMemento::Write(std::ostream &OutputStream) const
{
	for(std::deque<BCFMRep>::const_iterator it = rep.begin(); it != rep.end(); ++it)
		OutputStream << *it;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Dereference
/*!	\brief		Returns the BayesClassifier in the state it was stored in.
**	\param		none
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		3/1/2003 11:37:34 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
BayesClassifier BayesClassifierFileMemento::Dereference() const
{
	BayesClassifier ret;
	std::deque<BCFMRep>::const_iterator it = rep.begin();

	//skip past the total number of BCFMReps that comprised the stored BayesClassifier
	++it;

	//retrieve the total number of training examples
	ret.numTrainingExamples = (*it).theUnsignedInt;
	++it;
	
	//retrieve the number of Choices
	unsigned int numChoices = (*it).theUnsignedInt;
	++it;

	/* Retrieve all Choice information */
	for(unsigned int i=0; i < numChoices; ++i)
	{	
		//we're going to overwrite this zeroed out ID with something meaningful
		BayesClassifier::Choice choice(static_cast<Action_ID>(0));

		//retrieve choice's ID -- note that this cast should always be OK
		choice.id = static_cast<Bayes::Action_ID>((*it).theUnsignedInt);
		++it;

		//retrieve the number of times this Choice has occurred
		choice.numOccurrenceChoice = (*it).theUnsignedInt;
		++it;

		//retrieve the number of DataFilterKinds in this Choice
		unsigned int numDataFilterKinds = (*it).theUnsignedInt;
		++it;

		/* Retrieve all DataFilterKind information */
		for(unsigned int j=0; j < numDataFilterKinds; ++j)
		{
			//we're going to overwrite this zeroed out ID with something meaningful
			BayesClassifier::DataFilterKind dataFilterKind(0);

			//retrieve this DataFilterKind's ID
			dataFilterKind.id = (*it).theUnsignedInt;
			++it;

			//retrieve the number of Categories in this DataFilterKind
			unsigned int numCategories = (*it).theUnsignedInt;
			++it;

			/* Retrieve all Category information */
			for(unsigned int k=0; k < numCategories; ++k)
			{
				BayesClassifier::Category category;

				//retrieve the lower boundary of the Category
				category.lowerBound = (*it).theFloat;
				++it;

				//record the upper boundary of the Category
				category.upperBound = (*it).theFloat;
				++it;

				//retrieve the number of times this Category has occurred
				category.numOccurrences = (*it).theUnsignedInt;
				++it;

				dataFilterKind.category.push_back(category);
			}

			choice.dataFilterKind.push_back(dataFilterKind);
		}

		ret.choice.push_back(choice);
	}

	return ret;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	BayesClassifierFileMemento
/*!	\brief		Constructor
**	\param		none
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					
**	\author		Nathan Frost
**	\date		3/1/2003 1:08:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
BayesClassifierFileMemento::BayesClassifierFileMemento()
{
	;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Read
/*!	\brief		Reads the stored state of a BayesClassifier into *this BayesClassifierFileMemento
**	\param		InputStream: the stream to read from
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**					- any previous BayesClassifier state will be overwritten

**	\author		Nathan Frost
**	\date		3/1/2003 3:20:11 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void BayesClassifierFileMemento::Read(std::istream &InputStream)
{
	BCFMRep bcfmRep;

	InputStream >> bcfmRep;
	rep.push_back(bcfmRep);

	int numBCFMReps = bcfmRep.theUnsignedInt;
	for(int i=0; i < numBCFMReps; ++i)
	{
		InputStream >> bcfmRep;
		rep.push_back(bcfmRep);
	}
}


