////////////////////////////////////////////////////////////////////////////////
/*!	\file		BayesADN.h
**	\brief		The BayesClassifier Action Decision Node definition.
**	\author		Nathan Frost
**	\date		11/21/2002 12:53:59 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
					- Action_ID and AI_Attribute MUST be 32bit integral datatypes.
					- Every argument passed to an Action_ID or AI_Attribute parameters MUST be
					unique.
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once

//#include "../ai_core/AI_System.h"
//#include "../ai_core/AI_ADNBase.h"
//#include "BayesClassifier.h"


namespace Bayes
{
	using AI_System::AI_Action;
	using AI_System::AI_ID;


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		BayesADN
	**	\brief		Action Decision Node that uses a Bayes Classifier to learn and make decisions
	**	\author		Nathan Frost
	**	\date		11/21/2002 1:07:02 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	class BayesADN:public AI_System::AI_ADN
	{
		BayesClassifier bayesClassifier;


	protected:
		///overridden member functions
		AI_ID process() throw();
		void saveTrainingExample(	bool response, float fitness, AI_ID action, 
									std::vector<AI_Attribute>& Attributes);

		void subClassInit();


	public:
		BayesADN();


		void DebugOut(ostream &Os);


		//overridden member functions
		bool save() throw();
		bool load() throw();
		void Write(std::ostream&)const;
		void Read(std::istream&);
	};
}
