////////////////////////////////////////////////////////////////////////////////
/*!	\file		BayesClassifierFileMemento.h
**	\brief		The FileMemento that can store a BayesClassifier's internal state, as well as 
				restore that BayesClassifier to that state
**	\author		Nathan Frost
**	\date		2/28/2003 7:15:35 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:

	\todo		
				- eliminate all dependencies on outside datatypes (AI_System::Action_ID, to name 
				but one) by switching to equivalent internal datatypes
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


#include "../fileIO/memento.h"
#include <deque>


namespace Bayes
{
	//forward declarations
	class BayesClassifier;


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		BayesClassifierFileMemento
	**	\brief		Can save a BayesClassifier, and then, at a later date, restore it
	**	\author		Nathan Frost
	**	\date		2/28/2003 7:56:56 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	class BayesClassifierFileMemento:public FileMemento
	{
	public:
		//forward declaration
		union BCFMRep;

	private:
		void PushBack(int &numBCFMReps, BCFMRep &bcfmRep);

	public:
		///nested class declarations
		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		BCFMRep
		**	\brief		The internal representation of a BayesClassifier consists of a series of
						double's and unsigned int's.
		**	\author		Nathan Frost
		**	\date		2/28/2003 8:15:38 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		union BCFMRep
		{
			double theDouble;
			unsigned int theUnsignedInt;
		};


		BayesClassifierFileMemento(std::istream &InputStream);
		void Read(std::istream &InputStream);
		MakeFileMemento(BayesClassifier &TheBayesClassifier);

		void Write(std::ostream &OutputStream) const;
		BayesClassifier Dereference() const;


    private:
        std::deque<BCFMRep> rep;///<the internal state of the BayesClassifier
	};
}

//ancillary functions
std::ostream &operator<<(std::ostream &Os,
						 const Bayes::BayesClassifierFileMemento::BCFMRep &TheBCFMRep);
std::istream &operator>>(std::istream &Is, Bayes::BayesClassifierFileMemento::BCFMRep &TheBCFMRep);
