////////////////////////////////////////////////////////////////////////////////
/*!	\file		BayesUnit.h
**	\brief		The BayesClassifier AI Unit definition.
**	\author		Nathan Frost
**	\date		11/20/2002 8:57:02 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
					- Action_ID and AI_Attribute MUST be 32bit integral datatypes.
					- Every argument passed to an Action_ID or AI_Attribute parameters MUST be
					unique.
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once

//#include "../ai_core/AI_System.h"
//#include "BayesClassifier.h"


namespace Bayes
{
	using AI_System::Action_ID;
	using AI_System::AI_Unit;


	///The only two options available to an AI_Unit
	const Action_ID NO = static_cast<Action_ID>(0);
	const Action_ID YES = static_cast<Action_ID>(1);


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		BayesUnit
	**	\brief		AI Unit that uses a Bayes Classifier to learn and make decisions
	**	\author		Nathan Frost
	**	\date		11/20/2002 9:11:44 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	class BayesUnit:public AI_Unit
	{
		BayesClassifier bayesClassifier;


	protected:
		//overridden member functions
		bool triggerAction() throw();
		void saveTrainingExample(	bool response, float fitness, 
											std::vector<AI_Attribute>& Attributes) throw();
		void subClassInit();

	public:
		BayesUnit();


		void DebugOut(ostream &Os);


		//overridden member functions
		bool save() throw();
		bool load() throw();
		void Write(std::ostream&)const;
		void Read(std::istream&);
	};
}
