////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_DataFilter_Attributes.cpp
**	\brief		Defines all methods declared in AI_DataFilter_Attributes.h
**	\author		Nate Cleveland
**	\date		11/25/2002 5:53:06 PM
**	\par		DESCRIPTION:
**				 Please see AI_DataFilter_Attributes.h for more information.	
**					
*/
////////////////////////////////////////////////////////////////////////////////
#include "../AI_System.h"
using namespace AI_System::DataFilter;
using namespace AI_System;
using namespace InterpretationModule;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////Metabolism///////////////////////////////
//Only need to set the ID of the filter
Metabolism::Metabolism() throw() {ID = AI_System::DF_METABOLISM;}

//See class documentation for details.
float Metabolism::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Metabolism.
		return fishBody->m_pAttributes->GetMetabolism();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Metabolism::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Armor///////////////////////////////
//Only need to set the ID of the filter
Armor::Armor() throw() {ID = AI_System::DF_ARMOR;}

//See class documentation for details.
float Armor::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Armor.
		return fishBody->m_pAttributes->GetArmor();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Armor::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Eyes///////////////////////////////
//Only need to set the ID of the filter
Eyes::Eyes() throw() {ID = AI_System::DF_EYES;}

//See class documentation for details.
float Eyes::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Eyes.
		return fishBody->m_pAttributes->GetEyes();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Eyes::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Teeth_Baleen///////////////////////////////
//Only need to set the ID of the filter
Teeth_Baleen::Teeth_Baleen() throw() {ID = AI_System::DF_TEETH_BALEEN;}

//See class documentation for details.
float Teeth_Baleen::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Teeth_Baleen.
		return fishBody->m_pAttributes->GetTeeth_Baleen();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Teeth_Baleen::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Size///////////////////////////////
//Only need to set the ID of the filter
Size::Size() throw() {ID = AI_System::DF_SIZE;}

//See class documentation for details.
float Size::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Size.
		return fishBody->m_pAttributes->GetSize();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Size::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////ReacionTime///////////////////////////////
//Only need to set the ID of the filter
ReactionTime::ReactionTime() throw() {ID = AI_System::DF_REACTIONTIME;}

//See class documentation for details.
float ReactionTime::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its ReactionTime.
		return fishBody->m_pAttributes->GetReactionTime();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void ReactionTime::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////MaxSpeed///////////////////////////////
//Only need to set the ID of the filter
MaxSpeed::MaxSpeed() throw() {ID = AI_System::DF_MAXSPEED;}

//See class documentation for details.
float MaxSpeed::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its MaxSpeed.
		return fishBody->m_pAttributes->GetMaxSpeed();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void MaxSpeed::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Becum///////////////////////////////
//Only need to set the ID of the filter
Becum::Becum() throw() {ID = AI_System::DF_BECUM;}

//See class documentation for details.
float Becum::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Becum.
		return fishBody->m_pAttributes->GetBecum();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Becum::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////LifeSpan///////////////////////////////
//Only need to set the ID of the filter
LifeSpan::LifeSpan() throw() {ID = AI_System::DF_LIFESPAN;}

//See class documentation for details.
float LifeSpan::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its LifeSpan.
		return fishBody->m_pAttributes->GetLifeSpan();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void LifeSpan::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////AttackStrength///////////////////////////////
//Only need to set the ID of the filter
AttackStrength::AttackStrength() throw() {ID = AI_System::DF_ATTACKSTRENGTH;}

//See class documentation for details.
float AttackStrength::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its AttackStrength.
		return fishBody->m_pAttributes->GetAttackStrength();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void AttackStrength::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Suckability///////////////////////////////
//Only need to set the ID of the filter
Suckability::Suckability() throw() {ID = AI_System::DF_SUCKABILITY;}

//See class documentation for details.
float Suckability::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Suckability.
		return fishBody->m_pAttributes->GetSuckability();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Suckability::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////EnergyLevel///////////////////////////////
//Only need to set the ID of the filter
EnergyLevelObject::EnergyLevelObject() throw() {ID = AI_System::DF_ENERGYLEVEL_OBJECT;}

//See class documentation for details.
float EnergyLevelObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its EnergyLevel.
		return fishBodyObject->m_pAttributes->GetEnergyLevel_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyLevelObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}




/////////////////////////////EnergyLevelTarget///////////////////////////////
//Only need to set the ID of the filter
EnergyLevelTarget::EnergyLevelTarget() throw() {ID = AI_System::DF_ENERGYLEVEL_TARGET;}

//See class documentation for details.
float EnergyLevelTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its EnergyLevel.
		return fishBodyTarget->m_pAttributes->GetEnergyLevel_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyLevelTarget::endPass()
{
	//nothing to do
	//...
}



/////////////////////////////Stomach///////////////////////////////
//Only need to set the ID of the filter
StomachObject::StomachObject() throw() {ID = AI_System::DF_STOMACH_OBJECT;}

//See class documentation for details.
float StomachObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its Stomach.
		return fishBodyObject->m_pAttributes->GetStomach_Normalized() ;
	}

	//otherwise return NULL.
	return 0;
}


//See class documentation for details.
void StomachObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////StomachTarget///////////////////////////////
//Only need to set the ID of the filter
StomachTarget::StomachTarget() throw() {ID = AI_System::DF_STOMACH_TARGET;}

//See class documentation for details.
float StomachTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its Stomach.
		return fishBodyTarget->m_pAttributes->GetStomach_Normalized();
	}

	//otherwise return NULL.
	return 0;
}


//See class documentation for details.
void StomachTarget::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////HealthObject///////////////////////////////
//Only need to set the ID of the filter
HealthObject::HealthObject() throw() {ID = AI_System::DF_HEALTH_OBJECT;}

//See class documentation for details.
float HealthObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its Health.
		return fishBodyObject->m_pAttributes->GetHealth_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}




/////////////////////////////HealthTarget///////////////////////////////
//Only need to set the ID of the filter
HealthTarget::HealthTarget() throw() {ID = AI_System::DF_HEALTH_TARGET;}

//See class documentation for details.
float HealthTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its Health.
		return fishBodyTarget->m_pAttributes->GetHealth_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthTarget::endPass()
{
	//nothing to do
	//...
}



/////////////////////////////MatingInclineObject///////////////////////////////
//Only need to set the ID of the filter
MatingInclineObject::MatingInclineObject() throw() {ID = AI_System::DF_MATINGINCLINE_OBJECT;}

//See class documentation for details.
float MatingInclineObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its MatingIncline.
		return fishBodyObject->m_pAttributes->GetIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void MatingInclineObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}




/////////////////////////////MatingInclineTarget///////////////////////////////
//Only need to set the ID of the filter
MatingInclineTarget::MatingInclineTarget() throw() {ID = AI_System::DF_MATINGINCLINE_TARGET;}

//See class documentation for details.
float MatingInclineTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its MatingIncline.
		return fishBodyTarget->m_pAttributes->GetIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void MatingInclineTarget::endPass()
{
	//nothing to do
	//...
}



/////////////////////////////StomachNormalizedObject///////////////////////////////
//Only need to set the ID of the filter
StomachNormalizedObject::StomachNormalizedObject() throw() {ID = AI_System::DF_STOMACH_NORMALIZED_OBJECT;}

//See class documentation for details.
float StomachNormalizedObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its StomachNormalized.
		return fishBodyObject->m_pAttributes->GetStomach_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachNormalizedObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}



/////////////////////////////StomachNormalizedTarget///////////////////////////////
//Only need to set the ID of the filter
StomachNormalizedTarget::StomachNormalizedTarget() throw() {ID = AI_System::DF_STOMACH_NORMALIZED_TARGET;}

//See class documentation for details.
float StomachNormalizedTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its StomachNormalized.
		return fishBodyTarget->m_pAttributes->GetStomach_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachNormalizedTarget::endPass()
{
	//nothing to do
	//...
}



/////////////////////////////HealthNormalizedObject///////////////////////////////
//Only need to set the ID of the filter
HealthNormalizedObject::HealthNormalizedObject() throw() {ID = AI_System::DF_HEALTH_NORMALIZED_OBJECT;}

//See class documentation for details.
float HealthNormalizedObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its HealthNormalized.
		return fishBodyObject->m_pAttributes->GetHealth_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthNormalizedObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}



/////////////////////////////HealthNormalizedTarget///////////////////////////////
//Only need to set the ID of the filter
HealthNormalizedTarget::HealthNormalizedTarget() throw() {ID = AI_System::DF_HEALTH_NORMALIZED_TARGET;}

//See class documentation for details.
float HealthNormalizedTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its HealthNormalized.
		return fishBodyTarget->m_pAttributes->GetHealth_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthNormalizedTarget::endPass()
{
	//nothing to do
	//...
}



/////////////////////////////EnergyNormalizedObject///////////////////////////////
//Only need to set the ID of the filter
EnergyNormalizedObject::EnergyNormalizedObject() throw() {ID = AI_System::DF_ENERGY_NORMALIZED_OBJECT;}

//See class documentation for details.
float EnergyNormalizedObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyObject = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyObject)
	{
		//since this object was a fishBody, return its EnergyNormalized.
		return fishBodyObject->m_pAttributes->GetEnergyLevel_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyNormalizedObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}



/////////////////////////////EnergyNormalized///////////////////////////////
//Only need to set the ID of the filter
EnergyNormalizedTarget::EnergyNormalizedTarget() throw() {ID = AI_System::DF_ENERGY_NORMALIZED_TARGET;}

//See class documentation for details.
float EnergyNormalizedTarget::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBodyTarget = MH->GetFishBody(PG->getTarget());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBodyTarget)
	{
		//since this object was a fishBody, return its EnergyNormalized.
		return fishBodyTarget->m_pAttributes->GetEnergyLevel_Normalized();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyNormalizedTarget::endPass()
{
	//nothing to do
	//...
}