////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_DataFilterBase.h
**	\brief		Declares all Data filter class dealing with Fish Attributes (not 
**				 AI Attributes, they are two different things).
**
**	\author		Nate Cleveland
**	\date		11/25/2002 1:22:13 PM
**	\par		DESCRIPTION: 
				 Defines filters for the following items. If a filter is for fish
				  only it will return 0 for all non-fish objects that are evaluated.
					- Object Metabolism						(fish only)
					- Object Armor							(fish only)
					- Object Eyes							(fish only)
					- Object teeth/baleen					(fish only)
					- Object size 
					- Object reaction time					(fish only)
					- Object max speed						(fish only)
					- Object BECUM							(fish only)
					- Object life span						(fish only)
					- Object attack strength				(fish only)
					- Object Suckablity						(fish only)
					- Object and Target energy level		(fish only)
					- Object and Target health				(fish only)
					- Object and Target stomach				(fish only)
					- Object and Target mating inclinition	(fish only)
					- Object and Target normalized energy level	(fish only)
					- Object and Target normalized health	(fish only)
					- Object and Target normalized stomach 	(fish only)
**					
**	\par		REMARKS:
				 Recall that "Object" refers to the object that the fish is thinking
				  about and that "Target" is the fish that is doing the thinking.

	\todo		At this time size only works for fish. In the future it should also 
				 be extended to understand chum and items. (item is defined to be
				 anything that can be picked up).
**					
*/
////////////////////////////////////////////////////////////////////////////////
//#include "AI_System.h"
#pragma once


namespace AI_System
{
	namespace DataFilter
	{

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Metabolism
		**	\brief		Processes the Metabolism of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Metabolism: public AI_System::AI_DataFilter
		{
		public:
			Metabolism() throw();
			float processData();
			void endPass();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Armor
		**	\brief		Processes the Armor of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Armor: public AI_System::AI_DataFilter
		{
		public:
			Armor()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Eyes
		**	\brief		Processes the Eyes of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Eyes: public AI_System::AI_DataFilter
		{
		public:
			Eyes()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Teeth_Baleen
		**	\brief		Processes the Teeth_Baleen of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Teeth_Baleen: public AI_System::AI_DataFilter
		{
		public:
			Teeth_Baleen()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Size
		**	\brief		Processes the Size of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Size: public AI_System::AI_DataFilter
		{
		public:
			Size()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		ReactionTime
		**	\brief		Processes the ReactionTime of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class ReactionTime: public AI_System::AI_DataFilter
		{
		public:
			ReactionTime()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		MaxSpeed
		**	\brief		Processes the MaxSpeed of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class MaxSpeed: public AI_System::AI_DataFilter
		{
		public:
			MaxSpeed()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Becum
		**	\brief		Processes the Becum of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Becum: public AI_System::AI_DataFilter
		{
		public:
			Becum()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		LifeSpan
		**	\brief		Processes the LifeSpan of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class LifeSpan: public AI_System::AI_DataFilter
		{
		public:
			LifeSpan()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		AttackStrength
		**	\brief		Processes the AttackStrength of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class AttackStrength: public AI_System::AI_DataFilter
		{
		public:
			AttackStrength()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Suckability
		**	\brief		Processes the Suckability of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Suckability: public AI_System::AI_DataFilter
		{
		public:
			Suckability()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyLevelObject
		**	\brief		Processes the EnergyLevel of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyLevelObject: public AI_System::AI_DataFilter
		{
		public:
			EnergyLevelObject()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyLevelTarget
		**	\brief		Processes the EnergyLevel of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyLevelTarget: public AI_System::AI_DataFilter
		{
		public:
			EnergyLevelTarget()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachObject
		**	\brief		Processes the Stomach of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachObject: public AI_System::AI_DataFilter
		{
		public:
			StomachObject()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachTarget
		**	\brief		Processes the Stomach of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachTarget: public AI_System::AI_DataFilter
		{
		public:
			StomachTarget()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthObject
		**	\brief		Processes the Health of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthObject: public AI_System::AI_DataFilter
		{
		public:
			HealthObject()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthTarget
		**	\brief		Processes the Health of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthTarget: public AI_System::AI_DataFilter
		{
		public:
			HealthTarget()throw();
			float processData();
			void endPass();			
		};

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		MatingIncline
		**	\brief		Processes the MatingIncline of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class MatingInclineObject: public AI_System::AI_DataFilter
		{
		public:
			MatingInclineObject()throw();
			float processData();
			void endPass();			
		};
		

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		MatingIncline
		**	\brief		Processes the MatingIncline of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class MatingInclineTarget: public AI_System::AI_DataFilter
		{
		public:
			MatingInclineTarget()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachNormalizedObject
		**	\brief		Processes the StomachNormalized of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachNormalizedObject: public AI_System::AI_DataFilter
		{
		public:
			StomachNormalizedObject()throw();
			float processData();
			void endPass();			
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachNormalized
		**	\brief		Processes the StomachNormalized of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachNormalizedTarget: public AI_System::AI_DataFilter
		{
		public:
			StomachNormalizedTarget()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthNormalizedObject
		**	\brief		Processes the HealthNormalized of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthNormalizedObject: public AI_System::AI_DataFilter
		{
		public:
			HealthNormalizedObject()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthNormalizedTarget
		**	\brief		Processes the HealthNormalized of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthNormalizedTarget: public AI_System::AI_DataFilter
		{
		public:
			HealthNormalizedTarget()throw();
			float processData();
			void endPass();			
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyNormalizedObject
		**	\brief		Processes the EnergyNormalized of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyNormalizedObject: public AI_System::AI_DataFilter
		{
		public:
			EnergyNormalizedObject()throw();
			float processData();
			void endPass();			
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyNormalizedTarget
		**	\brief		Processes the EnergyNormalized of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 2:14:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyNormalizedTarget: public AI_System::AI_DataFilter
		{
		public:
			EnergyNormalizedTarget()throw();
			float processData();
			void endPass();			
		};
	}//end namespace DataFilter.
}//end namespace AI_System



