////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_DataFilter_Stats.cpp
**	\brief		Defines all methods declared in AI_DataFilter_Stats.h
**	\author		Nate Cleveland
**	\date		11/25/2002 5:53:06 PM
**	\par		DESCRIPTION:
**				 Please see AI_DataFilter_Attributes.h for more information.	
**					
*/
////////////////////////////////////////////////////////////////////////////////
#include "../AI_System.h"
using namespace AI_System::DataFilter;
using namespace AI_System;
using namespace InterpretationModule;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif




/////////////////////////////SpeedCountObject///////////////////////////////
//Only need to set the ID of the filter
SpeedCountObject::SpeedCountObject() throw() {ID = AI_System::DF_SPEED_COUNT_OBJECT;}

//See class documentation for details.
float SpeedCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its SpeedCountObject.
		return fishBody->m_pStats->GetSpeed();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void SpeedCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////SpeedCountTarget///////////////////////////////
//Only need to set the ID of the filter
SpeedCountTarget::SpeedCountTarget() throw() {ID = AI_System::DF_SPEED_COUNT_TARGET;}

//See class documentation for details.
float SpeedCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its SpeedCountTarget.
		return fishBody->m_pStats->GetSpeed();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void SpeedCountTarget::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////DistanceCountObject///////////////////////////////
//Only need to set the ID of the filter
DistanceCountObject::DistanceCountObject() throw() {ID = AI_System::DF_DISTANCE_COUNT_OBJECT;}

//See class documentation for details.
float DistanceCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its DistanceCountObject.
		return fishBody->m_pStats->GetDistance();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void DistanceCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////DistanceCountTarget///////////////////////////////
//Only need to set the ID of the filter
DistanceCountTarget::DistanceCountTarget() throw() {ID = AI_System::DF_DISTANCE_COUNT_TARGET;}

//See class documentation for details.
float DistanceCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its DistanceCountTarget.
		return fishBody->m_pStats->GetAvgDistance();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void DistanceCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////HealthCountObject///////////////////////////////
//Only need to set the ID of the filter
HealthCountObject::HealthCountObject() throw() {ID = AI_System::DF_HEALTH_COUNT_OBJECT;}

//See class documentation for details.
float HealthCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its HealthCountObject.
		return fishBody->m_pStats->GetHealth();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////HealthCountTarget///////////////////////////////
//Only need to set the ID of the filter
HealthCountTarget::HealthCountTarget() throw() {ID = AI_System::DF_HEALTH_COUNT_TARGET;}

//See class documentation for details.
float HealthCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its HealthCountTarget.
		return fishBody->m_pStats->GetHealth();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////InclineCountObject///////////////////////////////
//Only need to set the ID of the filter
InclineCountObject::InclineCountObject() throw() {ID = AI_System::DF_INCLINE_COUNT_OBJECT;}

//See class documentation for details.
float InclineCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its InclineCountObject.
		return fishBody->m_pStats->GetIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void InclineCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////InclineCountTarget///////////////////////////////
//Only need to set the ID of the filter
InclineCountTarget::InclineCountTarget() throw() {ID = AI_System::DF_INCLINE_COUNT_TARGET;}

//See class documentation for details.
float InclineCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its InclineCountTarget.
		return fishBody->m_pStats->GetIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void InclineCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////StomachCountObject///////////////////////////////
//Only need to set the ID of the filter
StomachCountObject::StomachCountObject() throw() {ID = AI_System::DF_STOMACH_COUNT_OBJECT;}

//See class documentation for details.
float StomachCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its StomachCountObject.
		return fishBody->m_pStats->GetStomach();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////StomachCountTarget///////////////////////////////
//Only need to set the ID of the filter
StomachCountTarget::StomachCountTarget() throw() {ID = AI_System::DF_STOMACH_COUNT_TARGET;}

//See class documentation for details.
float StomachCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its StomachCountTarget.
		return fishBody->m_pStats->GetStomach();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////EnergyCountObject///////////////////////////////
//Only need to set the ID of the filter
EnergyCountObject::EnergyCountObject() throw() {ID = AI_System::DF_ENERGY_COUNT_OBJECT;}

//See class documentation for details.
float EnergyCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its EnergyCountObject.
		return fishBody->m_pStats->GetEnergy();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////EnergyCountTarget///////////////////////////////
//Only need to set the ID of the filter
EnergyCountTarget::EnergyCountTarget() throw() {ID = AI_System::DF_ENERGY_COUNT_TARGET;}

//See class documentation for details.
float EnergyCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its EnergyCountTarget.
		return fishBody->m_pStats->GetEnergy();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////KillsCountObject///////////////////////////////
//Only need to set the ID of the filter
KillsCountObject::KillsCountObject() throw() {ID = AI_System::DF_KILLS_COUNT_OBJECT;}

//See class documentation for details.
float KillsCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its KillsCountObject.
		return fishBody->m_pStats->GetKills();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void KillsCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////KillsCountTarget///////////////////////////////
//Only need to set the ID of the filter
KillsCountTarget::KillsCountTarget() throw() {ID = AI_System::DF_KILLS_COUNT_TARGET;}

//See class documentation for details.
float KillsCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its KillsCountTarget.
		return fishBody->m_pStats->GetKills();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void KillsCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////OffspringCountObject///////////////////////////////
//Only need to set the ID of the filter
OffspringCountObject::OffspringCountObject() throw() {ID = AI_System::DF_OFFSPRING_COUNT_OBJECT;}

//See class documentation for details.
float OffspringCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its OffspringCountObject.
		return fishBody->m_pStats->GetOffspring();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void OffspringCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////OffspringCountTarget///////////////////////////////
//Only need to set the ID of the filter
OffspringCountTarget::OffspringCountTarget() throw() {ID = AI_System::DF_OFFSPRING_COUNT_TARGET;}

//See class documentation for details.
float OffspringCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its OffspringCountTarget.
		return fishBody->m_pStats->GetOffspring();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void OffspringCountTarget::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Been_AttackedCountObject///////////////////////////////
//Only need to set the ID of the filter
Been_AttackedCountObject::Been_AttackedCountObject() throw() {ID = AI_System::DF_BEEN_ATTACKED_COUNT_OBJECT;}

//See class documentation for details.
float Been_AttackedCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Been_AttackedCountObject.
		return fishBody->m_pStats->GetBeen_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Been_AttackedCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Been_AttackedCountTarget///////////////////////////////
//Only need to set the ID of the filter
Been_AttackedCountTarget::Been_AttackedCountTarget() throw() {ID = AI_System::DF_BEEN_ATTACKED_COUNT_TARGET;}

//See class documentation for details.
float Been_AttackedCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Been_AttackedCountTarget.
		return fishBody->m_pStats->GetBeen_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Been_AttackedCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////Has_AttackedCountObject///////////////////////////////
//Only need to set the ID of the filter
Has_AttackedCountObject::Has_AttackedCountObject() throw() {ID = AI_System::DF_HAS_ATTACKED_COUNT_OBJECT;}

//See class documentation for details.
float Has_AttackedCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Has_AttackedCountObject.
		return fishBody->m_pStats->GetHas_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Has_AttackedCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Has_AttackedCountTarget///////////////////////////////
//Only need to set the ID of the filter
Has_AttackedCountTarget::Has_AttackedCountTarget() throw() {ID = AI_System::DF_HAS_ATTACKED_COUNT_TARGET;}

//See class documentation for details.
float Has_AttackedCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Has_AttackedCountTarget.
		return fishBody->m_pStats->GetHas_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Has_AttackedCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////PlanktonCountObject///////////////////////////////
//Only need to set the ID of the filter
PlanktonCountObject::PlanktonCountObject() throw() {ID = AI_System::DF_PLANKTON_COUNT_OBJECT;}

//See class documentation for details.
float PlanktonCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its PlanktonCountObject.
		return fishBody->m_pStats->GetPlankton();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void PlanktonCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////PlanktonCountTarget///////////////////////////////
//Only need to set the ID of the filter
PlanktonCountTarget::PlanktonCountTarget() throw() {ID = AI_System::DF_PLANKTON_COUNT_TARGET;}

//See class documentation for details.
float PlanktonCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its PlanktonCountTarget.
		return fishBody->m_pStats->GetPlankton();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void PlanktonCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////ChumCountObject///////////////////////////////
//Only need to set the ID of the filter
ChumCountObject::ChumCountObject() throw() {ID = AI_System::DF_CHUM_COUNT_OBJECT;}

//See class documentation for details.
float ChumCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its ChumCountObject.
		return fishBody->m_pStats->GetChum();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void ChumCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////ChumCountTarget///////////////////////////////
//Only need to set the ID of the filter
ChumCountTarget::ChumCountTarget() throw() {ID = AI_System::DF_CHUM_COUNT_TARGET;}

//See class documentation for details.
float ChumCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its ChumCountTarget.
		return fishBody->m_pStats->GetChum();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void ChumCountTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////Total_FoodCountObject///////////////////////////////
//Only need to set the ID of the filter
Total_FoodCountObject::Total_FoodCountObject() throw() {ID = AI_System::DF_TOTAL_FOOD_COUNT_OBJECT;}

//See class documentation for details.
float Total_FoodCountObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Total_FoodCountObject.
		return fishBody->m_pStats->GetTotal_Food();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Total_FoodCountObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Total_FoodCountTarget///////////////////////////////
//Only need to set the ID of the filter
Total_FoodCountTarget::Total_FoodCountTarget() throw() {ID = AI_System::DF_TOTAL_FOOD_COUNT_TARGET;}

//See class documentation for details.
float Total_FoodCountTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Total_FoodCountTarget.
		return fishBody->m_pStats->GetTotal_Food();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Total_FoodCountTarget::endPass()
{
	//nothing to do
	//...
}


/////////////////////////////////////////////
/////////////////////////RATE////////////////
/////////////////////////////////////////////


/////////////////////////////SpeedRateObject///////////////////////////////
//Only need to set the ID of the filter
SpeedRateObject::SpeedRateObject() throw() {ID = AI_System::DF_SPEED_RATE_OBJECT;}

//See class documentation for details.
float SpeedRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its SpeedRateObject.
		return fishBody->m_pStats->GetAvgSpeed();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void SpeedRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////SpeedRateTarget///////////////////////////////
//Only need to set the ID of the filter
SpeedRateTarget::SpeedRateTarget() throw() {ID = AI_System::DF_SPEED_RATE_TARGET;}

//See class documentation for details.
float SpeedRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its SpeedRateTarget.
		return fishBody->m_pStats->GetAvgSpeed();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void SpeedRateTarget::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////DistanceRateObject///////////////////////////////
//Only need to set the ID of the filter
DistanceRateObject::DistanceRateObject() throw() {ID = AI_System::DF_DISTANCE_RATE_OBJECT;}

//See class documentation for details.
float DistanceRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its DistanceRateObject.
		return fishBody->m_pStats->GetAvgDistance();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void DistanceRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////DistanceRateTarget///////////////////////////////
//Only need to set the ID of the filter
DistanceRateTarget::DistanceRateTarget() throw() {ID = AI_System::DF_DISTANCE_RATE_TARGET;}

//See class documentation for details.
float DistanceRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its DistanceRateTarget.
		return fishBody->m_pStats->GetAvgDistance();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void DistanceRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////HealthRateObject///////////////////////////////
//Only need to set the ID of the filter
HealthRateObject::HealthRateObject() throw() {ID = AI_System::DF_HEALTH_RATE_OBJECT;}

//See class documentation for details.
float HealthRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its HealthRateObject.
		return fishBody->m_pStats->GetAvgHealth();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////HealthRateTarget///////////////////////////////
//Only need to set the ID of the filter
HealthRateTarget::HealthRateTarget() throw() {ID = AI_System::DF_HEALTH_RATE_TARGET;}

//See class documentation for details.
float HealthRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its HealthRateTarget.
		return fishBody->m_pStats->GetAvgHealth();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void HealthRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////InclineRateObject///////////////////////////////
//Only need to set the ID of the filter
InclineRateObject::InclineRateObject() throw() {ID = AI_System::DF_INCLINE_RATE_OBJECT;}

//See class documentation for details.
float InclineRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its InclineRateObject.
		return fishBody->m_pStats->GetAvgIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void InclineRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////InclineRateTarget///////////////////////////////
//Only need to set the ID of the filter
InclineRateTarget::InclineRateTarget() throw() {ID = AI_System::DF_INCLINE_RATE_TARGET;}

//See class documentation for details.
float InclineRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its InclineRateTarget.
		return fishBody->m_pStats->GetAvgIncline();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void InclineRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////StomachRateObject///////////////////////////////
//Only need to set the ID of the filter
StomachRateObject::StomachRateObject() throw() {ID = AI_System::DF_STOMACH_RATE_OBJECT;}

//See class documentation for details.
float StomachRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its StomachRateObject.
		return fishBody->m_pStats->GetAvgStomach();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////StomachRateTarget///////////////////////////////
//Only need to set the ID of the filter
StomachRateTarget::StomachRateTarget() throw() {ID = AI_System::DF_STOMACH_RATE_TARGET;}

//See class documentation for details.
float StomachRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its StomachRateTarget.
		return fishBody->m_pStats->GetAvgStomach();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void StomachRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////EnergyRateObject///////////////////////////////
//Only need to set the ID of the filter
EnergyRateObject::EnergyRateObject() throw() {ID = AI_System::DF_ENERGY_RATE_OBJECT;}

//See class documentation for details.
float EnergyRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its EnergyRateObject.
		return fishBody->m_pStats->GetAvgEnergy();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////EnergyRateTarget///////////////////////////////
//Only need to set the ID of the filter
EnergyRateTarget::EnergyRateTarget() throw() {ID = AI_System::DF_ENERGY_RATE_TARGET;}

//See class documentation for details.
float EnergyRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its EnergyRateTarget.
		return fishBody->m_pStats->GetAvgEnergy();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void EnergyRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////KillsRateObject///////////////////////////////
//Only need to set the ID of the filter
KillsRateObject::KillsRateObject() throw() {ID = AI_System::DF_KILLS_RATE_OBJECT;}

//See class documentation for details.
float KillsRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its KillsRateObject.
		return fishBody->m_pStats->GetAvgKills();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void KillsRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////KillsRateTarget///////////////////////////////
//Only need to set the ID of the filter
KillsRateTarget::KillsRateTarget() throw() {ID = AI_System::DF_KILLS_RATE_TARGET;}

//See class documentation for details.
float KillsRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its KillsRateTarget.
		return fishBody->m_pStats->GetAvgKills();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void KillsRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////OffspringRateObject///////////////////////////////
//Only need to set the ID of the filter
OffspringRateObject::OffspringRateObject() throw() {ID = AI_System::DF_OFFSPRING_RATE_OBJECT;}

//See class documentation for details.
float OffspringRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its OffspringRateObject.
		return fishBody->m_pStats->GetAvgOffspring();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void OffspringRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////OffspringRateTarget///////////////////////////////
//Only need to set the ID of the filter
OffspringRateTarget::OffspringRateTarget() throw() {ID = AI_System::DF_OFFSPRING_RATE_TARGET;}

//See class documentation for details.
float OffspringRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its OffspringRateTarget.
		return fishBody->m_pStats->GetAvgOffspring();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void OffspringRateTarget::endPass()
{
	//nothing to do
	//...
}




/////////////////////////////Been_AttackedRateObject///////////////////////////////
//Only need to set the ID of the filter
Been_AttackedRateObject::Been_AttackedRateObject() throw() {ID = AI_System::DF_BEEN_ATTACKED_RATE_OBJECT;}

//See class documentation for details.
float Been_AttackedRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Been_AttackedRateObject.
		return fishBody->m_pStats->GetAvgBeen_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Been_AttackedRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Been_AttackedRateTarget///////////////////////////////
//Only need to set the ID of the filter
Been_AttackedRateTarget::Been_AttackedRateTarget() throw() {ID = AI_System::DF_BEEN_ATTACKED_RATE_TARGET;}

//See class documentation for details.
float Been_AttackedRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Been_AttackedRateTarget.
		return fishBody->m_pStats->GetAvgBeen_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Been_AttackedRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////Has_AttackedRateObject///////////////////////////////
//Only need to set the ID of the filter
Has_AttackedRateObject::Has_AttackedRateObject() throw() {ID = AI_System::DF_HAS_ATTACKED_RATE_OBJECT;}

//See class documentation for details.
float Has_AttackedRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Has_AttackedRateObject.
		return fishBody->m_pStats->GetAvgHas_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Has_AttackedRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Has_AttackedRateTarget///////////////////////////////
//Only need to set the ID of the filter
Has_AttackedRateTarget::Has_AttackedRateTarget() throw() {ID = AI_System::DF_HAS_ATTACKED_RATE_TARGET;}

//See class documentation for details.
float Has_AttackedRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Has_AttackedRateTarget.
		return fishBody->m_pStats->GetAvgHas_Attacked();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Has_AttackedRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////PlanktonRateObject///////////////////////////////
//Only need to set the ID of the filter
PlanktonRateObject::PlanktonRateObject() throw() {ID = AI_System::DF_PLANKTON_RATE_OBJECT;}

//See class documentation for details.
float PlanktonRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its PlanktonRateObject.
		return fishBody->m_pStats->GetAvgPlankton();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void PlanktonRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////PlanktonRateTarget///////////////////////////////
//Only need to set the ID of the filter
PlanktonRateTarget::PlanktonRateTarget() throw() {ID = AI_System::DF_PLANKTON_RATE_TARGET;}

//See class documentation for details.
float PlanktonRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its PlanktonRateTarget.
		return fishBody->m_pStats->GetAvgPlankton();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void PlanktonRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////ChumRateObject///////////////////////////////
//Only need to set the ID of the filter
ChumRateObject::ChumRateObject() throw() {ID = AI_System::DF_CHUM_RATE_OBJECT;}

//See class documentation for details.
float ChumRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its ChumRateObject.
		return fishBody->m_pStats->GetAvgChum();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void ChumRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////ChumRateTarget///////////////////////////////
//Only need to set the ID of the filter
ChumRateTarget::ChumRateTarget() throw() {ID = AI_System::DF_CHUM_RATE_TARGET;}

//See class documentation for details.
float ChumRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its ChumRateTarget.
		return fishBody->m_pStats->GetAvgChum();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void ChumRateTarget::endPass()
{
	//nothing to do
	//...
}





/////////////////////////////Total_FoodRateObject///////////////////////////////
//Only need to set the ID of the filter
Total_FoodRateObject::Total_FoodRateObject() throw() {ID = AI_System::DF_TOTAL_FOOD_RATE_OBJECT;}

//See class documentation for details.
float Total_FoodRateObject::processData()
{
	//Get a pointer to this object iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getCurrentObject());

	//If the object ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this object was a fishBody, return its Total_FoodRateObject.
		return fishBody->m_pStats->GetAvgTotal_Food();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Total_FoodRateObject::endPass()
{
	//Since this is an object being evaluated make sure to always recalculate.
	dataProcessed = false;
}


/////////////////////////////Total_FoodRateTarget///////////////////////////////
//Only need to set the ID of the filter
Total_FoodRateTarget::Total_FoodRateTarget() throw() {ID = AI_System::DF_TOTAL_FOOD_RATE_TARGET;}

//See class documentation for details.
float Total_FoodRateTarget::processData()
{
	//Get a pointer to this Target iff it is a fishBody.
	const FishBody* fishBody = MH->GetFishBody(PG->getTarget());

	//If the Target ID wasn't a FishBody, the var fishBody will be null.
	if(fishBody)
	{
		//since this Target was a fishBody, return its Total_FoodRateTarget.
		return fishBody->m_pStats->GetAvgTotal_Food();
	}

	//otherwise return NULL.
	return 0;
}

//See class documentation for details.
void Total_FoodRateTarget::endPass()
{
	//nothing to do
	//...
}
