////////////////////////////////////////////////////////////////////////////////
/*!	\file		AI_DataFilterBase.h
**	\brief		Declares all Data filter classes dealing with Fish Statistics.
**
**	\author		Nate Cleveland
**	\date		11/25/2002 1:22:13 PM
**	\par		DESCRIPTION: 
				 Defines filters for the following items. 
**					
**	\par		REMARKS:
				 Recall that "Object" refers to the object that the fish is thinking
				  about and that "Target" is the fish that is doing the thinking.

**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once

namespace AI_System
{
	namespace DataFilter
	{

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		SpeedCountObject
		**	\brief		Processes the Speed Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class SpeedCountObject: public AI_System::AI_DataFilter
		{
		public:
			SpeedCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		SpeedCountTarget
		**	\brief		Processes the Speed Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class SpeedCountTarget: public AI_System::AI_DataFilter
		{
		public:
			SpeedCountTarget() throw();
			float processData();
			void endPass();
		};	





		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		DistanceCountObject
		**	\brief		Processes the Distance Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class DistanceCountObject: public AI_System::AI_DataFilter
		{
		public:
			DistanceCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		DistanceCountTarget
		**	\brief		Processes the Distance Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class DistanceCountTarget: public AI_System::AI_DataFilter
		{
		public:
			DistanceCountTarget() throw();
			float processData();
			void endPass();
		};





		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthCountObject
		**	\brief		Processes the Health Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthCountObject: public AI_System::AI_DataFilter
		{
		public:
			HealthCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthCountTarget
		**	\brief		Processes the Health Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthCountTarget: public AI_System::AI_DataFilter
		{
		public:
			HealthCountTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		InclineCountObject
		**	\brief		Processes the Incline Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class InclineCountObject: public AI_System::AI_DataFilter
		{
		public:
			InclineCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		InclineCountTarget
		**	\brief		Processes the Incline Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class InclineCountTarget: public AI_System::AI_DataFilter
		{
		public:
			InclineCountTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachCountObject
		**	\brief		Processes the Stomach Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachCountObject: public AI_System::AI_DataFilter
		{
		public:
			StomachCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachCountTarget
		**	\brief		Processes the Stomach Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachCountTarget: public AI_System::AI_DataFilter
		{
		public:
			StomachCountTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyCountObject
		**	\brief		Processes the Energy Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyCountObject: public AI_System::AI_DataFilter
		{
		public:
			EnergyCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyCountTarget
		**	\brief		Processes the Energy Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyCountTarget: public AI_System::AI_DataFilter
		{
		public:
			EnergyCountTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		KillsCountObject
		**	\brief		Processes the Kills Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class KillsCountObject: public AI_System::AI_DataFilter
		{
		public:
			KillsCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		KillsCountTarget
		**	\brief		Processes the Kills Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class KillsCountTarget: public AI_System::AI_DataFilter
		{
		public:
			KillsCountTarget() throw();
			float processData();
			void endPass();
		};		
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		OffspringCountObject
		**	\brief		Processes the Offspring Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class OffspringCountObject: public AI_System::AI_DataFilter
		{
		public:
			OffspringCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		OffspringCountTarget
		**	\brief		Processes the Offspring Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class OffspringCountTarget: public AI_System::AI_DataFilter
		{
		public:
			OffspringCountTarget() throw();
			float processData();
			void endPass();
		};
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Been_AttackedCountObject
		**	\brief		Processes the Been_Attacked Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Been_AttackedCountObject: public AI_System::AI_DataFilter
		{
		public:
			Been_AttackedCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Been_AttackedCountTarget
		**	\brief		Processes the Been_Attacked Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Been_AttackedCountTarget: public AI_System::AI_DataFilter
		{
		public:
			Been_AttackedCountTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Has_AttackedCountObject
		**	\brief		Processes the Has_Attacked Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Has_AttackedCountObject: public AI_System::AI_DataFilter
		{
		public:
			Has_AttackedCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Has_AttackedCountTarget
		**	\brief		Processes the Has_Attacked Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Has_AttackedCountTarget: public AI_System::AI_DataFilter
		{
		public:
			Has_AttackedCountTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		PlanktonCountObject
		**	\brief		Processes the Plankton Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class PlanktonCountObject: public AI_System::AI_DataFilter
		{
		public:
			PlanktonCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		PlanktonCountTarget
		**	\brief		Processes the Plankton Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class PlanktonCountTarget: public AI_System::AI_DataFilter
		{
		public:
			PlanktonCountTarget() throw();
			float processData();
			void endPass();
		};		
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		ChumCountObject
		**	\brief		Processes the Chum Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class ChumCountObject: public AI_System::AI_DataFilter
		{
		public:
			ChumCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		ChumCountTarget
		**	\brief		Processes the Chum Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class ChumCountTarget: public AI_System::AI_DataFilter
		{
		public:
			ChumCountTarget() throw();
			float processData();
			void endPass();
		};
		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Total_FoodCountObject
		**	\brief		Processes the Total_Food Count of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Total_FoodCountObject: public AI_System::AI_DataFilter
		{
		public:
			Total_FoodCountObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Total_FoodCountTarget
		**	\brief		Processes the Total_Food Count of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Total_FoodCountTarget: public AI_System::AI_DataFilter
		{
		public:
			Total_FoodCountTarget() throw();
			float processData();
			void endPass();
		};	
		

		///////////////////////////////RATE/////////////////////////////
		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		SpeedRateObject
		**	\brief		Processes the Speed Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class SpeedRateObject: public AI_System::AI_DataFilter
		{
		public:
			SpeedRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		SpeedRateTarget
		**	\brief		Processes the Speed Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class SpeedRateTarget: public AI_System::AI_DataFilter
		{
		public:
			SpeedRateTarget() throw();
			float processData();
			void endPass();
		};	





		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		DistanceRateObject
		**	\brief		Processes the Distance Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class DistanceRateObject: public AI_System::AI_DataFilter
		{
		public:
			DistanceRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		DistanceRateTarget
		**	\brief		Processes the Distance Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class DistanceRateTarget: public AI_System::AI_DataFilter
		{
		public:
			DistanceRateTarget() throw();
			float processData();
			void endPass();
		};





		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthRateObject
		**	\brief		Processes the Health Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthRateObject: public AI_System::AI_DataFilter
		{
		public:
			HealthRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		HealthRateTarget
		**	\brief		Processes the Health Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class HealthRateTarget: public AI_System::AI_DataFilter
		{
		public:
			HealthRateTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		InclineRateObject
		**	\brief		Processes the Incline Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class InclineRateObject: public AI_System::AI_DataFilter
		{
		public:
			InclineRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		InclineRateTarget
		**	\brief		Processes the Incline Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class InclineRateTarget: public AI_System::AI_DataFilter
		{
		public:
			InclineRateTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachRateObject
		**	\brief		Processes the Stomach Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachRateObject: public AI_System::AI_DataFilter
		{
		public:
			StomachRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		StomachRateTarget
		**	\brief		Processes the Stomach Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class StomachRateTarget: public AI_System::AI_DataFilter
		{
		public:
			StomachRateTarget() throw();
			float processData();
			void endPass();
		};




		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyRateObject
		**	\brief		Processes the Energy Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyRateObject: public AI_System::AI_DataFilter
		{
		public:
			EnergyRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		EnergyRateTarget
		**	\brief		Processes the Energy Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class EnergyRateTarget: public AI_System::AI_DataFilter
		{
		public:
			EnergyRateTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		KillsRateObject
		**	\brief		Processes the Kills Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class KillsRateObject: public AI_System::AI_DataFilter
		{
		public:
			KillsRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		KillsRateTarget
		**	\brief		Processes the Kills Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class KillsRateTarget: public AI_System::AI_DataFilter
		{
		public:
			KillsRateTarget() throw();
			float processData();
			void endPass();
		};		
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		OffspringRateObject
		**	\brief		Processes the Offspring Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class OffspringRateObject: public AI_System::AI_DataFilter
		{
		public:
			OffspringRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		OffspringRateTarget
		**	\brief		Processes the Offspring Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class OffspringRateTarget: public AI_System::AI_DataFilter
		{
		public:
			OffspringRateTarget() throw();
			float processData();
			void endPass();
		};
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Been_AttackedRateObject
		**	\brief		Processes the Been_Attacked Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Been_AttackedRateObject: public AI_System::AI_DataFilter
		{
		public:
			Been_AttackedRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Been_AttackedRateTarget
		**	\brief		Processes the Been_Attacked Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Been_AttackedRateTarget: public AI_System::AI_DataFilter
		{
		public:
			Been_AttackedRateTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Has_AttackedRateObject
		**	\brief		Processes the Has_Attacked Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Has_AttackedRateObject: public AI_System::AI_DataFilter
		{
		public:
			Has_AttackedRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Has_AttackedRateTarget
		**	\brief		Processes the Has_Attacked Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Has_AttackedRateTarget: public AI_System::AI_DataFilter
		{
		public:
			Has_AttackedRateTarget() throw();
			float processData();
			void endPass();
		};	
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		PlanktonRateObject
		**	\brief		Processes the Plankton Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class PlanktonRateObject: public AI_System::AI_DataFilter
		{
		public:
			PlanktonRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		PlanktonRateTarget
		**	\brief		Processes the Plankton Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class PlanktonRateTarget: public AI_System::AI_DataFilter
		{
		public:
			PlanktonRateTarget() throw();
			float processData();
			void endPass();
		};		
	



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		ChumRateObject
		**	\brief		Processes the Chum Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class ChumRateObject: public AI_System::AI_DataFilter
		{
		public:
			ChumRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		ChumRateTarget
		**	\brief		Processes the Chum Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class ChumRateTarget: public AI_System::AI_DataFilter
		{
		public:
			ChumRateTarget() throw();
			float processData();
			void endPass();
		};
		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Total_FoodRateObject
		**	\brief		Processes the Total_Food Rate of the current object.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Total_FoodRateObject: public AI_System::AI_DataFilter
		{
		public:
			Total_FoodRateObject() throw();
			float processData();
			void endPass();
		};		



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Total_FoodRateTarget
		**	\brief		Processes the Total_Food Rate of the Target.
		**	\author		Nate Cleveland
		**	\date		11/25/2002 1:26:15 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Total_FoodRateTarget: public AI_System::AI_DataFilter
		{
		public:
			Total_FoodRateTarget() throw();
			float processData();
			void endPass();
		};	
	}//end namespace DataFilter
}//end namespace AI_System