#include "AI_FitnessFunctions.h"
using namespace AI_System;
using namespace AI_System::FitnessFunctions;
using namespace InterpretationModule;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif



/*******************************************************************************
 * External Prototypes                                                             
 *//////////////////////////////////////////////////////////////////////////////
template<typename Value_type>
Value_type Normalize(Value_type val)
{
	if(val <= 0 )
		return 0;
	if(val >= 1)
		return 1;
	return val;
}



const float FitnessFunctions::FITNESS_BOUNDRY		= 0.5f;

namespace
{
	const float FITNESS_BOUNDRY_MAX_HEALTH				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_STOMACH				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_SPEED				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_ENERGY				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_SPEED_ENERGY		= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_STOMACH_HEALTH		= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_INCLINE				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MIN_BEEN_ATTACKED		= 0.2f;
	const float FITNESS_BOUNDRY_MAX_PLANKTON			= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_DISTANCE			= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MIN_DISTANCE			= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_MAX_CHUM				= FITNESS_BOUNDRY;
	const float FITNESS_BOUNDRY_PLANKTON_DENSITY		= FITNESS_BOUNDRY;
}


/////////////////////////////Max Health///////////////////////////////
//See class header for documentation.
MaxHealth::MaxHealth()throw(){ID = FF_MAX_HEALTH;}

//See class header for documentation.
bool MaxHealth::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//Get the normalized health of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_HEALTH_TARGET);
	float healthNormal = aiAtt.value;

	//set emphasis,
	emphasis = 1.0f - healthNormal;

	//Decide the fitness of this fish. 
	fitness = healthNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_HEALTH) 
		return response = true;
	return response = false;
}//end MaxHealth




/////////////////////////////Max Stomach///////////////////////////////
//See class header for documentation.
MaxStomach::MaxStomach()throw(){ID = FF_MAX_STOMACH;}

//See class header for documentation.
bool MaxStomach::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//Get the normalized Stomach of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_STOMACH_TARGET);
	float StomachNormal = aiAtt.value;

	//set emphasis,
	emphasis = 1.0f - StomachNormal;

	//Decide the fitness of this fish. 
	fitness = StomachNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_STOMACH) 
		return response = true;
	return response = false;
}//end MaxStomach




/////////////////////////////Max Speed///////////////////////////////
//See class header for documentation.
MaxSpeed::MaxSpeed()throw(){ID = FF_MAX_SPEED;}

//See class header for documentation.
bool MaxSpeed::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//Get the normalized Speed of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_TARGETSPEED);
	float SpeedNormal = aiAtt.value;

	//set emphasis,
	emphasis = 1.0f - SpeedNormal;

	//Decide the fitness of this fish. 
	fitness = SpeedNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_SPEED) 
		return response = true;
	return response = false;
}//end MaxSpeed


/*******************************************************************************
 * BEGIN NEW STUFF                                                             
 *//////////////////////////////////////////////////////////////////////////////

/////////////////////////////Max Energy Level///////////////////////////////
//See class header for documentation.
MaxEnergyLevel::MaxEnergyLevel()throw(){ID = FF_MAX_ENERGY;}

//See class header for documentation.
bool MaxEnergyLevel::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//Get the normalized Speed of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_ENERGY_NORMALIZED_TARGET);
	float EnergyNormal = aiAtt.value;
	EnergyNormal = Normalize<float>(EnergyNormal);

	//set emphasis,
	emphasis = 1.0f - EnergyNormal;

	//Decide the fitness of this fish. 
	fitness = EnergyNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_ENERGY) 
		return response = true;
	return response = false;
}//end MaxEnergyLevel




/////////////////////////////Max Incline///////////////////////////////
//See class header for documentation.
MaxIncline::MaxIncline()throw(){ID = FF_MAX_INCLINE;}

//See class header for documentation.
bool MaxIncline::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//Get the normalized Speed of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_MATINGINCLINE_TARGET);
	float inclineNormal = aiAtt.value;
	inclineNormal = Normalize<float>(inclineNormal);

	//set emphasis,
	emphasis = 1.0f - inclineNormal;

	//Decide the fitness of this fish. 
	fitness = inclineNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_INCLINE) 
		return response = true;
	return response = false;
}//end MaxIncline




/////////////////////////////Max Speed and Energy///////////////////////////////
//See class header for documentation.
MaxSpeedEnergy::MaxSpeedEnergy()throw(){ID = FF_MAX_SPEED_ENERGY;}

//See class header for documentation.
bool MaxSpeedEnergy::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//energy
	AI_Attribute aiAtt		= PG->requestData(DF_ENERGY_NORMALIZED_TARGET);
	float currNormal		= aiAtt.value;

	//speed
	aiAtt = PG->requestData(DF_TARGETSPEED);
	currNormal += aiAtt.value;

	//average the two
	currNormal /= 2;
	currNormal = Normalize<float>(currNormal);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_SPEED_ENERGY) 
		return response = true;
	return response = false;
}//end MaxSpeedEnergy




/////////////////////////////Max Stomach and Health///////////////////////////////
//See class header for documentation.
MaxStomachHealth::MaxStomachHealth()throw(){ID = FF_MAX_STOMACH_HEALTH;}

//See class header for documentation.
bool MaxStomachHealth::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	//health
	AI_Attribute aiAtt		= PG->requestData(DF_HEALTH_NORMALIZED_TARGET);
	float currNormal		= aiAtt.value;

	//stomach
	aiAtt = PG->requestData(DF_STOMACH_NORMALIZED_TARGET);
	currNormal += aiAtt.value;

	//average the two
	currNormal /= 2;
	currNormal = Normalize<float>(currNormal);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_STOMACH_HEALTH) 
		return response = true;
	return response = false;
}//end MaxIncline




/////////////////////////////Max Speed and Energy///////////////////////////////
//See class header for documentation.
MinBeenAttacked::MinBeenAttacked()throw(){ID = FF_MIN_BEEN_ATTACKED;}

//See class header for documentation.
bool MinBeenAttacked::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	AI_Attribute aiAtt		= PG->requestData(DF_BEEN_ATTACKED_RATE_TARGET);
	float currNormal		= aiAtt.value;
	currNormal				= Normalize<float>(currNormal);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MIN_BEEN_ATTACKED) 
		return response = true;
	return response = false;
}//end MinBeenAttacked




/////////////////////////////Min Been Attacked///////////////////////////////
//See class header for documentation.
MaxPlankton::MaxPlankton()throw(){ID = FF_MAX_PLANKTON;}

//See class header for documentation.
bool MaxPlankton::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	AI_Attribute aiAtt		= PG->requestData(DF_PLANKTON_RATE_TARGET);
	float currNormal		= aiAtt.value;
	currNormal				= Normalize<float>(currNormal);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MIN_BEEN_ATTACKED) 
		return response = true;
	return response = false;
}//end MaxPlankton




/////////////////////////////Max Distance///////////////////////////////
//See class header for documentation.
MaxDistance::MaxDistance()throw(){ID = FF_MAX_DISTANCE;}

//See class header for documentation.
bool MaxDistance::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	const float idealDist   = 10;
	AI_Attribute aiAtt		= PG->requestData(DF_DISTANCE);
	float currNormal		= aiAtt.value / idealDist;
	currNormal				= Normalize<float>(currNormal);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_DISTANCE) 
		return response = true;
	return response = false;
}//end MaxDistance




/////////////////////////////Min Distance///////////////////////////////
//See class header for documentation.
MinDistance::MinDistance()throw(){ID = FF_MIN_DISTANCE;}

//See class header for documentation.
bool MinDistance::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	const float idealDist   = 10;
	AI_Attribute aiAtt		= PG->requestData(DF_DISTANCE);
	float currNormal		= aiAtt.value / idealDist;
	currNormal				= Normalize<float>(currNormal);
	currNormal				= 1 - currNormal;

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MIN_DISTANCE) 
		return response = true;
	return response = false;
}//end MinDistance





/////////////////////////////Max Chum///////////////////////////////
//See class header for documentation.
MaxChum::MaxChum()throw(){ID = FF_MAX_CHUM;}

//See class header for documentation.
bool MaxChum::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	AI_Attribute aiAtt		= PG->requestData(DF_CHUM_RATE_TARGET);
	float currNormal		= Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_MAX_CHUM) 
		return response = true;
	return response = false;
}//end MaxChum





/////////////////////////////Plankton Density///////////////////////////////
//See class header for documentation.
PlanktonDensity::PlanktonDensity()throw(){ID = FF_MAX_PLANKTON_DENSITY;}

//See class header for documentation.
bool PlanktonDensity::evaluateFitness(bool fitnessReturned/* = false*/, float prevFitness/* = 0.0f*/)
{
	AI_Attribute aiAtt		= PG->requestData(DF_PLANKTONDENSITY_TARGET);
	float currNormal		= Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	//check to see if we were given a prevous fitness level.
	if(fitnessReturned)
	{
		//if the fitness has increased since this action was completed, given an 
		// emphasis bonus. else give a emphasis decrement.
		emphasis += prevFitness - fitness;


		//If the fitnessReturned is less then our current fitness then the fish did good.
		if(prevFitness <= fitness)
			return response = true;

		//otherwise the fish did bad
		return response = false;
	}

	//Use different logic if we didn't get a prevous fitness...
	// Assume that any action is better than no action iff our fitness is low.
	if(fitness < FITNESS_BOUNDRY_PLANKTON_DENSITY) 
		return response = true;
	return response = false;
}//end MaxChum

