#include "AI_HigherGoals.h"
using namespace AI_System;
using namespace AI_System::HigherGoals;
using namespace InterpretationModule;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif


const float HigherGoals::FITNESS_BOUNDRY		= 0.5f;
const float FITNESS_BOUNDRY_MAX_KILLS			= 0.2f;


////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	
/*!	\brief		Forces values to the range of 0 to 1.
**	\param		val		Value to cap.
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:
**				 Forces the passed value to the limits of 0 to 1.	
**	\author		Nate Cleveland
**	\date		11/27/2002 5:48:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
template<typename Value_type>
Value_type Normalize(Value_type val)
{
	if(val <= 0 )
		return 0;
	if(val >= 1)
		return 1;
	return val;
}



/////////////////////////////Max Kills///////////////////////////////
//See class header for documentation.
MaxKills::MaxKills()throw(){ID = HG_KILLS;}

//See class header for documentation.
bool MaxKills::evaluateFitness()
{
	//Get the normalized Kills of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_KILLS_RATE_TARGET);
	float KillsNormal = Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - KillsNormal;

	//Decide the fitness of this fish. 
	fitness = KillsNormal;

	if(KillsNormal < 0.3f)
		return false;
	return true;
}//end MaxKills



/////////////////////////////Nomad///////////////////////////////
//See class header for documentation.
Nomad::Nomad()throw(){ID = HG_NOMAD;}

//See class header for documentation.
bool Nomad::evaluateFitness()
{
	AI_Attribute aiAtt = PG->requestData(DF_DISTANCE_COUNT_TARGET);
	float distAvg = Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - distAvg;

	//Decide the fitness of this fish. 
	fitness = distAvg;

	if(distAvg < 0.3f)
		return false;
	return true;
}//end Nomad



/////////////////////////////DukeWellington///////////////////////////////
//See class header for documentation.
DukeWellington::DukeWellington()throw(){ID = HG_DUKE_WELLINGTON;}

//See class header for documentation.
bool DukeWellington::evaluateFitness()
{
	//set emphasis,
	fitness = emphasis = static_cast<float>(rand()%2);

	if(emphasis == 0)
		return false;
	return true;
}//end DukeWellington



/////////////////////////////Random///////////////////////////////
//See class header for documentation.
Random::Random()throw(){ID = HG_RANDOM;}

//See class header for documentation.
bool Random::evaluateFitness()
{
	emphasis = RandFloat();

	//Decide the fitness of this fish. 
	fitness = RandFloat();

	if(RandFloat() < 0.5f)
		return false;
	return true;
}//end Random



/////////////////////////////Glutton///////////////////////////////
//See class header for documentation.
Glutton::Glutton()throw(){ID = HG_GLUTTON;}

//See class header for documentation.
bool Glutton::evaluateFitness()
{
	AI_Attribute aiAtt = PG->requestData(DF_TOTAL_FOOD_RATE_TARGET);
	float currNormal = Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	if(currNormal < 0.5f)
		return false;
	return true;
}//end Glutton



/////////////////////////////Plankton Density///////////////////////////////
//See class header for documentation.
PlanktonDensity::PlanktonDensity()throw(){ID = HG_PLANKTON_DENSITY;}

//See class header for documentation.
bool PlanktonDensity::evaluateFitness()
{
	//Get the normalized Kills of this fish.
	AI_Attribute aiAtt = PG->requestData(DF_PLANKTONDENSITY_TARGET);
	float currNormal = Normalize<float>(aiAtt.value);

	//set emphasis,
	emphasis = 1.0f - currNormal;

	//Decide the fitness of this fish. 
	fitness = currNormal;

	if(currNormal < 0.6f)
		return false;
	return true;
}//end PlanktonDensity



/////////////////////////////Best Case///////////////////////////////
//See class header for documentation.
BestCase::BestCase()throw(){ID = HG_BEST_CASE;}

//See class header for documentation.
bool BestCase::evaluateFitness()
{
	///find the best reactive attribute
	AI_Attribute	currAtt	 = PG->requestData(DF_ENERGY_NORMALIZED_TARGET);
	float			currBest = Normalize<float>(currAtt.value);
	float			currTemp = 0;

	//go through each attribute, one at a time, trying to find the best.
	currAtt	= PG->requestData(DF_HEALTH_NORMALIZED_TARGET);
	if(currBest > (currTemp = Normalize<float>(currAtt.value)))
		currBest = currTemp;
	
	currAtt	= PG->requestData(DF_MATINGINCLINE_TARGET);
	if(currBest > (currTemp = Normalize<float>(currAtt.value)))
		currBest = currTemp;

	currAtt	= PG->requestData(DF_STOMACH_NORMALIZED_TARGET);
	if(currBest > (currTemp = Normalize<float>(currAtt.value)))
		currBest = currTemp;


	////now evaluate the fist on that attribute
	//set emphasis,
	emphasis = 1.0f - currBest;

	//Decide the fitness of this fish. 
	fitness = currBest;

	if(currBest < 0.5f)
		return false;
	return true;
}//end BestCase



/////////////////////////////Worst Case///////////////////////////////
//See class header for documentation.
WorstCase::WorstCase()throw(){ID = HG_WORST_CASE;}

//See class header for documentation.
bool WorstCase::evaluateFitness()
{
	///find the worst reactive attribute
	AI_Attribute	currAtt	 = PG->requestData(DF_ENERGY_NORMALIZED_TARGET);
	float			currWorst = Normalize<float>(currAtt.value);
	float			currTemp = 0;

	//go through each attribute, one at a time, trying to find the worst.
	currAtt	= PG->requestData(DF_HEALTH_NORMALIZED_TARGET);
	if(currWorst < (currTemp = Normalize<float>(currAtt.value)))
		currWorst = currTemp;
	
	currAtt	= PG->requestData(DF_MATINGINCLINE_TARGET);
	if(currWorst < (currTemp = Normalize<float>(currAtt.value)))
		currWorst = currTemp;

	currAtt	= PG->requestData(DF_STOMACH_NORMALIZED_TARGET);
	if(currWorst < (currTemp = Normalize<float>(currAtt.value)))
		currWorst = currTemp;


	////now evaluate the fist on that attribute
	//set emphasis,
	emphasis = 1.0f - currWorst;

	//Decide the fitness of this fish. 
	fitness = currWorst;

	if(currWorst < 0.5f)
		return false;
	return true;
}//end WorstCase



/////////////////////////////Average Case///////////////////////////////
//See class header for documentation.
AverageCase::AverageCase()throw(){ID = HG_AVERAGE_CASE;}

//See class header for documentation.
bool AverageCase::evaluateFitness()
{
	///find the average of the reactive attribute
	AI_Attribute	currAtt	 = PG->requestData(DF_ENERGY_NORMALIZED_TARGET);
	float			currAvg = Normalize<float>(currAtt.value);

	//go through each attribute, one at a time, summing them
	currAtt	= PG->requestData(DF_HEALTH_NORMALIZED_TARGET);
	currAvg += Normalize<float>(currAtt.value);
	
	currAtt	= PG->requestData(DF_MATINGINCLINE_TARGET);
	currAvg += Normalize<float>(currAtt.value);

	currAtt	= PG->requestData(DF_STOMACH_NORMALIZED_TARGET);
	currAvg += Normalize<float>(currAtt.value);

	//div by 4 to complete the average
	currAvg /= 4;


	////now evaluate the fist on that attribute
	//set emphasis,
	emphasis = 1.0f - currAvg;

	//Decide the fitness of this fish. 
	fitness = currAvg;

	if(currAvg < 0.5f)
		return false;
	return true;
}//end AverageCase



/////////////////////////////Herbivore///////////////////////////////
//See class header for documentation.
Herbivore::Herbivore()throw(){ID = HG_HERBIVORE;}

//See class header for documentation.
bool Herbivore::evaluateFitness()
{
	///find the average of the reactive attribute
	AI_Attribute	currAtt;
	float			currVal = 0;
	float			tempVal = 0;

	//factor in not eating chum (10%)
	currAtt	 = PG->requestData(DF_CHUM_RATE_TARGET);
	currVal += (1 - Normalize<float>(currAtt.value)) * 0.1f;
	
	//not attacking
	currAtt	 = PG->requestData(DF_HAS_ATTACKED_RATE_TARGET);
	currVal += (1 - Normalize<float>(currAtt.value)) * 0.2f;

	//not being attacked
	currAtt	 = PG->requestData(DF_BEEN_ATTACKED_RATE_TARGET);
	currVal += (1 - Normalize<float>(currAtt.value)) * 0.15f;

	//eating plankton
	currAtt	 = PG->requestData(DF_PLANKTON_RATE_TARGET);
	currVal += Normalize<float>(currAtt.value) * 0.35f;

	//moving.
	currAtt	 = PG->requestData(DF_SPEED_RATE_TARGET);
	currVal += Normalize<float>(currAtt.value) * 0.20f;	


	////now evaluate the fist on that attribute
	//set emphasis,
	emphasis = 1.0f - currVal;

	//Decide the fitness of this fish. 
	fitness = currVal;

	if(currVal < 0.5f)
		return false;
	return true;
}//end Herbivore



/////////////////////////////Carnivore///////////////////////////////
//See class header for documentation.
Carnivore::Carnivore()throw(){ID = HG_CARNIVORE;}

//See class header for documentation.
bool Carnivore::evaluateFitness()
{
	///find the average of the reactive attribute
	AI_Attribute	currAtt;
	float			currVal = 0;
	float			tempVal = 0;

	//factor in eating chum (30%)
	currAtt	 = PG->requestData(DF_CHUM_RATE_TARGET);
	currVal += Normalize<float>(currAtt.value) * 0.3f;
	
	//attacking (30%)
	currAtt	 = PG->requestData(DF_HAS_ATTACKED_RATE_TARGET);
	currVal += Normalize<float>(currAtt.value) * 0.3f;

	//not eating plankton
	currAtt	 = PG->requestData(DF_PLANKTON_RATE_TARGET);
	currVal += (1-Normalize<float>(currAtt.value)) * 0.1f;

	//moving.
	currAtt	 = PG->requestData(DF_SPEED_RATE_TARGET);
	currVal += Normalize<float>(currAtt.value) * 0.30f;	


	////now evaluate the fist on that attribute
	//set emphasis,
	emphasis = 1.0f - currVal;

	//Decide the fitness of this fish. 
	fitness = currVal;

	if(currVal < 0.5f)
		return false;
	return true;
}//end Carnivore