#pragma once
#include "../AI_System.h"

namespace AI_System
{

	namespace HigherGoals
	{
		extern const float FITNESS_BOUNDRY;		//!< If less then this value result
												//!<  will be set to false in several
												//!<  FitnessFunctions.

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		MaxKills
		**	\brief		Higher Goal function that max's the fish's kill rate.
		**	\author		Nate Cleveland
		**	\date		11/27/2002 2:08:21 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class MaxKills: public AI_System::AI_HigherGoal
		{
		public:
			MaxKills()throw();
			bool evaluateFitness();
		};

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Nomad
		**	\brief		Values fish that like to wander.
		**	\author		Nate Cleveland
		**	\date		11/27/2002 2:08:21 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Nomad: public AI_System::AI_HigherGoal
		{
		public:
			Nomad()throw();
			bool evaluateFitness();
		};

		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		DukeWellington
		**	\brief		Randomly sets emphasis and fitness to either 0 or 1
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
			\note
					Edmund Blackadder:	Tell me, do you ever stop bullying and shouting at 
										 the lower orders?
					Duke Wellington:	NEVER! There's only one way to win a campaign: 
										 shout, shout and shout again.
					Edmund Blackadder:	You don't think then that inspired leadership and 
										 tactical ability have anything to do with it?
					Duke Wellington:	NO! It's all down to shouting. WAAGGHH!
		*/
		////////////////////////////////////////////////////////////////////////////////
		class DukeWellington: public AI_System::AI_HigherGoal
		{
		public:
			DukeWellington()throw();
			bool evaluateFitness();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Random
		**	\brief		Randomly sets emphasis and fitness to either 0 or 1
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Random: public AI_System::AI_HigherGoal
		{
		public:
			Random()throw();
			bool evaluateFitness();
		};


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Glutton
		**	\brief		Tries to eat as much food as possible.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Glutton: public AI_System::AI_HigherGoal
		{
		public:
			Glutton()throw();
			bool evaluateFitness();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		PlanktonDensity
		**	\brief		Tries to eat as much food as possible.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		class PlanktonDensity: public AI_System::AI_HigherGoal
		{
		public:
			PlanktonDensity()throw();
			bool evaluateFitness();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		BestCase
		**	\brief		Evaluates the fish upon the best reactive attribute.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		**	\note		
						Reactive Attibutes looked at are:
							- Energy Level Normalized
							- Stomach Normalized
							- Health Normalized
							- Mating Inclination
		*/
		////////////////////////////////////////////////////////////////////////////////
		class BestCase: public AI_System::AI_HigherGoal
		{
		public:
			BestCase()throw();
			bool evaluateFitness();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		WorstCase
		**	\brief		Evaluates the fish upon the worst reactive attribute.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		**	\note		
						Reactive Attibutes looked at are:
							- Energy Level Normalized
							- Stomach Normalized
							- Health Normalized
							- Mating Inclination
		*/
		////////////////////////////////////////////////////////////////////////////////
		class WorstCase: public AI_System::AI_HigherGoal
		{
		public:
			WorstCase()throw();
			bool evaluateFitness();
		};



		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		AverageCase
		**	\brief		Evaluates the fish upon the average reactive attribute.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		**	\note		
						Reactive Attibutes looked at are:
							- Energy Level Normalized
							- Stomach Normalized
							- Health Normalized
							- Mating Inclination
		*/
		////////////////////////////////////////////////////////////////////////////////
		class AverageCase: public AI_System::AI_HigherGoal
		{
		public:
			AverageCase()throw();
			bool evaluateFitness();
		};
	


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Herbivore
		**	\brief		Hates: Eating chum, attacking, being attacked. Loves: Eating
		**				 plankton, moving.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		**	\note
		**				Importance ratings
 		**					10% - Not eating chum		
		**					20% - Not attacking 
		**					15% - Not being attacked
		**					35% - Eating plankton
		**					20% - Moving (high speed).
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Herbivore: public AI_System::AI_HigherGoal
		{
		public:
			Herbivore()throw();
			bool evaluateFitness();
		};		
	


		////////////////////////////////////////////////////////////////////////////////
		/*!	\class		Carnivore
		**	\brief		Loves: Eating chum, attacking, moving. Hates: Eating
		**				 plankton.
		**	\author		Nate Cleveland
		**	\date		4/14/2002 2:08:21 PM
		**	\warning	none
		**	\note
		**				Importance ratings
 		**					30% - Eating chum		
		**					30% - Attacking 
		**					10% - Not Eating plankton
		**					30% - Moving (high speed).
		*/
		////////////////////////////////////////////////////////////////////////////////
		class Carnivore: public AI_System::AI_HigherGoal
		{
		public:
			Carnivore()throw();
			bool evaluateFitness();
		};	
	}//end FitnessFunctions
}//end AI_System