////////////////////////////////////////////////////////////////////////////////
/*! \file           NeuralADN.h
**  \brief          The interface file for the NeuralNetwork based Action
**                  Decision Node.
**  \author         Austin Spafford
**  \date           2/14/2003 8:58:00 PM
**  \par            REMARKS:
**                    
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once
////////////////////////////////////////////////////////////////////////////////
#include <hash_map>
#include <iostream>
#include "ai_core/AI_System.h"
////////////////////////////////////////////////////////////////////////////////

namespace Neural
{
	////////////////////////////////////////////////////////////////////////////
	/*! \class          NeuralADN
	**  \brief          A NeuralADN is an implementation of an AI_ADN that
	**                  uses a NeuralNetwork to make its decisions.
	**  \author         Austin Spafford
	**  \date           2/14/2003 8:59:12 PM
	**  \warning        none
	*/
	////////////////////////////////////////////////////////////////////////////
	class NeuralADN : public AI_System::AI_ADN
	{
		public:

		/// default constructer
		NeuralADN(AI_System::TT_ID Technique) : m_neuralNetworkPtr(NULL), AI_ADN(Technique) {};

        /// copy constructor
        NeuralADN(const NeuralADN& source);

		// deconstructer
		~NeuralADN(void);

		////////////////////////////////////////////////////////////////////////

		// writes the object's contents to an ostream in a human readable format
		virtual void Write(std::ostream & outputStream) const;

		// reads the object's contents from an istream that is using a human readable format
		virtual void Read(std::istream & inputStream);

		// old interface
		virtual bool save() throw() { return false; }
		virtual bool load() throw() { return false; }

		////////////////////////////////////////////////////////////////////////

		protected:

		/// selects a unit to acknowledge
		AI_System::AI_ID process() throw();

		/// directly trains *m_neuralNetworkPtr
		void saveTrainingExample(
				bool trainingResponse,
				float trainingFitness,
				AI_System::AI_ID trainingUnitID,
				std::vector<AI_System::AI_Attribute> & trainingInputs
				) throw();

		/// instantiates *m_neuralNetworkPtr and builds the input/output mapping
		void subClassInit() throw();
	
		/// this NeuralNetwork is used to make decisions for the NeuralADN
		NeuralNetwork * m_neuralNetworkPtr;

		/// used to quickly map the input vector recieved by the NeuralADN
		/// to the input vector of *m_neuralNetworkPtr
		std::hash_map<AI_System::Attribute_ID, unsigned int> m_inputMapping;

		/// used to map output values from *m_neuralNetworkPtr to the IDs of the 
		/// AI_Units whose outputs that were used as input values 
		std::vector<AI_System::Attribute_ID> m_outputMapping;

		////////////////////////////////////////////////////////////////////////////

        private:
		
		const NeuralADN & operator=(const NeuralADN & notBeingCopied); // intentional
	};
}
