////////////////////////////////////////////////////////////////////////////////
/*! \file           NeuralUnit.h
**  \brief          The interface file for the NeuralNetwork based AI_Unit.
**  \author         Austin Spafford
**  \date           2/14/2003 8:58:00 PM
**  \par            REMARKS:
**                    
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once
////////////////////////////////////////////////////////////////////////////////
#include <hash_map>
#include <iostream>
#include "ai_core/AI_System.h"
////////////////////////////////////////////////////////////////////////////////

namespace Neural
{
	////////////////////////////////////////////////////////////////////////////
	/*! \class          NeuralUnit
	**  \brief          A NeuralUnit is an implementation of an AI_Unit that
	**                  uses a NeuralNetwork to make decisions.
	**  \author         Austin Spafford
	**  \date           2/14/2003 8:59:12 PM
	**  \warning        none
	*/
	////////////////////////////////////////////////////////////////////////////
	class NeuralUnit : public AI_System::AI_Unit
	{
		public:

		/// default constructer
		NeuralUnit(AI_System::TT_ID Technique) : m_neuralNetworkPtr(NULL), AI_Unit(Technique) {};

        /// copy constructor
		NeuralUnit(const NeuralUnit & neuralUnitToCopy);

		// deconstructer
		~NeuralUnit(void);

		////////////////////////////////////////////////////////////////////////////

		// writes the object's contents to an ostream in a human readable format
		virtual void Write(std::ostream & outputStream) const;

		// reads the object's contents from an istream that is using a human readable format
		virtual void Read(std::istream & inputStream);

		// old interface
		virtual bool save() throw() { return false; }
		virtual bool load() throw() { return false; }

		////////////////////////////////////////////////////////////////////////

		protected:

		/// generates a decision based on the current inputs
		bool triggerAction() throw();

		/// directly trains *m_neuralNetworkPtr
		void saveTrainingExample(
				bool trainingResponse,
				float trainingFitness,
				std::vector<AI_System::AI_Attribute> & trainingInputs
				) throw();

		/// instantiates *m_neuralNetworkPtr and builds the input mapping
		void subClassInit() throw();		

		/// this NeuralNetwork is used to make decisions for the NeuralUnit
		NeuralNetwork * m_neuralNetworkPtr;

		/// used to quickly map the input vector recieved by the NeuralUnit
		/// to the input vector of *m_neuralNetworkPtr
		std::hash_map<AI_System::Attribute_ID, unsigned int> m_inputMapping;

		////////////////////////////////////////////////////////////////////////////

        private:

		const NeuralUnit & operator=(const NeuralUnit & notBeingCopied); // intentional
	};
}
