#include <time.h>
#include <assert.h>
#include "waterMark.h"
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

WaterMark::WaterMark(const char *Message/* = NULL*/, float versionNumber/* = 0.0f*/)/* throw()*/
:m_message(NULL), m_ver(versionNumber)
{
	
	if(Message)
	{
		//create an array of chars for the message string
		m_message = new char[strlen(Message) + 1];

		//copy the message over, handle errors.
		if(!strcpy(m_message, Message))
		{
			delete m_message;
			m_message = NULL;
		}//end if
	}//end if
}//end WaterMark::WaterMark
////////////////////////////////////////////////////////////////////////////////
/*!	\file		waterMark.cpp
**	\brief		Defines all the classes contained in waterMark.h
**	\author		Nate Cleveland
**	\date		10/7/2002 5:19:42 PM
**	\par		DESCRIPTION: See waterMark.h for details on any function.
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////

//See header for documentation.
WaterMark::~WaterMark()
{
	//delete the array.
	if(m_message)
		delete m_message;
}

//See header for documentation.
void WaterMark::operator()(std::ostream &outStream)/* throw()*/
{
	time_t currTimeSec;			//Current time in seconds
	tm *pDate = NULL;			//Will point to current tm struct with current date.

	//get the current time
	time(&currTimeSec);
	pDate = localtime(&currTimeSec);
	assert(NULL != pDate);


	//Stream out the message we're given at creation
	if(m_message)
		outStream << m_message << std::endl;

	//stream out the version number we're on.
	outStream << m_ver << std::endl;

	//stream out the current date.
	outStream << pDate->tm_mon + 1 << " // " << pDate->tm_mday << " // " 
		<< (pDate->tm_year - 100) + 2000 << std::endl;
}//end WaterMark::operator()

//See header for documentation.
void WaterMark::operator ()(std::istream& inStream)
{
	const int MessageSize = 256;		//Size of the message buffer
	char buffer[MessageSize];			//Temp buffer for messages.

	/////1. first line is the message. Read it in, maybe stream it to console.
	inStream.getline(buffer, MessageSize, '\n');

	//delete any old message buffer
	if(m_message)
		delete m_message;
	m_message = new char[strlen(buffer) + 1];

	//copy the message over, handle errors.
	if(!strcpy(m_message, buffer))
	{
		delete m_message;
		m_message = NULL;
	}//end if


	////2. Next line is the version number.
	inStream.getline(buffer, MessageSize, '\n');
	this->m_ver = static_cast<float>(atof(buffer));


	////3. last line is the date.
	inStream.getline(buffer, MessageSize, '\n');
	this->m_date(buffer);
}




//See header for documentation.
void Date::operator ()(const char* str)
{
	//check for null. set to defualt if it is null.
	if(str)
	{
		sscanf(str, "%d // %d // %d", &this->m_month, &this->m_day, &this->m_year);
	}

	else
	{
		this->m_month = this->m_day = this->m_year = 0;
	}
}//end Date::operator ()





