////////////////////////////////////////////////////////////////////////////////
/*! \file       anim_model.h
**  \brief      Animated 3D models
**  \author     M Jared Finder
**  \date       1/30/2003 8:03:38 PM
**  \par        DESCRIPTION:
**                  Loads and renders animated 3D models from a file.  Currently
**                  supports MD2 format only.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include"../loader/interp_model.h"
namespace math { namespace linear_alg
{
    template<typename type> class Vector3;
} }
namespace gfx
{
    class Texture;
}


namespace gfx
{
////////////////////////////////////////////////////////////////////////////////
/*! \class      AnimModel
**  \brief      An animated 3D model consists of multiple polygons in ``object
**              space'' that look like something.  These can be textured.
**  \author     M Jared Finder
**  \date       11/26/2002 6:06:27 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class AnimModel
{
public:
    ///load a model from an MD2 file
    AnimModel(char fileName[]);
    ///clear the model
    ~AnimModel(void);

    ///render a specific frame of the model
    void Draw(
            int frame,
            gfx::Texture& texture,
            math::linear_alg::Vector3<float> position,
            math::linear_alg::Vector3<float> scale,
            math::linear_alg::Vector3<float> axis, float angle,
            float r=1, float g=1, float b=1
            );

    ///render a "tween" frame of the model
    void Draw(
            int frameA, int frameB, float interpAmount,
            gfx::Texture& texture,
            math::linear_alg::Vector3<float> position,
            math::linear_alg::Vector3<float> scale,
            math::linear_alg::Vector3<float> axis, float angle,
            float r=1, float g=1, float b=1
            );
private:
    loader::InterpModel m_modelData;
};

}