////////////////////////////////////////////////////////////////////////////////
/*! \file       gfx/api_loader.h
**  \brief      Loads the graphical API functionality
**  \author     M Jared Finder
**  \date       2/14/2003 12:47:09 PM
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////



namespace gfx {
    struct ApiLoader
    {
        ///constructor
        ApiLoader(void);

        ///major version
        int majorVersion;
        ///minor version
        int minorVersion;

        ///ARB extensions
        bool pointParameters;
        bool vertexBlend;
        bool matrixPalette;
        bool textureEnvCrossbar;
        bool textureMirroredRepeat;
        bool depthTexture;
        bool shadow;
        bool shadowAmbient;
        bool windowPos;

        ///EXT extensions
        bool compiledVertexArray;

        ///NV extensions
        bool NVvertexArray;
        bool NVpointSprite;

        ///ATI extensions
        bool ATIvertexArray;

    private:
        ///detect if an extension is enabled
        bool IsExtensionAvailable(const char*);

        ///Get OpenGL procedures
        void InitializeGL_1_0(void);
        void InitializeGL_1_1(void);
        void InitializeGL_1_2(void);
        void InitializeGL_1_3(void);

        ///Get ARB procedures
        void InitializeARB_point_parameters(void);
        void InitializeARB_vertex_blend(void);
        void InitializeARB_matrix_palette(void);
        void InitializeARB_texture_env_crossbar(void);
        void InitializeARB_texture_mirrored_repeat(void);
        void InitializeARB_depth_texture(void);
        void InitializeARB_shadow(void);
        void InitializeARB_shadow_ambient(void);
        void InitializeARB_window_pos(void);

        ///Get EXT procedures
        void InitializeEXT_compiled_vertex_array(void);
        
        ///Get NV procedures
        void InitializeNV_vertex_array_range(void);
        void InitializeNV_point_sprite(void);

        ///Get ATI procegures
        void InitializeATI_vertex_array_object(void);
    };
}



