////////////////////////////////////////////////////////////////////////////////
/*! \file       font.h
**  \brief      Provide basic font functionality
**  \author     M Jared Finder
**  \date       10/28/2002 8:51:05 PM
**  \par        DESCRIPTION:
**                  A font that represents certain text allows text in any
**                  format to be displayed.
**  \par        REMARKS:
**                  Fonts are cuurently loaded from 256 x 256 textures, with
**                  16 characters per row and 16 rows. (256 chars total)
**
**                  Overall, this looks relatively okay, but it might be more
**                  useful in the future to use something like FreeType
**                  (www.freetype.org).
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include"texture.h"


namespace gfx
{
////////////////////////////////////////////////////////////////////////////////
/*! \class      Font
**  \brief      Allows displaying of characters in any position with a printf
**              style format string.
**  \author     M Jared Finder
**  \date       10/28/2002 8:53:34 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class Font
{
public:
    ///create a font from a fontname
    Font(const char* fontName);
    ///clean up
    ~Font(void);

    ///printf to a specified location and the specified color
    int Write(
        float scaleX, float scaleY,
        float x, float y,
        float r, float g, float b,
        char formatString[], ...);

private:
    int             m_fontBase;     //< base in OGL for the display list
    gfx::Texture    m_texture;      //< texture that holds the font
};

}