////////////////////////////////////////////////////////////////////////////////
/*! \file       md2.h
**  \brief      Format for MD2 (Quake 2) model file format
**  \author     M Jared Finder
**  \date       11/26/2002 9:35:52 PM
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
//#include"stdheaders.h"


namespace gfx
{

////////////////////////////////////////////////////////////////////////////////
/*! \struct     MD2Header
**  \brief      The header for an MD2 file
**  \author     M Jared Finder
**  \date       11/26/2002 9:42:14 PM
**  \warning    Makes the following size assumptions:
**                  - sizeof(int) == 4
*/
////////////////////////////////////////////////////////////////////////////////
struct MD2Header
{
    int identifier;         ///< Identifier, should be IDP2 or 0x32504449 in hex
    int version;            ///< Version number of the file.  Always 8.
    int skinWidth;          ///< Width of the skins in pixels
    int skinHeight;         ///< Height of the skins in pixels
    int frameSize;          ///< Size of each frame in bytes.
    int numSkins;           ///< Number of skins this model contains referances to
    int numVertices;        ///< Number of vertices in each frame
    int numTexCoords;       ///< Number of texture coordinates
    int numTriangles;       ///< Number of triangles in each frame
    int numGLCommands;      ///< Number of DWORDs (4 bytes) in the GL command list
    int numFrames;          ///< Number of frames of animation
    int offsetSkins;        ///< Offset in bytes to the skin names, 64 bytes each
    int offsetTexCoords;    ///< Offset in bytes to the list of texture coordinates
    int offsetTriangles;    ///< Offset in bytes to the list of triangles
    int offsetFrames;       ///< Offset in bytes to the list of frames
    int offsetGLCommands;   ///< Offset in bytes to the list of frames
    int offsetEnd;          ///< Offset in bytes to the end of the file (size of the file)
};

}