////////////////////////////////////////////////////////////////////////////////
/*! \file       model.h
**  \brief      Static 3D models
**  \author     M Jared Finder
**  \date       11/26/2002 6:04:53 PM
**  \par        DESCRIPTION:
**                  Loads and renders 3D models from a file.  Currently supports
**                  MD2 format only.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include"../loader/interp_model.h"
namespace math { namespace linear_alg
{
    template<typename type> class Vector3;
} }
namespace gfx
{
    class Texture;
}


namespace gfx
{

class Texture;
////////////////////////////////////////////////////////////////////////////////
/*! \class      Model
**  \brief      A static 3D model consists of multiple polygons in ``object
**              space'' that look like something.  These can be textured, but
**              not animated.
**  \author     M Jared Finder
**  \date       11/26/2002 6:06:27 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class Model
{
public:
    ///load a model from an MD2 file
    Model(const char fileName[], int frameNumber);
    ///clear the model
    ~Model(void);

    ///draw the model
    void Draw(
            gfx::Texture& texture,
            math::linear_alg::Vector3<float> position,
            math::linear_alg::Vector3<float> scale,
            math::linear_alg::Vector3<float> axis, float angle,
            float r=1, float g=1, float b=1
            );
private:
    loader::InterpModel m_modelData;
    int m_frameNumber;
};

}