////////////////////////////////////////////////////////////////////////////////
/*! \file       multi_model.h
**  \brief      Aquarius's Multiple model interpolation method
**  \author     M Jared Finder
**  \date       4/8/2003 1:14:00 PM
**  \par        DESCRIPTION:
**                  Use this to display the fish so that interpolation between
**                  different models is doable.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include"loader/interp_model.h"
namespace math { namespace linear_alg
{
    template<typename type> class Vector3;
} }
namespace gfx
{
    class Texture;
}


namespace gfx
{
////////////////////////////////////////////////////////////////////////////////
/*! \class      MultiModel
**  \brief      
**  \author     M Jared Finder
**  \date       4/8/2003 1:16:10 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class MultiModel
{
public:
    ///load four models for interpolation between
    MultiModel(
            const char* fileMinAMinB,
            const char* fileMinAMaxB,
            const char* fileMaxAMinB,
            const char* fileMaxAMaxB
            );
    ///free models
    ~MultiModel(void);

    ///render a specific frame
    void Draw(
            float weightA, float weightB, float weightTex,
            int frame,
            gfx::Texture& tex1, gfx::Texture& tex2,
            math::linear_alg::Vector3<float> position,
            math::linear_alg::Vector3<float> scale,
            math::linear_alg::Vector3<float> axis, float angle,
            float r=1, float g=1, float b=1
            );
    ///render an interpolated frame
    void Draw(
            float weightA, float weightB, float weightTex,
            int frame1, int frame2, float interpAmount,
            gfx::Texture& tex1, gfx::Texture& tex2,
            math::linear_alg::Vector3<float> position,
            math::linear_alg::Vector3<float> scale,
            math::linear_alg::Vector3<float> axis, float angle,
            float r=1, float g=1, float b=1
            );
private:
    loader::InterpModel m_modelMinAMinB;
    loader::InterpModel m_modelMinAMaxB;
    loader::InterpModel m_modelMaxAMinB;
    loader::InterpModel m_modelMaxAMaxB;
};

//end namespace gfx
}