////////////////////////////////////////////////////////////////////////////////
/*! \file       gfx/rendering_window.h
**  \brief      Maintains the window that is used for rendering.
**  \author     M Jared Finder
**  \date       2/13/2003 3:00:02 PM
**  \par        DESCRIPTION:
**                  The RenderingWindow class maintains the window that is
**                  used for rendering.  This will create a window with
**                  "sutable" properties for rendering to with the graphics API
**                  being used.
**  \par        REMARKS:
**                  RenderingWindow assumes that the graphics API being used
**                  will render the entire viewable area with the exception of
**                  the title bar and icon.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include<vector>
#include<windows.h>
namespace gfx
{
    struct Resolution;
}


namespace gfx {
////////////////////////////////////////////////////////////////////////////////
/*! \class      RenderingWindow
**  \brief      Maintains the window that is used for rendering.
**  \author     M Jared Finder
**  \date       10/15/2002 7:34:08 PM
*/
////////////////////////////////////////////////////////////////////////////////
class RenderingWindow
{
public:
    /// constructor
    RenderingWindow(void);
    /// destructor
    ~RenderingWindow(void);

    /// pump all OS messages
    bool MessagePump(void);

    /// get all supported resolutions
    void GetSupportedResolutions(std::vector<Resolution>& displayModeList) const;
    /// resize the window
    bool ChangeResolution(const Resolution& mode, bool fullscreen);
    /// swap the double buffer
    void SwapBuffers(void);

private:
    /// make a window suitable for the graphics subsystem
    void MakeWindow(void);
    /// attach the graphics renderer to the window
    void AttachGraphics(void);
    /// resize the window so that it is the proper drawable size
    void ResizeWindow(int w, int h, bool fullscreen);


private:
    /// window handle
    HWND        m_hWnd;
    /// window device context
    HDC         m_hdc;
    /// is the window fullscreen?
    bool        m_isFullscreen;
    /// Window Procedure
    static LRESULT CALLBACK WindowProc(HWND,UINT,WPARAM,LPARAM);
};


}