////////////////////////////////////////////////////////////////////////////////
/*! \file       sprite.h
**  \brief      Implement basic (rectangular) sprite functionality.
**  \author     M Jared Finder
**  \date       10/30/2002 11:59:46 AM
**  \par        DESCRIPTION:
**                  
**  \par        REMARKS:
**                  
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
namespace gfx
{
    class Texture;
}


namespace gfx {
////////////////////////////////////////////////////////////////////////////////
/*! \class      Sprite
**  \brief      2D drawing with texturing.  Each sprite is rendered with a quad
**              or two triangles
**  \author     M Jared Finder
**  \date       10/30/2002 12:01:06 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class Sprite
{
public:
    ///set the texture and size
    Sprite(const Texture& tex,
           float width, float height,
           float originX = 0, float originY = 0);
    ///clean up
    ~Sprite(void) {}

    ///set the texture
    void SetTexture(const Texture&);
    ///set the size
    void SetSize(float width, float height);
    ///set the centerpoint (the origin)
    void SetOrigin(float x, float y);

    ///draw at a specified spot with a specified rotation amount
    void Draw(float left, float top,
              float rotation = 0,
              float scaleX = 1, float scaleY = 1,
              float r = 1, float g = 1, float b = 1);
private:
    //origin, for calculation
    float m_originX;
    float m_originY;

    ///texture coordinates
    float m_textureCoords[4*2];
    ///vertex coordinates
    float m_vertexCoords[4*2];
    ///current texture handle
    unsigned int m_textureHandle;
};

}
