////////////////////////////////////////////////////////////////////////////////
/*! \file       texture.cpp
**  \brief      Implemets texture loading/saving
**  \author     M Jared Finder
**  \date       10/28/2002 9:05:29 PM
*/
////////////////////////////////////////////////////////////////////////////////
#include"texture.h"

////////////////////////////////////////////////////////////////////////////////
// Needed includes
////////////////////////////////////////////////////////////////////////////////
//#include"stdheaders.h"
#include<windows.h>
#include<gl/gl.h>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////////////////
//  FUNCTION:   
/*! \brief      Create a texture.  Keep this texture around for the duration
**              of its use.
**  \param      fileName    The file to load the texture from.  The extension
**                          states the image format of the texture
**  \param      hasAlpha    A flag denoting if the texture has an alpha channel.
**  \param      repeat      A flag denoting if the texture should repeat
**  \return     none
**  \exception  none
**  \par        DESCRIPTION:
**                  
**  \author     M Jared Finder
**  \date       10/29/2002 3:51:18 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
gfx::Texture::Texture(const char fileName[], bool hasAlpha, bool repeat)
{
    //set the data from the picture
    loader::Image picture(fileName);
    m_width     = picture.GetWidth();
    m_height    = picture.GetHeight();

    //add it to OGL
    glGenTextures(1, &m_object);
    glBindTexture(GL_TEXTURE_2D, m_object);
    glTexImage2D(
            GL_TEXTURE_2D, 0,
            hasAlpha ? GL_RGBA : GL_RGB,
            m_width, m_height, 0,
            GL_RGBA,    ///< assumes the image format is 32bit, from the member class
            GL_UNSIGNED_BYTE, picture.GetData());

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     repeat ? GL_REPEAT : GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     repeat ? GL_REPEAT : GL_CLAMP);
}


////////////////////////////////////////////////////////////////////////////////
//  FUNCTION:   
/*! \brief      Delete the texture.
**  \param      
**  \return     none
**  \exception  none
**  \par        DESCRIPTION:
**                  
**  \author     M Jared Finder
**  \date       10/29/2002 3:53:39 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
gfx::Texture::~Texture(void)
{
    glDeleteTextures(1, &m_object);
}
