////////////////////////////////////////////////////////////////////////////////
/*! \file       texture.h
**  \brief      Implementation of 2d-textures for graphics.
**  \author     M Jared Finder
**  \date       10/29/2002 1:06:20 PM
**  \par        DESCRIPTION:
**                  Loads from any format that Image supports.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
#include"../loader/image.h"


namespace gfx
{

////////////////////////////////////////////////////////////////////////////////
/*! \class      Texture
**  \brief      A texture consists of one rectangular picture that will be
**              ``mapped'' to a polygonal surface.
**  \author     M Jared Finder
**  \date       10/29/2002 1:24:58 PM
**  \warning    A texture must always be a power of two in its dimensions,
**              though it does not have to be square.
*/
////////////////////////////////////////////////////////////////////////////////
class Texture
{
public:
    ///load a texture from a file and possibly mipmap it
    Texture(const char fileName[], bool hasAlpha = false, bool repeat = false);
    ///clean up
    ~Texture(void);
    //accessor functions
    float GetLeft(void) const   { return 0; }
    float GetRight(void) const  { return 1; }
    float GetBottom(void) const { return 0; }
    float GetTop(void) const    { return 1; }

    ///get the texture handle (an unsigned in in OGL)
    unsigned int GetTextureHandle(void) const   { return m_object; }

private:
    ///texture name
    unsigned int   m_object;

    ///width of the texture
    unsigned int m_width;
    ///height of the texture
    unsigned int m_height;
};

}