////////////////////////////////////////////////////////////////////////////////
/*! \file       vtx_buffer.h
**  \brief      Places vertex buffers at accelerated memory (AGP memory or on
**              the card) for faster rendering.
**  \author     M Jared Finder
**  \date       2/27/2003 3:12:33 PM
**  \note       This is a quick hack to get quicker rendering by placing vertex
**              data near the video card.  Later, this should be changed to
**              something more elegent.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////


namespace gfx {
////////////////////////////////////////////////////////////////////////////////
/*! \class      VertexBuffer
**  \brief      Storage for vertecies, texture data, etc.  Acts pretty much
**              like a std::vector.
**  \author     M Jared Finder
**  \date       2/27/2003 3:14:04 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
class VertexBuffer
{
public:
    ///default constructor
    VertexBuffer(int size, bool isDynamic);
    ///destructor
    ~VertexBuffer(void);

    ///lock buffer, finishing changed
    void Lock(void);
    ///unlock buffer so changes can be made
    void* Unlock(void);

    ///activate the vertex buffer with the respective array
    void Activate(bool vertexArray, bool texCoordArray, bool clrArray, int stride);
    ///deactivate the buffer
    void Deactivate();

private:
    
public:
    ///data for NVidia vertex array extension or if VertexBuffers don't exist
    void* m_pData;
private:
    ///handle to ATI vertex buffer
    unsigned int m_atiHandle;
    ///number of elements in buffer
    int m_numElements;
    
    ///are we using NV extensions in this buffer?
    bool m_usingNV;
    ///are we using ATI extensions in this buffer?
    bool m_usingATI;
};


}