////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Aquarius_Screens.cpp
**	\brief		Implementation of Functions to create screens in Aquarius
**	\author		John Corpening
**	\date		1/18/2003 1:22:10 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
**				The steps to follow in order to make a screen in the game are:
**				-Create the screen
**				-Create the menus
**				-Create the menu items
**				-Add the menu items to the menus
**				-add the menu to the screen
**				-Create Actions
**				-Connect menu items with actions (when applicable)
**				-Set hotkeys for actions (when applicable)
**				-Create the keymap
**				-Add actions to the keymap
**				-Set the keymap to the screen
**				-set the Camera (if necessary)
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)
#include "../GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace UserInput;


//This is still here because it shows how to set up tool tips
UI_Screen* UserInput::createTestScreen()
{
	//Initialize the PlayerInput module
	//Create the screens
	
	UI_Screen		*test = new UI_Screen;
	UI_Menu			*mTest = new UI_Menu(test);
	UI_Pushbutton	*pTest = new UI_Pushbutton(test);
//	UI_ToolTip		*ttTest = new UI_ToolTip(test);
//	UI_ToolTip		*vtTest = new UI_ToolTip(test);
//	UI_EditBox		*eTest = new UI_EditBox(test);
	UI_Console		*cTest = UI_Console::getInstance();

	cTest->setScreen(test);
	cTest->setCursor(test->getCursor());
//	ttTest->setText("Terse");
//	vtTest->setText("Verbose");
//	pTest->setTipTerse(ttTest);
//	pTest->setTipVerbose(vtTest);
	pTest->setCaption("NextFish");
	mTest->addMenuItem(pTest, 0.01f,0.05f);
//	mTest->setPosition(0.85f,0.0f);
//	mTest->addMenuItem(eTest, 0.0,0.75);
	test->addMenu(cTest,0.01f,0.01f);
	test->addMenu(mTest, 0.75f, 0.0f);

	//create the keymap
	UI_Keymap					*keymap			= new UI_Keymap(test);
	UI_Action_SwitchScreens		*aTest			= new UI_Action_SwitchScreens(GUI_MAINMENU, test);
	UI_Action_SwitchScreens		*aToObserver	= new UI_Action_SwitchScreens(GUI_OBSERVERMODE, test);
	UI_Action_NextFish			*nextFish		= new UI_Action_NextFish(test);
	
	nextFish->setMenuItem(pTest);
//	UI_TestEditBox	*aTest2 = new UI_TestEditBox(test);

	aTest->setKey(DIK_ESCAPE);
	aToObserver->setKey(DIK_O);

//	aTest->setMenuItem(pTest);

//	aTest2->setMenuItem(eTest);

	keymap->addMenuItem(aTest, 0.0f, 0.0f);
	keymap->addMenuItem(nextFish, 0.0f, 0.0f);
	keymap->addMenuItem(aToObserver, 0.0f, 0.0f);
//	keymap->addMenuItem(aTest2,0.0f, 0.1f);
	
	test->setKeymap(keymap);

	test->setCamera(UI_Camera::getInstance());
	
	return test;
}

//Creates the main menu
UI_Screen* UserInput::createMainMenu()
{
	//Create the screen
	UI_Screen		*screen			= new UI_Screen;

	//Create the menus
	UI_Menu			*menu			= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();
	console->activate(false);
	

	//Create the menu items
	UI_Pushbutton	*pbEnter		= new UI_Pushbutton(screen);
		pbEnter->setCaption("Enter Aquarium");

	UI_Pushbutton	*pbNew			= new UI_Pushbutton(screen);
		pbNew->setCaption("New Aquarium");

	UI_Pushbutton	*pbSave			= new UI_Pushbutton(screen);
		pbSave->setCaption("Save Aquarium");

	UI_Pushbutton	*pbLoad			= new UI_Pushbutton(screen);
		pbLoad->setCaption("Load Aquarium");

	UI_Pushbutton	*pbOffline		= new UI_Pushbutton(screen);
		pbOffline->setCaption("Offline Mode");

	UI_Pushbutton	*pbMultiplayer	= new UI_Pushbutton(screen);
		pbMultiplayer->setCaption("Multiplayer");

	UI_Pushbutton	*pbGameOptions	= new UI_Pushbutton(screen);
		pbGameOptions->setCaption("Game Options");

	UI_Pushbutton	*pbSysOptions	= new UI_Pushbutton(screen);
		pbSysOptions->setCaption("System Options");

	UI_Pushbutton	*pbTitleScreen	= new UI_Pushbutton(screen);
		pbTitleScreen->setCaption("Title Screen");

	UI_Pushbutton	*pbQuit			= new UI_Pushbutton(screen);
		pbQuit->setCaption("Quit");

	//add the menu items to the menu
	menu->addMenuItem(pbEnter,			0.05f, 0.9f);
	menu->addMenuItem(pbNew,			0.05f, 0.8f);
	menu->addMenuItem(pbSave,			0.05f, 0.7f);
	menu->addMenuItem(pbLoad,			0.05f, 0.6f);
	menu->addMenuItem(pbOffline,		0.05f, 0.5f);
	menu->addMenuItem(pbMultiplayer,	0.05f, 0.4f);
	menu->addMenuItem(pbGameOptions,	0.05f, 0.3f);
	menu->addMenuItem(pbSysOptions,		0.05f, 0.2f);
	menu->addMenuItem(pbTitleScreen,	0.05f, 0.1f);
	menu->addMenuItem(pbQuit,			0.05f, 0.0f);

	//add the menu to the screen
	screen->addMenu(menu, 0.3f, 0.005f);

	//Create Actions
	UI_Action_Quit			*aQuit		= new UI_Action_Quit(screen);
	UI_Action_SwitchScreens *aEnter		= new UI_Action_SwitchScreens(GUI_EDITMODE, screen);
	UI_Action_SwitchScreens *aOffline	= new UI_Action_SwitchScreens(GUI_OFFLINEMODE, screen);
	UI_Action_SwitchScreens *aTest		= new UI_Action_SwitchScreens(GUI_TESTMODE, screen); //This is temporary
	UI_Action_SwitchScreens *aGameOptions = new UI_Action_SwitchScreens(GUI_GAMEOPTIONS, screen);
	UI_Action_SwitchScreens *aSysOptions = new UI_Action_SwitchScreens(GUI_SYSTEMOPTIONS, screen);
	UI_Action_SwitchScreens *aMultiplayer = new UI_Action_SwitchScreens(GUI_MULTIPLAYER, screen);
	UI_Action_SwitchScreens *aSaveScreen = new UI_Action_SwitchScreens(GUI_SAVESCREEN, screen);
	UI_Action_SwitchScreens *aLoadScreen = new UI_Action_SwitchScreens(GUI_LOADSCREEN, screen);
	UI_Action_SwitchScreens *aTitleScreen = new UI_Action_SwitchScreens(GUI_TITLESCREEN, screen);
	UI_Action_NewAquarium	*aNew		= new UI_Action_NewAquarium(screen);

	UI_Action_PrevScreen	*aPrev		= new UI_Action_PrevScreen(screen);
	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);

//	UI_Action_FullScreen	*afullScreen	= new UI_Action_FullScreen(screen);
	
	//Connect menu items with actions (when applicable)
	aQuit->setMenuItem(pbQuit);
	aEnter->setMenuItem(pbEnter);
	aOffline->setMenuItem(pbOffline);
	aGameOptions->setMenuItem(pbGameOptions);
	aSysOptions->setMenuItem(pbSysOptions);
	aMultiplayer->setMenuItem(pbMultiplayer);
	aSaveScreen->setMenuItem(pbSave);
	aLoadScreen->setMenuItem(pbLoad);
	aTitleScreen->setMenuItem(pbTitleScreen);
	aNew->setMenuItem(pbNew);

	//Set hotkeys for actions (when applicable)
	aQuit->setKey(DIK_Q);
	aPrev->setKey(DIK_ESCAPE);
	aTest->setKey(DIK_T);
	aConsole->setKey(DIK_GRAVE);
//	afullScreen->setKey(DIK_F);


	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aQuit, 0.0f, 0.0f);
	keymap->addMenuItem(aEnter, 0.0f, 0.0f);
	keymap->addMenuItem(aPrev, 0.0f, 0.0f);
	keymap->addMenuItem(aTest, 0.0f, 0.0f);
	keymap->addMenuItem(aOffline, 0.0f, 0.0f);
	keymap->addMenuItem(aGameOptions, 0.0f, 0.0f);
	keymap->addMenuItem(aSysOptions, 0.0f, 0.0f);
	keymap->addMenuItem(aMultiplayer, 0.0f, 0.0f);
	keymap->addMenuItem(aConsole, 0.0f,0.0f);
//	keymap->addMenuItem(afullScreen, 0, 0);
	keymap->addMenuItem(aSaveScreen, 0, 0);
	keymap->addMenuItem(aLoadScreen, 0, 0);
	keymap->addMenuItem(aTitleScreen,0, 0);
	keymap->addMenuItem(aNew ,0,0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the observer mode screen
UI_Screen* UserInput::createObserverScreen()
{
	//Create the screen
	UI_Screen *screen	= new UI_Screen;

	//Create the menus
	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items

	//add the menu to the screen

	//Create Actions
	UI_Action_SwitchScreens *aToMainMenu = new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_SwitchScreens *aToEditMode = new UI_Action_SwitchScreens(GUI_EDITMODE, screen);

	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);
	
	//Connect menu items with actions (when applicable)

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);
	aToEditMode->setKey(DIK_RBUTTON);
	aConsole->setKey(DIK_GRAVE);

	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0.0f, 0.0f);
	keymap->addMenuItem(aToEditMode, 0.0f, 0.0f);
	keymap->addMenuItem(aConsole, 0.0f,0.0f);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//set the Camera
	screen->setCamera(UI_Camera::getInstance());
	screen->drawCursor(false);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the swim mode screen
UI_Screen* UserInput::createSwimModeScreen()
{
	//Create the screen
	UI_Screen *screen	= new UI_Screen;

	//Create the menus
	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items

	//add the menu to the screen

	//Create Actions
	UI_Action_SwitchScreens			*aToMainMenu	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_SwitchScreens			*aToEditMode	= new UI_Action_SwitchScreens(GUI_EDITMODE, screen);
	UI_Action_FishForward			*aFishForward	= new UI_Action_FishForward(screen);
	UI_Action_FishBackward			*aFishBackward	= new UI_Action_FishBackward(screen);
	UI_Action_FishLeft				*aFishLeft		= new UI_Action_FishLeft(screen);
	UI_Action_FishRight				*aFishRight		= new UI_Action_FishRight(screen);
	UI_Action_IssueCommand			*aAttack		= new UI_Action_IssueCommand(AI_System::ATTACK, screen);
	UI_Action_IssueCommand			*aAttackAlt		= new UI_Action_IssueCommand(AI_System::ATTACK, screen);
//	UI_Action_Eat					*aEat			= new UI_Action_Eat(screen);
//	UI_Action_DropPosMarker			*aDropPos		= new UI_Action_DropPosMarker(screen);
//	UI_Action_DropNegMarker			*aDropNeg		= new UI_Action_DropNegMarker(screen);
	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);
	
	//Connect menu items with actions (when applicable)

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);
	aToEditMode->setKey(DIK_RBUTTON);
	aFishForward->setKey(DIK_W);
	aFishBackward->setKey(DIK_S);
	aFishLeft->setKey(DIK_A);
	aFishRight->setKey(DIK_D);
//	aEat->setKey(DIK_1);
	aAttack->setKey(DIK_LBUTTON);
	aAttackAlt->setKey(DIK_SPACE);
//	aDropPos->setKey(DIK_3);
//	aDropNeg->setKey(DIK_4);
	aConsole->setKey(DIK_GRAVE);

	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0.0f, 0.0f);
	keymap->addMenuItem(aToEditMode, 0.0f, 0.0f);
	keymap->addMenuItem(aFishForward, 0.0f, 0.0f);
	keymap->addMenuItem(aAttack, 0.0f, 0.0f);
//	keymap->addMenuItem(aEat, 0.0f, 0.0f);
//	keymap->addMenuItem(aDropPos, 0.0f, 0.0f);
//	keymap->addMenuItem(aDropNeg, 0.0f, 0.0f);
	keymap->addMenuItem(aFishBackward, 0.0f, 0.0f);
	keymap->addMenuItem(aFishLeft, 0.0f, 0.0f);
	keymap->addMenuItem(aFishRight, 0.0f, 0.0f);
	keymap->addMenuItem(aAttackAlt, 0,0);
	keymap->addMenuItem(aConsole, 0.0f,0.0f);


	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//set the Camera
	screen->setCamera(UI_Camera::getInstance());
	screen->drawCursor(false);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the offline mode screen
UI_Screen* UserInput::createOfflineModeScreen()
{
	//Create the screen
	UI_Screen	*screen	= new UI_Screen;

	//Create the menus
	UI_Menu	*menu = new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	UI_TextBox	*tbNumFish		= new UI_TextBox(screen);
		tbNumFish->setActualHeight(0.05f);
		tbNumFish->setActualWidth(0.6f);
	UI_TextBox	*tbGameSpeed		= new UI_TextBox(screen);
		tbGameSpeed->setActualHeight(0.05f);
		tbGameSpeed->setActualWidth(0.6f);
	UI_TextBox	*tbTimeElapsed	= new UI_TextBox(screen);
		tbTimeElapsed->setActualHeight(0.05f);
		tbTimeElapsed->setActualWidth(0.6f);
	UI_TextBox	*tbVTimeElapsed	= new UI_TextBox(screen);
		tbVTimeElapsed->setActualHeight(0.05f);
		tbVTimeElapsed->setActualWidth(0.6f);
	UI_TextBox	*tbExit			= new UI_TextBox(screen);
		tbExit->setText("Hit ESCAPE to exit");
		tbExit->setActualHeight(0.05f);
		tbExit->setActualWidth(0.6f);

	//add menu items to the menus
	menu->addMenuItem(tbNumFish,		0.01f, 0.45f);
	menu->addMenuItem(tbGameSpeed,		0.01f, 0.35f);
	menu->addMenuItem(tbTimeElapsed,	0.01f, 0.25f);
	menu->addMenuItem(tbVTimeElapsed,	0.01f, 0.15f);
	menu->addMenuItem(tbExit,			0.01f, 0.05f);

	//add the menu to the screen
	screen->addMenu(menu, 0.175f, 0.25f);

	//Create Actions
	UI_Action_SwitchScreens		*aToMainMenu	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_ActivateConsole	*aConsole		= new UI_Action_ActivateConsole(screen);
	UI_Action_NumFish			*aNumFish		= new UI_Action_NumFish(screen);
	UI_Action_TimeElapsed		*aRealTime		= new UI_Action_TimeElapsed(screen);
	UI_Action_VTimeElapsed		*aGameTime		= new UI_Action_VTimeElapsed(screen);
	UI_Action_IncGameSpeed		*aIncSpeed		= new UI_Action_IncGameSpeed(screen);
	UI_Action_DecGameSpeed		*aDecSpeed		= new UI_Action_DecGameSpeed(screen);
	UI_Action_GameSpeedStat		*aSpeedStat		= new UI_Action_GameSpeedStat(screen);

	//Connect menu items with actions (when applicable)
	aNumFish->setMenuItem(tbNumFish);
	aRealTime->setMenuItem(tbTimeElapsed);
	aGameTime->setMenuItem(tbVTimeElapsed);
	aSpeedStat->setMenuItem(tbGameSpeed);

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);
	aConsole->setKey(DIK_GRAVE);
	aIncSpeed->setKey(DIK_ADD);
	aDecSpeed->setKey(DIK_SUBTRACT);
	
	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0.0f, 0.0f);
	keymap->addMenuItem(aConsole, 0.0f,0.0f);
	keymap->addMenuItem(aNumFish, 0, 0);
	keymap->addMenuItem(aRealTime, 0, 0);
	keymap->addMenuItem(aGameTime, 0, 0);
	keymap->addMenuItem(aIncSpeed, 0, 0);
	keymap->addMenuItem(aDecSpeed, 0, 0);
	keymap->addMenuItem(aSpeedStat, 0,0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the edit mode screen
UI_Screen* UserInput::createEditModeScreen()
{
	//Create the screen
	UI_Screen		*screen			= new UI_Screen;

	//Create the menus
	UI_Menu			*fishSideBar	= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	UI_Pushbutton	*pbRandomFish	= new UI_Pushbutton(screen);
	pbRandomFish->setCaption("Add Fish");
	pbRandomFish->setActualWidth(0.15f);

	UI_Pushbutton	*iNextFish		= new UI_Pushbutton(screen);
		iNextFish->setCaption("Next Fish");
		iNextFish->setActualWidth(0.15f);

	UI_Pushbutton	*iObserver		= new UI_Pushbutton(screen);
		iObserver->setCaption("Observe");
		iObserver->setActualWidth(0.15f);

	fishSideBar->addMenuItem(iObserver,		0.01f, 0.01f);
	fishSideBar->addMenuItem(iNextFish,		0.01f, 0.09f);
	fishSideBar->addMenuItem(pbRandomFish,	0.01f, 0.17f);
	fishSideBar->setActualWidth(0.18f);

	//fish options menu
	UI_Menu			*fishOptions	= new UI_Menu(screen);

	UI_Pushbutton	*iSurpress		= new UI_Pushbutton(screen);
		iSurpress->setCaption("Suppress");
		iSurpress->setActualWidth(0.15f);

	UI_Pushbutton	*iBrainView		= new UI_Pushbutton(screen);
		iBrainView->setCaption("Brains");
		iBrainView->setActualWidth(0.15f);

	UI_Pushbutton	*iBodyView			= new UI_Pushbutton(screen);
		iBodyView->setCaption("Body");
		iBodyView->setActualWidth(0.15f);

	UI_Pushbutton	*iStats			= new UI_Pushbutton(screen);
		iStats->setCaption("Stats");
		iStats->setActualWidth(0.15f);

	UI_Pushbutton *iAutoSwim = new UI_Pushbutton(screen);
		iAutoSwim->setCaption("Swim Mode");
		iAutoSwim->setActualWidth(0.15f);

	UI_Pushbutton *iCommandBar = new UI_Pushbutton(screen);
		iCommandBar->setCaption("Commands");
		iCommandBar->setActualWidth(0.15f);

	fishOptions->addMenuItem(iCommandBar,	0.01f, 0.01f);
	fishOptions->addMenuItem(iSurpress,		0.01f, 0.09f);
	fishOptions->addMenuItem(iStats,		0.01f, 0.17f);
	fishOptions->addMenuItem(iBrainView,	0.01f, 0.25f);
	fishOptions->addMenuItem(iBodyView,		0.01f, 0.33f);
	fishOptions->addMenuItem(iAutoSwim,		0.01f, 0.41f);

	fishOptions->setActualWidth(0.18f);
	fishOptions->activate(false);


	//add the menu to the screen
	screen->addMenu(fishOptions, 0.0f, 0.27f);
	screen->addMenu(fishSideBar, 0.0f, 0.75f);
	

	//Fish Body stats menu
	UI_Menu			*fishBodyStats	= new UI_Menu(screen);
	fishBodyStats->showBackground(false);
	fishBodyStats->activate(false);

	//Fish stats menu items
	/*	Metabolism
		Armor
		teeth/baleen
		size
		Mating Inclination
		health_normalized
		stomach_normalized
		energyLevel_normalized
	*/

	UI_TextBox		*iEnergy	= new UI_TextBox(screen);
		iEnergy->setText("Energy: 0.0%");
		iEnergy->setActualWidth(0.28f);
		iEnergy->setActualHeight(0.05f);

	UI_TextBox		*iHealth		= new UI_TextBox(screen);
		iHealth->setText("Health: 0.0");
		iHealth->setActualWidth(0.28f);
		iHealth->setActualHeight(0.05f);

	UI_TextBox		*iMetabolism		= new UI_TextBox(screen);
		iMetabolism->setText("Metab.: 0.0%");
		iMetabolism->setActualWidth(0.28f);
		iMetabolism->setActualHeight(0.05f);

	UI_TextBox		*iArmor		= new UI_TextBox(screen);
		iArmor->setText("Armor: 0.0%");
		iArmor->setActualWidth(0.28f);
		iArmor->setActualHeight(0.05f);

	UI_TextBox		*iTeeth		= new UI_TextBox(screen);
		iTeeth->setText("Teeth: 0.0%");
		iTeeth->setActualWidth(0.28f);
		iTeeth->setActualHeight(0.05f);

	UI_TextBox		*iSize		= new UI_TextBox(screen);
		iSize->setText("Size: 0.0%");
		iSize->setActualWidth(0.28f);
		iSize->setActualHeight(0.05f);

	UI_TextBox		*iMating		= new UI_TextBox(screen);
		iMating->setText("Mating: 0.0%");
		iMating->setActualWidth(0.28f);
		iMating->setActualHeight(0.05f);

	UI_TextBox		*iStomach		= new UI_TextBox(screen);
		iStomach->setText("Hunger: 0.0%");
		iStomach->setActualWidth(0.28f);
		iStomach->setActualHeight(0.05f);

	//Add the items to the menu
	fishBodyStats->addMenuItem(iEnergy,			0.01f, 0.01f);
	fishBodyStats->addMenuItem(iHealth,			0.01f, 0.07f);
	fishBodyStats->addMenuItem(iMetabolism,		0.01f, 0.13f);
	fishBodyStats->addMenuItem(iArmor,			0.01f, 0.19f);

	fishBodyStats->addMenuItem(iTeeth,			0.3f, 0.01f);
	fishBodyStats->addMenuItem(iSize,			0.3f, 0.07f);
	fishBodyStats->addMenuItem(iMating,			0.3f, 0.13f);
	fishBodyStats->addMenuItem(iStomach,		0.3f, 0.19f);

	//Add the menu to the screen
	screen->addMenu(fishBodyStats, 0.22f, 0.75f);

	//Fish Brain Viewer
	UI_Menu		*fishBrainStats	= new UI_Menu(screen);
		fishBrainStats->showBackground(false);
		fishBrainStats->activate(false);

	UI_TextBox	*iMoveAction		= new UI_TextBox(screen);
		iMoveAction->setActualWidth(0.6f);
		iMoveAction->setActualHeight(0.05f);
	UI_TextBox	*iMoveTarget		= new UI_TextBox(screen);
		iMoveTarget->setActualWidth(0.6f);
		iMoveTarget->setActualHeight(0.05f);
	UI_TextBox	*iMoveGoal		= new UI_TextBox(screen);
		iMoveGoal->setActualWidth(0.6f);
		iMoveGoal->setActualHeight(0.05f);
	UI_TextBox	*iSurpressedStatus	= new UI_TextBox(screen);
		iSurpressedStatus->setActualWidth(0.6f);
		iSurpressedStatus->setActualHeight(0.05f);

	fishBrainStats->addMenuItem(iSurpressedStatus,	0.01f, 0.19f);
	fishBrainStats->addMenuItem(iMoveAction,		0.01f, 0.13f);
	fishBrainStats->addMenuItem(iMoveTarget,		0.01f, 0.07f);
	fishBrainStats->addMenuItem(iMoveGoal,			0.01f, 0.01f);
	
	

	screen->addMenu(fishBrainStats, 0.22f, 0.75f);

	//Fish Stats Viewer
	//Age				Attacks		Distance
	//Avg Energy		Kills		Avg. Speed
	//Avg Health		Defends		Plankton

	UI_Menu	*fishStats = new UI_Menu(screen);
		fishStats->showBackground(false);
		fishStats->activate(false);

	UI_TextBox	*iAge	= new UI_TextBox(screen);
		iAge->setText("Age: ");
		iAge->setActualHeight(0.05f);
		iAge->setActualWidth(0.35f);

	UI_TextBox	*iAvgEnergy	= new UI_TextBox(screen);
		iAvgEnergy->setText("Avg. Energy: ");
		iAvgEnergy->setActualHeight(0.05f);
		iAvgEnergy->setActualWidth(0.35f);

	UI_TextBox	*iAvgHealth	= new UI_TextBox(screen);
		iAvgHealth->setText("Avg. Health: ");
		iAvgHealth->setActualHeight(0.05f);
		iAvgHealth->setActualWidth(0.35f);

	UI_TextBox	*iAttackCount	= new UI_TextBox(screen);
		iAttackCount->setText("Attacks: ");
		iAttackCount->setActualHeight(0.05f);
		iAttackCount->setActualWidth(0.35f);

	UI_TextBox	*iKillCount	= new UI_TextBox(screen);
		iKillCount->setText("Kills: ");
		iKillCount->setActualHeight(0.05f);
		iKillCount->setActualWidth(0.35f);

	UI_TextBox	*iDefendCount	= new UI_TextBox(screen);
		iDefendCount->setText("Defends:");
		iDefendCount->setActualHeight(0.05f);
		iDefendCount->setActualWidth(0.35f);

	UI_TextBox	*iDistance	= new UI_TextBox(screen);
		iDistance->setText("Distance: ");
		iDistance->setActualHeight(0.05f);
		iDistance->setActualWidth(0.35f);

	UI_TextBox	*iAvgSpeed	= new UI_TextBox(screen);
		iAvgSpeed->setText("Avg. Speed: ");
		iAvgSpeed->setActualHeight(0.05f);
		iAvgSpeed->setActualWidth(0.35f);

	UI_TextBox	*iPlankton	= new UI_TextBox(screen);
		iPlankton->setText("Plankton: ");
		iPlankton->setActualHeight(0.05f);
		iPlankton->setActualWidth(0.35f);

	UI_TextBox	*iChum	= new UI_TextBox(screen);
		iChum->setText("Chum: ");
		iChum->setActualHeight(0.05f);
		iChum->setActualWidth(0.35f);


	fishStats->addMenuItem(iAge,				0.01f, 0.25f);
	fishStats->addMenuItem(iAvgEnergy,			0.01f, 0.19f);
	fishStats->addMenuItem(iAvgHealth,			0.01f, 0.13f);
	fishStats->addMenuItem(iAvgSpeed,			0.01f, 0.07f);
	fishStats->addMenuItem(iDistance,			0.01f, 0.01f);
	
	fishStats->addMenuItem(iAttackCount,		0.37f, 0.25f);
	fishStats->addMenuItem(iKillCount,			0.37f, 0.19f);
	fishStats->addMenuItem(iDefendCount,		0.37f, 0.13f);
	fishStats->addMenuItem(iPlankton,			0.37f, 0.07f);
	fishStats->addMenuItem(iChum,				0.37f, 0.01f);
	
	

	screen->addMenu(fishStats, 0.22f, 0.68f);

	//Command bar menu
	UI_Menu			*commandBar	= new UI_Menu(screen);
	commandBar->activate(false);

	//Command Bar menu items
	UI_Pushbutton *iMoveTo	= new UI_Pushbutton(screen);
		iMoveTo->setCaption("Move To");
		iMoveTo->setActualWidth(0.13f);

	UI_Pushbutton *iFlee = new UI_Pushbutton(screen);
		iFlee->setCaption("Flee");
		iFlee->setActualWidth(0.13f);

	UI_Pushbutton *iFollow	= new UI_Pushbutton(screen);
		iFollow->setCaption("Follow");
		iFollow->setActualWidth(0.13f);

	UI_Pushbutton *iFlock	= new UI_Pushbutton(screen);
		iFlock->setCaption("Flock");
		iFlock->setActualWidth(0.13f);

	UI_Pushbutton *iRoam	= new UI_Pushbutton(screen);
		iRoam->setCaption("Roam");
		iRoam->setActualWidth(0.13f);

	UI_Pushbutton *iAttack	= new UI_Pushbutton(screen);
		iAttack->setCaption("Attack");
		iAttack->setActualWidth(0.13f);

	UI_Pushbutton *iMate	= new UI_Pushbutton(screen);
		iMate->setCaption("Mate");
		iMate->setActualWidth(0.13f);

	//Command bar second level
//Note: It was decided to remove marker ADNs therefore this portion of the command bar
//		is unnecessary
//	UI_Pushbutton *iDropPos	= new UI_Pushbutton(screen);
//		iDropPos->setCaption("Drop +");
//		iDropPos->setActualWidth(0.13f);

//	UI_Pushbutton *iDropNeg	= new UI_Pushbutton(screen);
//		iDropNeg->setCaption("Drop -");
//		iDropNeg->setActualWidth(0.13f);

//	UI_Pushbutton *iGoPos	= new UI_Pushbutton(screen);
//		iGoPos->setCaption("Go +");
//		iGoPos->setActualWidth(0.13f);

//	UI_Pushbutton *iFleeNeg	= new UI_Pushbutton(screen);
//		iFleeNeg->setCaption("Flee -");
//		iFleeNeg->setActualWidth(0.13f);

//	UI_Pushbutton *iHeed	= new UI_Pushbutton(screen);
//		iHeed->setCaption("Heed");
//		iHeed->setActualWidth(0.13f);

//	UI_Pushbutton *iIgnore	= new UI_Pushbutton(screen);
//		iIgnore->setCaption("Ignore");
//		iIgnore->setActualWidth(0.13f);



	//Add the menu items to the menus
//	commandBar->addMenuItem(iEat,		0.02f, 0.025f);
	commandBar->addMenuItem(iFlee,		0.01f, 0.09f);
	commandBar->addMenuItem(iFollow,	0.15f, 0.09f);
	commandBar->addMenuItem(iFlock,		0.29f, 0.09f);
	commandBar->addMenuItem(iAttack,	0.43f, 0.05f);
	commandBar->addMenuItem(iRoam,		0.01f, 0.01f);
	commandBar->addMenuItem(iMoveTo,	0.15f, 0.01f);
	commandBar->addMenuItem(iMate,		0.29f, 0.01f);


//	commandBar->addMenuItem(iDropPos,	0.57f, 0.09f);
//	commandBar->addMenuItem(iDropNeg,	0.71f, 0.09f);
//	commandBar->addMenuItem(iGoPos,		0.85f, 0.09f);
//	commandBar->addMenuItem(iFleeNeg,	0.57f, 0.01f);
//	commandBar->addMenuItem(iHeed,		0.71f, 0.01f);
//	commandBar->addMenuItem(iIgnore,	0.85f, 0.01f);
//	commandBar->addMenuItem(iAutoSwim,	0.82f, 0.025f);

	//add the menu to the screen
	commandBar->setActualHeight(0.17f);
	screen->addMenu(commandBar, 0.0f, 0.0f);

	//Create Actions
	UI_Action_SwitchScreens			*aToMainMenu	 = new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_SwitchScreens			*aToObserverMode = new UI_Action_SwitchScreens(GUI_OBSERVERMODE, screen);
	UI_Action_AddRandomFish			*aAddFish		 = new UI_Action_AddRandomFish(screen);
	UI_Action_ActivateStats			*aACB			 = new UI_Action_ActivateStats(screen);
	UI_Action_NextFish				*aNextFish		 = new UI_Action_NextFish(screen);
	UI_Action_SwitchScreens			*aToSwimMode	 = new UI_Action_SwitchScreens(GUI_SWIMMODE, screen);
	UI_Action_ActivateCommandBar	*aActivateOptions = new UI_Action_ActivateCommandBar(screen);
	UI_Action_ModeToggle			*aModeToggle	= new UI_Action_ModeToggle(screen);

	//Commands
	UI_Action_IssueCommand			*aAttack		 = new UI_Action_IssueCommand(AI_System::ATTACK, screen);
	UI_Action_IssueCommand			*aFlee			 = new UI_Action_IssueCommand(AI_System::FLEE, screen);
	UI_Action_IssueCommand			*aFollow		 = new UI_Action_IssueCommand(AI_System::FOLLOW, screen);
	UI_Action_IssueCommand			*aFlock			 = new UI_Action_IssueCommand(AI_System::MOVEWITH, screen);
	UI_Action_IssueCommand			*aRoam			 = new UI_Action_IssueCommand(AI_System::ROAM, screen);
	UI_Action_IssueCommand			*aMoveTo		 = new UI_Action_IssueCommand(AI_System::MOVETO, screen);
	UI_Action_Mate					*aMate			 = new UI_Action_Mate(screen);

//	UI_Action_IssueCommand			*aDropPos		 = new UI_Action_IssueCommand(AI_System::DROP_POS_MARKER, screen);
//	UI_Action_IssueCommand			*aDropNeg		 = new UI_Action_IssueCommand(AI_System::DROP_NEG_MARKER, screen);
//	UI_Action_IssueCommand			*aGoPos			 = new UI_Action_IssueCommand(AI_System::GOTO_POS_MARKER, screen);
//	UI_Action_IssueCommand			*aFleeNeg		 = new UI_Action_IssueCommand(AI_System::FLEE_NEG_MARKER, screen);
//	UI_Action_IssueCommand			*aHeed			 = new UI_Action_IssueCommand(AI_System::HEED_MARKERS, screen);
//	UI_Action_IssueCommand			*aIgnore		 = new UI_Action_IssueCommand(AI_System::IGNORE_MARKERS, screen);

	


	//Fish Options Actions
	UI_Action_SurpressBrain			*aSurpress		 = new UI_Action_SurpressBrain(screen);

	//Fish Body stats actions
	UI_Action_ActivateStats			*aActivateStats	 = new UI_Action_ActivateStats(screen);
	UI_Action_CheckHealth			*aHealth		 = new UI_Action_CheckHealth(screen);
	UI_Action_CheckEnergy			*aEnergy		 = new UI_Action_CheckEnergy(screen);
	UI_Action_CheckArmor			*aArmor			 = new UI_Action_CheckArmor(screen);
	UI_Action_CheckMetabolism		*aMetabolism	 = new UI_Action_CheckMetabolism(screen);
	UI_Action_CheckSize				*aSize			 = new UI_Action_CheckSize(screen);
	UI_Action_CheckStomach			*aStomach		 = new UI_Action_CheckStomach(screen);
	UI_Action_CheckTeeth			*aTeeth			 = new UI_Action_CheckTeeth(screen);
	UI_Action_CheckMating			*aMating		 = new UI_Action_CheckMating(screen);

	//Fish Brain Stats actions
	UI_Action_ActivateStats			*aBrainStats		= new UI_Action_ActivateStats(screen);
	UI_Action_SurpressedStatus		*aSurpStat			= new UI_Action_SurpressedStatus(screen);
	UI_Action_CheckMoveAction		*aCheckMove			= new UI_Action_CheckMoveAction(screen);
	UI_Action_CheckMoveTarget		*aCheckMoveTarget	= new UI_Action_CheckMoveTarget(screen);
	UI_Action_CheckMoveGoal			*aCheckMoveGoal		= new UI_Action_CheckMoveGoal(screen);

	//Fish Stat Viewer
	UI_Action_ActivateStats			*aFishStats		= new UI_Action_ActivateStats(screen);
	UI_Action_CheckAge				*aAge			= new UI_Action_CheckAge(screen);
	UI_Action_CheckAvgEnergy		*aAvgEnergy		= new UI_Action_CheckAvgEnergy(screen);
	UI_Action_CheckAvgHealth		*aAvgHealth		= new UI_Action_CheckAvgHealth(screen);
	UI_Action_CheckAttacks			*aAttacks		= new UI_Action_CheckAttacks(screen);
	UI_Action_CheckKills			*aKills			= new UI_Action_CheckKills(screen);
	UI_Action_CheckDefends			*aDefends		= new UI_Action_CheckDefends(screen);
	UI_Action_CheckPlankton			*aPlankton		= new UI_Action_CheckPlankton(screen);
	UI_Action_CheckDistance			*aDistance		= new UI_Action_CheckDistance(screen);
	UI_Action_CheckAvgSpeed			*aAvgSpeed		= new UI_Action_CheckAvgSpeed(screen);
	UI_Action_CheckChum				*aChum			= new UI_Action_CheckChum(screen);

	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);
	
	
	//Connect menu items with actions (when applicable)
	aAddFish->setMenuItem(pbRandomFish);
	aACB->setMenuItem(iCommandBar);
	aACB->setStatsDisplay(commandBar);
	aActivateOptions->setCommandBar(fishOptions);
	aNextFish->setMenuItem(iNextFish);
	aToObserverMode->setMenuItem(iObserver);

	aAttack->setMenuItem(iAttack);
	aFlee->setMenuItem(iFlee);
	aFlock->setMenuItem(iFlock);
	aFollow->setMenuItem(iFollow);
	aRoam->setMenuItem(iRoam);
	aMoveTo->setMenuItem(iMoveTo);
//	aGoPos->setMenuItem(iGoPos);
//	aFleeNeg->setMenuItem(iFleeNeg);
//	aHeed->setMenuItem(iHeed);
//	aIgnore->setMenuItem(iIgnore);
	aMate->setMenuItem(iMate);
//	aEat->setMenuItem(iEat);
//	aDropPos->setMenuItem(iDropPos);
//	aDropNeg->setMenuItem(iDropNeg);
	aToSwimMode->setMenuItem(iAutoSwim);

	aActivateStats->setMenuItem(iBodyView);
	aActivateStats->addOtherMenus(fishBrainStats);
	aActivateStats->addOtherMenus(fishStats);
	aActivateStats->setStatsDisplay(fishBodyStats);

	aBrainStats->setMenuItem(iBrainView);
	aBrainStats->addOtherMenus(fishBodyStats);
	aBrainStats->addOtherMenus(fishStats);
	aBrainStats->setStatsDisplay(fishBrainStats);

	aFishStats->setMenuItem(iStats);
	aFishStats->addOtherMenus(fishBrainStats);
	aFishStats->addOtherMenus(fishBodyStats);
	aFishStats->setStatsDisplay(fishStats);


	//Fish Body Stats
	aHealth->setMenuItem(iHealth);
	aEnergy->setMenuItem(iEnergy);
	aArmor->setMenuItem(iArmor);
	aMetabolism->setMenuItem(iMetabolism);
	aSize->setMenuItem(iSize);
	aStomach->setMenuItem(iStomach);
	aTeeth->setMenuItem(iTeeth);
	aMating->setMenuItem(iMating);
	aChum->setMenuItem(iChum);

	//Fish Brain Stats
	aSurpStat->setMenuItem(iSurpressedStatus);
	aCheckMove->setMenuItem(iMoveAction);
	aCheckMoveTarget->setMenuItem(iMoveTarget);
	aCheckMoveGoal->setMenuItem(iMoveGoal);

	//Fish Stats
	aAge->setMenuItem(iAge);
	aAvgEnergy->setMenuItem(iAvgEnergy);
	aAvgHealth->setMenuItem(iAvgHealth);
	aAttacks->setMenuItem(iAttackCount);
	aKills->setMenuItem(iKillCount);
	aDefends->setMenuItem(iDefendCount);
	aPlankton->setMenuItem(iPlankton);
	aDistance->setMenuItem(iDistance);
	aAvgSpeed->setMenuItem(iAvgSpeed);

	//Fish Options
	aSurpress->setMenuItem(iSurpress);

	aConsole->setKey(DIK_GRAVE);

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);
	aModeToggle->setKey(DIK_RBUTTON);
	aAttack->setKey(DIK_SPACE);


	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0.0f, 0.0f);
	keymap->addMenuItem(aToObserverMode, 0.0f, 0.0f);
	keymap->addMenuItem(aAddFish, 0.0f, 0.0f);
	keymap->addMenuItem(aACB, 0.0f, 0.0f);
	keymap->addMenuItem(aNextFish, 0.0f, 0.0f);
	keymap->addMenuItem(aAttack, 0.0f, 0.0f);
	keymap->addMenuItem(aFlee,0,0);
	keymap->addMenuItem(aFlock,0,0);
	keymap->addMenuItem(aFollow,0,0);
	keymap->addMenuItem(aRoam,0,0);
	keymap->addMenuItem(aMoveTo,0,0);
//	keymap->addMenuItem(aGoPos,0,0);
//	keymap->addMenuItem(aFleeNeg,0,0);
//	keymap->addMenuItem(aHeed,0,0);
//	keymap->addMenuItem(aIgnore,0,0);
//	keymap->addMenuItem(aEat, 0.0f, 0.0f);
//	keymap->addMenuItem(aDropPos, 0.0f, 0.0f);
//	keymap->addMenuItem(aDropNeg, 0.0f, 0.0f);
	keymap->addMenuItem(aToSwimMode, 0.0f, 0.0f);
	keymap->addMenuItem(aMate, 0,0);
	keymap->addMenuItem(aActivateStats, 0.0f, 0.0f);
	keymap->addMenuItem(aModeToggle,0,0);

	//Add fish options commands
	keymap->addMenuItem(aSurpress, 0, 0);

	//Add body stats actions
	keymap->addMenuItem(aHealth, 0.0f, 0.0f);
	keymap->addMenuItem(aEnergy, 0.0f, 0.0f);
	keymap->addMenuItem(aArmor, 0.0f, 0.0f);
	keymap->addMenuItem(aMetabolism, 0.0f, 0.0f);
	keymap->addMenuItem(aSize, 0.0f, 0.0f);
	keymap->addMenuItem(aStomach, 0.0f, 0.0f);
	keymap->addMenuItem(aTeeth, 0.0f, 0.0f);
	keymap->addMenuItem(aMating, 0.0f, 0.0f);

	//Add Brain stats actions
	keymap->addMenuItem(aSurpStat, 0.0f, 0.0f);
	keymap->addMenuItem(aCheckMove, 0,0);
	keymap->addMenuItem(aCheckMoveTarget, 0, 0);
	keymap->addMenuItem(aCheckMoveGoal, 0, 0);

	//Add Fish Stats actions
	keymap->addMenuItem(aAge, 0,0);
	keymap->addMenuItem(aAvgEnergy, 0,0);	
	keymap->addMenuItem(aAvgHealth	, 0,0);
	keymap->addMenuItem(aAttacks	, 0,0);
	keymap->addMenuItem(aKills		, 0,0);
	keymap->addMenuItem(aDefends	, 0,0);
	keymap->addMenuItem(aPlankton	, 0,0);
	keymap->addMenuItem(aDistance	, 0,0);
	keymap->addMenuItem(aAvgSpeed	, 0,0);
	keymap->addMenuItem(aChum		, 0,0);

	keymap->addMenuItem(aConsole, 0.0f,0.0f);
	keymap->addMenuItem(aActivateOptions, 0.0f, 0.0f);
	keymap->addMenuItem(aBrainStats, 0.0f, 0.0f);
	keymap->addMenuItem(aFishStats , 0, 0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	screen->setCamera(UI_Camera::getInstance());

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//creates the game options screen
UI_Screen* UserInput::createGameOptionsScreen()
{
	//Create the screen
	UI_Screen *screen = new UI_Screen;

	//Create the menus
	UI_Menu	*menu = new UI_Menu(screen);
	UI_Console	*console	= UI_Console::getInstance();
	

	//Create the menu items
	UI_CheckBox		*iYInvert		= new UI_CheckBox(screen);
		iYInvert->setCaption("Invert Y-Axis");

	UI_EditBox		*iMinEdit		= new UI_EditBox(screen);
		iMinEdit->setActualHeight(0.07f);
		iMinEdit->setActualWidth(0.3f);

	UI_TextBox		*iMinText		= new UI_TextBox(screen);
		iMinText->setText("Min. Fish");
		iMinText->setActualHeight(0.07f);
		iMinText->setActualWidth(0.3f);

	UI_EditBox		*iMaxEdit		= new UI_EditBox(screen);
		iMaxEdit->setActualHeight(0.07f);
		iMaxEdit->setActualWidth(0.3f);

	UI_TextBox		*iMaxText		= new UI_TextBox(screen);
		iMaxText->setText("Max. Fish");
		iMaxText->setActualHeight(0.07f);
		iMaxText->setActualWidth(0.3f);

	UI_EditBox		*iDepthEdit		= new UI_EditBox(screen);
		iDepthEdit->setActualHeight(0.07f);
		iDepthEdit->setActualWidth(0.3f);

	UI_TextBox		*iDepthText		= new UI_TextBox(screen);
		iDepthText->setText("Aquarium Depth");
		iDepthText->setActualHeight(0.07f);
		iDepthText->setActualWidth(0.3f);
	
	UI_EditBox		*iWidthEdit		= new UI_EditBox(screen);
		iWidthEdit->setActualHeight(0.07f);
		iWidthEdit->setActualWidth(0.3f);

	UI_TextBox		*iWidthText		= new UI_TextBox(screen);
		iWidthText->setActualHeight(0.07f);
		iWidthText->setActualWidth(0.3f);
		iWidthText->setText("Aquarium Width");

	UI_EditBox		*iHeightEdit	= new UI_EditBox(screen);
		iHeightEdit->setActualHeight(0.07f);
		iHeightEdit->setActualWidth(0.3f);

	UI_TextBox		*iHeightText	= new UI_TextBox(screen);
		iHeightText->setActualHeight(0.07f);
		iHeightText->setActualWidth(0.3f);
		iHeightText->setText("Aquarium Height");

	UI_Slider		*iBreeding		= new UI_Slider(screen); //Breeding Choosiness -----|-----
	UI_TextBox		*iBreedText		= new UI_TextBox(screen);
		iBreedText->setText("Mating Choosiness");
		iBreedText->setActualHeight(0.07f);
		iBreedText->setActualWidth(0.3f);

	UI_TextBox	*iTSAText		= new UI_TextBox(screen);
		iTSAText->setText("Height Map");
		iTSAText->setActualHeight(0.07f);
		iTSAText->setActualWidth(0.3f);

	UI_EditBox	*iTSAEdit		= new UI_EditBox(screen);
		iTSAEdit->setActualHeight(0.07f);
		iTSAEdit->setActualWidth(0.3f);

//	UI_Pushbutton	*iReturn		= new UI_Pushbutton(screen);
//		iReturn->setCaption("Main Menu");

	//add the menuitems to the menu 
//	menu->addMenuItem(iKeymappings,		0.1f, 0.85f);
	menu->addMenuItem(iYInvert,			0.1f, 0.75f);
//	menu->addMenuItem(iGFText,			0.1f, 0.65f);
//		menu->addMenuItem(iGlassFish,		0.41f, 0.67f);
//	menu->addMenuItem(iSwimPOV,			0.1f, 0.55f);

	menu->addMenuItem(iMaxText,			0.1f, 0.65f);
		menu->addMenuItem(iMaxEdit,			0.41f, 0.65f);
	menu->addMenuItem(iMinText,			0.1f, 0.55f);
		menu->addMenuItem(iMinEdit,			0.41f, 0.55f);
	menu->addMenuItem(iDepthText,		0.1f, 0.45f);
		menu->addMenuItem(iDepthEdit,		0.41f, 0.45f);
	menu->addMenuItem(iWidthText,		0.1f, 0.35f);
		menu->addMenuItem(iWidthEdit,		0.41f, 0.35f);
	menu->addMenuItem(iHeightText,		0.1f, 0.25f);
		menu->addMenuItem(iHeightEdit,		0.41f, 0.25f);
	menu->addMenuItem(iBreedText,		0.1f, 0.15f);
		menu->addMenuItem(iBreeding,		0.41f, 0.17f);
//	menu->addMenuItem(iReturn,			0.1f, 0.05f);
	menu->addMenuItem(iTSAText,			0.1f, 0.05f);
		menu->addMenuItem(iTSAEdit,			0.41f, 0.05f);

	//add the menu to the screen
	screen->addMenu(menu, 0.075f, 0.075f);

	//Create Actions
	UI_Action_SwitchScreens		*aToMainMenu	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_InvertYAxis		*aInvertY		= new UI_Action_InvertYAxis(screen);
	UI_Action_MatingChoosiness	*aMate			= new UI_Action_MatingChoosiness(screen);
	UI_Action_TankWidth			*aWidth			= new UI_Action_TankWidth(screen);
	UI_Action_TankHeight		*aHeight		= new UI_Action_TankHeight(screen);
	UI_Action_TankDepth			*aDepth			= new UI_Action_TankDepth(screen);
	UI_Action_FishMin			*aFishMin		= new UI_Action_FishMin(screen);
	UI_Action_FishMax			*aFishMax		= new UI_Action_FishMax(screen);
	UI_Action_SetHeightMap		*aSetMap		= new UI_Action_SetHeightMap(screen);
	
	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);
	//Connect menu items with actions (when applicable)
	aInvertY->setMenuItem(iYInvert);
	aMate->setMenuItem(iBreeding);
//	aToMainMenu->setMenuItem(iReturn);
	aWidth->setMenuItem(iWidthEdit);
	aHeight->setMenuItem(iHeightEdit);
	aDepth->setMenuItem(iDepthEdit);
	aFishMin->setMenuItem(iMinEdit);
	aFishMax->setMenuItem(iMaxEdit);
	aSetMap->setMenuItem(iTSAEdit);

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);
	aConsole->setKey(DIK_GRAVE);


	//Create the keymap
	UI_Keymap *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0, 0);
	keymap->addMenuItem(aInvertY, 0, 0);
	keymap->addMenuItem(aMate, 0, 0);
	keymap->addMenuItem(aWidth, 0,0);
	keymap->addMenuItem(aHeight, 0, 0);
	keymap->addMenuItem(aDepth, 0, 0);
	keymap->addMenuItem(aFishMin, 0, 0);
	keymap->addMenuItem(aFishMax, 0, 0);
	keymap->addMenuItem(aSetMap, 0, 0);

	keymap->addMenuItem(aConsole, 0.0f,0.0f);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the system options screen
UI_Screen* UserInput::createSystemOptionsScreen()
{
	//Create the screen
	UI_Screen	*screen = new UI_Screen;
	//Create the menus
	UI_Menu		*menu	= new UI_Menu(screen);
	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	UI_TextBox	*iSFXText		= new UI_TextBox(screen);
		iSFXText->setText("Effects Volume");
		iSFXText->setActualHeight(0.07f);
		iSFXText->setActualWidth(0.3f);
	UI_Slider	*iSFXSlider		= new UI_Slider(screen);

	UI_TextBox	*iMusicText		= new UI_TextBox(screen);
		iMusicText->setText("Music Volume");
		iMusicText->setActualHeight(0.07f);
		iMusicText->setActualWidth(0.3f);
	UI_Slider	*iMusicSlider	= new UI_Slider(screen);

	UI_TextBox	*iMasterText	= new UI_TextBox(screen);
		iMasterText->setText("Master Volume");
		iMasterText->setActualHeight(0.07f);
		iMasterText->setActualWidth(0.3f);
	UI_Slider	*iMasterSlider	= new UI_Slider(screen);

	UI_TextBox	*iMP3Text		= new UI_TextBox(screen);
		iMP3Text->setText("Song Settings");
		iMP3Text->setActualHeight(0.07f);
		iMP3Text->setActualWidth(0.3f);

	UI_TextBox	*iTitleText		= new UI_TextBox(screen);
		iTitleText->setText("Title Screen");
		iTitleText->setActualHeight(0.07f);
		iTitleText->setActualWidth(0.3f);
	UI_EditBox	*iTitleEdit		= new UI_EditBox(screen);
		iTitleEdit->setActualHeight(0.07f);
		iTitleEdit->setActualWidth(0.3f);

	UI_TextBox	*iAquariumText	= new UI_TextBox(screen);
		iAquariumText->setText("Aquarium");
		iAquariumText->setActualHeight(0.07f);
		iAquariumText->setActualWidth(0.3f);

	UI_EditBox	*iAquariumEdit	= new UI_EditBox(screen);
		iAquariumEdit->setActualHeight(0.07f);
		iAquariumEdit->setActualWidth(0.3f);

/*	UI_TextBox	*iBrainText		= new UI_TextBox(screen);
		iBrainText->setText("Brain Editor");
		iBrainText->setActualHeight(0.07f);
		iBrainText->setActualWidth(0.3f);

	UI_EditBox	*iBrainEdit		= new UI_EditBox(screen);
		iBrainEdit->setActualHeight(0.07f);
		iBrainEdit->setActualWidth(0.3f);
*/

	UI_Pushbutton	*iFullScreen	= new UI_Pushbutton(screen);
		iFullScreen->setCaption("Toggle Fullscreen");
		iFullScreen->setActualHeight(0.07f);
		iFullScreen->setActualWidth(0.3f);

	//Add the menu items to the menus
	menu->addMenuItem(iSFXText,			0.1f, 0.85f);
		menu->addMenuItem(iSFXSlider,		0.41f, 0.87f);

	menu->addMenuItem(iMusicText,		0.1f, 0.75f);
		menu->addMenuItem(iMusicSlider,		0.41f, 0.77f);

	menu->addMenuItem(iMasterText,		0.1f, 0.65f);
		menu->addMenuItem(iMasterSlider,	0.41f, 0.67f);

	menu->addMenuItem(iMP3Text,			0.25f, 0.25f);

	menu->addMenuItem(iTitleText,		0.1f, 0.15f);
		menu->addMenuItem(iTitleEdit,		0.41f, 0.15f);

	menu->addMenuItem(iAquariumText,	0.1f, 0.05f);
		menu->addMenuItem(iAquariumEdit,	0.41f, 0.05f);

//	menu->addMenuItem(iBrainText,		0.1f, 0.25f);
//		menu->addMenuItem(iBrainEdit,		0.41f, 0.25f);

//	menu->addMenuItem(iTSAText,			0.1f, 0.45f);
//		menu->addMenuItem(iTSAEdit,			0.41f, 0.45f);

	menu->addMenuItem(iFullScreen,		0.1f, 0.55f);

	//add the menu to the screen
	screen->addMenu(menu, 0.075f, 0.02f);

	//Create Actions
	UI_Action_SwitchScreens			*aToMainMenu	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_MasterVolume			*aMasterVolume	= new UI_Action_MasterVolume(screen);
	UI_Action_MusicVolume			*aMusicVolume	= new UI_Action_MusicVolume(screen);
	UI_Action_SFXVolume				*aSFXVolume		= new UI_Action_SFXVolume(screen);
	UI_Action_SetTitleScreenSong	*aTSSong		= new UI_Action_SetTitleScreenSong(screen);
	UI_Action_SetAquariumSong		*aIGSong		= new UI_Action_SetAquariumSong(screen);
	UI_Action_FullScreen			*afullScreen	= new UI_Action_FullScreen(screen);

	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);

	//Connect menu items with actions (when applicable)
	aMasterVolume->setMenuItem(iMasterSlider);
	aMusicVolume->setMenuItem(iMusicSlider);
	aSFXVolume->setMenuItem(iSFXSlider);
	aTSSong->setMenuItem(iTitleEdit);
	aIGSong->setMenuItem(iAquariumEdit);
	
	afullScreen->setMenuItem(iFullScreen);

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);

	aConsole->setKey(DIK_GRAVE);

	//Create the keymap
	UI_Keymap *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0, 0);
	keymap->addMenuItem(aMasterVolume, 0, 0);
	keymap->addMenuItem(aMusicVolume, 0, 0);
	keymap->addMenuItem(aSFXVolume, 0, 0);
	keymap->addMenuItem(aTSSong, 0, 0);
	keymap->addMenuItem(aIGSong, 0, 0);
	
	keymap->addMenuItem(afullScreen, 0, 0);

	keymap->addMenuItem(aConsole, 0.0f,0.0f);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Creates the multiplayer screen
UI_Screen* UserInput::createMultiplayerScreen()
{
	//Create the screen
	UI_Screen* screen = new UI_Screen;
	//Create the menus
	UI_Menu* menu = new UI_Menu(screen);

	UI_Menu* mReadout = new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	UI_TextBox		*iJoinText	= new UI_TextBox(screen);
		iJoinText->setActualHeight(0.07f);
		iJoinText->setActualWidth(0.3f);
		iJoinText->setText("IP of Server: ");
	
	UI_EditBox		*iJoinEdit	= new UI_EditBox(screen);
		iJoinEdit->setActualHeight(0.07f);
		iJoinEdit->setActualWidth(0.3f);
		iJoinEdit->setText("IP Goes Here");

	UI_TextBox		*iReadout = new UI_TextBox(screen);
		iReadout->setActualHeight(0.07f);
		iReadout->setActualWidth(0.9f);

	UI_Pushbutton	*iBoot		= new UI_Pushbutton(screen);
		iBoot->setCaption("Boot Clients");

	//Add the menu items to the menus
	menu->addMenuItem(iJoinText,		0.1f, 0.15f);
		menu->addMenuItem(iJoinEdit,		0.45f, 0.15f);

	menu->addMenuItem(iBoot,			0.1f, 0.05f);

	mReadout->addMenuItem(iReadout, 0, 0);
	mReadout->showBackground(false);

	//add the menu to the screen
	screen->addMenu(menu, 0.075f, 0.4f);
	screen->addMenu(mReadout, 0.05f, 0.85f);

	//Create Actions
	UI_Action_SwitchScreens				*aToMainMenu		= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_ConnectToServer			*aServerConnect		= new UI_Action_ConnectToServer(screen);
	UI_Action_NetReadout				*aReadout			= new UI_Action_NetReadout(screen);
	UI_Action_BootClients				*aBoot				= new UI_Action_BootClients(screen);
	
	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);


	//Connect menu items with actions (when applicable)
	aServerConnect->setMenuItem(iJoinEdit);
	aReadout->setMenuItem(iReadout);
	aBoot->setMenuItem(iBoot);

	//Set hotkeys for actions (when applicable)
	aToMainMenu->setKey(DIK_ESCAPE);

	aConsole->setKey(DIK_GRAVE);

	//Create the keymap
	UI_Keymap *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMainMenu, 0, 0);
	keymap->addMenuItem(aServerConnect, 0, 0);
	keymap->addMenuItem(aReadout, 0,0);
	keymap->addMenuItem(aBoot, 0, 0);

	keymap->addMenuItem(aConsole, 0.0f,0.0f);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Client permissions were dropped from the game
UI_Screen* UserInput::createClientPermissionsScreen()
{
	//Create the screen
	UI_Screen	*screen = new UI_Screen;

	//Create the menus
	UI_Menu		*menu	= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();
	

	//Create the menu items
	UI_CheckBox *iEditFish		= new UI_CheckBox(screen);
		iEditFish->setCaption("Edit Fish");
	UI_CheckBox *iEditTerrain	= new UI_CheckBox(screen);
		iEditTerrain->setCaption("Edit Terrain");
	UI_CheckBox *iDropInFish	= new UI_CheckBox(screen);
		iDropInFish->setCaption("Drop In Fish");
	UI_CheckBox *iDragOutFish	= new UI_CheckBox(screen);
		iDragOutFish->setCaption("Drag Out Fish");
	UI_CheckBox *iDropInObject	= new UI_CheckBox(screen);
		iDropInObject->setCaption("Drop In Objects");
	UI_CheckBox *iDragOutObject	= new UI_CheckBox(screen);
		iDragOutObject->setCaption("Drag Out Objects");
	UI_CheckBox *iPossessFish	= new UI_CheckBox(screen);
		iPossessFish->setCaption("Possess Fish");

	//Add the menu items to the menus
		menu->addMenuItem(iEditFish,		0.075f, 0.7f);
		menu->addMenuItem(iEditTerrain,		0.075f, 0.6f);
		menu->addMenuItem(iDropInFish,		0.075f, 0.5f);
		menu->addMenuItem(iDragOutFish,		0.075f, 0.4f);
		menu->addMenuItem(iDropInObject,	0.075f, 0.3f);
		menu->addMenuItem(iDragOutObject,	0.075f, 0.2f);
		menu->addMenuItem(iPossessFish,		0.075f, 0.1f);

	//add the menu to the screen
		screen->addMenu(menu, 0.3f, 0.05f);

	//Create Actions
	UI_Action_SwitchScreens *aToMultiplayer	 = new UI_Action_SwitchScreens(GUI_MULTIPLAYER, screen);

	UI_Action_ActivateConsole		*aConsole		= new UI_Action_ActivateConsole(screen);
	
	//Connect menu items with actions (when applicable)

	//Set hotkeys for actions (when applicable)
	aToMultiplayer->setKey(DIK_ESCAPE);

	aConsole->setKey(DIK_GRAVE);

	//Create the keymap
	UI_Keymap *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMultiplayer, 0, 0);
	keymap->addMenuItem(aConsole, 0.0f,0.0f);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Create the save screen
UI_Screen* UserInput::createSaveScreen()
{
	//Create the screen
	UI_Screen	*screen = new UI_Screen;

	//Create the menus
	UI_Menu		*menu	= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	//Slot 1
	UI_EditBox		*iFile1	= new UI_EditBox(screen);
		iFile1->setActualHeight(0.07f);
		iFile1->setActualWidth(0.3f);
	
	UI_Pushbutton	*iSave1 = new UI_Pushbutton(screen);
		iSave1->setCaption("Save");
		iSave1->setActualHeight(0.07f);
		iSave1->setActualWidth(0.15f);


	UI_Pushbutton	*iDel1	= new UI_Pushbutton(screen);
		iDel1->setCaption("Delete");
		iDel1->setActualHeight(0.07f);
		iDel1->setActualWidth(0.15f);

	//Slot 2
	UI_EditBox		*iFile2	= new UI_EditBox(screen);
		iFile2->setActualHeight(0.07f);
		iFile2->setActualWidth(0.3f);
	
	UI_Pushbutton	*iSave2 = new UI_Pushbutton(screen);
		iSave2->setCaption("Save");
		iSave2->setActualHeight(0.07f);
		iSave2->setActualWidth(0.15f);


	UI_Pushbutton	*iDel2	= new UI_Pushbutton(screen);
		iDel2->setCaption("Delete");
		iDel2->setActualHeight(0.07f);
		iDel2->setActualWidth(0.15f);

	//Slot 3
	UI_EditBox		*iFile3	= new UI_EditBox(screen);
		iFile3->setActualHeight(0.07f);
		iFile3->setActualWidth(0.3f);
	
	UI_Pushbutton	*iSave3 = new UI_Pushbutton(screen);
		iSave3->setCaption("Save");
		iSave3->setActualHeight(0.07f);
		iSave3->setActualWidth(0.15f);


	UI_Pushbutton	*iDel3	= new UI_Pushbutton(screen);
		iDel3->setCaption("Delete");
		iDel3->setActualHeight(0.07f);
		iDel3->setActualWidth(0.15f);

	//Slot 4
	UI_EditBox		*iFile4	= new UI_EditBox(screen);
		iFile4->setActualHeight(0.07f);
		iFile4->setActualWidth(0.3f);
	
	UI_Pushbutton	*iSave4 = new UI_Pushbutton(screen);
		iSave4->setCaption("Save");
		iSave4->setActualHeight(0.07f);
		iSave4->setActualWidth(0.15f);


	UI_Pushbutton	*iDel4	= new UI_Pushbutton(screen);
		iDel4->setCaption("Delete");
		iDel4->setActualHeight(0.07f);
		iDel4->setActualWidth(0.15f);

	//Slot 5
	UI_EditBox		*iFile5	= new UI_EditBox(screen);
		iFile5->setActualHeight(0.07f);
		iFile5->setActualWidth(0.3f);
	
	UI_Pushbutton	*iSave5 = new UI_Pushbutton(screen);
		iSave5->setCaption("Save");
		iSave5->setActualHeight(0.07f);
		iSave5->setActualWidth(0.15f);


	UI_Pushbutton	*iDel5	= new UI_Pushbutton(screen);
		iDel5->setCaption("Delete");
		iDel5->setActualHeight(0.07f);
		iDel5->setActualWidth(0.15f);



	//Add the menu items to the menus
	menu->addMenuItem(iSave1,	0.05f,	0.45f);
	menu->addMenuItem(iDel1,	0.25f,	0.45f);
	menu->addMenuItem(iFile1,	0.45f,	0.45f);

	menu->addMenuItem(iSave2,	0.05f,	0.35f);
	menu->addMenuItem(iDel2,	0.25f,	0.35f);
	menu->addMenuItem(iFile2,	0.45f,	0.35f);

	menu->addMenuItem(iSave3,	0.05f,	0.25f);
	menu->addMenuItem(iDel3,	0.25f,	0.25f);
	menu->addMenuItem(iFile3,	0.45f,	0.25f);

	menu->addMenuItem(iSave4,	0.05f,	0.15f);
	menu->addMenuItem(iDel4,	0.25f,	0.15f);
	menu->addMenuItem(iFile4,	0.45f,	0.15f);

	menu->addMenuItem(iSave5,	0.05f,	0.05f);
	menu->addMenuItem(iDel5,	0.25f,	0.05f);
	menu->addMenuItem(iFile5,	0.45f,	0.05f);

	//add the menu to the screen
	screen->addMenu(menu, 0.1f, 0.25f);

	//Create Actions
	UI_Action_SwitchScreens	*aToMain	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);

	UI_Action_SaveEdit		*aSave1		= new UI_Action_SaveEdit(1, screen);
	UI_Action_SaveFile		*aSaveF1	= new UI_Action_SaveFile(screen);
	UI_Action_DeleteFile	*aDel1		= new UI_Action_DeleteFile(screen);

	UI_Action_SaveEdit		*aSave2		= new UI_Action_SaveEdit(2, screen);
	UI_Action_SaveFile		*aSaveF2	= new UI_Action_SaveFile(screen);
	UI_Action_DeleteFile	*aDel2		= new UI_Action_DeleteFile(screen);

	UI_Action_SaveEdit		*aSave3		= new UI_Action_SaveEdit(3, screen);
	UI_Action_SaveFile		*aSaveF3	= new UI_Action_SaveFile(screen);
	UI_Action_DeleteFile	*aDel3		= new UI_Action_DeleteFile(screen);

	UI_Action_SaveEdit		*aSave4		= new UI_Action_SaveEdit(4, screen);
	UI_Action_SaveFile		*aSaveF4	= new UI_Action_SaveFile(screen);
	UI_Action_DeleteFile	*aDel4		= new UI_Action_DeleteFile(screen);

	UI_Action_SaveEdit		*aSave5		= new UI_Action_SaveEdit(5, screen);
	UI_Action_SaveFile		*aSaveF5	= new UI_Action_SaveFile(screen);
	UI_Action_DeleteFile	*aDel5		= new UI_Action_DeleteFile(screen);

	//Connect menu items with actions (when applicable)
	aSave1->setMenuItem(iFile1);
	aSaveF1->setMenuItem(iSave1);
	aSaveF1->setEditBox(iFile1);
	aDel1->setMenuItem(iDel1);
	aDel1->setEditBox(iFile1);

	aSave2->setMenuItem(iFile2);
	aSaveF2->setMenuItem(iSave2);
	aSaveF2->setEditBox(iFile2);
	aDel2->setMenuItem(iDel2);
	aDel2->setEditBox(iFile2);

	aSave3->setMenuItem(iFile3);
	aSaveF3->setMenuItem(iSave3);
	aSaveF3->setEditBox(iFile3);
	aDel3->setMenuItem(iDel3);
	aDel3->setEditBox(iFile3);

	aSave4->setMenuItem(iFile4);
	aSaveF4->setMenuItem(iSave4);
	aSaveF4->setEditBox(iFile4);
	aDel4->setMenuItem(iDel4);
	aDel4->setEditBox(iFile4);

	aSave5->setMenuItem(iFile5);
	aSaveF5->setMenuItem(iSave5);
	aSaveF5->setEditBox(iFile5);
	aDel5->setMenuItem(iDel5);
	aDel5->setEditBox(iFile5);

	//Set hotkeys for actions (when applicable)
	aToMain->setKey(DIK_ESCAPE);

	//Create the keymap
	UI_Keymap *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMain,	0, 0);
	
	keymap->addMenuItem(aSave1,		0, 0);
	keymap->addMenuItem(aSaveF1,	0, 0);
	keymap->addMenuItem(aDel1,		0, 0);

	keymap->addMenuItem(aSave2,		0, 0);
	keymap->addMenuItem(aSaveF2,	0, 0);
	keymap->addMenuItem(aDel2,		0, 0);

	keymap->addMenuItem(aSave3,		0, 0);
	keymap->addMenuItem(aSaveF3,	0, 0);
	keymap->addMenuItem(aDel3,		0, 0);

	keymap->addMenuItem(aSave4,		0, 0);
	keymap->addMenuItem(aSaveF4,	0, 0);
	keymap->addMenuItem(aDel4,		0, 0);

	keymap->addMenuItem(aSave5,		0, 0);
	keymap->addMenuItem(aSaveF5,	0, 0);
	keymap->addMenuItem(aDel5,		0, 0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	//return the screen
	return screen;
}

//Create load screen
UI_Screen* UserInput::createLoadScreen()
{
	//Create the screen
	UI_Screen*	screen = new UI_Screen;

	//Create the menus
	UI_Menu*	menu	= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	//Slot 1
	UI_Pushbutton	*iLoad1	= new UI_Pushbutton(screen);
		iLoad1->setCaption("Load");
		iLoad1->setActualHeight(0.07f);
		iLoad1->setActualWidth(0.15f);

	UI_TextBox		*iFile1	= new UI_TextBox(screen);
		iFile1->setActualHeight(0.07f);
		iFile1->setActualWidth(0.3f);

	//Slot 2
	UI_Pushbutton	*iLoad2	= new UI_Pushbutton(screen);
		iLoad2->setCaption("Load");
		iLoad2->setActualHeight(0.07f);
		iLoad2->setActualWidth(0.15f);

	UI_TextBox		*iFile2	= new UI_TextBox(screen);
		iFile2->setActualHeight(0.07f);
		iFile2->setActualWidth(0.3f);

	//Slot 3
	UI_Pushbutton	*iLoad3	= new UI_Pushbutton(screen);
		iLoad3->setCaption("Load");
		iLoad3->setActualHeight(0.07f);
		iLoad3->setActualWidth(0.15f);

	UI_TextBox		*iFile3	= new UI_TextBox(screen);
		iFile3->setActualHeight(0.07f);
		iFile3->setActualWidth(0.3f);

	//Slot 4
	UI_Pushbutton	*iLoad4	= new UI_Pushbutton(screen);
		iLoad4->setCaption("Load");
		iLoad4->setActualHeight(0.07f);
		iLoad4->setActualWidth(0.15f);

	UI_TextBox		*iFile4	= new UI_TextBox(screen);
		iFile4->setActualHeight(0.07f);
		iFile4->setActualWidth(0.3f);

	//Slot 5
	UI_Pushbutton	*iLoad5	= new UI_Pushbutton(screen);
		iLoad5->setCaption("Load");
		iLoad5->setActualHeight(0.07f);
		iLoad5->setActualWidth(0.15f);

	UI_TextBox		*iFile5	= new UI_TextBox(screen);
		iFile5->setActualHeight(0.07f);
		iFile5->setActualWidth(0.3f);

	//Add the menu items to the menus
	menu->addMenuItem(iLoad1,	0.04f,	0.45f);
	menu->addMenuItem(iFile1,	0.21f,	0.45f);

	menu->addMenuItem(iLoad2,	0.04f,	0.35f);
	menu->addMenuItem(iFile2,	0.21f,	0.35f);

	menu->addMenuItem(iLoad3,	0.04f,	0.25f);
	menu->addMenuItem(iFile3,	0.21f,	0.25f);

	menu->addMenuItem(iLoad4,	0.04f,	0.15f);
	menu->addMenuItem(iFile4,	0.21f,	0.15f);

	menu->addMenuItem(iLoad5,	0.04f,	0.05f);
	menu->addMenuItem(iFile5,	0.21f,	0.05f);

	//add the menu to the screen
	screen->addMenu(menu, 0.2f, 0.25f);

	//Create Actions
	UI_Action_SwitchScreens	*aToMain	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);

	UI_Action_LoadTextBox	*aFile1		= new UI_Action_LoadTextBox(1, screen);
	UI_Action_Load			*aLoadF1	= new UI_Action_Load(screen);
	
	UI_Action_LoadTextBox	*aFile2		= new UI_Action_LoadTextBox(2, screen);
	UI_Action_Load			*aLoadF2	= new UI_Action_Load(screen);

	UI_Action_LoadTextBox	*aFile3		= new UI_Action_LoadTextBox(3, screen);
	UI_Action_Load			*aLoadF3	= new UI_Action_Load(screen);

	UI_Action_LoadTextBox	*aFile4		= new UI_Action_LoadTextBox(4, screen);
	UI_Action_Load			*aLoadF4	= new UI_Action_Load(screen);

	UI_Action_LoadTextBox	*aFile5		= new UI_Action_LoadTextBox(5, screen);
	UI_Action_Load			*aLoadF5	= new UI_Action_Load(screen);

	//Connect menu items with actions (when applicable)
	aFile1->setMenuItem(iFile1);
	aLoadF1->setMenuItem(iLoad1);
	aLoadF1->setTextBox(iFile1);

	aFile2->setMenuItem(iFile2);
	aLoadF2->setMenuItem(iLoad2);
	aLoadF2->setTextBox(iFile2);

	aFile3->setMenuItem(iFile3);
	aLoadF3->setMenuItem(iLoad3);
	aLoadF3->setTextBox(iFile3);

	aFile4->setMenuItem(iFile4);
	aLoadF4->setMenuItem(iLoad4);
	aLoadF4->setTextBox(iFile4);

	aFile5->setMenuItem(iFile5);
	aLoadF5->setMenuItem(iLoad5);
	aLoadF5->setTextBox(iFile5);


	//Set hotkeys for actions (when applicable)
	aToMain->setKey(DIK_ESCAPE);

	//Create the keymap
	UI_Keymap  *keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMain,	0, 0);
	
	keymap->addMenuItem(aFile1,		0, 0);
	keymap->addMenuItem(aLoadF1,	0, 0);

	keymap->addMenuItem(aFile2,		0, 0);
	keymap->addMenuItem(aLoadF2,	0, 0);

	keymap->addMenuItem(aFile3,		0, 0);
	keymap->addMenuItem(aLoadF3,	0, 0);

	keymap->addMenuItem(aFile4,		0, 0);
	keymap->addMenuItem(aLoadF4,	0, 0);

	keymap->addMenuItem(aFile5,		0, 0);
	keymap->addMenuItem(aLoadF5,	0, 0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	//return the screen
	return screen;
}

//Create title screen
UI_Screen* UserInput::createTitleScreen()
{
	//Create the screen
	UI_Screen	*screen = new UI_Screen;

	//Create the menus
//	UI_Menu		*mTitle = new UI_Menu(screen);
	UI_Menu		*menu	= new UI_Menu(screen);

	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items
	UI_Pushbutton	*iToMain	= new UI_Pushbutton(screen);
		iToMain->setCaption("Main Menu");
		iToMain->setActualHeight(0.07f);
		iToMain->setActualWidth(0.2f);

	UI_Pushbutton	*iToCredits	= new UI_Pushbutton(screen);
		iToCredits->setCaption("Credits");
		iToCredits->setActualHeight(0.07f);
		iToCredits->setActualWidth(0.2f);

	//Add the menu items to the menus
	menu->addMenuItem(iToMain,		0.05f,	0.05f);
	menu->addMenuItem(iToCredits,	0.3f,	0.05f);

	//add the menu to the screen
	screen->addMenu(menu,	0.22f, 0.02f);

	//Create Actions
	UI_Action_SwitchScreens	*aToMain	= new UI_Action_SwitchScreens(GUI_MAINMENU, screen);
	UI_Action_SwitchScreens *aToCredits	= new UI_Action_SwitchScreens(GUI_CREDITS, screen);

	//Connect menu items with actions (when applicable)
	aToMain->setMenuItem(iToMain);
	aToCredits->setMenuItem(iToCredits);

	//Set hotkeys for actions (when applicable)
	aToMain->setKey(DIK_ESCAPE);


	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToMain,	0,	0);
	keymap->addMenuItem(aToCredits,	0,	0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);

	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}

//Create credits screen
UI_Screen* UserInput::createCreditsScreen()
{
	//Create the screen
	UI_Screen	*screen	= new UI_Screen;

	//Create the menus
	UI_Console	*console	= UI_Console::getInstance();

	//Create the menu items

	//Create Actions
	UI_Action_SwitchScreens	*aToTitle	= new UI_Action_SwitchScreens(GUI_TITLESCREEN, screen);

	//Connect menu items with actions (when applicable)

	//Set hotkeys for actions (when applicable)
	aToTitle->setKey(DIK_ESCAPE);

	//Create the keymap
	UI_Keymap	*keymap = new UI_Keymap(screen);

	//Add actions to the keymap
	keymap->addMenuItem(aToTitle, 0, 0);

	//Set the keymap to the screen
	screen->setKeymap(keymap);
	
	//Ensure that the console is added last so that it will be drawn on top of everything else
	screen->addMenu(console, 0.01f, 0.01f);

	return screen;
}