////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Aquarius_Screens.h
**	\brief		screens used for the game
**	\author		John Corpening
**	\date		1/14/2003 3:46:08 PM
**	\par		DESCRIPTION:
**				This file contains functions for creating different screens in the game
**				These screens include:
**				- Title
**				- credits
**				- main menu
**				- offline mode
**				- observer mode
**				- editor/command mode
**				- swim mode
**				- fish editor
**				- multiplayer
**				- gameplay options
**				- system options
**				- keymappings
**				- client permissions
**
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace UserInput
{
	//Screen ID numbers
	enum GUI_ScreenID{	GUI_NONE, 
						GUI_MAINMENU, 
						GUI_OBSERVERMODE,
						GUI_EDITMODE,
						GUI_TESTMODE,
						GUI_OFFLINEMODE,
						GUI_GAMEOPTIONS,
						GUI_SYSTEMOPTIONS,
						GUI_MULTIPLAYER,
						GUI_CLIENTPERMISSIONS,
						GUI_SWIMMODE,
						GUI_TITLESCREEN,
						GUI_BRAINEDITOR,
						GUI_SAVESCREEN,
						GUI_LOADSCREEN,
						GUI_CREDITS
					};

	UI_Screen* createTestScreen();
	UI_Screen* createMainMenu();
	UI_Screen* createObserverScreen();
	UI_Screen* createEditModeScreen();
	UI_Screen* createOfflineModeScreen();
	UI_Screen* createGameOptionsScreen();
	UI_Screen* createSystemOptionsScreen();
	UI_Screen* createMultiplayerScreen();
	UI_Screen* createClientPermissionsScreen();
	UI_Screen* createSwimModeScreen();
	UI_Screen* createSaveScreen();
	UI_Screen* createLoadScreen();
	UI_Screen* createTitleScreen();
	UI_Screen* createCreditsScreen();
}