////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Aquarius_actions.cpp
**	\brief		implementation file for Aquarius actions
**	\author		John Corpening
**	\date		1/18/2003 1:14:17 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////
#include"../GUI_Includes.h"
#include"../../ai_core/AI_System.h"
#include <io.h>
#include <direct.h>

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;
using namespace Debugging;

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Quit::takeAction
/*!	\brief		calls the quit game function of the message handler
**	\author		John Corpening
**	\date		1/18/2003 1:15:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Quit::takeAction()
{
	Debugger::Instance()->output(GEN, L1, "UI calling quitGame() function.");
	MH->QuitGame();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_NextFish::takeAction
/*!	\brief		calls the UI_Handler's getNextFish function
**	\author		John Corpening
**	\date		1/18/2003 1:15:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_NextFish::takeAction()
{
	Debugger::Instance()->output(GEN, L3, "UI Action: Next Fish.");
	UI_Handler::instance()->getNextFish();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SwitchScreens::takeAction
/*!	\brief		calls the UI_Handler's switchScreens function with the internal
**				screen ID
**	\author		John Corpening
**	\date		1/18/2003 1:15:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SwitchScreens::takeAction()
{
	UI_Handler::instance()->switchScreens(screenID);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_PrevScreen::takeAction
/*!	\brief		calls the UI_Handler's previousScreen function
**	\author		John Corpening
**	\date		1/18/2003 1:15:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_PrevScreen::takeAction()
{
	UI_Handler::instance()->previousScreen();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_AddRandomFish::takeAction
/*!	\brief		calls the MessageHandler's AddRandomFish function
**	\author		John Corpening
**	\date		1/18/2003 1:15:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_AddRandomFish::takeAction()
{
	MsgHandler::MessageHandler::Instance()->AddRandomFish();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_InvertYAxis::takeAction
/*!	\brief		calls the Camera's invertYAxis function
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_InvertYAxis::takeAction()
{
	UI_Camera::getInstance()->invertYAxis();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_InvertYAxis::onEnter
/*!	\brief		sets the GUI checkbox with the current state of the y-axis
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_InvertYAxis::onEnter()
{
	if(!checkbox)
		return;
	checkbox->setCheckState(UI_Camera::getInstance()->isYAxisInverted());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateCommandBar::checkMenuItem
/*!	\brief		activates the command bar if a fish is selected
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/14/2003 3:39:03 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_ActivateCommandBar::checkMenuItem()
{
	//HACK: I'm ashamed of misusing the system but time is short
	if(UI_Handler::instance()->getFishTarget())
	{
		commandBar->activate(true);
		return false;
	}
	commandBar->activate(false);
	return false;
}


////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Attack::takeAction
/*!	\brief		sends the fish an ATTACK command
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Mate::takeAction()
{
	MsgHandler::MessageHandler::Instance()->addToMatingList(UI_Handler::instance()->getFishTarget());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_IssueCommand::takeAction
/*!	\brief		Issues commands to the fish
**	\author		John Corpening
**	\date		4/9/2003 10:51:48 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_IssueCommand::takeAction()
{
	MsgHandler::MessageHandler::Instance()->issueCommand(UI_Handler::instance()->getFishTarget(), action);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishForward::takeAction
/*!	\brief		moves the fish to a position 15 units in front of the fish
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishForward::takeAction()
{
	UI_Camera* temp = UI_Camera::getInstance();
	MsgHandler::MessageHandler::Instance()->SwimModeGotoPoint(UI_Handler::instance()->getFishTarget(), 
		temp->getPosition()+(15.0f+temp->getOrbitDist())*temp->getViewDir());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishForward::checkMenuItem
/*!	\brief		used to make the motion continue as long as key is pressed
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_FishForward::checkMenuItem()
{
	return pInput->isPressed(key);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishBackward::takeAction
/*!	\brief		moves the fish to a position 15 units towards the viewer
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishBackward::takeAction()
{
	UI_Camera* temp = UI_Camera::getInstance();
	MsgHandler::MessageHandler::Instance()->SwimModeGotoPoint(UI_Handler::instance()->getFishTarget(), 
		temp->getPosition()+(temp->getOrbitDist()-15.0f)*temp->getViewDir());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishBackward::checkMenuItem
/*!	\brief		used to make the motion continue as long as key is pressed
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_FishBackward::checkMenuItem()
{
	return pInput->isPressed(key);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishLeft::takeAction
/*!	\brief		moves the fish to a position 15 untis left of the fish
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishLeft::takeAction()
{
	UI_Camera* temp = UI_Camera::getInstance();
	math::linear_alg::Vector3<float> fishPos, vec;
	MsgHandler::Key fishKey = UI_Handler::instance()->getFishTarget();
	MsgHandler::MessageHandler::Instance()->GetPosition(fishKey, fishPos);
	vec = math::linear_alg::Cross(temp->getUpVector(), temp->getViewDir());
	MsgHandler::MessageHandler::Instance()->SwimModeGotoPoint(fishKey, fishPos+15.0f*vec.Normalize());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishLeft::checkMenuItem
/*!	\brief		used to make the motion continue as long as key is pressed
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_FishLeft::checkMenuItem()
{
	return pInput->isPressed(key);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishRight::takeAction
/*!	\brief		moves the fish to a position right of the fish
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishRight::takeAction()
{
	UI_Camera* temp = UI_Camera::getInstance();
	math::linear_alg::Vector3<float> fishPos, vec;
	MsgHandler::Key fishKey = UI_Handler::instance()->getFishTarget();
	MsgHandler::MessageHandler::Instance()->GetPosition(fishKey, fishPos);
	vec = math::linear_alg::Cross(temp->getViewDir(), temp->getUpVector());
	MsgHandler::MessageHandler::Instance()->SwimModeGotoPoint(fishKey, fishPos+15.0f*vec.Normalize());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishRight::checkMenuItem
/*!	\brief		used to make the motion continue as long as key is pressed
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_FishRight::checkMenuItem()
{
	return pInput->isPressed(key);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MasterVolume::takeAction
/*!	\brief		sets the master volume
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MasterVolume::takeAction()
{
	AcqariusSound::UIClient::Instance()->SetGlobalVolume((int)(slider->getValue()*255.0f));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MasterVolume::onEnter
/*!	\brief		sets the slider bar to reflect the current volume
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MasterVolume::onEnter()
{
	slider->setIndexValue((float)AcqariusSound::UIClient::Instance()->GetGlobalVolume()/255.0f);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MusicVolume::takeAction
/*!	\brief		sets the volume of the music
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MusicVolume::takeAction()
{
	AcqariusSound::UIClient::Instance()->SetMusicVolume((int)(slider->getValue()*255.0f));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MusicVolume::onEnter
/*!	\brief		sets the slider bar to reflect the current volume
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MusicVolume::onEnter()
{
	slider->setIndexValue((float)AcqariusSound::UIClient::Instance()->GetMusicVolume()/255.0f);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SFXVolume::takeAction
/*!	\brief		sets the volume of the sound effects
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SFXVolume::takeAction()
{
	AcqariusSound::UIClient::Instance()->SetEffectsVolume((int)(slider->getValue()*255.0f));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SFXVolume::onEnter
/*!	\brief		sets the slider bar to reflect the current volume
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SFXVolume::onEnter()
{
	slider->setIndexValue((float)AcqariusSound::UIClient::Instance()->GetEffectsVolume()/255.0f);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetTitleScreenSong::takeAction
/*!	\brief		sets the song played during the title screen
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetTitleScreenSong::takeAction()
{
	UI_Handler::instance()->setTitleScreenSong(edit->getText());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetTitleScreenSong::onEnter
/*!	\brief		sets the editbox text with the current title screen
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetTitleScreenSong::onEnter()
{
	edit->setText(UI_Handler::instance()->getTitleScreenSong());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetAquariumSong::takeAction
/*!	\brief		sets the music played in game
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetAquariumSong::takeAction()
{
	if(AcqariusSound::UIClient::Instance()->LoadMusic(edit->getText(), AcqariusSound::UISTATE_IN_GAME))
	{
		UI_Handler::instance()->setInGameSong(edit->getText());
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetAquariumSong::onEnter
/*!	\brief		sets the editbox text with the name of the current ingame song
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetAquariumSong::onEnter()
{
	edit->setText(UI_Handler::instance()->getInGameSong());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetBrainEditorSong::takeAction
/*!	\brief		sets the song played during brain editing
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetBrainEditorSong::takeAction()
{
	UI_Handler::instance()->setBrainEditorSong(edit->getText());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetBrainEditorSong::onEnter
/*!	\brief		sets the editbox text with the song played during brain editing
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetBrainEditorSong::onEnter()
{
	edit->setText(UI_Handler::instance()->getBrainEditorSong());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MatingChoosiness::takeAction
/*!	\brief		sets how choosy fish are when they mate
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MatingChoosiness::takeAction()
{
	AI_System::AI_BrainManager::getInstance()->setMatingChoosiness(slider->getValue());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_MatingChoosiness::onEnter
/*!	\brief		sets the mating choosiness slider to reflect the current value
**	\author		John Corpening
**	\date		2/22/2003 9:57:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_MatingChoosiness::onEnter()
{
	slider->setIndexValue(AI_System::AI_BrainManager::getInstance()->getMatingChoosiness());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankWidth::takeAction()
/*!	\brief		sets the tank width
**	\author		John Corpening
**	\date		2/27/2003 9:14:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankWidth::takeAction()
{
	float val = (float)atof(edit->getText());
	//If the value does not convert to a float then bail
	if(!val) return;
	//Otherwise set the tank's width
	MsgHandler::MessageHandler::Instance()->setTankWidth(val);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankWidth::onEnter()
/*!	\brief		Retrieves the current tank width and displays it
**	\author		John Corpening
**	\date		2/27/2003 9:14:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankWidth::onEnter()
{
	math::linear_alg::Vector3<float> min, max;
	MsgHandler::MessageHandler::Instance()->getBoundary(min, max);
	char temp[128];
	gcvt(max[0] - min[0], 10, temp);
	edit->setText(temp);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankHeight::takeAction()
/*!	\brief		sets the tank height
**	\author		John Corpening
**	\date		2/27/2003 9:14:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankHeight::takeAction()
{
	float val = (float)atof(edit->getText());
	//If the value does not convert to a float then bail
	if(!val) return;
	//Otherwise set the tank's width
	MsgHandler::MessageHandler::Instance()->setTankHeight(val);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankHeight::onEnter()
/*!	\brief		Retrieves the current tank height and displays it
**	\author		John Corpening
**	\date		2/27/2003 9:14:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankHeight::onEnter()
{
	math::linear_alg::Vector3<float> min, max;
	MsgHandler::MessageHandler::Instance()->getBoundary(min, max);

	char temp[128];
	gcvt(max[1] - min[1], 10, temp);
	edit->setText(temp);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankDepth::takeAction()
/*!	\brief		set's the tank depth
**	\author		John Corpening
**	\date		2/27/2003 9:14:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankDepth::takeAction()
{
	float val = (float)atof(edit->getText());
	//If the value does not convert to a float then bail
	if(!val) return;
	//Otherwise set the tank's Depth
	MsgHandler::MessageHandler::Instance()->setTankDepth(val);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TankDepth::onEnter
/*!	\brief		gets the current depth and sets it in the edit box
**	\author		John Corpening
**	\date		4/17/2003 2:18:59 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TankDepth::onEnter()
{
	math::linear_alg::Vector3<float> min, max;
	MsgHandler::MessageHandler::Instance()->getBoundary(min, max);
	char temp[128];
	gcvt(max[2] - min[2], 10, temp);
	edit->setText(temp);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateStats::takeAction
/*!	\brief		toggles the stats
**	\author		John Corpening
**	\date		3/10/2003 1:17:12 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ActivateStats::takeAction()
{
	if(stats && UI_Handler::instance()->getFishTarget())
	{
		for(unsigned int i = 0; i < otherMenus.size(); i++)
		{
			otherMenus[i]->activate(false);
		}
		stats->activate(!stats->getActiveState());
	}
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateStats::onDeselect
/*!	\brief		Deactivates the stats when a fish is deselected
**	\author		John Corpening
**	\date		4/17/2003 2:19:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ActivateStats::onDeselect()
{
	if(stats && !UI_Handler::instance()->getFishTarget())
	{
		stats->activate(false);
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateStats
/*!	\brief		deactivates stats when the screen is entered
**	\author		John Corpening
**	\date		4/17/2003 2:21:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ActivateStats::onEnter()
{
	if(stats && !UI_Handler::instance()->getFishTarget())
	{
		stats->activate(false);
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckHealth
/*!	\brief		Updates the Health display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckHealth::checkMenuItem()
{
	float health = MsgHandler::MessageHandler::Instance()->FishBody_Health(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Health: ";
	char  buf[128];
	strcat(temp, gcvt(health, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckHealth
/*!	\brief		Updates the Health display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckEnergy::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_EnergyLevel(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Energy: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckMetabolism
/*!	\brief		Updates the Metabolism display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckMetabolism::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Meta(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Metab.: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckSize
/*!	\brief		Updates the Size display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckSize::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Size(
		UI_Handler::instance()->getFishTarget());
	char  temp[100] = "Size: ";
	char  buf[10];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckTeeth
/*!	\brief		Updates the Teeth display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckTeeth::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Teeth_Baleen(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Teeth: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckMating
/*!	\brief		Updates the Mating display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckMating::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_MatingIncline(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Mating: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckArmor
/*!	\brief		Updates the Armor display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckArmor::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Armor(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Armor: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckStomach
/*!	\brief		Updates the Stomach display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	This is a hack based on knowledge of how the menu system works
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckStomach::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Stomach(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Hunger: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	strcat(temp, "%%");
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ConnectToServer::takeAction
/*!	\brief		Connects to a server
**	\exception	none
**	\author		John Corpening
**	\date		3/11/2003 3:58:16 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ConnectToServer::takeAction()
{
	if(!MsgHandler::MessageHandler::Instance()->ConnectToServer(edit->getText()))
	{
		edit->setText("Invalid IP address");
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	 UI_Action_NetReadout::checkMenuItem
/*!	\brief		readout for networking feedback messages
**	\author		John Corpening
**	\date		4/17/2003 2:22:16 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_NetReadout::checkMenuItem()
{
	textbox->setText(MsgHandler::MessageHandler::Instance()->NetworkingState().c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateConsole::takeAction
/*!	\brief		Activates the debug console
**	\author		John Corpening
**	\date		4/17/2003 2:23:05 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ActivateConsole::takeAction()
{
	console->activate(!console->getActiveState());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ActivateConsole
/*!	\brief		sets the screen, and cursor and deactivates the console
**	\author		John Corpening
**	\date		4/17/2003 2:24:26 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ActivateConsole::onEnter()
{
	console->setScreen(screen);
	console->setCursor(screen->getCursor());
	console->activate(false);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_BootClients::takeAction
/*!	\brief		Boots clients if server, disconnects if not
**	\author		John Corpening
**	\date		4/17/2003 2:25:27 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_BootClients::takeAction()
{
	if(MsgHandler::MessageHandler::Instance()->NetworkingIsServer())
	{
		MsgHandler::MessageHandler::Instance()->BootAllClients();
	}
	else
	{
		MsgHandler::MessageHandler::Instance()->AbortClientConnection();
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_BootClients::checkMenuItem
/*!	\brief		Provides feedback for booting clients or quiting server
**	\author		John Corpening
**	\date		4/17/2003 2:27:02 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_BootClients::checkMenuItem()
{
	if(MsgHandler::MessageHandler::Instance()->NetworkingIsServer())
	{
		button->setCaption("Boot Clients");
	}
	else
	{
		button->setCaption("Quit Server");
	}
	if(button && button->isClicked())
	{
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SurpressBrain::checkMenuItem
/*!	\brief		Provides feedback as to whether a brain is free or not
**	\author		John Corpening
**	\date		4/17/2003 2:28:53 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_SurpressBrain::checkMenuItem()
{
	MsgHandler::Key temp = UI_Handler::instance()->getFishTarget();
	if(AI_System::AI_BrainManager::getInstance()->isSuppressed(temp))
	{
		button->setCaption("Free");
	}
	else
	{
		button->setCaption("Suppress");
	}

	if(button && button->isClicked())
		return true;

	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SurpressBrain::setMenuItem
/*!	\brief		sets the button used for suppressing a brain
**	\param		Item, the pushbutton to associate this with
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		4/17/2003 2:31:03 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SurpressBrain::setMenuItem(UI_MenuItem *Item)
{
	button = dynamic_cast<UI_Pushbutton*>(Item);
	if(!button)
	{
		throw(Nexception("Item is not a pushbutton", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SurpressBrain::takeAction
**	\brief		Toggles whether the selected fish thinks or not
**	\author		John Corpening
**	\date		4/9/2003 10:19:47 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SurpressBrain::takeAction()
{
	MsgHandler::Key temp = UI_Handler::instance()->getFishTarget();
	//If it is surpressed then unsuppress it
	if(AI_System::AI_BrainManager::getInstance()->isSuppressed(temp))
	{
		//AI_System::AI_BrainManager::getInstance()->SuppressBrain(0);
		MH->UnsuppressRequest(temp);
	}
	//Otherwise surpress it
	else
	{
		//AI_System::AI_BrainManager::getInstance()->SuppressBrain(temp);
		MH->SuppressRequest(temp);
	}
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SurpressedStatus::checkMenuItem
/*!	\brief		provides feedback for the suppression status of the brain
**	\author		John Corpening
**	\date		4/17/2003 2:33:43 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_SurpressedStatus::checkMenuItem()
{
	if(AI_System::AI_BrainManager::getInstance()->isSuppressed(UI_Handler::instance()->getFishTarget()))
	{
		textbox->setText("Suppressed");
	}
	else
	{
		textbox->setText("Free Thinking");
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckMoveAction::checkMenuItem
/*!	\brief		outputs the action the fish is trying to perform
**	\author		John Corpening
**	\date		4/17/2003 2:34:30 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckMoveAction::checkMenuItem()
{
	std::string	*temp = 0;

	MsgHandler::MessageHandler::Instance()->GetActionString(UI_Handler::instance()->getFishTarget(), NULL, &temp, NULL);
	if(temp)
	{
		std::string output("Action: ");
		output += *temp;
		textbox->setText(output.c_str());
	}

	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckMoveTarget::checkMenuItem
/*!	\brief		feedback for the target of the current action
**	\author		John Corpening
**	\date		4/17/2003 2:35:20 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckMoveTarget::checkMenuItem()
{
	std::string *temp2 = 0;

	MsgHandler::MessageHandler::Instance()->GetTargetString(UI_Handler::instance()->getFishTarget(), NULL, &temp2, NULL);
	
	if(temp2)
	{
		std::string output("Target: ");
		output += *temp2;
		textbox->setText(output.c_str());
	}

	return false;
}
bool UI_Action_CheckMarkerAction::checkMenuItem()
{
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FullScreen::init
/*!	\brief		gets the most likely 600X800 resolution
**	\author		John Corpening
**	\date		4/17/2003 2:36:14 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FullScreen::init()
{
	resolutions = gfx::Renderer::Get().GetResolutions();
	index = 0;
	flag = false;
	for(unsigned int i = 0; i < resolutions.size(); i++)
	{
		if((resolutions[i].height) == 600 && (resolutions[i].width == 800))
		{
			if(resolutions[i].frequency >= resolutions[index].frequency)
			{
				if(resolutions[i].bpp >= resolutions[index].bpp)
				{
					index = i;
				}
			}
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FullScreen::takeAction
/*!	\brief		toggles between fullscreen and windowed mode
**	\author		John Corpening
**	\date		4/17/2003 2:37:11 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FullScreen::takeAction()
{
	flag = !flag;
	gfx::Renderer::Get().ChangeResolution(resolutions[index], flag);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_AbortConnectionAttempt::takeAction
/*!	\brief		Aborts the cliendt connection
**	\author		John Corpening
**	\date		4/17/2003 2:37:54 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_AbortConnectionAttempt::takeAction()
{
	MsgHandler::MessageHandler::Instance()->AbortClientConnection();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ToggleCameraControls::takeAction
/*!	\brief		Toggles between locked and unlocked camera controls
**	\author		John Corpening
**	\date		4/17/2003 2:38:36 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ToggleCameraControls::takeAction()
{
	lock = !lock;
	if(lock)
	{
		UI_Camera::getInstance()->lockControls();
	}
	else
	{
		UI_Camera::getInstance()->unlockControls();
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_ModeToggle::takeAction
/*!	\brief		if fish selected goes to swimmode, otherwise it goes to observermode
**	\author		John Corpening
**	\date		4/17/2003 2:39:12 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_ModeToggle::takeAction()
{
	if(UI_Handler::instance()->getFishTarget())
	{
		UI_Handler::instance()->switchScreens(GUI_SWIMMODE);
	}
	else
	{
		UI_Handler::instance()->switchScreens(GUI_OBSERVERMODE);
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SaveEdit::onEnter
/*!	\brief		gets the name of the nth file in the direcory
**	\author		John Corpening
**	\date		4/17/2003 2:40:20 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SaveEdit::onEnter()
{
	unsigned int temp = 0;
	struct _finddata_t file;
	long hFile;

	_chdir("Aquariums");
	//Get the first file in the group
	if((hFile = _findfirst("*.sav", &file)) == -1)
	{
		//We didn't find a file of this sort so bail
		return;
	}

	//Iterate through the list until the file is found
	while((++temp != num)&&(_findnext(hFile, &file) == 0))
	{
	}
	//put the text in the box
	if(temp == num)
	{
		edit->setText(file.name);
	}
	_findclose(hFile);
	//I don't know if this is necessary but change back
	_chdir("..");
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SaveEdit::takeAction
/*!	\brief		ensures that the proper extension is on the file to be saved
**	\author		John Corpening
**	\date		4/17/2003 2:41:14 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SaveEdit::takeAction()
{
	if(!edit)
		return;

	char temp[255];
	strcpy(temp, edit->getText());
	if((strlen(temp) < 4)|| strcmp(temp + strlen(temp)-4, ".sav"))
	{
		strcat(temp,".sav");
		edit->setText(temp);
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SaveFile::setEditBox
/*!	\brief		sets the edit box for the save file edit actions
**	\param		item, the edit box to set
**	\exception	"Item is not an edit box" Nexception
**	\author		John Corpening
**	\date		4/17/2003 2:42:25 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SaveFile::setEditBox(UI_MenuItem *item)
{
	editBox = dynamic_cast<UI_EditBox*>(item);
	if(!editBox)
	{
		throw(Nexception("Item is not an editbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SaveFile::takeAction
/*!	\brief		Saves the file from the associated edit box
**	\author		John Corpening
**	\date		4/17/2003 2:43:25 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SaveFile::takeAction()
{
	MsgHandler::MessageHandler::Instance()->SaveGame(editBox->getText());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_DeleteFile::setEditBox
/*!	\brief		sets the edit box that the action is associated with
**	\param		item, the edit box to associate with
**	\exception	"Item is not an edit box" Nexception
**	\author		John Corpening
**	\date		4/17/2003 2:43:58 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_DeleteFile::setEditBox(UI_MenuItem *item)
{
	editBox = dynamic_cast<UI_EditBox*>(item);
	if(!editBox)
	{
		throw(Nexception("Item is not an editbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_DeleteFile::takeAction
/*!	\brief		Deletes the file specified in the associated edit box
**	\author		John Corpening
**	\date		4/17/2003 2:45:01 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_DeleteFile::takeAction()
{
	if(!editBox)
		return;
	char temp[255];
	strcpy(temp, "Aquariums\\");
	strcat(temp, editBox->getText());
	if(!remove(temp))
	{
		editBox->clearText();
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_LoadTextBox::setMenuItem
/*!	\brief		sets the associated text box
**	\param		item, the text box that is associated with this
**	\exception	"Item is not a textbox" Nexception
**	\author		John Corpening
**	\date		4/17/2003 2:45:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_LoadTextBox::setMenuItem(UI_MenuItem *item)
{
	textbox = dynamic_cast<UI_TextBox*>(item);
	if(!textbox)
	{
		throw(Nexception("Item is not a textbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_LoadTextBox::onEnter
/*!	\brief		displays the nth file title in the Aquariums folder
**	\author		John Corpening
**	\date		4/17/2003 2:46:58 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_LoadTextBox::onEnter()
{
		unsigned int temp = 0;
	struct _finddata_t file;
	long hFile;

	_chdir("Aquariums");
	//Get the first file in the group
	if((hFile = _findfirst("*.sav", &file)) == -1)
	{
		//We didn't find a file of this sort so bail
		return;
	}

	//Iterate through the list until the file is found
	while((++temp != num)&&(_findnext(hFile, &file) == 0))
	{
	}
	//put the text in the box
	if(temp == num)
	{
		textbox->setText(file.name);
	}
	else
	{
		textbox->setText("empty");
	}
	_findclose(hFile);
	//I don't know if this is necessary but change back
	_chdir("..");
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Load::takeAction
/*!	\brief		Loads the file
**	\author		John Corpening
**	\date		4/17/2003 2:48:03 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Load::takeAction()
{
	MsgHandler::MessageHandler::Instance()->LoadGame(textbox->getText());
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Load::setTextBox
/*!	\brief		associates the load action with a text box
**	\param		item, the textbox to associate with
**	\exception	"Item is not a textbox" Nexception
**	\author		John Corpening
**	\date		4/17/2003 2:48:20 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Load::setTextBox(UI_MenuItem *item)
{
	textbox = dynamic_cast<UI_TextBox*>(item);
	if(!textbox)
	{
		throw(Nexception("Item is not a textbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckAge
/*!	\brief		Updates the Age display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckAge::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Age(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Age: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckAvgEnergy
/*!	\brief		Updates the AvgEnergy display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckAvgEnergy::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_AvgEnergy(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Avg. Energy: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckAvgHealth
/*!	\brief		Updates the AvgHealth display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckAvgHealth::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_AvgHealth(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Avg. Health: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckAttacks
/*!	\brief		Updates the Attacks display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckAttacks::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_HasAttacked(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Attacks: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckDefends
/*!	\brief		Updates the Defends display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckDefends::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_BeenAttacked(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Defends: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckKills
/*!	\brief		Updates the Kills display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckKills::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_KillCount(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Kills: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckPlankton
/*!	\brief		Updates the Plankton display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckPlankton::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_PlanktonEaten(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Plankton: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckDistance
/*!	\brief		Updates the Distance display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckDistance::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_Distance(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Distance: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckAvgSpeed
/*!	\brief		Updates the AvgSpeed display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckAvgSpeed::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_AvgSpeed(
			UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Avg. Speed: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckChum
/*!	\brief		Updates the Chum display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckChum::checkMenuItem()
{
	float num = MsgHandler::MessageHandler::Instance()->FishBody_ChumEaten(
		UI_Handler::instance()->getFishTarget());
	char  temp[128] = "Chum: ";
	char  buf[128];
	strcat(temp, gcvt(num, 3, buf));
	textbox->setText(temp);
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckMoveGoal
/*!	\brief		Updates the Goal display
**	\return		false
**	\exception	none
**	\author		John Corpening
**	\date		3/10/2003 5:59:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckMoveGoal::checkMenuItem()
{
	std::string temp;
	std::string temp2("Goal: ");
	MsgHandler::MessageHandler::Instance()->getFishMoveGoal(UI_Handler::instance()->getFishTarget(), temp);
	temp2 += temp;
	textbox->setText(temp2.c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_NewAquarium::takeAction
/*!	\brief		Creates a new Aquarium
**	\author		John Corpening
**	\date		4/17/2003 2:52:49 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_NewAquarium::takeAction()
{
	MsgHandler::MessageHandler::Instance()->NewAquarium();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishMin::onEnter
/*!	\brief		displays the current minimum in the tank
**	\author		John Corpening
**	\date		4/17/2003 2:53:08 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishMin::onEnter()
{
	char temp[64];
	edit->setText(itoa(MsgHandler::MessageHandler::Instance()->getMinFish(), temp, 10));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishMin::takeAction
/*!	\brief		Sets the minimum fish in the tank
**	\author		John Corpening
**	\date		4/17/2003 2:54:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishMin::takeAction()
{
	MsgHandler::MessageHandler::Instance()->setMinFish(atoi(edit->getText()));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishMax::onEnter
/*!	\brief		displays the current maximum in the tank
**	\author		John Corpening
**	\date		4/17/2003 2:54:28 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishMax::onEnter()
{
	char temp[64];
	edit->setText(itoa(MsgHandler::MessageHandler::Instance()->getMaxFish(), temp, 10));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_FishMax::takeAction
/*!	\brief		Sets the maximum number of fish in the tank
**	\author		John Corpening
**	\date		4/17/2003 2:54:57 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_FishMax::takeAction()
{
	MsgHandler::MessageHandler::Instance()->setMaxFish(atoi(edit->getText()));
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_NumFish::checkMenuItem
/*!	\brief		Displays the number of fish in the tank
**	\author		John Corpening
**	\date		4/17/2003 2:55:21 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_NumFish::checkMenuItem()
{
	char temp[64];
	std::string fish("Number of Fish: ");
	fish += itoa(MsgHandler::MessageHandler::Instance()->numFish(), temp, 10);
	textbox->setText(fish.c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TimeElapsed::onEnter
/*!	\brief		clears out the elapsed time
**	\author		John Corpening
**	\date		4/17/2003 2:55:57 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_TimeElapsed::onEnter()
{
	time = 0.0;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_TimeElapsed::checkMenuItem
/*!	\brief		displays the current time elapsed
**	\author		John Corpening
**	\date		4/17/2003 2:56:21 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_TimeElapsed::checkMenuItem()
{
	time += TimeObject::Instance()->getRealTimeStep();
	double temp = time;
	int hours = (int)(temp/3600.0);
	temp -= hours*3600.0;
	int minutes = (int)(temp / 60.0);
	temp -= minutes*60.0;
	int sec = (int)temp;
	char h[128];
	std::string output("Real Time Elapsed: ");
	output += itoa(hours, h, 10);
	output += (minutes < 10 ? ":0" : ":");
	output += itoa(minutes, h, 10);
	output += (sec < 10 ? ":0" : ":");
	output += itoa(sec, h, 10);
	textbox->setText(output.c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_VTimeElapsed::onEnter
/*!	\brief		clears out the virtual time counter
**	\author		John Corpening
**	\date		4/17/2003 2:57:06 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_VTimeElapsed::onEnter()
{
	time = 0.0;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_VTimeElapsed::checkMenuItem
/*!	\brief		Displays the virtual time elapsed
**	\author		John Corpening
**	\date		4/17/2003 2:57:31 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_VTimeElapsed::checkMenuItem()
{
	time += TimeObject::Instance()->getGameTimeStep();
	double temp = time;
	int hours = (int)(temp/3600.0);
	temp -= hours*3600.0;
	int minutes = (int)(temp / 60.0);
	temp -= minutes*60.0;
	int sec = (int)temp;
	char h[128];
	std::string output("Game Time Elapsed: ");
	output += itoa(hours, h, 10);
	output += (minutes < 10 ? ":0" : ":");
	output += itoa(minutes, h, 10);
	output += (sec < 10 ? ":0" : ":");
	output += itoa(sec, h, 10);
	textbox->setText(output.c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	 UI_Action_DecGameSpeed::takeAction
/*!	\brief		decreased the game speed
**	\author		John Corpening
**	\date		4/17/2003 2:58:36 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_DecGameSpeed::takeAction()
{
	MsgHandler::MessageHandler::Instance()->decreaseGameSpeed();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_IncGameSpeed::takeAction
/*!	\brief		increases the game speed
**	\author		John Corpening
**	\date		4/17/2003 2:58:56 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_IncGameSpeed::takeAction()
{
	MsgHandler::MessageHandler::Instance()->increaseGameSpeed();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_GameSpeedStat::checkMenuItem
/*!	\brief		Displays the current game speed multiplier
**	\author		John Corpening
**	\date		4/17/2003 2:59:32 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_GameSpeedStat::checkMenuItem()
{
	std::string temp("Game Speed: ");
	char temp2[128];
	temp += gcvt(TimeObject::Instance()->getGameSpeed(), 5, temp2);
	textbox->setText(temp.c_str());
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_SetHeightMap::takeAction
/*!	\brief		changes the height map
**	\author		John Corpening
**	\date		4/17/2003 3:00:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_SetHeightMap::takeAction()
{
	std::string temp(edit->getText());
	MsgHandler::MessageHandler::Instance()->setHeightMap(temp);
}