////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Aquarius_actions.h
**	\brief		classes for every action possible in Aquarius
**	\author		John Corpening
**	\date		1/14/2003 7:15:16 PM
**	\par		DESCRIPTION:
**				Actions:
**				-quit
**					-exits the program
**				-NextFish
**					-Cycles to the next fish in the tank
**				-SwitchScreen
**					-Switches between the current screen and the screen that is set
**				-PrevScreen
**					-Switches between the current screen and the previous screen
**				-AddRandomFish
**					-Adds a new randomized fish to the tank
**				-InvertYAxis
**					-Inverts the Y axis in First Person Modes
**				-ActivateCommandBar
**					-Activates Command Bar if a fish is selected in EditMode
**				-Attack
**					-Sends fish an ATTACK command
**				-Eat
**					-Sends fish an EAT command
**				-DropPosMarker
**					-Sends fish a DROP_POS_MARKER command
**				-DropNegMarker
**					-Sends fish a DROP_NEG_MARKER command
**				-FishForward
**					-Gives the fish a point to move to in the direction of the user's view
**				-FishBackward
**					-Gives the fish a point to move to towards the viewer
**				-FishLeft
**					-Gives the fish a point to move to left of the fish's position
**				-FishRight
**					-Gives the fish a point to move to the right of the fish's position
**				-MasterVolume
**					-Sets the master volume
**				-MusicVolume
**					-Sets the music volume
**				-SFXVolume
**					-Sets ths sound effects volume
**				-SetTitleScreenSong
**					-Sets the song played in the title screen
**				-SetAquariumScreenSong
**					-Sets the in game song
**				-SetBrainEditorSong
**					-Sets the song played in the brain editor
**				-MatingChoosiness
**					-Adjusts how choosy fish are when they mate
**				-TankWidth
**					-Adjusts tank width
**				-TankHeight
**					-Adjusts tank height
**				-TankDepth
**					-Adjusts tank depth
**				-ActivateStats
**					-Activates the stats display
**				-CheckHealth
**					-Updates health display
**				-CheckEnergy
**					-Updates energy display
**				-CheckMetabolism
**					-Updates metabolism display
**				-CheckSize
**					-Updates size display
**				-CheckTeeth
**					-Updates teeth display
**				-CheckMating
**					-Updates mating display
**				-CheckArmor
**					-Updates armor display
**				-CheckStomach
**					-Updates stomach display
**				-ConnectToServer
**					-Connects to a server given an IP
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once

namespace UserInput
{

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_Quit
**	\brief		pushbutton action class that quits the game
**	\author		John Corpening
**	\date		1/18/2003 1:10:23 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_Quit : public UI_Action_Pushbutton
{	
public:
	UI_Action_Quit(UI_Screen* Screen) : UI_Action_Pushbutton(Screen) {}
	UI_Action_Quit(){}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_NextFish
**	\brief		pushbutton action class that cycles through the fish
**	\author		John Corpening
**	\date		1/18/2003 1:10:50 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_NextFish : public UI_Action_Pushbutton
{
public:
	UI_Action_NextFish(UI_Screen* Screen) : UI_Action_Pushbutton(Screen) {}
	UI_Action_NextFish(){}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SwitchScreens
**	\brief		pushbutton action class that swithces between screens
**	\author		John Corpening
**	\date		1/18/2003 1:11:29 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SwitchScreens : public UI_Action_Pushbutton
{
	GUI_ScreenID screenID;
public:
	UI_Action_SwitchScreens(GUI_ScreenID ScreenID, UI_Screen* Screen) : UI_Action_Pushbutton(Screen), screenID(ScreenID) {}
	UI_Action_SwitchScreens(GUI_ScreenID ScreenID) : screenID(ScreenID) {}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_PrevScreen
**	\brief		pushbutton action class that switches to the previous screen
**	\author		John Corpening
**	\date		1/18/2003 1:12:02 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_PrevScreen : public UI_Action
{
public:
	UI_Action_PrevScreen(UI_Screen* Screen) : UI_Action(Screen){}
	UI_Action_PrevScreen(){}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_AddRandomFish
**	\brief		pushbutton action class that adds a random fish to the tank
**	\author		John Corpening
**	\date		1/18/2003 1:13:05 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_AddRandomFish : public UI_Action_Pushbutton
{
public:
	UI_Action_AddRandomFish(UI_Screen* Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_AddRandomFish(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_InvertYAxis	
**	\brief		checkbox that inverts the y-axis
**	\author		John Corpening
**	\date		2/5/2003 12:16:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_InvertYAxis : public UI_Action_CheckBox
{
public:
	UI_Action_InvertYAxis(UI_Screen* Screen) : UI_Action_CheckBox(Screen){}
	UI_Action_InvertYAxis(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ActivateCommandBar
**	\brief		action that determines whether or not there is a selected
**				fish and activates the command bar as necessary
**	\author		John Corpening
**	\date		2/15/2003 12:37:18 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ActivateCommandBar : public UI_Action
{
	UI_Menu*	commandBar;
public:
	UI_Action_ActivateCommandBar(UI_Screen* Screen) : UI_Action(Screen), commandBar(0) {}
	UI_Action_ActivateCommandBar() : commandBar(0){}
	bool checkMenuItem();
	void takeAction(){}
	void setCommandBar(UI_Menu* Menu){commandBar = Menu;}
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_Mate
**	\brief		Puts the fish on the mating list
**	\author		John Corpening
**	\date		2/15/2003 12:40:30 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_Mate : public UI_Action_Pushbutton
{
public:
	UI_Action_Mate(UI_Screen* Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_Mate(){}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_IssueCommand
**	\brief		issues commands to the selected fish
**	\author		John Corpening
**	\date		4/09/2003 12:40:30 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_IssueCommand : public UI_Action_Pushbutton
{
	AI_System::Action_ID action;
public:
	UI_Action_IssueCommand(AI_System::Action_ID Action, UI_Screen* Screen) : action(Action), UI_Action_Pushbutton(Screen){}
	UI_Action_IssueCommand(AI_System::Action_ID Action) : action(Action){}
	void takeAction();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishForward
**	\brief		Gives the fish a point to move to in away from the viewer
**	\author		John Corpening
**	\date		2/22/2003 9:49:17 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishForward : public UI_Action
{
public:
	UI_Action_FishForward(UI_Screen* screen) : UI_Action(screen){}
	UI_Action_FishForward(){}
	void takeAction();
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishBackward
**	\brief		Gives the fish a point to move to towards the viewer
**	\author		John Corpening
**	\date		2/22/2003 9:49:21 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishBackward : public UI_Action
{
public:
	UI_Action_FishBackward(UI_Screen* screen) : UI_Action(screen){}
	UI_Action_FishBackward(){}
	void takeAction();
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishLeft
**	\brief		Gives the fish a point to move to left of the fish's position
**	\author		John Corpening
**	\date		2/22/2003 9:49:25 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishLeft : public UI_Action
{
public:
	UI_Action_FishLeft(UI_Screen* screen) : UI_Action(screen){}
	UI_Action_FishLeft(){}
	void takeAction();
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishRight
**	\brief		Gives the fish a point to move to the right of the fish's position
**	\author		John Corpening
**	\date		2/22/2003 9:49:29 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishRight : public UI_Action
{
public:
	UI_Action_FishRight(UI_Screen* screen) : UI_Action(screen){}
	UI_Action_FishRight(){}
	void takeAction();
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_MasterVolume
**	\brief		Sets the master volume
**	\author		John Corpening
**	\date		2/22/2003 9:49:32 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_MasterVolume : public UI_Action_Slider
{
public:
	UI_Action_MasterVolume(UI_Screen* screen) : UI_Action_Slider(screen){}
	UI_Action_MasterVolume(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_MusicVolume
**	\brief		Sets the music volume
**	\author		John Corpening
**	\date		2/22/2003 9:49:36 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_MusicVolume : public UI_Action_Slider
{
public:
	UI_Action_MusicVolume(UI_Screen* screen) : UI_Action_Slider(screen){}
	UI_Action_MusicVolume(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SFXVolume
**	\brief		Sets the sound effects volume
**	\author		John Corpening
**	\date		2/22/2003 9:49:40 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SFXVolume : public UI_Action_Slider
{
public:
	UI_Action_SFXVolume(UI_Screen* screen) : UI_Action_Slider(screen){}
	UI_Action_SFXVolume(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SetTitleScreenSong
**	\brief		Sets the song played in the title screen
**	\author		John Corpening
**	\date		2/22/2003 9:49:44 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SetTitleScreenSong : public UI_Action_Edit
{
public:
	UI_Action_SetTitleScreenSong(UI_Screen* screen) : UI_Action_Edit(screen){}
	UI_Action_SetTitleScreenSong(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SetAquariumSong
**	\brief		sets the in game song
**	\author		John Corpening
**	\date		2/22/2003 9:49:47 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SetAquariumSong	: public UI_Action_Edit
{
public:
	UI_Action_SetAquariumSong(UI_Screen* screen) : UI_Action_Edit(screen){}
	UI_Action_SetAquariumSong(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SetBrainEditorSong
**	\brief		sets the brain editor song
**	\author		John Corpening
**	\date		2/22/2003 9:49:51 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SetBrainEditorSong	: public UI_Action_Edit
{
public:
	UI_Action_SetBrainEditorSong(UI_Screen* screen) : UI_Action_Edit(screen){}
	UI_Action_SetBrainEditorSong(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_MatingChoosiness
**	\brief		Sets how choosy fish are when they mate
**	\author		John Corpening
**	\date		2/22/2003 9:49:55 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_MatingChoosiness : public UI_Action_Slider
{
public:
	UI_Action_MatingChoosiness(UI_Screen* screen) : UI_Action_Slider(screen){}
	UI_Action_MatingChoosiness(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_TankWidth
**	\brief		Sets the tank's width
**	\author		John Corpening
**	\date		2/27/2003 9:03:19 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_TankWidth : public UI_Action_Edit
{
public:
	UI_Action_TankWidth(UI_Screen* Screen) : UI_Action_Edit(Screen){}
	UI_Action_TankWidth(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_TankHeight
**	\brief		Sets the tank's height
**	\author		John Corpening
**	\date		2/27/2003 9:03:19 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_TankHeight : public UI_Action_Edit
{
public:
	UI_Action_TankHeight(UI_Screen* Screen) : UI_Action_Edit(Screen){}
	UI_Action_TankHeight(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_TankDepth
**	\brief		Sets the tank's depth
**	\author		John Corpening
**	\date		2/27/2003 9:03:19 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_TankDepth : public UI_Action_Edit
{
public:
	UI_Action_TankDepth(UI_Screen* Screen) : UI_Action_Edit(Screen){}
	UI_Action_TankDepth(){}
	void takeAction();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ActivateStats
**	\brief		Activates the fish stats display
**	\author		John Corpening
**	\date		2/15/2003 12:37:18 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ActivateStats : public UI_Action_Pushbutton
{
	UI_Menu*	stats;
	std::vector<UI_Menu*>	otherMenus;
public:
	UI_Action_ActivateStats(UI_Screen* Screen) : UI_Action_Pushbutton(Screen), stats(0) {}
	UI_Action_ActivateStats() : stats(0){}
	void takeAction();
	void setStatsDisplay(UI_Menu* Menu){stats = Menu;}
	void addOtherMenus(UI_Menu* Menu){otherMenus.push_back(Menu);}
	void onDeselect();
	void onEnter();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckHealth
**	\brief		Updates Health display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckHealth	: public UI_Action_Stat
{
public:
	UI_Action_CheckHealth(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckHealth(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckEnergy
**	\brief		Updates Energy display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckEnergy	: public UI_Action_Stat
{
public:
	UI_Action_CheckEnergy(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckEnergy(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMetabolism
**	\brief		Updates Metabolism display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMetabolism	: public UI_Action_Stat
{
public:
	UI_Action_CheckMetabolism(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckMetabolism(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckArmor
**	\brief		Updates Armor display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckArmor	: public UI_Action_Stat
{
public:
	UI_Action_CheckArmor(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckArmor(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckSize
**	\brief		Updates Size display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckSize	: public UI_Action_Stat
{
public:
	UI_Action_CheckSize(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckSize(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckTeeth
**	\brief		Updates Teet display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckTeeth	: public UI_Action_Stat
{
public:
	UI_Action_CheckTeeth(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckTeeth(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMating
**	\brief		Updates Mating display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMating	: public UI_Action_Stat
{
public:
	UI_Action_CheckMating(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckMating(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckStomach
**	\brief		Updates Stomach display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckStomach	: public UI_Action_Stat
{
public:
	UI_Action_CheckStomach(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckStomach(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ConnectToServer
**	\brief		Connects to the server
**	\author		John Corpening
**	\date		4/9/2003 10:18:49 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ConnectToServer : public UI_Action_Edit
{
public:
	UI_Action_ConnectToServer(UI_Screen* Screen) : UI_Action_Edit(Screen){}
	UI_Action_ConnectToServer(){}
	void takeAction();
	void onEnter(){}
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_NetReadout
**	\brief		Retrieves data from the networking module for the feedback readout
**	\author		John Corpening
**	\date		4/9/2003 10:19:05 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_NetReadout	: public UI_Action_Stat
{
public:
	UI_Action_NetReadout(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_NetReadout(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ActivateConsole
**	\brief		Activates the Debugging Console
**	\author		John Corpening
**	\date		4/9/2003 10:19:14 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ActivateConsole : public UI_Action
{
	UI_Console *console;
public:
	UI_Action_ActivateConsole(UI_Screen* Screen) : UI_Action(Screen)
	{
		console = UI_Console::getInstance();
	}
	UI_Action_ActivateConsole()
	{
		console = UI_Console::getInstance();
	}
	void takeAction();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_BootClients
**	\brief		Disconnects all clients from the server
**	\author		John Corpening
**	\date		4/9/2003 10:19:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_BootClients : public UI_Action_Pushbutton
{
public:
	UI_Action_BootClients(UI_Screen* Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_BootClients(){}
	void takeAction();
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SurpressBrain
**	\brief		Toggles whether the selected fish thinks or not
**	\author		John Corpening
**	\date		4/9/2003 10:19:47 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SurpressBrain : public UI_Action
{
	UI_Pushbutton*	button;
public:
	UI_Action_SurpressBrain(UI_Screen* Screen) : button(0),  UI_Action(Screen){}
	UI_Action_SurpressBrain() : button(0){}
	void takeAction();
	bool checkMenuItem() throw();
	void setMenuItem(UI_MenuItem *Item);
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SurpressedStatus
**	\brief		Stat class to display whether the brain is suppressed or not
**	\author		John Corpening
**	\date		4/9/2003 10:20:01 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SurpressedStatus : public UI_Action_Stat
{
public:
	UI_Action_SurpressedStatus(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_SurpressedStatus(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMoveAction
**	\brief		Stat class to display what action the fish is trying to do
**	\author		John Corpening
**	\date		4/9/2003 10:22:31 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMoveAction	: public UI_Action_Stat
{
public:
	UI_Action_CheckMoveAction(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckMoveAction(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMoveTarget
**	\brief		Stat class what the fish is trying to do its action to
**	\author		John Corpening
**	\date		4/9/2003 10:23:25 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMoveTarget : public UI_Action_Stat
{
public:
	UI_Action_CheckMoveTarget(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckMoveTarget(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMarkerAction
**	\brief		Stat class to determine what marker action the fish is performing
**	\author		John Corpening
**	\date		4/9/2003 10:23:36 AM
**	\warning	This class currently does nothing
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMarkerAction : public UI_Action_Stat
{
public:
	UI_Action_CheckMarkerAction(UI_Screen* Screen) : UI_Action_Stat(){}
	UI_Action_CheckMarkerAction(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FullScreen
**	\brief		Changes the game to fullscreen
**	\author		John Corpening
**	\date		4/9/2003 10:24:10 AM
**	\warning	Fails is the card does not support 800X600
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FullScreen : public UI_Action_Pushbutton
{
	void init();
	std::vector<gfx::Resolution> resolutions;
	unsigned int index;
	bool flag;
public:
	UI_Action_FullScreen(UI_Screen* screen) : UI_Action_Pushbutton(screen){init();}
	UI_Action_FullScreen(){init();}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_AbortConnectionAttempt
**	\brief		Aborts the client connection
**	\author		John Corpening
**	\date		4/9/2003 10:24:27 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_AbortConnectionAttempt : public UI_Action_Pushbutton
{
public:
	UI_Action_AbortConnectionAttempt(UI_Screen* Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_AbortConnectionAttempt(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ToggleCameraControls
**	\brief		Toggles whether controls are locked or unlocked
**	\author		John Corpening
**	\date		4/9/2003 10:24:36 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ToggleCameraControls : public UI_Action
{
	bool lock;
public:
	UI_Action_ToggleCameraControls(UI_Screen* Screen) : UI_Action(Screen), lock(false){}
	UI_Action_ToggleCameraControls(): lock(false){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_ModeToggle
**	\brief		Toggles between EditorMode and ObserverMode or SwimMode
**				depending on whether a fish is selected or not
**	\author		John Corpening
**	\date		4/9/2003 10:24:46 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_ModeToggle : public UI_Action
{
public:
	UI_Action_ModeToggle(UI_Screen* Screen) : UI_Action(Screen){}
	UI_Action_ModeToggle(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SaveEdit
**	\brief		Edit box functionality for saving the game
**	\author		John Corpening
**	\date		4/9/2003 10:24:53 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SaveEdit : public UI_Action_Edit
{
	unsigned int num;	//!< The slot number of the save
public:
	UI_Action_SaveEdit(unsigned int Num, UI_Screen* Screen) : num(Num), UI_Action_Edit(Screen){}
	UI_Action_SaveEdit(unsigned int Num) : num(Num){}
	void takeAction();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SaveFile
**	\brief		Saves the current game in the file specified by the associated
**				Editbox
**	\author		John Corpening
**	\date		4/9/2003 10:25:01 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SaveFile : public UI_Action_Pushbutton
{
	UI_EditBox	*editBox;
public:
	UI_Action_SaveFile(UI_Screen* Screen) : editBox(0), UI_Action_Pushbutton(Screen){}
	UI_Action_SaveFile() : editBox(0){}
	void takeAction();
	void setEditBox(UI_MenuItem* item);
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_DeleteFile
**	\brief		Deletes the file specified by the associated edit box
**	\author		John Corpening
**	\date		4/9/2003 10:25:08 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_DeleteFile : public UI_Action_Pushbutton
{
	UI_EditBox *editBox;
public:
	UI_Action_DeleteFile(UI_Screen *Screen) : editBox(0), UI_Action_Pushbutton(Screen){}
	UI_Action_DeleteFile() : editBox(0){}
	void takeAction();
	void setEditBox(UI_MenuItem* item);
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_LoadTextBox
**	\brief		Displays a file in the saved aquariums directory
**	\author		John Corpening
**	\date		4/9/2003 10:25:16 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_LoadTextBox : public UI_Action
{
	UI_TextBox	*textbox;
	unsigned int num;
public:
	UI_Action_LoadTextBox(unsigned int Num, UI_Screen* Screen) : num(Num), UI_Action(Screen){}
	UI_Action_LoadTextBox(unsigned int Num) : num(Num) {}
	void takeAction(){};
	void setMenuItem(UI_MenuItem* item);
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_Load
**	\brief		Loads the file specified in the associated load textbox
**	\author		John Corpening
**	\date		4/9/2003 10:25:22 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_Load : public UI_Action_Pushbutton
{
	UI_TextBox	*textbox;
public:
	UI_Action_Load(UI_Screen *Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_Load(){}
	void takeAction();
	void setTextBox(UI_MenuItem* item);
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckAge
**	\brief		Updates Age display
**	\author		John Corpening
**	\date		3/10/2003 5:56:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckAge	: public UI_Action_Stat
{
public:
	UI_Action_CheckAge(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckAge(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckAvgEnergy
**	\brief		Updates average energy display
**	\author		John Corpening
**	\date		4/9/2003 10:25:30 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckAvgEnergy	: public UI_Action_Stat
{
public:
	UI_Action_CheckAvgEnergy(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckAvgEnergy(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckAvgHealth
**	\brief		Updates the average health display
**	\author		John Corpening
**	\date		4/9/2003 10:25:37 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckAvgHealth	: public UI_Action_Stat
{
public:
	UI_Action_CheckAvgHealth(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckAvgHealth(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckAttacks
**	\brief		Updates the attack count display
**	\author		John Corpening
**	\date		4/9/2003 10:25:43 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckAttacks	: public UI_Action_Stat
{
public:
	UI_Action_CheckAttacks(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckAttacks(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckKills
**	\brief		Updates the kill count display
**	\author		John Corpening
**	\date		4/9/2003 10:25:54 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckKills	: public UI_Action_Stat
{
public:
	UI_Action_CheckKills(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckKills(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckDefends
**	\brief		Updates the Defend count display
**	\author		John Corpening
**	\date		4/9/2003 10:26:01 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckDefends	: public UI_Action_Stat
{
public:
	UI_Action_CheckDefends(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckDefends(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckPlankton
**	\brief		Updates the plankton display
**	\author		John Corpening
**	\date		4/9/2003 10:26:10 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckPlankton	: public UI_Action_Stat
{
public:
	UI_Action_CheckPlankton(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckPlankton(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckDistance
**	\brief		updates the distance covered display
**	\author		John Corpening
**	\date		4/9/2003 10:26:17 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckDistance	: public UI_Action_Stat
{
public:
	UI_Action_CheckDistance(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckDistance(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckAvgSpeed
**	\brief		updates the average speed display
**	\author		John Corpening
**	\date		4/9/2003 10:26:23 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckAvgSpeed	: public UI_Action_Stat
{
public:
	UI_Action_CheckAvgSpeed(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckAvgSpeed(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckChum
**	\brief		Updates the check chum display
**	\author		John Corpening
**	\date		4/9/2003 10:26:30 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckChum	: public UI_Action_Stat
{
public:
	UI_Action_CheckChum(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckChum(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_CheckMoveGoal
**	\brief		Displays the movement goal display
**	\author		John Corpening
**	\date		4/9/2003 10:26:38 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_CheckMoveGoal	: public UI_Action_Stat
{
public:
	UI_Action_CheckMoveGoal(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_CheckMoveGoal(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_NewAquarium
**	\brief		Nukes the aquarium
**	\author		John Corpening
**	\date		4/9/2003 10:26:44 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_NewAquarium	: public UI_Action_Pushbutton
{
public:
	UI_Action_NewAquarium(UI_Screen *Screen) : UI_Action_Pushbutton(Screen){}
	UI_Action_NewAquarium(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishMin
**	\brief		Sets the minimum number of fish held in the tank
**	\author		John Corpening
**	\date		4/9/2003 10:26:50 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishMin : public UI_Action_Edit
{
public:
	UI_Action_FishMin(UI_Screen *Screen) : UI_Action_Edit(Screen){}
	UI_Action_FishMin(){}
	void takeAction();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_FishMax
**	\brief		Updates the maximum number of fish allowed in the tank
**	\author		John Corpening
**	\date		4/9/2003 10:26:59 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_FishMax : public UI_Action_Edit
{
public:
	UI_Action_FishMax(UI_Screen *Screen) : UI_Action_Edit(Screen){}
	UI_Action_FishMax(){}
	void takeAction();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_NumFish
**	\brief		Updates the number of fish display
**	\author		John Corpening
**	\date		4/9/2003 10:27:05 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_NumFish : public UI_Action_Stat
{
public:
	UI_Action_NumFish(UI_Screen *Screen) : UI_Action_Stat(Screen){}
	UI_Action_NumFish(){}
	bool checkMenuItem();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_TimeElapsed
**	\brief		Updates the actual time elapsed in the tank in offline mode
**	\author		John Corpening
**	\date		4/9/2003 10:27:10 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_TimeElapsed : public UI_Action_Stat
{
	double time;
public:
	UI_Action_TimeElapsed(UI_Screen *Screen) : UI_Action_Stat(Screen){}
	UI_Action_TimeElapsed(){}
	bool checkMenuItem();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_VTimeElapsed
**	\brief		Updates the virtual time elapsed in the tank in offline mode
**	\author		John Corpening
**	\date		4/9/2003 10:27:16 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_VTimeElapsed : public UI_Action_Stat
{
	double time;
public:
	UI_Action_VTimeElapsed(UI_Screen *Screen) : UI_Action_Stat(Screen){}
	UI_Action_VTimeElapsed(){}
	bool checkMenuItem();
	void onEnter();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_IncGameSpeed
**	\brief		Increments the game speed
**	\author		John Corpening
**	\date		4/9/2003 10:27:22 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_IncGameSpeed : public UI_Action
{
public:
	UI_Action_IncGameSpeed(UI_Screen* Screen) : UI_Action(Screen){}
	UI_Action_IncGameSpeed(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_DecGameSpeed
**	\brief		Decrements the game speed
**	\author		John Corpening
**	\date		4/9/2003 10:27:27 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_DecGameSpeed : public UI_Action
{
public:
	UI_Action_DecGameSpeed(UI_Screen* Screen) : UI_Action(Screen){}
	UI_Action_DecGameSpeed(){}
	void takeAction();
};
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_GameSpeedStat
**	\brief		Displays the current game speed
**	\author		John Corpening
**	\date		4/9/2003 10:27:32 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_GameSpeedStat : public UI_Action_Stat
{
public:
	UI_Action_GameSpeedStat(UI_Screen* Screen) : UI_Action_Stat(Screen){}
	UI_Action_GameSpeedStat(){}
	bool checkMenuItem();
};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action_SetHeightMap
**	\brief		Allows the user to change the height map
**	\author		John Corpening
**	\date		4/17/2003 12:25:49 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action_SetHeightMap : public UI_Action_Edit
{
public:
	UI_Action_SetHeightMap(UI_Screen* Screen) : UI_Action_Edit(Screen){}
	UI_Action_SetHeightMap(){}
	void takeAction();
	void onEnter(){}
};

}