////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Action_Subclasses.cpp
**	\brief		Implementation of checkMenuItem and setMenuItem functions for
**				Action Subclasses.
**	\author		John Corpening
**	\date		1/18/2003 1:34:55 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#include "GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;

//-----------Pushbutton Action----------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Pushbutton::checkMenuItem
/*!	\brief		determines whether or not the button has been clicked
**	\return		true if clicked, false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		1/18/2003 1:29:28 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_Pushbutton::checkMenuItem()
{
	if(button && button->isClicked())
		return true;
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Pushbutton::setMenuItem
/*!	\brief		Sets the given menuitem as a pushbutton
**	\param		item, a UI_MenuItem to set as a pushbutton
**	\return		none
**	\exception	"Item is not a pushbutton" Nexcpetion
**	\author		John Corpening
**	\date		1/18/2003 1:30:32 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Pushbutton::setMenuItem(UI_MenuItem *item)
{
	button = dynamic_cast<UI_Pushbutton*>(item);
	if(!button)
	{
		throw(Nexception("Item is not a pushbutton", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

//-----------Checkbox Action--------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckBox::checkMenuItem
/*!	\brief		determines if the checkbox has been updated or not
**	\return		true if updated, false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 10:47:25 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_CheckBox::checkMenuItem()
{
	if(checkbox && checkbox->isUpdated())
	{
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_CheckBox::setMenuItem
/*!	\brief		casts the paramter to a checkbox and sets it internally
**	\param		item, a pointer to a UI_MenuItem
**	\return		none
**	\exception	"Item is not a checkbox" Nexception
**	\author		John Corpening
**	\date		2/22/2003 10:47:29 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_CheckBox::setMenuItem(UI_MenuItem *item)
{
	checkbox = dynamic_cast<UI_CheckBox*>(item);
	if(!checkbox)
	{
		throw(Nexception("Item is not a checkbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

//-------------------Slider Action-----------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Slider::checkMenuItem
/*!	\brief		determines whether or not the slider state has been updated
**	\return		true if updated false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 10:47:57 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_Slider::checkMenuItem()
{
	if(slider && slider->isUpdated())
	{
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Slider::setMenuItem
/*!	\brief		casts the parameter as a UI_Slider and sets it internally
**	\param		item, pointer to the slider
**	\return		none
**	\exception	"Item is not a Slider" Nexception
**	\author		John Corpening
**	\date		2/22/2003 10:48:02 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Slider::setMenuItem(UI_MenuItem *item)
{
	slider = dynamic_cast<UI_Slider*>(item);
	if(!slider)
	{
		throw(Nexception("Item is not a slider", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

//--------------------Edit Action-------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Edit::checkMenuItem
/*!	\brief		determines whether or not the edit box was updated
**	\return		true if updated, false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 10:48:14 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action_Edit::checkMenuItem()
{
	if(edit && edit->isUpdated())
	{
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Edit::setMenuItem
/*!	\brief		casts the parameter to an editbox and sets it internally
**	\param		item, pointer to the item to cast as an edit box
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 10:48:20 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Edit::setMenuItem(UI_MenuItem *item)
{
	edit = dynamic_cast<UI_EditBox*>(item);
	if(!edit)
	{
		throw(Nexception("Item is not an editbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

//--------------------Stat Action-------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action_Stat::setMenuItem
/*!	\brief		casts the parameter to a textbox and sets it internally
**	\param		item, pointer to the item to cast as a text box
**	\return		none
**	\exception	"Item is not a textbox" Nexception
**	\author		John Corpening
**	\date		3/10/2003 2:34:01 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action_Stat::setMenuItem(UI_MenuItem *item)
{
	textbox = dynamic_cast<UI_TextBox*>(item);
	if(!textbox)
	{
			throw(Nexception("Item is not a textbox", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}