////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Camera.h
**	\brief		Class for GUI control of the camera
**	\author		John Corpening
**	\date		1/18/2003 1:35:43 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace UserInput
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Camera
**	\brief		Class for GUI control of the camera
**	\author		John Corpening
**	\date		1/18/2003 1:36:51 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Camera
{
	PlayerInput	*input;						//!< Used to for camera controls
	math::linear_alg::Vector3<float> view,  //!< The direction of the camera
									 pos,	//!< The position of the camera
									 up;	//!< The up vector
	InputConstant	forward,				//!< The key used for forward movement
					back,					//!< The key used for backward movement
					left,					//!< The key used for left strafing
					right;					//!< The key used for right strafing
	bool	update;							//!< Flag used to update the camera parameters
	bool	yAxisInvert;					//!< Flag used to indicate yAxisInversion
	bool	controlLock;					//!< Locks the camera controls
	bool	orbitFlag;						//!< Flag to indicate that the camera is orbiting
	float	orbitDist;						//!< Distance from orbit target
	math::linear_alg::Vector3<float> orbitPos;//!< Orbit target

	static UI_Camera* _instance;	//!< Singleton instance

protected:
	//Constructor
	UI_Camera();
public:
	//Singleton Accessor
	static UI_Camera* getInstance();
	//Moves the camera forward
	void moveForward();
	//Moves the camera backward
	void moveBackward();
	//Strafes the camera to the left
	void strafeLeft();
	//Strafes the camera to the right
	void strafeRight();
	//Rotates the view vector about the local X axis
	void pitch(float a);
	//Rotates the view vector about the local y axis
	void yaw(float a);
	//Moves the camera into a chase position
	void chase(math::linear_alg::Vector3<float>& target);

	//BEGIN Added by Nathan Frost <April 12, 2003>
	//Camera keeps selected Fish in center of user's view but does not alter its orientation at all
	void chaseWithStaticViewVector(math::linear_alg::Vector3<float>& target);
	//END Added by Nathan Frost <April 12, 2003>

	//Camera logic processing
	void logic();
	//Inverts the pitch rotation
	void invertYAxis(){yAxisInvert = !yAxisInvert;}
	//Accessor to determine whether or not pitch rotation has been inverted
	bool isYAxisInverted(){return yAxisInvert;}
	//locks the camera controls
	void lockControls() {controlLock = true;}
	//unlocks the camera controls
	void unlockControls() {controlLock = false;}
	//Makes the camera look at a point
	void orbit(math::linear_alg::Vector3<float>& target);

	//Accessor Functions
	math::linear_alg::Vector3<float> getViewDir(){return view;}
	math::linear_alg::Vector3<float> getUpVector(){return up;}
	math::linear_alg::Vector3<float> getPosition(){return pos;}
	void setOrbitDist(float dist){orbitDist = dist;}
	float getOrbitDist(){return orbitDist;}
};
}