////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Console.cpp
**	\brief		Implementation for GUI_Console.h
**	\author		John Corpening
**	\date		11/23/2002 4:29:24 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#include "GUI_Includes.h"
using namespace UserInput;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

//Initialization of static member variables
UI_Console*				UI_Console::_instance = 0;
std::deque<string>		UI_Console::output;
unsigned int			UI_Console::maxSize = 100;
unsigned int			UI_Console::maxOutput = 30;
unsigned int			UI_Console::refCount = 0;
Debugging::Debugger		*UI_Console::debug;

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Console::getInstance
/*!	\brief		Singleton access point
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 2:52:29 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Console* UI_Console::getInstance()
{
	refCount++;
	if(!_instance)
		 _instance = new UI_Console;
	return _instance;
}

void UI_Console::Destroy()
{
	if(_instance)
	{
		delete _instance;
		_instance = 0;
	}
	
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Console::UI_Console
/*!	\brief		Constructor
**	\param		Screen, pointer to the screen required for all children of Menus
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/23/2002 4:24:16 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Console::UI_Console()
{
	height = 0.5;
	width = 0.5;
	edit = new UI_EditBox;
	edit->setActualWidth(0.5f);
	edit->setActualHeight(0.05f);
	addMenuItem(edit, 0.0,0.0);
	debug = Debugging::Debugger::Instance();
	debug->setScreenOutputCallback(getOutputCopy);
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Console::getOutputCopy
/*!	\brief		callback function for the debugger to set output text
**	\param		check, output text sent from the debugger
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/23/2002 4:26:09 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Console::getOutputCopy(const char *check)
{
	if(check)
	{
		//Parse the output for newline characters
		char	*temp = new char[strlen(check) + 1];
		strcpy(temp,check);
		char	*tok;
		char	delimiters[] = "\n";
		tok = strtok(temp, delimiters);
		while(tok)
		{
			//Keep the deque from getting too big
			while(output.size() >= maxSize)
			{
				//Lose the last element
				output.pop_back();
			}

			//Add the line of text
			output.push_front(tok);
			tok = strtok(NULL, delimiters);
		}
		delete[] temp;
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Console::draw
/*!	\brief		draws the Console
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/23/2002 4:27:06 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Console::drawMenu()
{
	int size = output.size();
	unsigned int j = 1;
	//Draw the background
//	style->drawMenuBackground(x, y, width, height);

	//Draw the debug output
	for(std::deque<string>::iterator i = output.begin(); i < output.end(); i++)
	{
		//Draw the line of text
		style->drawText(x, y + 0.033f * ++j, (*i).c_str());

		//Limit the amount of output text
		if(j > maxOutput)
			break;
	}

	//Render it
	style->draw();	
}


////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Console::specificLogic
/*!	\brief		Checks edit box, updates Debugger options as necessary
**	\return		true if edit box has been updated otherwise false
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 2:42:07 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Console::specificLogic() throw()
{
	if(edit->isUpdated())
	{
		debug->setOptions(edit->getText());
		edit->clearText();
		return true;
	}
	return false;
}