////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Console.h
**	\brief		Displays Debugger output to the screen
**	\author		John Corpening
**	\date		11/23/2002 4:21:55 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace UserInput
{

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Console
**	\brief		displays debug output to the screen
**	\author		John Corpening
**	\date		11/23/2002 4:22:33 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Console : public UI_Menu
{
	static std::deque<std::string>	output;		//!< lines of text to output
	static unsigned int				maxSize;	//!< The Maximum lines of text to Keep
	static unsigned int				maxOutput;	//!< The Maximum lines of text to output
	static Debugging::Debugger		*debug;		//!< Handle to the debugger
	UI_EditBox						*edit;		//!< used to set debugger options
	static UI_Console				*_instance;	//!< Singleton instance
	static unsigned int				refCount;	//!< 

protected:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Console::UI_Console
	/*!	\brief		Constructor
	**	\param		Screen, pointer to the screen required for all children of Menus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/23/2002 4:24:16 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Console();
	~UI_Console(){};

public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Console::getInstance
	/*!	\brief		Singleton access point
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 2:52:29 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	static UI_Console* getInstance();

	static void Destroy();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Console::specificLogic
	/*!	\brief		Checks edit box, updates Debugger options as necessary
	**	\return		true if edit box has been updated otherwise false
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 2:42:07 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool specificLogic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Console::getOutputCopy
	/*!	\brief		callback function for the debugger to set output text
	**	\param		check, output text sent from the debugger
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/23/2002 4:26:09 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	static void getOutputCopy(const char *check);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Console::draw
	/*!	\brief		draws the Console
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/23/2002 4:27:06 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void drawMenu();
};
}