////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Cursor.cpp
**	\brief		implementation file for GUI_Cursor.h
**	\author		John Corpening
**	\date		10/30/2002 7:52:07 PM
*/
////////////////////////////////////////////////////////////////////////////////

#include "GUI_Includes.h"
using namespace UserInput;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Cursor::UI_Cursor
/*!	\brief		gains access to the low level mouse interface
**	\return		none
**	\exception	Null Mouse Pointer
**	\author		John Corpening
**	\date		10/30/2002 7:48:31 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Cursor::UI_Cursor() : x(0.5), y(0.5)//, tex("cursor.tga"), sprite(tex, 0.1f, 0.1f)
{
	style = new UI_Style;
	//sprite.SetOrigin(0.0f, 0.0f);
	mouse = Mouse::instance();
	timing = TimeObject::Instance();
	if(!mouse)
	{
		throw(Nexception("Null Mouse Pointer", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

UI_Cursor::~UI_Cursor()
{
	if(style)
		delete style;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Cursor::logic
/*!	\brief		queries input modules and updates the cursor
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		10/30/2002 7:50:47 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Cursor::logic()
{
	//LEFT MOUSE BUTTON
	//check for left button pressed
	if(mouse->isLeftButtonDown())
	{
		if(lPressed)
		{
			lPressTime += timing->getRealTimeStep();
		}
		else
		{
			lPressed = true;
			lPressTime = 0.0;
		}
	}
	else
	{
		lPressed = false;
		lPressTime = 0.0;
	}

	//RIGHT MOUSE BUTTON
	//check for right button pressed
	if(mouse->isRightButtonDown())
	{
		if(rPressed)
		{
			rPressTime += timing->getRealTimeStep();
		}
		else
		{
			rPressed = true;
			rPressTime = 0.0;
		}
	}
	else
	{
		rPressed = false;
		rPressTime = 0.0;
	}

	//MOUSE WHEEL
	//Check wheel pressed
	if(mouse->isWheelDown())
	{
		if(wPressed)
		{
			wPressTime += timing->getRealTimeStep();
		}
		else
		{
			wPressed = true;
		}
	}
	else
	{
		wPressed = false;
		wPressTime = 0.0;
	}

	//Update the cursor position
	positionTime += timing->getRealTimeStep();

	//Update position deltas
	dx = dy = 0.0f;
	if(mouse->getDeltaX())
	{
		dx = x;
		x += static_cast<float>(mouse->getDeltaX())/static_cast<float>(g_ScreenWidth);
		x = (x > 1.0f ? 1.0f : x);
		x = (x < 0.0f ? 0.0f : x);
		dx = x - dx;
		positionTime = 0.0;
	}

	if(mouse->getDeltaY())
	{
		dy = y;
		y -= static_cast<float>(mouse->getDeltaY())/static_cast<float>(g_ScreenHeight);
		y = (y > 1.0f ? 1.0f : y);
		y = (y < 0.0f ? 0.0f : y);
		dy = y - dy;
		positionTime = 0.0;
	}

}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Cursor::draw
/*!	\brief		draws the mouse cursor
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		10/30/2002 7:50:14 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Cursor::draw()
{
	//Render();
	style->drawCursor(x, y, .03f, .03f);
	style->draw();
}