////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Cursor.h
**	\brief		GUI interface for the mouse
**	\author		John Corpening
**	\date		10/30/2002 7:46:13 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Cursor
**	\brief		GUI interface for the mouse
**	\author		John Corpening
**	\date		10/30/2002 7:47:37 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Cursor 
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Cursor::UI_Cursor
	/*!	\brief		gains access to the low level mouse interface
	**	\return		none
	**	\exception	Null Mouse Pointer
	**	\author		John Corpening
	**	\date		10/30/2002 7:48:31 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Cursor();

	~UI_Cursor();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Cursor::draw
	/*!	\brief		draws the mouse cursor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 7:50:14 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void draw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Cursor::logic
	/*!	\brief		queries input modules and updates the cursor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 7:50:47 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void logic() throw();

	//inline accessors
	//These are self explanatory
	float X() {return x;}
	float Y() {return y+0.03f;}	//Hack to make the y position the upper left hand corner
	float deltaX(){return dx;}
	float deltaY(){return dy;}
	bool isLTriggered(){return mouse->isLeftButtonTriggered();}
	bool isRTriggered(){return mouse->isRightButtonTriggered();}
	bool isWTriggered(){return mouse->isWheelTriggered();}
	bool isLPressed(){return lPressed;}
	bool isRPressed(){return rPressed;}
	bool isWPressed() {return wPressed;}
	double getLPressTime(){return lPressTime;}
	double getRPressTime(){return rPressTime;}
	double getWPressTime(){return wPressTime;}
	double getPositionTime(){return positionTime;}
	double getDeltaWheel(){return mouse->getDeltaWheel();}

//------------Private---------------------------------------------------------
private:

	//Mouse data
	float			x;				//!< x coordinate in screen coordinates
	float			dx;
	float			y;				//!< y coordinate in screen coordinates
	float			dy;		
	double			lPressTime;		//!< seconds that the Left button has been pressed
	double			rPressTime;		//!< seconds that the right button has been pressed
	double			wPressTime;		//!< seconds that the wheel has been pressed
	double			positionTime;	//!< seconds since the cursor has changed position
	bool			lPressed;		//!< Left button pressed flag
	bool			rPressed;		//!< Rightt button pressed flag
	bool			wPressed;		//!< Left button pressed flag
	Mouse			*mouse;			//!< handle to flag
	TimeObject		*timing;		//!< handle to the timeobject
	UI_Style		*style;			//!< How to draw the cursor
};

}