////////////////////////////////////////////////////////////////////////////////
/*!	\file		UI_EditBox.cpp
**	\brief		Implementation of UI_EditBox.h
**	\author		John Corpening
**	\date		11/25/2002 3:21:57 PM
*/
////////////////////////////////////////////////////////////////////////////////

#include "GUI_Includes.h"
using namespace UserInput;

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_EditBox::editInit
/*!	\brief		common editbox initialization
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 3:19:39 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_EditBox::editInit()
{
	index = 0;
	maxSize = 19;
	pKeyboard = Keyboard::instance();
	if(!pKeyboard)
		throw(Nexception("Null Keyboard", __FILE__,__LINE__,__DATE__,__TIME__));
	text.reserve(maxSize+5);
	width = 0.2f;
	height = 0.1f;
	updated = false;
	blink = false;
	blinkTime = 0;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_EditBox::specificLogic
/*!	\brief		checks keyboard and changes the text accordingly
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 2:58:06 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_EditBox::specificLogic()
{
	updated = false;

	blinkTime += TimeObject::Instance()->getRealTimeStep();
	if(blinkTime > 0.5)
	{
		blinkTime = 0;
		blink = !blink;
	}

	//If clicked set as focus
	if( cursor 
		&& (cursor->X() >= x) 
		&& (cursor->X() < (x + width))
		&& (cursor->Y() >= y)
		&& (cursor->Y() < (y + height))
		&& (cursor->isLTriggered() || cursor->isRTriggered() || cursor->isWTriggered()))
	{
		getFocus();
		AcqariusSound::UIClient::Instance()->Beep();
		prevText = text;
		text[0] = 0;
		index = 0;
		return true;
	}

	if(!inFocus)
		return false;

	//If the edit box is in focus then turn the mouseover off
	mouseOver = false;

	//Process Characters
	char	C = pKeyboard->getLastCharacter();
	if(C)
	{
		if(index + 2 <= maxSize)
		{
			text[index++] = C;
		}
	}

	//Implement Backspace and Delete
	if(pKeyboard->isTriggered(DIK_BACK) || pKeyboard->isTriggered(DIK_DELETE))
	{
		if(index)
		{
			text[--index] = 0;
		}
	}

	if(blink)
		text[index] = '|';
	else
		text[index] = ' ';
	text[index + 1] = 0;
	//Clear the focus if the enter key is pressed
	if(pKeyboard->isTriggered(DIK_RETURN) || pKeyboard->isTriggered(DIK_NUMPADENTER))
	{
		giveUpFocus();
		updated = true;
		text[index] = 0;
		//Set the prevText to the current text
		AcqariusSound::UIClient::Instance()->Beep();
		return true;
	}
	//If the escape key is pressed then return to the previous text
	if(pKeyboard->isTriggered(DIK_ESCAPE))
	{
		giveUpFocus();
		updated = true;
		text.clear();
		text = prevText;
		AcqariusSound::UIClient::Instance()->Beep();
		return true;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_EditBox::clearText
/*!	\brief		clears the edit box text
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 3:18:04 PM
**	\warning	This doesn't delete memory, it sets the first elemet & index to 0
*/
////////////////////////////////////////////////////////////////////////////////
void UI_EditBox::clearText()
{
	text.clear();
	updated = false;
}

void UI_EditBox::draw()
{
	style->drawEditBackground(x, y, width, height, mouseOver);
	style->drawText(x+width*0.01f,y + height*0.4f, text.c_str());
	style->draw();
}