////////////////////////////////////////////////////////////////////////////////
/*!	\file		UI_EditBox
**	\brief		GUI primitive for accepting single line input text
**	\author		John Corpening
**	\date		11/25/2002 2:55:56 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace UserInput
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_EditBox
**	\brief		GUI primitive for accepting single line input text
**	\author		John Corpening
**	\date		11/25/2002 2:56:28 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_EditBox : public UI_TextBox
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::UI_EditBox
	/*!	\brief		Constructor constructs given a pointer to a Screen
	**	\param		Screen, a pointer to the associated screen
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 2:59:41 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_EditBox(UI_Screen* Screen) : UI_TextBox(Screen) { editInit(); }

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::UI_EditBox
	/*!	\brief		Constructor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 2:59:41 PM
	**	\warning	No screen or cursor pointer is set with this constructor
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_EditBox() { editInit(); }

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::isUpdated
	/*!	\brief		Updated flag is set when the ENTER key is hit 
	**	\return		true if updated, false otherwise
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 3:04:17 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool isUpdated(){return updated;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::getText
	/*!	\brief		returns the text currently in the edit box
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 3:17:36 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
//	char* getText() {return text;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::clearText
	/*!	\brief		clears the edit box text
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 3:18:04 PM
	**	\warning	This doesn't delete memory, it sets the first elemet & index to 0
	*/
	////////////////////////////////////////////////////////////////////////////////
	void clearText();

	void draw();

private:

	unsigned int	index;		//!< The current index into the character array
	unsigned int	maxSize;	//!< The maximum amount of text to accept
	Keyboard		*pKeyboard;	//!< Handle to the keyboard for player input
	std::string		prevText;	//!< saves the text before editing
	bool			blink;		//!< flag to indicate whether or not to blink the cursor
	double			blinkTime;	//!< the time since last blink
	bool			updated;	//!< flag to indicate whehther the edit box has been edited

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::specificLogic
	/*!	\brief		checks keyboard and changes the text accordingly
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 2:58:06 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool specificLogic();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_EditBox::editInit
	/*!	\brief		common editbox initialization
	**	\return		none
	**	\exception	Null Keyboard
	**	\author		John Corpening
	**	\date		11/25/2002 3:19:39 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void editInit();

};
}