////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Keymap.cpp
**	\brief		Implementation file for GUI_Keymap.h
**	\author		John Corpening
**	\date		10/30/2002 7:54:37 PM
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)

#include "GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action::UI_Action
/*!	\brief		Constructor gets a Message Handler Handle
**	\param		Screen screen pointer for MenuItem base class constructor
**	\return		none
**	\exception	NULL Message Handler
**	\author		John Corpening
**	\date		10/29/2002 8:31:40 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Action::UI_Action(UI_Screen* Screen) : UI_MenuItem(Screen)
{
	actionInit();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Action::actionInit
/*!	\brief		common initialization for UI_Actions
**	\return		none
**	\exception	Null Message Handler, Null Keyboard
**	\author		John Corpening
**	\date		11/25/2002 5:35:59 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Action::actionInit()
{
	key = DIK_NONE;

	if(!(MH = MsgHandler::MessageHandler::Instance()))
		throw(Nexception("NULL Message Handler", __FILE__,__LINE__,__DATE__,__TIME__));

	pInput = PlayerInput::instance();
	if(!pInput)
	{
		throw(Nexception("NULL Keyboard", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	specificLogic
/*!	\brief		Determines whether or not to trigger the action and does so
**	\return		true if the action is triggered, false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 5:20:42 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Action::specificLogic() throw()
{
	//If a key is assigned to the action and it is currently triggered
	//Or if the associated gui element indicates that the action will trigger
	if(((key != DIK_NONE) && pInput->isTriggered(key)) || checkMenuItem())
	{
		takeAction();
		return true;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Keymap::onEnter
/*!	\brief		executes the onEnter function for each action
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:37:19 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Keymap::onEnter()
{
	int size = menuItems.size();
	for(int i = 0; i < size; i++)
	{
		dynamic_cast<UI_Action*>(menuItems[i])->onEnter();
	}
}

void UI_Keymap::onDeselect()
{
		int size = menuItems.size();
	for(int i = 0; i < size; i++)
	{
		dynamic_cast<UI_Action*>(menuItems[i])->onDeselect();
	}
}

