////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Keymap.h
**	\brief		classes that map keys and buttons to actions
**	\author		John Corpening
**	\date		10/29/2002 8:20:57 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Action
**	\brief		single key and/or button to action map
**	\author		John Corpening
**	\date		10/29/2002 8:21:27 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Action : public UI_MenuItem
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::UI_Action
	/*!	\brief		Constructor gets a Message Handler Handle
	**	\param		Screen screen pointer for MenuItem base class constructor
	**	\return		none
	**	\exception	NULL Message Handler
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:40 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Action(UI_Screen* Screen);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::UI_Action
	/*!	\brief		Constructor gets a Message Handler Handle
	**	\return		none
	**	\exception	NULL Message Handler
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:40 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Action() {actionInit();}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::checkMenuItem
	/*!	\brief		Optional: allows action to take action based on menu item's status
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:45 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual bool checkMenuItem() throw() {return false;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::takeAction
	/*!	\brief		Required: takes the appropriate action
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:49 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void takeAction() throw() = 0;

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::setMenuItem
	/*!	\brief		sets the menu item
	**	\param		Item the new menu item for this action
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:53 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void setMenuItem(UI_MenuItem* Item) throw() {}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::setKey
	/*!	\brief		sets the key associated with this action
	**	\param		Key the key to set
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:31:57 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setKey(InputConstant Key) throw() {key = Key;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::getKey
	/*!	\brief		gets the key associated with this action
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:32:01 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	InputConstant getKey() throw() {return key;}


	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::draw
	/*!	\brief		currently UI_Actions don't display, this will change
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:19:48 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void draw(){};

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::specificLogic
	/*!	\brief		Determines whether or not to trigger the action and does so
	**	\return		true if the action is triggered, false otherwise
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:20:42 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool specificLogic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::onEnter
	/*!	\brief		code that is executed each time the UI switches to the screen
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:36:10 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void onEnter(){}

	virtual void onDeselect(){}


protected:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Action::actionInit
	/*!	\brief		common initialization for UI_Actions
	**	\return		none
	**	\exception	Null Message Handler, Null Keyboard
	**	\author		John Corpening
	**	\date		11/25/2002 5:35:59 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void actionInit();

	MsgHandler::MessageHandler	*MH;		//!< Used to take action
	InputConstant				key;		//!< key that triggers this action
	PlayerInput					*pInput;	//!< handle to keyboard


};

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Keymap
**	\brief		collection of UI_Actions
**	\author		John Corpening
**	\date		10/29/2002 8:22:13 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Keymap : public UI_Menu
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Keymap::UI_Keymap
	/*!	\brief		Constructor passes along screen pointer to Menu superclass
	**	\param		Screen pointer to Screen for menu constructor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 8:39:16 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Keymap(UI_Screen* Screen) : UI_Menu(Screen) {}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Keymap::onEnter
	/*!	\brief		executes the onEnter function for each action
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:37:19 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void onEnter();

	void onDeselect();
private:
	std::vector<UI_Action*>	actions;	//!< List of actions
};


}