////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Menu.h
**	\brief		Class for creating in-game menus
**	\author		John Corpening
**	\date		10/29/2002 6:32:18 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Menu
**	\brief		Class for creating in-game menus
**	\author		John Corpening
**	\date		10/29/2002 6:32:48 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Menu : public UI_MenuItem
{
//-------------VIRTUAL FUNCTIONS---------------------------------------------------
protected:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::specificLogic
	/*!	\brief		logic specific to the derived menu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:02:59 PM
	**	\warning	Abstract Function, You must override
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual bool specificLogic() throw(){return false;};

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::drawMenu
	/*!	\brief		draws the menu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:01 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void drawMenu() throw() 
	{
		if(displayBackground)
		{
			style->drawMenuBackground(x,y,width,height);
			style->draw();
		}
	}

//-------------END ABSTRACT FUNCTIONS-----------------------------------------------
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::UI_Menu
	/*!	\brief		Constructor constructs the menu given a Screen Pointer
	**	\param		Screen, a pointer to the UI_Screen that owns the menu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:39:22 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Menu(UI_Screen* Screen) : UI_MenuItem(Screen), active(true), displayBackground(true){}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::UI_Menu
	/*!	\brief		Constructor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:40:15 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Menu() : active(true), displayBackground(true){};

	~UI_Menu();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::menuLogic
	/*!	\brief		logic for processing player input
	**	\return		true for input processed
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:33:50 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool menuLogic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::addMenuItem
	/*!	\brief		adds a menu item to the menu
	**	\param		item the item to add
	**	\param		rX the x coordinate relative to the menu
	**	\param		rY the y coordinate relative to the menu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:37:46 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void addMenuItem(UI_MenuItem* item, float rX, float rY) throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::setPosition
	/*!	\brief		sets the menu position and updates all menuItem positions
	**	\param		X the new X coordinate
	**	\param		Y the new Y coordinate
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:41:16 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setPosition(float X, float Y) throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::setScreen
	/*!	\brief		sets the screen that owns the menu for the menu and all children
	**	\param		Screen, the pointer to the UI_Screen that owns this menu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:40:47 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setScreen(UI_Screen* Screen);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::setCursor
	/*!	\brief		sets the cursor for this menu and all of its children
	**	\param		Cursor, pointer to UI_Cursor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:41:27 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setCursor(UI_Cursor* Cursor);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::draw
	/*!	\brief		draws the menu and then all the menuItems
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:42:37 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void draw() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Menu::activate
	/*!	\brief		sets whether or not a menu is active
	**	\param		Activate, true to activate, false to deactivate
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:39:34 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void activate(bool Activate) {active = Activate;}

	bool getActiveState(){return active;}

	void showBackground(bool val) {displayBackground = val;}

protected:
	std::vector<UI_MenuItem*>	menuItems;	//!< The menuItems on this Menu
	bool active;
	bool displayBackground;
};
}
