////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_MenuItem.cpp
**	\brief		Implentation file for GUI_MenuItem.h
**	\author		John Corpening
**	\date		10/30/2002 8:12:07 PM
*/
////////////////////////////////////////////////////////////////////////////////


#include "GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::UI_MenuItem
/*!	\brief		constructs the MenuItem given a screen pointer
**	\param		Screen A pointer to a UI_Screen
**	\return		none
**	\exception	"Null Screen Pointer", "Null PlayerInput", "Null Cursor"
**	\author		John Corpening
**	\date		10/29/2002 8:13:27 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_MenuItem::UI_MenuItem(UI_Screen* Screen) : x(0.0f), y(0.0f), terseTip(0), 
verboseTip(0), inFocus(false), mouseOver(false), width(0.0f), height(0.0f)
{
	if(!Screen)
	{
		throw(Nexception("Null Screen Pointer", __FILE__,__LINE__,__DATE__,__TIME__));
	}
	screen = Screen;
	cursor = Screen->getCursor();
	if(!cursor)
	{
		throw(Nexception("Null Cursor", __FILE__,__LINE__,__DATE__,__TIME__));
	}

	style = new UI_Style;
	if(!style)
	{
		throw(Nexception("Null Style", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::UI_MenuItem
/*!	\brief		constructs the MenuItem 
**	\return		none
**	\exception	"Null PlayerInput"
**	\author		John Corpening
**	\date		10/29/2002 8:13:27 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_MenuItem::UI_MenuItem() : x(0.0f), y(0.0f), terseTip(0), 
verboseTip(0), inFocus(false), width(0.0f), height(0.0f), screen(0), cursor(0)
{
	style = new UI_Style;
	if(!style)
	{
		throw(Nexception("Null Style", __FILE__,__LINE__,__DATE__,__TIME__));
	}
}


////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::commonLogic
/*!	\brief		logic common to all menuItems
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		10/30/2002 8:02:54 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_MenuItem::commonLogic()
{
	//Tool Tip logic
	//If the position of the mouse is over the button
	//__asm{int 3}
	mouseOver = false;
	if(cursor && (cursor->X() >= x) 
		&& (cursor->X() < (x + width))
		&& (cursor->Y() >= y)
		&& (cursor->Y() < (y + height)))
	{
		if(!cursor->isLPressed())
		{
			mouseOver = true;
		}
		//And the mouse has not moved for a set time
		if(cursor->getPositionTime() > TIP_TIME_VERBOSE)
		{
			//Display the verbose Tool Tip
			if(verboseTip && screen)
			{
				screen->setToolTip(verboseTip);
			}
		}
		else if(cursor->getPositionTime() > TIP_TIME_TERSE)
		{
			//Display the terse Tool Tip
			if(terseTip && screen)
			{
				screen->setToolTip(terseTip);
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::processInput
/*!	\brief		Queries PlayerInput and processes input
**	\return		true if processed false otherwise
**	\exception	none
**	\author		John Corpening
**	\date		10/29/2002 6:47:54 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_MenuItem::logic()
{
	commonLogic();
	return specificLogic();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::getFocus
/*!	\brief		sets this items as the focus
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		10/30/2002 8:03:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_MenuItem::getFocus()
{
	if(!inFocus && screen)
	{
		screen->setFocus(this);
		inFocus = true;
	}
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_MenuItem::giveUpFocus
/*!	\brief		relinquishes the focus
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		10/30/2002 8:03:13 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_MenuItem::giveUpFocus()
{
	if(inFocus && screen)
	{
		screen->clearFocus();
		inFocus = false;
	}
}

UI_MenuItem::~UI_MenuItem()
{
	if(style)
	{
		delete style;
		style = 0;
	}
}