////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_MenuItem.h
**	\brief		The base class of GUI primitives
**	\author		John Corpening
**	\date		10/29/2002 6:45:52 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_MenuItem
**	\brief		The base class of GUI primitives
**	\author		John Corpening
**	\date		10/29/2002 6:47:15 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_MenuItem
{
//-------------ABSTRACT FUNCTIONS----------------------------------------------------
protected:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::specificLogic
	/*!	\brief		logic specific to the derived menuItem
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:02:59 PM
	**	\warning	Abstract Function, You must override
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual bool specificLogic() throw() = 0;

public:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::draw
	/*!	\brief		draws the menuItem
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:01 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void draw() throw() = 0;

//--------------END ABSTRACT FUNCTIONS-----------------------------------------------

public:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::UI_MenuItem
	/*!	\brief		constructs the MenuItem given a screen pointer
	**	\param		Screen A pointer to a UI_Screen
	**	\return		none
	**	\exception	"Null Screen Pointer", "Null PlayerInput", "Null Cursor"
	**	\author		John Corpening
	**	\date		10/29/2002 8:13:27 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_MenuItem(UI_Screen *Screen);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::UI_MenuItem
	/*!	\brief		constructs the MenuItem 
	**	\return		none
	**	\exception	"Null PlayerInput"
	**	\author		John Corpening
	**	\date		10/29/2002 8:13:27 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_MenuItem();

	~UI_MenuItem();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::logic
	/*!	\brief		Queries PlayerInput and processes input
	**	\return		true if processed false otherwise
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:47:54 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool logic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::setPosition
	/*!	\brief		sets the position of the Menu_Item in screen coordinates
	**	\param		X the new x coordinate in screen coordinates
	**	\param		Y the new y coordinate in screen coordinats
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:05 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void setPosition(float X, float Y) throw() {x = X; y = Y;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::shiftPosition
	/*!	\brief		shifts the position of the Menu_Item
	**	\param		dX the amount to shift the x coordinate
	**	\param		dY the amount to shift the y coordinate
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:10 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void shiftPosition(float dX, float dY) {x += dX; y += dY;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::X
	/*!	\brief		returns the x coordinate of the MenuItem
	**	\param		the x coordinate of the MenuItem
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:14 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	float X(){return x;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::Y
	/*!	\brief		returns the y coordinate of the MenuItem
	**	\param		the y coordinate of the MenuItem
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:48:19 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	float Y(){return y;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::setTipTerse
	/*!	\brief		sets the terse tool tip
	**	\param		tip a pointer to a tool tip
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:08:03 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setTipTerse(UI_ToolTip *tip) throw() {terseTip = tip;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::setTipVerbose
	/*!	\brief		sets the verbose tool tip
	**	\param		tip a pointer to a tool tip
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:08:03 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setTipVerbose(UI_ToolTip *tip) throw() {verboseTip = tip;}

	//Accessor Functions
	void setActualWidth(float Width) {width = Width;}
	void setActualHeight(float Height) {height = Height;}
	void setRelativePosition(float RX, float RY) {rX = RX, rY = RY;}
	float getRelativeWidth() {return rWidth;}
	float getRelativeHeight() {return rHeight;}
	float relX(){return rX;}
	float relY(){return rY;}
	float getWidth(){return width;}
	float getHeight(){return height;}
	virtual void setScreen(UI_Screen *Screen){screen = Screen;}
	virtual void setCursor(UI_Cursor *Cursor){cursor = Cursor;}
	
protected:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::commonLogic
	/*!	\brief		logic common to all menuItems
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:02:54 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void commonLogic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::getFocus
	/*!	\brief		sets this items as the focus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:03:04 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void getFocus() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_MenuItem::giveUpFocus
	/*!	\brief		relinquishes the focus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:03:13 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void giveUpFocus() throw();

	float			x;			//!< The x position in screen coordinates
	float			y;			//!< The y position in screen coordinates
	float			width;		//!< The width of the menuItem
	float			height;		//!< The height of the menuItem
	float			rWidth;		//!< The relative width of the menuItem to the menu
	float			rHeight;	//!< The relative height of the menuItem to the menu
	float			rX;			//!< The relative x position of the menuItem to the menu
	float			rY;			//!< The relative y position of the menuItem to the menu
	UI_ToolTip		*terseTip;	//!< A tool tip with basic information
	UI_ToolTip		*verboseTip;	//!< A tool tip with extended information
	UI_Cursor		*cursor;		//!< Handle to the screen's mouse cursor
	UI_Screen		*screen;		//!< Handle to the screen this menuItem belongs to
	UI_Style		*style;
	bool			inFocus;	//!< flag to indicate if we have the focus
	bool			mouseOver;
	
	//Internal constants
	enum 
	{
		TIP_TIME_TERSE		= 2,	//!< Time in seconds till the terse tip is displayed
		TIP_TIME_VERBOSE	= 5,	//!< Time in seconds till the verbose tip is displayed
	};
};
}
