////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Pushbutton.cpp
**	\brief		Implementation file for GUI_Pushbutton.h
**	\author		John Corpening
**	\date		11/25/2002 6:06:44 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)

#include "GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Pushbutton::pushbuttonInit()
/*!	\brief		common initialization for pushbuttons
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 5:59:07 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Pushbutton::pushbuttonInit()
{
	width = 0.3f;
	height = 0.07f;
	clicked = false;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Pushbutton::UI_Pushbutton
/*!	\brief		Constructs the pushbutton given a pointer to the screen that owns it
**	\param		Screen, the pointer to the UI_Screen that owns the pushbutton
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 5:56:52 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Pushbutton::UI_Pushbutton(UI_Screen *Screen) : UI_MenuItem(Screen)
{
	pushbuttonInit();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Pushbutton::UI_Pushbutton
/*!	\brief		Constructs the pushbutton
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 5:56:52 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
UI_Pushbutton::UI_Pushbutton()
{
	pushbuttonInit();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Pushbutton::draw()
/*!	\brief		draws the pushbutton
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 6:03:38 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Pushbutton::draw()
{
	style->drawItemBackground(x, y, width, height, mouseOver);
	style->drawText(x+width*0.01f,y+height*0.4f, caption.c_str());
	style->draw();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Pushbutton::specificLogic()
/*!	\brief		sets clicked if the mouse clicks the pushbutton
**	\return		true if clicked otherwise false
**	\exception	none
**	\author		John Corpening
**	\date		11/25/2002 6:02:04 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Pushbutton::specificLogic()
{
	//set the clicked flag if the mouse is within our bounds and
	//any mouse button has been clicked
	clicked = false;
	if((cursor->X() >= x) 
		&& (cursor->X() < (x + width))
		&& (cursor->Y() >= y)
		&& (cursor->Y() < (y + height))
		&& (cursor->isLTriggered() || cursor->isRTriggered() || cursor->isWTriggered()))
	{
		clicked = true;
		AcqariusSound::UIClient::Instance()->Beep();
		return true;
	}
	return false;
}