////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Pushbotton.h
**	\brief		pushbuttons
**	\author		John Corpening
**	\date		10/30/2002 8:13:06 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////


#pragma once
namespace UserInput
{

////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Pushbutton
**	\brief		class for pushbuttons
**	\author		John Corpening
**	\date		10/30/2002 8:13:24 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Pushbutton : public UI_MenuItem
{

public:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::UI_Pushbutton
	/*!	\brief		Constructs the pushbutton given a pointer to the screen that owns it
	**	\param		Screen, the pointer to the UI_Screen that owns the pushbutton
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:56:52 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Pushbutton(UI_Screen* Screen);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::UI_Pushbutton
	/*!	\brief		Constructs the pushbutton
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:56:52 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Pushbutton();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::specificLogic()
	/*!	\brief		sets clicked if the mouse clicks the pushbutton
	**	\return		true if clicked otherwise false
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:02:04 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual bool specificLogic();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::draw()
	/*!	\brief		draws the pushbutton
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:03:38 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	virtual void draw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::isClicked
	/*!	\brief		returns the status of the clicked flag
	**	\return		true if clicked otherwise false
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:04:20 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool isClicked() {return clicked;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::setCaption
	/*!	\brief		sets the button caption
	**	\param		Caption, string to set as caption
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:04:57 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setCaption(string Caption) {caption = Caption;}

protected:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Pushbutton::pushbuttonInit()
	/*!	\brief		common initialization for pushbuttons
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 5:59:07 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void pushbuttonInit();

	bool		clicked;	//!< flag for whether or not the button was clicked
	string		caption;	//!< The caption on the button
};
}