////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Screen.h
**	\brief		The base class for creating screens for the game
**	\author		John Corpening
**	\date		10/29/2002 6:55:22 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{
class UI_MenuItem;
class UI_ToolTip;
class UI_Keymap;
class UI_PullDownMenu;
class UI_Menu;
class UI_Cursor;
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Screen
**	\brief		the base class for creating screens
**	\author		John Corpening
**	\date		10/29/2002 6:55:52 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Screen
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::UI_Screen
	/*!	\brief		Constructor for the screen
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:09:28 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Screen();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::logic
	/*!	\brief		regulates processing of input for menus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:28 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool logic() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::setFocus
	/*!	\brief		sets the given menuItem as the focus for UI input
	**	\param		item the menuItem to set as the focus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:33 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setFocus(UI_MenuItem* item) throw() {focus = item;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::clearFocus
	/*!	\brief		clears the focus
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:37 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void clearFocus() throw() {focus = NULL;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::draw
	/*!	\brief		draws the menus, then the pulldown menu then the toolTip
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:41 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void draw() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::setToolTip
	/*!	\brief		sets the given toolTip as the current toolTip
	**	\param		tip the toolTip to set
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:49 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setToolTip(UI_ToolTip* tip) throw() {toolTip = tip;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::clearToolTip
	/*!	\brief		clears the current toolTip
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:53 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void clearToolTip() throw() {toolTip = 0;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::setKeyMap
	/*!	\brief		sets the keymap
	**	\param		map the keymap to set as the current keymap
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:56:57 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setKeymap(UI_Keymap* map) throw() {keyMap = map;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::setPullDown
	/*!	\brief		sets the given pullDownMenu as the current pullDown
	**	\param		menu the pullDownMenu to set
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:57:02 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setPullDown(UI_PullDownMenu* menu) throw() {pullDown = menu;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::clearPullDown
	/*!	\brief		clears the current pullDownMenu
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:57:06 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void clearPullDown() throw() {pullDown = 0;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::addMenu
	/*!	\brief		adds a menu to the screen
	**	\param		menu the menu to add
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/29/2002 6:58:06 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void addMenu(UI_Menu *menu, float x = 0.0f, float y = 0.0f) throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::getCursor
	/*!	\brief		cursor accessor
	**	\return		the screen's cursor
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 8:14:46 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Cursor* getCursor() throw() {return cursor;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::setCamera
	/*!	\brief		sets the screen's handle to a camera
	**	\param		Camera, a UI_Camera pointer to set
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		1/18/2003 1:54:42 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setCamera(UI_Camera* Camera) {camera = Camera;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::drawCursor
	/*!	\brief		sets flag as to whether or not to draw the cursor
	**	\param		Draw, true to draw the cursor, false to not draw cursor
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:41:52 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void drawCursor(bool Draw = true){m_drawCursor = Draw;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Screen::onEnter
	/*!	\brief		code that is executed whenever the screen is entered
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:43:10 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void onEnter();

	void onDeselect();

	bool checkCursorVsMenus();

	~UI_Screen();

private:
	UI_PullDownMenu		*pullDown;	//!< The pullDownMenu to display
	UI_ToolTip			*toolTip;	//!< The toolTip to display
	UI_MenuItem			*focus;		//!< The menu item with the focus
	UI_Keymap			*keyMap;	//!< Maps buttons and keys to actions
	UI_Cursor			*cursor;	//!< Mouse Cursor
	UI_Camera			*camera;	//!< The camera object
	bool				m_drawCursor;	//!< flag to determine whether or not to draw the cursor
	std::vector<UI_Menu*>	menus;		//!< Menus that make up this screen	

};
}