////////////////////////////////////////////////////////////////////////////////
/*!	\file		UI_Slider.cpp
**	\brief		implementation file for UI_Slider.h
**	\author		John Corpening
**	\date		2/22/2003 12:03:00 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)

#include "GUI_Includes.h"

// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif
using namespace UserInput;


UI_Slider::UI_Slider(UI_Screen* Screen) : UI_MenuItem(Screen)
{
	init();
}

UI_Slider::UI_Slider()
{
	init();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::init
/*!	\brief		initialization function
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:49:40 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Slider::init()
{
	height = 0.025f;
	width = 0.4f;
	updated = false;
	dragging = false;
	initIndex();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::initIndex
/*!	\brief		initializes the slider index
**				(needs to be called if slider is resized)
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:55:09 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Slider::initIndex()
{
	iheight = height * 1.2f;
	iwidth = 0.01f;
	ix = x;
	iy = y - 0.1f*height;
}


////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::draw
/*!	\brief		draws the slider bar
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:55:27 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Slider::draw()
{
	style->drawItemBackground(x,y,width,height);
	style->drawSliderIndex(x+ix, y+iy, iwidth, iheight);
	style->draw();
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::getValue
/*!	\brief		returns the relative position of the index to the slider
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:55:31 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
float UI_Slider::getValue()
{
	return ix/width;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::setIndexValue
/*!	\brief		sets the relative position of the index to the slider
**	\param		val, float between 0.0f and 1.0f
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:55:36 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void UI_Slider::setIndexValue(float val)
{
	if((val < 0.0f) || (val > 1.0f))
		return;
	ix = width*val;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	UI_Slider::specificLogic
/*!	\brief		logic for the slider bar
**	\return		none
**	\exception	none
**	\author		John Corpening
**	\date		2/22/2003 11:55:41 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
bool UI_Slider::specificLogic()
{
	updated = false;
	//Check for clicked condition
	if((cursor->X() >= x+ix) 
		&& (cursor->X() < (x+ix + iwidth))
		&& (cursor->Y() >= y+iy)
		&& (cursor->Y() < (y+iy + iheight))
		&& cursor->isLPressed())// || cursor->isRTriggered() || cursor->isWTriggered()))
	{
		dragging = true;
		return true;
	}

	//Check for Release condition
	if(dragging && !cursor->isLPressed())
	{
		dragging = false;
		updated = true;
		AcqariusSound::UIClient::Instance()->Beep();
		return true;
	}

	//Check for dragging condition
	if(dragging)
	{
		ix += cursor->deltaX();
		if(ix > width-(iwidth*0.5f)) 
		{
			ix = width-(iwidth*0.5f);
		}

		if(ix < -iwidth*0.5f) 
		{
			ix = -iwidth*0.5f;
		}
		
		return true;
	}

	return false;
}