////////////////////////////////////////////////////////////////////////////////
/*!	\file		GUI_Slider.h
**	\brief		GUI slider bar primitive
**	\author		John Corpening
**	\date		2/22/2003 11:44:29 AM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once

namespace UserInput
{
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Slider
**	\brief		GUI slider bar primitive
**	\author		John Corpening
**	\date		2/22/2003 11:45:14 AM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Slider : public UI_MenuItem
{
	float	ix,			//!< Index x coordinate
			iy;			//!< Index y coordinate
	float	iwidth,		//!< Index width
			iheight;	//!< Index height
	bool	updated;	//!< flag to indicate state change
	bool	dragging;	//!< flag to indicate that the user is dragging the index

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::init
	/*!	\brief		initialization function
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:49:40 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void init();

public:
	UI_Slider(UI_Screen* Screen);
	UI_Slider();

	//returns the updated flag
	bool isUpdated(){return updated;}

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::draw
	/*!	\brief		draws the slider bar
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:55:27 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void draw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::getValue
	/*!	\brief		returns the relative position of the index to the slider
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:55:31 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	float getValue();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::initIndex
	/*!	\brief		initializes the slider index
	**				(needs to be called if slider is resized)
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:55:09 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void initIndex();

	
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::setIndexValue
	/*!	\brief		sets the relative position of the index to the slider
	**	\param		val, float between 0.0f and 1.0f
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:55:36 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void setIndexValue(float val);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Slider::specificLogic
	/*!	\brief		logic for the slider bar
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		2/22/2003 11:55:41 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	bool specificLogic();
};
}