////////////////////////////////////////////////////////////////////////////////
/*!	\file		UI_Handler
**	\brief		Class to manage screens
**	\author		John Corpening
**	\date		10/30/2002 6:53:14 PM
**	\par		DESCRIPTION:
**	\par		REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////

#pragma once


namespace UserInput
{
	typedef unsigned int PlayerID;
class UI_Screen;
////////////////////////////////////////////////////////////////////////////////
/*!	\class		UI_Handler
**	\brief		Class to manage which screen is currently displayed
**	\author		John Corpening
**	\date		10/29/2002 9:38:24 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
class UI_Handler
{
public:

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler:: instance
	/*!	\brief		access point for the UI_Handler
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 6:55:28 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	static UI_Handler*	instance();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler::logic
	/*!	\brief		allows the current screen to process input
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 6:56:04 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void logic();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Hanlder::draw
	/*!	\brief		draws the current screen
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		11/25/2002 6:16:12 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void draw() throw();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler::getNextFish
	/*!	\brief		gets the next Fish in the tank
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		1/18/2003 1:55:47 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void getNextFish();

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler::switchScreens
	/*!	\brief		switches to the given screen if it exists
	**	\param		screenID, the ID of the screen to switch to
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		1/18/2003 1:56:22 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void switchScreens(GUI_ScreenID screenID);

	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler::previousScreen
	/*!	\brief		switches to the previousScreen
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		1/18/2003 1:57:01 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	void previousScreen();

	bool confirmSelect(const MsgHandler::Key& select, PlayerID id);
	bool confirmDeselect(const MsgHandler::Key& select, PlayerID id);

	void GenerateCompleteNetMessage(UDPNet::NetMessage &ServerUpdate);
	void LoadNetMessage(UDPNet::NetMessage &ServerUpdate);

	MsgHandler::Key getFishTarget() {return fish;}
	void setFishTarget(const MsgHandler::Key& Fish);

	void setTitleScreenSong(const char* song);
	const char* getTitleScreenSong(){return songTS;}
	void setInGameSong(const char* song);
	const char* getInGameSong(){return songIG;}
	void setBrainEditorSong(const char* song);
	const char* getBrainEditorSong(){return songBE;}
	MsgHandler::Key getFishFromIP(PlayerID id);
	void deselectAllButThisIP(PlayerID id);
	MsgHandler::Key FindFirstDeselectedFish(MsgHandler::Key FirstFish);

	bool isSelected(MsgHandler::Key key);
	~UI_Handler();

protected:
	////////////////////////////////////////////////////////////////////////////////
	//	FUNCTION:	UI_Handler::UI_Handler
	/*!	\brief		Constructor creates all screens for the game
	**	\return		none
	**	\exception	none
	**	\author		John Corpening
	**	\date		10/30/2002 6:54:22 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	UI_Handler();

private:
	std::map<GUI_ScreenID, UI_Screen*>		screens;		//!< All the screens in the game
	std::map<MsgHandler::Key, PlayerID>		possessed;		//!< The possessed fish
	GUI_ScreenID							prevScreenID;	//!< ID of previous screen
	GUI_ScreenID							curScreenID;	//!< ID of current screen
	PlayerID								myID;
	UI_Screen								*currentScreen;	//!< The current screen
	PlayerInput								*input;			//!< The low level player input
	static UI_Handler*						_instance;		//!< Singleton instance

	MsgHandler::Key							fish;			//!< Current Fish
	MsgHandler::MessageHandler				*MH;			//!< Handle to the MessageHandler
	char									*songTS,		//!< Title Screen Song
											*songIG,		//!< In Game Song
											*songBE;		//!< Brain Editor Song
	UI_Style*								style;			//!< Used for Frame Counter Display
};
}