////////////////////////////////////////////////////////////////////////////////
/*!	\file		
**	\brief		
**	\author		Nate Cleveland
**	\date		10/22/2002 1:46:58 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)
#include<math.h>
#include "action.hpp"
#include "../networking/NetData.h"
#include "../Nexception/Nexception.h"
#include "../debugging/Debug.h"
#include <fstream>
char buff[512] ={0};
using namespace InterpretationModule;
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

const float InterpretationModule::FOLLOW_ACTION_MIN_DISTANCE	= 10.0f;
const float InterpretationModule::GOTO_ACTION_MIN_DISTANCE		= 5.0f;
const float InterpretationModule::EVAL_MARKER_ACTION_RANGE		= 1000.0f;
const float InterpretationModule::ATTACK_ACTION_TIME_DELAY		= 8.0f/30;	
const float InterpretationModule::EAT_ACTION_TIME_DELAY			= 0.0f;
const float	InterpretationModule::ATTACK_ACTION_MIN_DISTANCE	= 20.0f;
const float	InterpretationModule::EAT_ACTION_MIN_DISTANCE		= 20.0f;
const float	InterpretationModule::ROAM_ACTION_MIN_DISTANCE		= 10.0f;	
const float InterpretationModule::FLEE_ACTION_MIN_DISTANCE		= 50.0f;
const float						  FLEE_ACTION_MIN_TIME			= 30.0f;

const float InterpretationModule::MOVE_WITH_TIME				= 30.0f;	
const float MARKER_ACTION_TIME									= 30.0f;


#define FISH_BODY_LOG
#ifdef FISH_BODY_LOG
#	define FISH_BODY_ACTIONS_LOG					//!< Remove to stop all logging by actions modules

#	ifdef FISH_BODY_ACTIONS_LOG
#		define	FISH_BODY_ACTIONS_LOG_TO_CONSOLE	//!< Remove to stop actions from logging to console
#		define FISH_BODY_ACTIONS_LOG_TO_FILE		//!< Remove to stop actions from logging to file.
#	endif
#endif


//see header for documenation.
Action::Action(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
:m_hFishBody(hFishBody), m_cmdPack(cmdPack), m_isDone(false), m_isComplete(false),
m_elapsedTime(0.0f), m_strObj(strObj)
{
	
}


//See header for documentation.
#pragma warning(push)
#pragma warning(disable: 4800)
Action* Action::CreateAction(StrObj& strObj, HFishBody& hFishBody, UDPNet::NetMessage& netMessage) throw()
{
	Action *newAction = NULL;		//Eventual pntr to the new Action to be created.
	CommandPacket cmdPack;

	//read in the command packet.
	cmdPack << netMessage;


	//Create the correct action, check for error.
	if(NULL ==(newAction = CreateAction(strObj, hFishBody, cmdPack)))
		throw Nexception("Fatal error in Action::CreateAction while trying to read netMessage.");

	//fill state flags and timer.
	newAction->m_isDone = *reinterpret_cast<const bool*>(netMessage.ReadNBytes(sizeof(bool)));
	newAction->m_isComplete = *reinterpret_cast<const bool*>(netMessage.ReadNBytes(sizeof(bool)));
	newAction->m_elapsedTime = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));

	//give action a chance to load other data.
	newAction->Read(netMessage);

	//we're done, return the newAction pntr to the caller
	return newAction;	
}
#pragma warning(pop)

//See header for documentation.
void Action::SetString(void) throw()
{
	m_strObj.SetMove_Action("Unable to deduce Action");
	m_strObj.SetMove_Target("Unable to deduce Target");	
}


//See header for documentation.
void Action::Write(UDPNet::NetMessage& netMessage)
{
	////cmdPack
	m_cmdPack >> netMessage;

	////isDone
	netMessage.WriteNBytes(sizeof(bool), reinterpret_cast<char*>(&m_isDone));

	////isComplete
	netMessage.WriteNBytes(sizeof(bool), reinterpret_cast<char*>(&m_isComplete));

	////time
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_elapsedTime));
}



//See header for documentation.
Action* Action::CreateAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack) throw()
{
	Action *newAction = NULL;		//Eventual pntr to the new Action to be created.
	
	try{
		//switch through the cmdPacket's commandID to find the correct Action child class
		switch(cmdPack.GetCommandID())
		{
		case AQUARIUS_COMMAND_FOLLOW:							//follow action.
			newAction = new FollowAction(strObj, hFishBody, cmdPack);
			break;
			
		case AQUARIUS_COMMAND_GOTO:								//goto action.
			newAction = new GotoAction(strObj, hFishBody, cmdPack);
			break;
			
		case AQUARIUS_COMMAND_FLEE:								//flee action.
			newAction = new FleeAction(strObj, hFishBody, cmdPack);
			break;

		case AQUARIUS_COMMAND_ATTACK:							//attack action
			newAction = new AttackAction(strObj, hFishBody, cmdPack);
			break;

		case AQUARIUS_COMMAND_EAT:
			newAction = new EatAction(strObj, hFishBody, cmdPack);		//eat action
			break;

		case AQUARIUS_COMMAND_MOVE_FORWARD:						//move forward action.
			newAction = new MoveForwardAction(strObj, hFishBody, cmdPack);
			break;

		case AQUARIUS_COMMAND_STOP_FIN:							//stop fin action
			newAction = new StopFinAction(strObj, hFishBody, cmdPack);			
			break;

		case AQUARIUS_COMMAND_STOP_MOUTH:						//stop mouth action
			newAction = new StopMouthAction(strObj, hFishBody,cmdPack);
			break;

		case AQUARIUS_COMMAND_EVALUATE_MARKERS:					//evaluate markers action
			newAction = new EvaluateMarkersAction(strObj, hFishBody, cmdPack);	
			break;

		case AQUARIUS_COMMAND_EVALUATE_POSITIVE:				//evaluate positive markers action
			newAction = new EvaluateMarkersAction_Pos(strObj, hFishBody, cmdPack);	
			break;

		case AQUARIUS_COMMAND_EVALUATE_NEGATIVE:				//evaluate negative markers action
			newAction = new EvaluateMarkersAction_Neg(strObj, hFishBody, cmdPack);	
			break;


		case AQUARIUS_COMMAND_DROP_MARKER:						//drop markers action
			newAction = new DropMarkersAction(strObj, hFishBody, cmdPack);
			break;

		case AQUARIUS_COMMAND_ROAM:								//roam action
			newAction = new RoamAction(strObj, hFishBody, cmdPack);
			break;

		case AQUARIUS_COMMAND_MOVE_WITH:
			newAction = new MoveWithAction(strObj, hFishBody, cmdPack);
			break;

		default:
			//if we're given a commandId we don't understand
			// log the error and make sure newAction is set 
			// to NULL.
			newAction = NULL;
		}//end switch
	} catch (...) {
		//there was an error in the cmdPack, log the error.
		newAction = NULL;	
	}//end catch


	//we're done, return the newAction pntr to the caller.
	return newAction;	
}//end Action::CreateAction



//See header for documentation.
Action* Action::CreateAction(StrObj& strObj, HFishBody& hFishBody, std::istream&inStream) throw()
{
	Action *newAction = NULL;		//Eventual pntr to the new Action to be created.
	CommandPacket cmdPack;
	bool isDone = false;
	float elapsedTime = 0;

	Read(inStream, cmdPack, isDone, elapsedTime);

	
	try{
		//switch through the cmdPacket's commandID to find the correct Action child class
		switch(cmdPack.GetCommandID())
		{
		case AQUARIUS_COMMAND_FOLLOW:							//follow action.
			newAction = new FollowAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;
			
		case AQUARIUS_COMMAND_GOTO:								//goto action.
			newAction = new GotoAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;
			
		case AQUARIUS_COMMAND_FLEE:								//flee action.
			newAction = new FleeAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_ATTACK:							//attack action
			newAction = new AttackAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			newAction->Read(inStream);
			break;

		case AQUARIUS_COMMAND_EAT:
			newAction = new EatAction(strObj, hFishBody, cmdPack);		//eat action
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			newAction->Read(inStream);
			break;

		case AQUARIUS_COMMAND_MOVE_FORWARD:						//move forward action.
			newAction = new MoveForwardAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_STOP_FIN:							//stop fin action
			newAction = new StopFinAction(strObj, hFishBody, cmdPack);			
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_STOP_MOUTH:						//stop mouth action
			newAction = new StopMouthAction(strObj, hFishBody,cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_EVALUATE_MARKERS:					//evaluate markers action
			newAction = new EvaluateMarkersAction(strObj, hFishBody, cmdPack);	
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_EVALUATE_POSITIVE:				//evaluate positive markers action
			newAction = new EvaluateMarkersAction_Pos(strObj, hFishBody, cmdPack);	
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_EVALUATE_NEGATIVE:				//evaluate negative markers action
			newAction = new EvaluateMarkersAction_Neg(strObj, hFishBody, cmdPack);	
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_DROP_MARKER:						//drop markers action
			newAction = new DropMarkersAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

		case AQUARIUS_COMMAND_ROAM:								//roam action
			newAction = new RoamAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;

			//next line stop the RoamAction from creating a new
			//random point
			newAction->m_cmdPack = cmdPack;	
			break;

		case AQUARIUS_COMMAND_MOVE_WITH:
			newAction = new MoveWithAction(strObj, hFishBody, cmdPack);
			newAction->m_isDone = isDone;
			newAction->m_elapsedTime = elapsedTime;
			break;

			
		default:
			//if we're given a commandId we don't understand
			// log the error and make sure newAction is set 
			// to NULL.
			newAction = NULL;
		}//end switch
	} catch (...) {
		
		//there was an error in the cmdPack, log the error.
		newAction = NULL;	
	}//end catch


	//we're done, return the newAction pntr to the caller.
	return newAction;	
}//end Action::CreateAction


//See header for documentation.
bool Action::GetTargetPoint(InternalPoint_type& point)const throw()
{
	AquariusKey_type targetKey;		//var used if target is has a key.


	try{
	//switch between the different types of commandTargets there are.
		switch(m_cmdPack.GetTargetFlag())
		{
		case ActionTarget::ACTION_TARGET_FLAG_KEY:					//key'd target
			targetKey = m_cmdPack.GetTargetKey();
			return m_hFishBody.GetPos(targetKey, point);

		case ActionTarget::ACTION_TARGET_FLAG_NONE:					//no target.
			//unable to deduce location from this target.
			return false;	

		case ActionTarget::ACTION_TARGET_FLAG_POINT:				//point target
			m_cmdPack.GetTargetPoint(point);
			return true;


		case ActionTarget::ACTION_TARGET_FLAG_VOXEL:
			//at this time unable to deduce voxel locations
#			ifdef FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			//log an error to the console.
			//...
#			endif

#			ifdef FISH_BODY_ACTIONS_LOG_TO_FILE
			//log error to file
			//...
#			endif
			return false;
		}//end switch
	} catch(...)
	{
		//there was an error in the cmdPack, log the error.
#		ifdef FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			//log an error to the console.
			//...
#		endif

#		ifdef FISH_BODY_ACTIONS_LOG_TO_FILE
			//log error to file
			//...
#		endif
	}//end try/catch

	//if we reach this point in the code there
	// was an error.
	return false;
}//end Action::GetTargetPoint


//See header for documentation.
//!	\note	Formula for generation is PQ = Q - P
bool Action::GetVectorToTarget(const InternalPoint_type& toPoint, InternalVector_type& answerVector) 
	const throw()
{
	//first, get the target point, check for errors.
	InternalPoint_type targetPoint;

	if(!GetTargetPoint(targetPoint))
		return false;

	//compute the vector with simple subtraction.
	answerVector.x = targetPoint.x - toPoint.x;
	answerVector.y = targetPoint.y - toPoint.y;
	answerVector.z = targetPoint.z - toPoint.z;
	return true;
}//end Action::GetVectorToTarget(const InternalPoint_type&, InternalVector_type& ) 


//See header for documentation.
float Action::GotoTarget(InternalVector_type& answerVector) const
{
	//compute a vector to the target, check for error and log it.
	InternalVector_type	vecToTarget;

	if(!this->GetVectorToTarget(vecToTarget))
	{
#		ifdef FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			//log an error to the console.
			//...
#		endif

#		ifdef FISH_BODY_ACTIONS_LOG_TO_FILE
			//log error to file
			//...
#		endif

		return -1.0f;
	}


	//Transform to unit vector
	UnitizeVector<float>(vecToTarget, vecToTarget);

	//scale by the wieght given to us in the command packet.
	ScaleVector<float>(vecToTarget, answerVector, this->GetCommandPackWeight());

	return FindDistToTarget();
}//end Action::GotoTarget



//see header for documentation
void Action::VectorToPoint(const InternalVector_type& fromPoint, 
						   InternalVector_type& answerVector)  const throw()
{
	//get this fish's position
	InternalPoint_type pos;
	m_hFishBody.GetPos(pos);

	//compute a vector from the fromPoint to the fish position
	answerVector.x = fromPoint.x - pos.x;
	answerVector.y = fromPoint.y - pos.y;
	answerVector.z = fromPoint.z - pos.z;
}


//See header for documentation.
void Action::Write(std::ostream& outStream) const
{
	char buffer[256] = {0};

	outStream << "##Begin Action \n";
	//stream in the command packet.
	outStream << m_cmdPack;

	//stream in everything else
	sprintf(buffer, "IsDone: %d\t LifeTime: %.2f\t\n", m_isDone, m_elapsedTime);
	outStream << buffer;

	outStream << "##End Action\n";
}	

//See header for documentation.
void Action::Read(std::istream& inStream, CommandPacket&cmdPack, bool& isDone, float& elapsedTime)
{
	char buffer[512] = {0};

	//read in the header.
	inStream.getline(buffer, 512, '\n');

	//read in command packet
	inStream >> cmdPack;

	//read in everything else
	inStream.getline(buffer, 512, '\n');
	sscanf(buffer,"IsDone: %d\t LifeTime: %.2f\t ", &isDone, &elapsedTime);

	//read in the footer.
	inStream.getline(buffer, 512, '\n');
}




//See header for documentation.
GotoAction::GotoAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_GOTO/* ||
		cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_NONE*/) 
	{
		throw 0;
	}
	
	//if this is true then we need to attack the nearest fish.
	if(cmdPack.GetTargetFlag() ==  ActionTarget::ACTION_TARGET_FLAG_KEY &&
	   0 == GetCommandPackTargetKey())
	{
		//Get the closest fish ID from the HFishBody
		AquariusKey_type nearFish = m_hFishBody.GetNearestFish();

		//if there is a near by fish, set that as the target,
		if(nearFish)
			SetCommandPackTarget(nearFish);

		//otherwise just ignore the action as there is nothing to attack.
		else
			this->SetDone(true, false);
	}//end if


	// Added by Nate Cleveland [4/17/2003]
	// If this is a point clip it to the world boundry
	if(cmdPack.GetTargetFlag() ==  ActionTarget::ACTION_TARGET_FLAG_POINT)
	{
		InternalPoint_type pnt;
		GetCommandPacktTargetPoint(pnt);
		
		ActionTarget actTarg;

		actTarg.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_POINT;



		m_hFishBody.ClipPoint(pnt);
		actTarg.m_ActionTarget.m_point = pnt;
		this->SetCommandPackTarget(actTarg);
	}

}

void GotoAction::SetString(void) throw()
{
	m_strObj.SetMove_Action("Goto/Roam");

	////Set the StrObj Target
	//as a key
	if(m_cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_KEY) 
		sprintf(buff, "Key %d", GetCommandPackTargetKey());

	//as a point.
	else if(m_cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_POINT)  
	{	
		InternalPoint_type pnt;
		GetCommandPacktTargetPoint(pnt);
		sprintf(buff, "Point (%.2f, %.2f, %.2f)", pnt.x, pnt.y, pnt.z);
	}

	//as something else
	else
		buff[0] = 0;

	//read in the data.
	m_strObj.SetMove_Target(buff);
	
}

//See header for documentation.
bool GotoAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, goto \n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	float distToTarget;

	//find a vector to the target.
	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;

	//see if we're close enough to stop moving. if we are done
	// set the isDone flag to true and return out.
	if(distToTarget <= GOTO_ACTION_MIN_DISTANCE) 
	{
		this->SetDone();
		return true;
	}

	InternalPoint_type targPoint;
	GetTargetPoint(targPoint);



			sprintf(buff, "Goto targ(%.2f,%.2f,%.2f) dir(%.2f,%.2f,%.2f)\n",
				targPoint.x, targPoint.y, targPoint.z,
				potenFieldVec.x, potenFieldVec.y, potenFieldVec.z);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);

	//Add the vector to the summation
	this->m_hFishBody.AddVector(potenFieldVec);
	return true;
}

//See header for documentation.
FollowAction::FollowAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_FOLLOW ||
		cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_NONE) 
	{
		throw 0;
	}

	//if this is true then we need to attack the nearest fish.
	if(0 == GetCommandPackTargetKey())
	{
		//Get the closest fish ID from the HFishBody
		AquariusKey_type nearFish = m_hFishBody.GetNearestFish();

		//if there is a near by fish, set that as the target,
		if(nearFish)
			SetCommandPackTarget(nearFish);

		//otherwise just ignore the action as there is nothing to attack.
		else
			this->SetDone(true, false);
	}//end if

}
void FollowAction::SetString(void) throw()
{
	if(GetCommandPackTargetKey()) 
	{
		m_strObj.SetMove_Action("Follow");
		sprintf(buff, "Key %d", GetCommandPackTargetKey());
		m_strObj.SetMove_Target(buff);
	}
}


//See header for documentation.
bool FollowAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Follow\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//find a vector to the target.
	float distToTarget;

	//swim towards the object we're trailing.
	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;


	//see if we're close enough to the object to stop following it
	if(distToTarget <= FOLLOW_ACTION_MIN_DISTANCE) 
	{
		this->SetDone();
		return true;	
	}


	//Add this vector to the summation
	this->m_hFishBody.AddVector(potenFieldVec);
	return true;
}

//See header for documentation.
FleeAction::FleeAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_FLEE ||
		cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_NONE) 
	{
		throw 0;
	}
	//if this is true then we need to attack the nearest fish.
	if(0 == GetCommandPackTargetKey())
	{
		//Get the closest fish ID from the HFishBody
		AquariusKey_type nearFish = m_hFishBody.GetNearestFish();

		//if there is a near by fish, set that as the target,
		if(nearFish)
			SetCommandPackTarget(nearFish);

		//otherwise just ignore the action as there is nothing to attack.
		else
			this->SetDone(true, false);
	}//end if


	if(0 == GetCommandPackTargetKey())
		this->SetDone(true, false);


}

void FleeAction::SetString(void) throw()
{
	if(GetCommandPackTargetKey()) 
	{
		//Set the StrObj data
		m_strObj.SetMove_Action("Flee");
		sprintf(buff, "Key %d", GetCommandPackTargetKey());
		m_strObj.SetMove_Target(buff);
	}
}


//See header for documentation.
bool FleeAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Fleeing from %d\n", 
				this->GetCommandPackBodyID(), this->GetCommandPackTargetKey());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif


	float distToTarget;

	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;

	//check to see if we're far enough away from the target not
	//to worry about it.
	if(distToTarget >= FLEE_ACTION_MIN_DISTANCE || FLEE_ACTION_MIN_TIME <= ElapsedTime()) 
	{
			sprintf(buff, "Done Fleeing from %d. dist is %.2f\n", 
				this->GetCommandPackTargetKey(), distToTarget);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
		this->SetDone();
		return true;	
	}

	//since we want to flee from the target, not run to it,
	//negate the vector.
	UnitizeVector<float>(potenFieldVec, potenFieldVec);
	ScaleVector<float>(potenFieldVec, potenFieldVec, -1.0f*this->GetCommandPackWeight());

	//Add this vector to the summation
	this->m_hFishBody.AddVector(potenFieldVec);
	return true;
}


//See header for documentation.
AttackAction::AttackAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_ATTACK ||
		cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_NONE) 
	{
		throw 0;
	}

	//if this is true then we need to attack the nearest fish.
	if(0 == GetCommandPackTargetKey())
	{
		//Get the closest fish ID from the HFishBody
		AquariusKey_type nearFish = m_hFishBody.GetNearestFish();

		//if there is a near by fish, set that as the target,
		if(nearFish)
			SetCommandPackTarget(nearFish);

		//otherwise just ignore the action as there is nothing to attack.
		else
			this->SetDone(true, false);
	}//end if

	

//	cout << "Attack " << buff << endl;
}

void AttackAction::SetString(void) throw()
{

	//Set the StrObj data
	if(GetCommandPackTargetKey()) 
	{
		m_strObj.SetMove_Action("Attack");
		sprintf(buff, "Key %d", GetCommandPackTargetKey());
		m_strObj.SetMove_Target(buff);
	}
}


//See header for documentation.
bool AttackAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Attack dist %.2f\n", 
				this->GetCommandPackBodyID(), FindDistToTarget());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	float distToTarget;
	
	//swim to the thing we're trying to attack.
	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;

	//no matter what swim to the target.
	this->m_hFishBody.AddVector(potenFieldVec);

	//See if we've already begun the attacking proceedure.
	if(this->m_hFishBody.IsAttacking()) 
	{
		//see if we can land the attack now
		if(this->ElapsedTime() - this->m_attackStartTime >= ATTACK_ACTION_TIME_DELAY) 
		{
			//get the target from the command pack.
			ActionTarget actTarget;
			this->GetCommandPackTarget(actTarget);

			//before dealing damage to the target make sure it was a key'd target.
			if(actTarget.m_ActionFlag == actTarget.ACTION_TARGET_FLAG_KEY)
				this->m_hFishBody.EndAttack(actTarget.m_ActionTarget.m_key);
			

			//ignore the ret-value of the EndAttack call. At this time we don't
			//evaluate whether or not the attack was successfull, only if it happend.
			//therefore, all we need do is set the isDone flag to true. With that in
			//mind we still mark the isDone flag to true even if the target didn't
			//have a key. Who are we to say biting a random chunk of water is bad?
			this->SetDone(true);
		}//end if
	}//end if

	//if the above wasn't true, then we need to begin/restart the attack
	else if(m_hFishBody.CheckCollsion(GetCommandPackTargetKey()) && this->m_hFishBody.BeginAttack())
	{
		//set our start time to now.
		this->m_attackStartTime = this->ElapsedTime();
	}//end else if	


	return true;
}

//See header for documentation.
void AttackAction::Read(std::istream& inStream)
{
	char buffer[512] = {0};

	inStream.getline(buffer, 512, '\n');
	sscanf(buffer, "Attack Timer: %.2f\n", &m_attackStartTime);
}

//See header for documentation.
void AttackAction::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_attackStartTime = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void AttackAction::Write(UDPNet::NetMessage& netMessage)
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_attackStartTime));
}




//See header for documentation.
EatAction::EatAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack), m_eatStartTime(0)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_EAT/* ||
		cmdPack.GetTargetFlag() == ActionTarget::ACTION_TARGET_FLAG_NONE*/) 
	{
		throw 0;
	}

	m_eatStartTime = this->ElapsedTime();


	//create a new target item.
	ActionTarget newTarg;

	//find the nearest piece of chum.
	m_hFishBody.GetNearestChum(newTarg.m_ActionTarget.m_point);

	//values will all be zero if we're unable to find a chunk of chum.
	if( newTarg.m_ActionTarget.m_point.x != 0 &&
		newTarg.m_ActionTarget.m_point.y != 0 &&
		newTarg.m_ActionTarget.m_point.z != 0   ) 
	{
		//set the flag
		newTarg.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_POINT;

		//set newTarg as the new target
		SetTarget(newTarg);

		//also set the eat timer.
	}//end if
	
	//if above was false then we're unable to find any chum.
	else
	{
		//complete the action.
		this->SetDone(true, false);
	}//end else



//	cout << "Eat " << buff << endl;
}

void EatAction::SetString(void) throw()
{

	//Set the StrObj data
	m_strObj.SetMouth_Action("Eat");
//	sprintf(buff, "Point (%.2f, %.2f, %.2f)",newTarg.m_ActionTarget.m_point.x,
//											 newTarg.m_ActionTarget.m_point.y,
//											 newTarg.m_ActionTarget.m_point.z);
	m_strObj.SetMouth_Target("N/A");
}

//See header for documentation.
bool EatAction::operator() (InternalVector_type& potenFieldVec)
{

			InternalPoint_type	chumPnt;
			GetTargetPoint(chumPnt);
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Eat Target (%.2f, %.2f, %.2f)\n", 
				this->GetCommandPackBodyID(), chumPnt.x, chumPnt.y, chumPnt.z);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	float distToTarget;

	
	//swim to the thing we're trying to eat
	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;

	//no matter what swim to the target.
	this->m_hFishBody.AddVector(potenFieldVec);


	//See if we've already begun the eating proceedure.
	if(this->m_hFishBody.IsEating()) 
	{
		//see if we can land the eat command now
		if(this->ElapsedTime() - this->m_eatStartTime >= EAT_ACTION_TIME_DELAY) 
		{
			//get the target from the command pack.
			ActionTarget actTarget;
			this->GetCommandPackTarget(actTarget);

			//before dealing damage to the target make sure it was a key'd target.
			if(actTarget.m_ActionFlag == actTarget.ACTION_TARGET_FLAG_POINT)
				this->m_hFishBody.EndEat(actTarget.m_ActionTarget.m_key);
			
			//same logic as attack command.
			this->SetDone(true);
		}//end if
	}//end if

	//if the above wasn't true, then we need to begin/restart the eatting. But we must
	//make sure we're close enough to the target before we begin.
	else if(FindDistToTarget() <= EAT_ACTION_MIN_DISTANCE && this->m_hFishBody.BeginEat())
	{
		//set our start time to now.
		this->m_eatStartTime = this->ElapsedTime();
	}//end else if	


	return true;
}

//See header for documentation.
void EatAction::Read(std::istream& inStream)
{
	char buffer[512] = {0};

	inStream.getline(buffer, 512, '\n');
	sscanf(buffer, "Eat Timer: %.2f\n", &m_eatStartTime);
}

//See header for documentation.
void EatAction::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_eatStartTime = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void EatAction::Write(UDPNet::NetMessage& netMessage) 
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_eatStartTime));
}




//See header for documentation.
MoveForwardAction::MoveForwardAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_MOVE_FORWARD) 
	{
		throw 0;
	}



}

void MoveForwardAction::SetString(void) throw()
{
	//Set the StrObj data
	m_strObj.SetMove_Action("Move Forward");
	buff[0] = 0;
	m_strObj.SetMove_Target(buff);
}

//See header for documentation.
bool MoveForwardAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Move Forward\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//Get the fish's current direction
	this->m_hFishBody.GetCurrentDirectionVector(potenFieldVec);

	//unitize that vector
	UnitizeVector<float>(potenFieldVec, potenFieldVec);

	//scale that vector by the wieght of the cmdPack
	ScaleVector<float>(potenFieldVec, potenFieldVec, this->GetCommandPackWeight());

	//add that vector to the summation
	this->m_hFishBody.AddVector(potenFieldVec);

	return true;
}

//See header for documentation.
StopFinAction::StopFinAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_STOP_FIN) 
	{
		throw 0;
	}

}

void StopFinAction::SetString(void) throw()
{
	//Set the StrObj data
	m_strObj.SetMove_Action("Stop Movement");
	buff[0] = 0;
	m_strObj.SetMove_Target(buff);
}


//See header for documentation.
bool StopFinAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Stop fin\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif
	//poison all other command's fin related actions.
	this->m_hFishBody.SetFinStopped(true);
	return true;
}

//See header for documentation.
StopMouthAction::StopMouthAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_STOP_MOUTH) 
	{
		throw 0;
	}

}

void StopMouthAction::SetString(void) throw()
{
	//Set the StrObj data
	m_strObj.SetMouth_Action("Stop Mouth");
	buff[0] = 0;
	m_strObj.SetMouth_Target(buff);
}

//See header for documentation.
bool StopMouthAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Stop Mouth\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif
	//poison all other command's mouth related actions.
	this->m_hFishBody.SetMouthStopped(true);
	return true;
}

RoamAction::RoamAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_ROAM) 
	{
		throw 0;
	}


}

void RoamAction::SetString(void) throw()
{
	InternalPoint_type pnt;
	GetCommandPacktTargetPoint(pnt);

	//Set the StrObj data
	m_strObj.SetMove_Action("Roam");
	sprintf(buff, "Point (%.2f, %.2f, %.2f)",pnt.x, pnt.y, pnt.z);
	m_strObj.SetMove_Target(buff);
}


//See header for documentation.
bool RoamAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, Roam\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	float distToTarget;
	
	//swim to the thing we're trying to attack.
	if(-1 == (distToTarget = this->GotoTarget(potenFieldVec)))
		return false;


	//unitize and scale vector
	UnitizeVector<float>(potenFieldVec, potenFieldVec);
	ScaleVector<float>(potenFieldVec, potenFieldVec, this->GetCommandPackWeight());


	InternalPoint_type pnt;
	this->GetTargetPoint(pnt);
			
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "Targ Pnt (%.2f, %.2f, %.2f) weight %.2f\n", pnt.x, pnt.y, pnt.z, this->GetCommandPackWeight());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
			sprintf(buff, "Roam dir (%.2f, %.2f, %.2f)\n", potenFieldVec.x, potenFieldVec.y, potenFieldVec.z);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	if(distToTarget <= ROAM_ACTION_MIN_DISTANCE) 
	{
		this->SetDone();
		return true;	
	}

	//no matter what swim to the target.
	this->m_hFishBody.AddVector(potenFieldVec);


	return true;
}




//See header for documentation.
EvaluateMarkersAction::EvaluateMarkersAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack), m_markerTimer(0)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_EVALUATE_MARKERS) 
	{
		throw 0;
	}

	//cheesy hack so I display how many marker there are.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);


	//Set the StrObj data
	m_strObj.SetMarker_Action("Evaluate All");
	sprintf(buff, "Count %d", elementCount);
	m_strObj.SetMarker_Target(buff);
}

//See header for documentation.
bool EvaluateMarkersAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, EvaluateMarkersAction\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif


	//check to see if we should init the timer, also check to see if we're done
	if(0 == m_markerTimer)
		m_markerTimer= ElapsedTime();

	else if(ElapsedTime()-m_markerTimer >= MARKER_ACTION_TIME) 
	{
		this->SetDone();
		return true;
	}



	////1.0 Get the marker array.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);

	//if we have zero elements, set command done and incomplete
	if(!elementCount)
	{
		SetDone(true, false);
		return true;
	}
	

	//sanity check
	if(!markerArr) 
		return false;


	////2.0 Sum vectors to all markers
	InternalVector_type markerSum;					//holds the sum of all relevent vectors.
	markerSum.x = markerSum.y = markerSum.z = 0;

	for(int i = 0 ; i < elementCount ; i++)
	{
		//compute the unit vector to that location.
		InternalVector_type temp;
		VectorToPoint(markerArr[i], temp);

		//unitize that vector.
		UnitizeVector<float>(temp, temp);

		//scale it by its weight.
		ScaleVector<float>(temp, temp, weightArr[i]);

		//add this vector to the marker sum
		markerSum.x += temp.x;
		markerSum.y += temp.y;
		markerSum.z += temp.z;
	}

	//check to see if we're done.
	if((fabs(markerSum.x) > 0 && fabs(markerSum.y) > 0 && fabs(markerSum.z) > 0) || ElapsedTime() >= 5.0f)
		this->SetDone();
	

#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "Eval_All (%.2f,%.2f,%.2f) elements, %d\n", 
				markerSum.x, markerSum.y, markerSum.z, elementCount);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//set the potenFieldVec as the sum vector and add it.
	potenFieldVec = markerSum;
	m_hFishBody.AddVector(potenFieldVec);
	return true;
}


//See header for documentation.
void EvaluateMarkersAction::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_markerTimer = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void EvaluateMarkersAction::Write(UDPNet::NetMessage& netMessage) 
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_markerTimer));
}





//See header for documentation.
EvaluateMarkersAction_Pos::EvaluateMarkersAction_Pos(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack), m_markerTimer(0)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_EVALUATE_POSITIVE) 
	{
		throw 0;
	}
	//cheesy hack so I display how many marker there are.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);


	//Set the StrObj data
	m_strObj.SetMarker_Action("Evaluate Positive");
	sprintf(buff, "Count %d", elementCount);
	m_strObj.SetMarker_Target(buff);
}

//See header for documentation.
bool EvaluateMarkersAction_Pos::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, EvaluateMarkersAction_Pos\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif


	//check to see if we should init the timer, also check to see if we're done
	if(0 == m_markerTimer)
		m_markerTimer= ElapsedTime();

	else if(ElapsedTime()-m_markerTimer >= MARKER_ACTION_TIME) 
	{
		this->SetDone();
		return true;
	}



	////1.0 Get the marker array.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);


	//if we have zero elements, set command done and incomplete
	if(!elementCount)
	{
		SetDone(true, false);
		return true;
	}

	//sanity check
	if(!markerArr) 
		return false;


	////2.0 Sum vectors to all positive markers
	InternalVector_type markerSum;					//holds the sum of all relevent vectors.
	markerSum.x = markerSum.y = markerSum.z = 0;
	bool isPosMarkers = false;						//will be set to true if we ever evaluate a marker.

	for(int i = 0 ; i < elementCount ; i++)
	{
		//check to see if weight is positive.
		if(0 < weightArr[i])
		{
			isPosMarkers = true;

			//compute the unit vector to that location.
			InternalVector_type temp;
			VectorToPoint(markerArr[i], temp);

			//unitize that vector.
			UnitizeVector<float>(temp, temp);

			//scale it by its weight.
			ScaleVector<float>(temp, temp, weightArr[i]);

			//add this vector to the marker sum
			markerSum.x += temp.x;
			markerSum.y += temp.y;
			markerSum.z += temp.z;
		}//end if
	}//end for
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "Eval_Pos (%.2f,%.2f,%.2f)\n", 
				markerSum.x, markerSum.y, markerSum.z);
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//if there are no markers set this done
	if(!isPosMarkers)
	{
		SetDone(true, false);
		return false;
	}
			
	//set the potenFieldVec as the sum vector and add it.
	potenFieldVec = markerSum;
	m_hFishBody.AddVector(potenFieldVec);
	return true;
}

//See header for documentation.
void EvaluateMarkersAction_Pos::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_markerTimer = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void EvaluateMarkersAction_Pos::Write(UDPNet::NetMessage& netMessage) 
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_markerTimer));
}




//See header for documentation.
EvaluateMarkersAction_Neg::EvaluateMarkersAction_Neg(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack), m_markerTimer(0)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_EVALUATE_NEGATIVE) 
	{
		throw 0;
	}

	//cheesy hack so I display how many marker there are.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);


	//Set the StrObj data
	m_strObj.SetMarker_Action("Evaluate Negative");
	sprintf(buff, "Count %d", elementCount);
	m_strObj.SetMarker_Target(buff);
}
//See header for documentation.
bool EvaluateMarkersAction_Neg::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, EvaluateMarkersAction_Neg\n", this->GetCommandPackBodyID());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//check to see if we should init the timer, also check to see if we're done
	if(0 == m_markerTimer)
		m_markerTimer= ElapsedTime();

	else if(ElapsedTime()-m_markerTimer >= MARKER_ACTION_TIME) 
	{
		this->SetDone();
		return true;
	}

	////1.0 Get the marker array.
	int					elementCount = 0;	//holds number of elements in marker array
	const float*		weightArr = NULL;	//filled with pointer to the weight array.
	const InterpPoint*	markerArr = NULL;	//filled with pointer to the marker array.

	markerArr = m_hFishBody.GetMarkerArray(elementCount, &weightArr);


	//if we have zero elements, set command done and incomplete
	if(!elementCount)
	{
		SetDone(true, false);
		return true;
	}

	//sanity check
	if(!markerArr) 
		return false;


	////2.0 Sum vectors to all positive markers
	InternalVector_type markerSum;					//holds the sum of all relevent vectors.
	markerSum.x = markerSum.y = markerSum.z = 0;
	bool isNegMarkers = false;						//will be set to true if we ever evaluate a marker.


	for(int i = 0 ; i < elementCount ; i++)
	{
		//check to see if weight is positive.
		if(0 > weightArr[i])
		{
			isNegMarkers = true;

			//compute the unit vector to that location.
			InternalVector_type temp;
			VectorToPoint(markerArr[i], temp);

			//unitize that vector.
			UnitizeVector<float>(temp, temp);

			//scale it by its weight.
			ScaleVector<float>(temp, temp, weightArr[i]);

			//add this vector to the marker sum
			markerSum.x += temp.x;
			markerSum.y += temp.y;
			markerSum.z += temp.z;
		}//end if
	}//end for

	//if there are no markers set this done
	if(!isNegMarkers)
	{
		SetDone(true, false);
		return false;
	}

	//set the potenFieldVec as the sum vector and add it.
	potenFieldVec = markerSum;
	m_hFishBody.AddVector(potenFieldVec);
	return true;
}

//See header for documentation.
void EvaluateMarkersAction_Neg::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_markerTimer = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void EvaluateMarkersAction_Neg::Write(UDPNet::NetMessage& netMessage)
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_markerTimer));
}






//See header for documentation.
DropMarkersAction::DropMarkersAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_DROP_MARKER) 
	{
		throw 0;
	}

	InternalPoint_type pnt;
	GetPos(pnt);



	//Set the StrObj data
	m_strObj.SetMarker_Action("Drop Marker");
	sprintf(buff, "Point (%.2f, %.2f, %.2f)",pnt.x, pnt.y, pnt.z);
	m_strObj.SetMarker_Target(buff);
}

//See header for documentation.
bool DropMarkersAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "***ID %d, Drop Marker\n", this->GetCommandPackBodyID());
//			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif

	//get the point referenced in the cmdPack. If we're given a false just
	// assume that drop marker was meant for here.
	InternalPoint_type targetPoint;

	if(!this->GetTargetPoint(targetPoint))
		m_hFishBody.GetPos(targetPoint);

	//drop the marker into the system
	this->m_hFishBody.AddMarker(targetPoint, this->GetCommandPackWeight());
	this->SetDone();
	return true;
}







//See header for documentation.
MoveWithAction::MoveWithAction(StrObj& strObj, HFishBody& hFishBody, CommandPacket& cmdPack)
			:Action(strObj, hFishBody, cmdPack), m_moveWithStartTime(0)
{
	//check for illegal cmdPacks.
	if(cmdPack.GetCommandID() != AQUARIUS_COMMAND_MOVE_WITH) 
	{
		throw 0;
	}

	//if this is true then we need to attack the nearest fish.
	if(0 == GetCommandPackTargetKey())
	{
		//Get the closest fish ID from the HFishBody
		AquariusKey_type nearFish = m_hFishBody.GetNearestFish();

		//if there is a near by fish, set that as the target,
		if(nearFish)
			SetCommandPackTarget(nearFish);

		//otherwise just ignore the action as there is nothing to attack.
		else
			this->SetDone(true, false);
	}//end if

}

void MoveWithAction::SetString(void) throw()
{
	//Set the StrObj data
	if(GetCommandPackTargetKey()) 
	{
		m_strObj.SetMove_Action("Move With");
		sprintf(buff, "Key %d", GetCommandPackTargetKey());
		m_strObj.SetMove_Target(buff);
	}
}

//See header for documentation.
bool MoveWithAction::operator() (InternalVector_type& potenFieldVec)
{
#ifdef	FISH_BODY_ACTIONS_LOG_TO_CONSOLE
			sprintf(buff, "ID %d, MoveWith %d,  timer: %.2f, elapsed: %.2f\n", 
				this->GetCommandPackBodyID(), this->GetCommandPackTargetKey(),
				m_moveWithStartTime, ElapsedTime());
			(Debugging::Debugger::Instance())->output(Debugging::BOD, Debugging::L4, buff);
#endif
	//check to see if we should init the timer, also check to see if we're done
	if(0 == m_moveWithStartTime)
		m_moveWithStartTime = ElapsedTime();

	else if(ElapsedTime()-m_moveWithStartTime >= MOVE_WITH_TIME) 
	{
		this->SetDone();
		return true;
	}
	

	//otherwise, plot a course for where this fish is going.
	try{
	
		//Get a pointer to the target fish.
		FishBody* pTargetFish = NULL;
		if(!(pTargetFish = m_hFishBody.GetFishBodyPtr(GetCommandPackTargetKey())))
			throw 0;

		//find its current direction and position.
		InterpVec_type		fishDir;
		InterpPnt_type		fishPos;
		
		pTargetFish->GetDirection(fishDir);
		pTargetFish->GetPos(fishPos);
		
		//add position and vector together, this'll be the target.
		InterpPnt_type		targetPoint = fishPos;
		targetPoint += fishDir;

		//compute a vector to this target, this'll be our direction.
		VectorToPoint(targetPoint, potenFieldVec);


		//unitize and scale this vector
		UnitizeVector<float>(potenFieldVec, potenFieldVec);
		ScaleVector<float>(potenFieldVec, potenFieldVec, this->GetCommandPackWeight());

		//add this vector in.
		m_hFishBody.AddVector(potenFieldVec);
		
	} catch(...)
	{
		//target was not found.
	}	 
	


	
	return true;
}

//See header for documentation.
void MoveWithAction::Read(UDPNet::NetMessage& netMessage)
{
	this->Action::Read(netMessage);
	m_moveWithStartTime = *reinterpret_cast<const float*>(netMessage.ReadNBytes(sizeof(float)));
}

//See header for documentation.
void MoveWithAction::Write(UDPNet::NetMessage& netMessage)
{
	this->Action::Write(netMessage);
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_moveWithStartTime));
}