#include "attributeUtils.hpp"
#include <assert.h>
#include <math.h>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace InterpretationModule;

//define the const's defined in the header.
const Attribute_type InterpretationModule::MAX_ATTRIBUTE_VALUE = 1.0f;
const Attribute_type InterpretationModule::MIN_ATTRIBUTE_VALUE = 0.0f;



//see header for documentation.
Attribute_type InterpretationModule::NormalizeAttributeValue(Attribute_type at)
{
	if(at >= MAX_ATTRIBUTE_VALUE) 
		return MAX_ATTRIBUTE_VALUE;

	else if(at < MIN_ATTRIBUTE_VALUE)
		return MIN_ATTRIBUTE_VALUE;

	return at;
}

//see header for documentation.
void InterpretationModule::GenerateAttributeTable(Attribute_type primarySeed, Attribute_type secondarySeed, 
							Attribute_type minVal, Attribute_type maxVal,
							std::vector<Attribute_type>& attribTable, int& levels)
{
	levels = 0;
	
	//after every interation of loop will alter value by the seeds passed
	//continues proccess until we've reached the maxValue.
	for(Attribute_type val = minVal; val < maxVal ; val = static_cast<float>(pow(val * primarySeed, secondarySeed)), levels++)
	{
		attribTable.push_back(val);
	}
}

//see header for documentation.
Attribute_type InterpretationModule::FindTableEntry(Attribute_type inputValue,  
							std::vector<Attribute_type> attribTable, int levels)
{
	//a one will mess up the index. hack a value for one
	if(inputValue >= 1.0f) 
		inputValue = 0.99999f;

	Attribute_type index = static_cast<Attribute_type>(levels) * inputValue;
	assert(index >= 0 && index <= levels);
	Attribute_type ans = attribTable[static_cast<int>(index)];
	return ans;
}
