////////////////////////////////////////////////////////////////////////////////
/*!	\file		attributes.cpp
**	\brief		Defines all the methods declared in attributes.hpp.
**	\author		Nate Cleveland
**	\date		10/16/2002 4:59:07 PM
**	\par		DESCRIPTION: Please see attributes.hpp for description.
**					
**	\par		REMARKS: Please see attributes.hpp for description.
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)
#include "attributes.hpp"
#include "attributeGenerator.hpp"
#include "../networking/NetData.h"
#include <fstream>
#include <stdio.h>
using namespace InterpretationModule;
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif



//See header for documentation
Attributes::Attributes(void) throw()
:m_metabolism(0.2f), m_armor(0.1f), m_eyes(0.5f),m_teeth_baleen(0.5f), m_size(0.2f)
{
	this->SetStomach_Normalized(0.5f);
	this->SetHealth_Normalized(1.0f);
	this->SetIncline(0);
	this->SetEnergyLevel_Normalized(0.5f);	
	
	AttributeGenerator::Instance().VerifyValues(*this);

	this->SetStomach_Normalized(0.5f);
	this->SetHealth_Normalized(1.0f);
	this->SetIncline(0);
	this->SetEnergyLevel_Normalized(0.5f);	
}//end Attributes::Attributes(void)

//See header for documentation
Attributes::Attributes(Attribute_type metabolism, Attribute_type armor, Attribute_type eyes,
					   Attribute_type teeth_baleen, Attribute_type size)
:m_metabolism(metabolism), m_armor(armor), m_eyes(eyes), 
m_teeth_baleen(teeth_baleen), m_size(size)
{
	this->SetStomach_Normalized(0.5f);
	this->SetHealth_Normalized(1.0f);
	this->SetEnergyLevel_Normalized(0.5f);
	this->SetInclineIncrement(0);

	AttributeGenerator::Instance().VerifyValues(*this);

	this->SetStomach_Normalized(0.5f);
	this->SetHealth_Normalized(1.0f);
	this->SetIncline(0);
	this->SetEnergyLevel_Normalized(0.5f);	
}//end Attributes::Attributes(Attribute_type ...)


//See header for documentation.
AttributeGenerator& Attributes::GetGenerator(void) throw()
{
	return AttributeGenerator::Instance();
}



//See header for documentation.
Attributes::~Attributes(void)  throw ()
{
	//nothing to do
	//...
}//end Attributes::~Attributes


//See header for documentation
void Attributes::VerifyValues(void) throw()
{
	AttributeGenerator::Instance().VerifyValues(*this);
}//end Attributes::VerifyValues


//See header for documentation
void Attributes::Update(float elapsedTime_seconds) throw()
{
	float elapsedTime_minutes = elapsedTime_seconds / 60.0f;
	AttributeGenerator::Instance().Update(*this, elapsedTime_minutes);
}//end Attributes::Update(float)

//See header for documentation
void Attributes::Write(std::ostream& outStream) const
{
	outStream << *this;
}

//See header for documentation
void Attributes::Read(std::istream& inStream)
{
	inStream  >> *this;
}

//See header for documentation
void Attributes::operator << (UDPNet::NetMessage& netMessage)
{
	////Base Attributes
	m_metabolism	= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_armor			= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_eyes			= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_teeth_baleen	= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_size			= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));

	////Derived Attributes
	//no need to send over stream, can deduce.


	////Reactive Attribute Modifier/Limiters.
	//no need to send over stream, can deduce.


	////Reactive Attributes
	m_energyLevel		= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_stomach			= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_health			= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
	m_matingInclination	= *reinterpret_cast<const Attribute_type*>(netMessage.ReadNBytes(sizeof(Attribute_type)));
}


void Attributes::operator >> (UDPNet::NetMessage& netMessage)
{
	////Base Attributes
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_metabolism));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_armor));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_eyes));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_teeth_baleen));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_size));


	////Derived Attributes
	//no need to send over stream, can deduce.


	////Reactive Attribute Modifier/Limiters.
	//no need to send over stream, can deduce.


	////Reactive Attributes
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_energyLevel));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_stomach));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_health));
	netMessage.WriteNBytes(sizeof(Attribute_type),  reinterpret_cast<char*>(&m_matingInclination));
}







using namespace std;

//See header for documentation
ostream& InterpretationModule::operator << (ostream &outStream, const InterpretationModule::Attributes &at)
{
	char buffer[512] = {0};		//buffer to store the data in before writing it to file.

	//fill the buffer with this fish's Base Attributes then stream it to file.
	sprintf(buffer, "Metabolism = %f\t Armor = %f\t Eyes = %f\t Teeth/Baleen = %f\t Size = %f\n",
					at.GetMetabolism(), at.GetArmor(), at.GetEyes(), at.GetTeeth_Baleen(), at.GetSize());

	outStream << buffer;

	//fill the buffer with this fish's Reactive Attributes then stream it to file.
	sprintf(buffer, "Health = %f\t Energy = %f\t Mating Inclination = %f\t Stomach = %f\n",
					at.GetHealth(), at.GetEnergyLevel(), at.GetIncline(), at.GetStomach());

	outStream << buffer;
	return outStream;
}

//See header for documentation
istream& InterpretationModule::operator>> (istream& inStream, Attributes &at)
{
	char buffer[512] = {0};		//buffer to store each line from the file.

	//read the first line of data and use sscanf to read it back into the class.
	inStream.getline(buffer, 512, '\n');
	sscanf(buffer, "Metabolism = %f\t Armor = %f\t Eyes = %f\t Teeth/Baleen = %f\t Size = %f\n",
					&at.m_metabolism, &at.m_armor, &at.m_eyes, &at.m_teeth_baleen, &at.m_size);

	//read in the next line and use sscanf to read it back into the class.
	memset(buffer, 0, 512);
	inStream.getline(buffer, 512, '\n');
	sscanf(buffer, "Health = %f\t Energy = %f\t Mating Inclination = %f\t Stomach = %f\n",
					&at.m_health, &at.m_energyLevel, &at.m_matingInclination, &at.m_stomach);

	//call verify to figure out the rest of the data and make sure this data is legal
	at.VerifyValues();
	return inStream;
}


Attribute_type Attributes::GetReactionTime(void) const throw()
{
	return m_reactionTime;
}




