#pragma warning(disable:4786)
#include "commandPacket.hpp"
#include "../networking/NetData.h"
#include "../Nexception/Nexception.h"
#include <memory.h>
#include <fstream>
#include <stdio.h>
using namespace InterpretationModule;
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif



//! target is an object
CommandPacket::CommandPacket(CommandID_type cmdID, AquariusKey_type senderID, AquariusKey_type bodyID,
							 float weight, unsigned int decisionID, float fitness, 
							 AquariusKey_type targetKey)throw()
:m_senderID(senderID), m_bodyID(bodyID), m_weight(weight), m_commandID(cmdID), 
m_decisionID(decisionID), m_fitness(fitness)
{
	this->m_target.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_KEY;
	this->m_target.m_ActionTarget.m_key = targetKey;
}

//! target is a point
CommandPacket::CommandPacket(CommandID_type cmdID, AquariusKey_type senderID, AquariusKey_type bodyID,
							 float weight, unsigned int decisionID, float fitness, 
							 const InternalPoint_type& point)throw()
:m_senderID(senderID), m_bodyID(bodyID), m_weight(weight), m_commandID(cmdID), 
m_decisionID(decisionID), m_fitness(fitness)
{
	this->m_target.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_POINT;
	this->m_target.m_ActionTarget.m_point = point;
}

//! target is a voxel
CommandPacket::CommandPacket(CommandID_type cmdID, AquariusKey_type senderID, AquariusKey_type bodyID,
							 float weight, unsigned int decisionID, float fitness, 
							 const Voxel_type& voxel)throw()
:m_senderID(senderID), m_bodyID(bodyID), m_weight(weight), m_commandID(cmdID), 
m_decisionID(decisionID), m_fitness(fitness)
{
	this->m_target.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_VOXEL;
	this->m_target.m_ActionTarget.m_voxel = voxel;
}

//! target must be infered from the command.
CommandPacket::CommandPacket(CommandID_type cmdID, AquariusKey_type senderID, AquariusKey_type bodyID,
							 float weight, unsigned int decisionID,float fitness)throw()
:m_senderID(senderID), m_bodyID(bodyID), m_weight(weight), m_commandID(cmdID), 
m_decisionID(decisionID), m_fitness(fitness)
{
	this->m_target.m_ActionFlag = ActionTarget::ACTION_TARGET_FLAG_NONE;
}

//See header for documentation
void CommandPacket::GetTarget(ActionTarget& actTarget)const throw()
{
	//switch between all possible keys and fill out the proper member of the union.
	switch(actTarget.m_ActionFlag = this->m_target.m_ActionFlag)
	{

	case ActionTarget::ACTION_TARGET_FLAG_VOXEL:
		actTarget.m_ActionTarget.m_voxel = this->m_target.m_ActionTarget.m_voxel;
		break;

	case ActionTarget::ACTION_TARGET_FLAG_KEY:
		actTarget.m_ActionTarget.m_key = this->m_target.m_ActionTarget.m_key;
		break;

	case ActionTarget::ACTION_TARGET_FLAG_POINT:
		actTarget.m_ActionTarget.m_point = this->m_target.m_ActionTarget.m_point;
		break;

	//special case or error in class. either way zero out the actTarget.
	case ActionTarget::ACTION_TARGET_FLAG_NONE:
	default:
		memset(&actTarget, 0, sizeof(actTarget));
		break;	
	}
}

//See header for documentation
AquariusKey_type CommandPacket::GetTargetKey(void) const
{
	//Make sure this is a key type attribute, throw if it isn't.
	if(ActionTarget::ACTION_TARGET_FLAG_KEY == this->m_target.m_ActionFlag) 
	{
		return this->m_target.m_ActionTarget.m_key;
	}

	throw 0;
}

//See header for documentation
void CommandPacket::GetTargetPoint(InternalPoint_type& p) const
{
	//Make sure this is a key type attribute, throw if it isn't.
	if(ActionTarget::ACTION_TARGET_FLAG_POINT == this->m_target.m_ActionFlag) 
	{
		p = this->m_target.m_ActionTarget.m_point;
		return ;
	}

	throw 0;
}

//See header for documentation
void CommandPacket::GetTargetVoxel(Voxel_type& v) const
{
	//Make sure this is a key type attribute, throw if it isn't.
	if(ActionTarget::ACTION_TARGET_FLAG_VOXEL == this->m_target.m_ActionFlag) 
	{
		v = this->m_target.m_ActionTarget.m_voxel;
		return ;
	}

	throw 0;
}


//See header for documentation
bool CommandPacket::operator != (const CommandPacket& cp) const throw()
{
	return !(*this == cp);
}

//See header for documentation
bool CommandPacket::operator == (const CommandPacket& cp) const throw()
{
	//if not using the roam command evaluate the ActionTarget member
	// removed the check for the body id. seems unneeded. [1/16/2003]
	if(this->m_commandID != AQUARIUS_COMMAND_ROAM) 
	{
		return	(m_senderID == cp.m_senderID) && /*(m_bodyID == cp.m_bodyID) &&*/
				(m_commandID == cp.m_commandID) && (m_target == cp.m_target);
	}

	return	(m_senderID == cp.m_senderID) && /*(m_bodyID == cp.m_bodyID) &&*/
			(m_commandID == cp.m_commandID);
}


//See header for documentation (read)
void CommandPacket::operator << (UDPNet::NetMessage& netMessage)
{
	const char *currChunk = NULL;	//temp point to store data location

	//command ID
	currChunk = netMessage.ReadNBytes(sizeof(CommandID_type));
	m_commandID = *reinterpret_cast<const CommandID_type*>(currChunk);

	//brain ID
	currChunk = netMessage.ReadNBytes(sizeof(AquariusKey_type));
	m_senderID = *reinterpret_cast<const AquariusKey_type*>(currChunk);

	//weight of command
	currChunk = netMessage.ReadNBytes(sizeof(float));
	m_weight = *reinterpret_cast<const float*>(currChunk);

	//target
	m_target << netMessage;
}

//See header for documentation (write)
void CommandPacket::operator >> (UDPNet::NetMessage& netMessage)
{
	//command ID
	netMessage.WriteNBytes(sizeof(CommandID_type), reinterpret_cast<char*>(&m_commandID));

	//brain ID
	netMessage.WriteNBytes(sizeof(AquariusKey_type), reinterpret_cast<char*>(&m_senderID));

	//weight of command
	netMessage.WriteNBytes(sizeof(float), reinterpret_cast<char*>(&m_weight));

	//target
	m_target >> netMessage;
}


//See header for documentation (read)
void ActionTarget::operator << (UDPNet::NetMessage& netMessage)
{
	const char *currChunk = NULL;	//temp point to store data location

	//flag
	currChunk = netMessage.ReadNBytes(sizeof(ActionTargetFlag));
	m_ActionFlag = *reinterpret_cast<const ActionTargetFlag*>(currChunk);

	//target
	switch(m_ActionFlag)
	{

	case ACTION_TARGET_FLAG_KEY:		//target is stored as a key.
		currChunk = netMessage.ReadNBytes(sizeof(AquariusKey_type));
		m_ActionTarget.m_key = *reinterpret_cast<const AquariusKey_type*>(currChunk);
		break;

	case ACTION_TARGET_FLAG_VOXEL:		//target is a voxel.
		currChunk = netMessage.ReadNBytes(sizeof(Voxel_type));
		m_ActionTarget.m_voxel = *reinterpret_cast<const Voxel_type*>(currChunk);
		break;

	case ACTION_TARGET_FLAG_POINT:		//target is a point.
		currChunk = netMessage.ReadNBytes(sizeof(InternalPoint_type));
		m_ActionTarget.m_point = *reinterpret_cast<const InternalPoint_type*>(currChunk);
		break;
	
	case ACTION_TARGET_FLAG_NONE:		//target is implied from command.	
	default:
		break;
	}	
}

//See header for documentation (write)
void ActionTarget::operator >> (UDPNet::NetMessage& netMessage)
{
	//flag
	netMessage.WriteNBytes(sizeof(ActionTargetFlag), reinterpret_cast<char*>(&m_ActionFlag));

	//target
	switch(m_ActionFlag)
	{
	case ACTION_TARGET_FLAG_NONE:		//target is implied from command.
		break;

	case ACTION_TARGET_FLAG_KEY:		//target is stored as a key.
		netMessage.WriteNBytes(sizeof(AquariusKey_type), 
							   reinterpret_cast<char*>(&m_ActionTarget.m_key));
		break;

	case ACTION_TARGET_FLAG_VOXEL:		//target is a voxel.
		netMessage.WriteNBytes(sizeof(Voxel_type), 
							   reinterpret_cast<char*>(&m_ActionTarget.m_voxel));
		break;

	case ACTION_TARGET_FLAG_POINT:		//target is a point.
		netMessage.WriteNBytes(sizeof(InternalPoint_type), 
							   reinterpret_cast<char*>(&m_ActionTarget.m_point));
		break;		

	default:
		throw Nexception("Fatal Error: Unknown CommandPak Target Flag coming from netStream.");
	}	
}


using namespace std;

//See header for documentation
ostream& InterpretationModule::operator << (ostream &outStream, const InterpretationModule::CommandPacket &cmdPack)
{
	char buffer[512] = {0};		//buffer to store the data in before writing it to file.

	//stream out a header
	outStream << "#Begin CommandPacket\n";

	//stream out everything but the target itself.
	sprintf(buffer, "CommandID == %d,\t BodyID == %d,\t SenderID == %d,\t Weight == %f.\t TargetFlag == %d\n",
			cmdPack.m_commandID, cmdPack.m_bodyID, cmdPack.m_senderID, cmdPack.m_weight, 
			cmdPack.m_target.m_ActionFlag);

	outStream << buffer;

	//stream out the target data itself.
	switch(cmdPack.m_target.m_ActionFlag)
	{
	case ActionTarget::ACTION_TARGET_FLAG_KEY:
		sprintf(buffer, "Target Key == %d\n", cmdPack.m_target.m_ActionTarget.m_key);
		break;

	case ActionTarget::ACTION_TARGET_FLAG_POINT:
		sprintf(buffer, "Target Point == (%f, %f, %f)\n", cmdPack.m_target.m_ActionTarget.m_point.x,
														  cmdPack.m_target.m_ActionTarget.m_point.y,
														  cmdPack.m_target.m_ActionTarget.m_point.z);
		break;

	case ActionTarget::ACTION_TARGET_FLAG_VOXEL:
	default:
		sprintf(buffer, "Either a currently unsuppoted target or the target must be inferred by the command.\n");
	}//end case

	outStream << buffer;


	//stream out the footer.
	outStream << "#End CommandPacket\n";
	
	return outStream;
}

//See header for documentation
istream& InterpretationModule::operator>> (istream& inStream, CommandPacket &cmdPack)
{
	char buffer[512] = {0};		//buffer to store each line from the file.
	
	//stream in the header
	inStream.getline(buffer, 512, '\n');

	//Stream in the first line of data
	inStream.getline(buffer, 512, '\n');
	sscanf(buffer, "CommandID == %d,\t BodyID == %d,\t SenderID == %d,\t Weight == %f.\t TargetFlag == %d\n",
			&cmdPack.m_commandID, &cmdPack.m_bodyID, &cmdPack.m_senderID, &cmdPack.m_weight, 
			&cmdPack.m_target.m_ActionFlag);


	//stream in the actual target.
	inStream.getline(buffer, 512, '\n');
	switch(cmdPack.m_target.m_ActionFlag)
	{
	case ActionTarget::ACTION_TARGET_FLAG_KEY:
		sscanf(buffer, "Target Key == %d\n", &cmdPack.m_target.m_ActionTarget.m_key);
		break;

	case ActionTarget::ACTION_TARGET_FLAG_POINT:
		sscanf(buffer, "Target Point == (%f, %f, %f)\n", &cmdPack.m_target.m_ActionTarget.m_point.x,
														 &cmdPack.m_target.m_ActionTarget.m_point.y,
														 &cmdPack.m_target.m_ActionTarget.m_point.z);
		break;

	case ActionTarget::ACTION_TARGET_FLAG_VOXEL:
	default:
		break;
		
	}//end case	

	//stream in the footer.
	inStream.getline(buffer, 512, '\n');	
	return inStream;
}


