#pragma warning(disable:4786)
#include <stdlib.h>
#include <math.h>
#include "fishBodyHandle.hpp"
#include "action.hpp"
#include "../tankSimulator/tankSimulator.h"
#include <iostream>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace InterpretationModule;



//see header for documentation
bool HFishBody::GetFishBody(AquariusKey_type key, FishBody& fishBody) const throw()
{
	return m_fishBody.GetFishBody(key, fishBody);
}

FishBody* HFishBody::GetFishBodyPtr(AquariusKey_type key) throw(){return m_fishBody.GetFishBodyPtr(key);}
bool HFishBody::IsAttacking(void)const throw(){return this->m_fishBody.m_isAttacking;}
bool HFishBody::IsEating(void)const throw(){return this->m_fishBody.m_isEating;}
bool HFishBody::IsMouthStopped(void)const throw(){return this->m_fishBody.m_IsMouthStopped;}
void HFishBody::AddMarker(const InternalPoint_type& newMarker, float weight) throw() {this->m_fishBody.AddMarker(newMarker, weight);}
const InternalPoint_type *HFishBody::GetMarkerArray(int& elementCount, const float **weightArr) const throw()
{
	elementCount = this->m_fishBody.m_markerArrSize;
	*weightArr = this->m_fishBody.m_markerArrWeight;
	return this->m_fishBody.m_markerArr;
}

bool HFishBody::GetCurrentDirectionVector(InternalVector_type& vec) const throw()
		{this->m_fishBody.GetCurrVector(vec); return true;}

//see header for documentation
void HFishBody::Flush() throw()
{
	this->m_vectorSummation.x = m_vectorSummation.y = m_vectorSummation.z = 0.0f;
}

//see header for documentation
void HFishBody::SetMouthStopped(bool isMouthStopped /* = true */)
{
	if(isMouthStopped)
	{	
		this->m_fishBody.m_IsMouthStopped = true;
		this->m_fishBody.m_isEating = this->m_fishBody.m_isAttacking = false;
	}

	else
		this->m_fishBody.m_IsMouthStopped = false;
}

//see header for documentation
void HFishBody::SetFinStopped(bool isFinStopped /* = true */)
{
	if(isFinStopped)
		this->m_fishBody.m_IsFinStopped = true;

	else
		this->m_fishBody.m_IsFinStopped = false;
}

//see header for documentation
void HFishBody::GetSummationVector(InternalVector_type& vec) const throw()
{
///	if(m_fishBody.m_IsFinStopped == false)
		vec = this->m_vectorSummation;

//	else
//		vec.x = vec.y = vec.z = 0.0f;
}

//see header for documentation
void HFishBody::AddVector(const InternalVector_type& newField) throw()
{
	//only bother adding if the fin isn't stopped.
	if(false == this->m_fishBody.m_IsFinStopped) 
	{
		this->m_vectorSummation.x += newField.x;
		this->m_vectorSummation.y += newField.y;
		this->m_vectorSummation.z += newField.z;		
	}//end if
}//end HFishBody::AddVector


//see header for documentation
void HFishBody::GetPos(InternalPoint_type& point) const throw()
{
	point.x = m_fishBody.m_phyData.GetXPos();
	point.y = m_fishBody.m_phyData.GetYPos();
	point.z = m_fishBody.m_phyData.GetZPos();	
}

//see header for documentation
bool HFishBody::GetPos(AquariusKey_type key, InternalPoint_type& point) const throw()
{
	FishBody* pFishBody = NULL;
	if(!(pFishBody = this->m_fishBody.m_pTankClient->GetFishBodyPtr(key)))
		return false;

	InterpPnt_type pos;
	pFishBody->GetPos(pos);

	point.x = pos[0];
	point.y = pos[1];
	point.z = pos[2];

	return true;
}


//see header for documentation
bool HFishBody::EndAttack(AquariusKey_type key)
{
	if(this->IsMouthStopped())
		return false;
	

	//check to make sure this is a fish.
	if(!m_fishBody.IsFishKey(key))
		return false;
	

	//Find the object we're attacking
	FishBody* attackieFish = m_fishBody.GetFishBodyPtr(key);
	if(!attackieFish)
		return false;

	//make sure this fish is in range
	InterpPnt_type a;
	InterpPnt_type b;
	attackieFish->GetPos(a);
	m_fishBody.GetPos(b);
	
	float dist = ComputeDistance<float>(a,b);

	if(!CheckCollsion(key)) 
	{
		return false;
	}


	//store the old stomach size
	float oldStomachNorm = m_fishBody.m_attributes.GetStomach_Normalized();
	
	
	//Deal damage to the target
	const AttributeGenerator& ag = m_fishBody.m_attributes.GetGenerator();
	float damageDealt = ag.DamageDealt(m_fishBody.m_attributes.GetAttackStrength(), 
									   attackieFish->m_attributes.GetArmor(),
									   m_fishBody.m_attributes.GetMaxHealth());

	//update Health
	float oldHealthNorm = attackieFish->m_attributes.GetHealth_Normalized();
	float oldHealth = attackieFish->m_attributes.GetHealth();
	float newHealth = __max(oldHealth - damageDealt,0);
	attackieFish->m_attributes.SetHealth(newHealth);

	//update the attackie fish's energy level
	AttributeGenerator& atG = attackieFish->m_attributes.GetGenerator();
	attackieFish->m_attributes.SetEnergyLevel(attackieFish->m_attributes.GetEnergyLevel() - atG.DefendingEnergyLoss());

	//upated the attacking fish's energy level.
	float attackEnergyLoss = atG.AttackingEnergyLoss(m_fishBody.m_attributes.GetSize());
	m_fishBody.m_attributes.SetEnergyLevel(m_fishBody.m_attributes.GetEnergyLevel() - attackEnergyLoss);
	
	//update all stats.
	attackieFish->m_stats.IncBeen_Attacked(1);
	this->m_fishBody.m_stats.IncChum(damageDealt);
	this->m_fishBody.m_stats.IncHas_Attacked(1);

	//check to see if we need to credit the attacking fish with a kill. also play death sound.
	if(attackieFish->m_attributes.GetHealth() <= 0)
	{
		//award the fish for the kill
		if(oldHealth > 0) 
		{
			m_fishBody.m_stats.IncKills(1);
			attackieFish->Death();
		}


		//decrement the attackie fish's size, if the other fish's stomach isn't full
		else if(oldStomachNorm < 1) 
		{
			float sizeLost = damageDealt / attackieFish->m_attributes.GetMaxStomachSize()*2;
			attackieFish->m_attributes.SetSize(__max(attackieFish->m_attributes.GetSize() - sizeLost, 0));
		}//end else
	}
	

	//Generate Chum from the damage delt.
///	attackieFish->AddChum(damageDealt);

	//Eat chum in the ammount of damage we dealt
	attackieFish->m_attributes.SetStomach(damageDealt + attackieFish->m_attributes.GetStomach());
	if(attackieFish->m_attributes.GetStomach_Normalized() > 1)
		attackieFish->m_attributes.SetStomach_Normalized(1);

	// Added by Nate Cleveland [4/10/2003]
	// Dump chum should either deal
//	damageDealt = __min(1, oldHealthNorm);

	//or
	damageDealt = damageDealt / (m_fishBody.m_attributes.GetGenerator().FindStomachSize(1)*2);

	//I can't quite remember. if the second is too big, use the first.
	
	float chumPercentage = damageDealt / (m_fishBody.m_attributes.GetGenerator().FindStomachSize(0.4f)*2);
	chumPercentage *= 500;
	chumPercentage = __min(1, chumPercentage);
///	cout << chumPercentage << endl;
	m_fishBody.m_pTankClient->DumpChum(a, chumPercentage);

	//reset state flags.
	this->m_fishBody.m_isAttacking = false;
	attackieFish->Hit();
	
	//play the bite sound too.
	m_fishBody.Bite();


	//reset the logic flags
	m_fishBody.m_isAttacking = false;
	return true;
}

//see header for documentation
bool HFishBody::BeginAttack(void)
{
	// The first condition of this test has been worked around
	// and is no longer even needed [3/18/2003]
	if(/*!m_fishBody.m_isEating && */!this->m_fishBody.m_IsMouthStopped) 
	{
		m_fishBody.m_isAttacking = true;
		m_fishBody.m_graphicClient.BeginBite();
	}
	else
	{
		m_fishBody.m_isAttacking = false;
//		cout << "Sorry, we are already eating." << endl;
	}

	return m_fishBody.m_isAttacking;
}


//see header for documentation
bool HFishBody::BeginEat(void) throw()
{
	// The first condition of this test has been worked around
	// and is no longer even needed [3/18/2003]
	if(/*!m_fishBody.m_isAttacking && */!this->m_fishBody.m_IsMouthStopped) 
	{
		m_fishBody.m_isEating = true;
		m_fishBody.m_graphicClient.BeginBite();
	}
	else
	{
//		cout << "Sorry, we are already attacking." << endl;
		m_fishBody.m_isEating = false;
	}
	return m_fishBody.m_isEating;
}


//see header for documentation
bool HFishBody::EndEat(AquariusKey_type key) throw()
{
	if(!m_fishBody.m_pTankClient)
		return false;
	

	//sanity check
	if(this->IsMouthStopped() || !this->IsEating())
		return false;


	////find our distance to the nearest chunk, make sure it is 
	//// close enough.
	//get our location and that of the chum.
	InterpPnt_type	pos, chumPos;
	m_fishBody.GetPos(pos);
	m_fishBody.m_pTankClient->GetClosestChumPosition(pos, chumPos);

	//find the distance between the two points.
	float dist = ComputeDistance<float>(pos, chumPos);

	//return out if the distance is too far.
	if(dist > EAT_ACTION_MIN_DISTANCE) 
		return false;

	//consume the chum.
	float chumConsumed = m_fishBody.m_pTankClient->EatChum(chumPos, m_fishBody.m_pAttributes->GetMaxStomachSize() 
														   - m_fishBody.m_pAttributes->GetStomach());

	//award the fish the food it ate
	Attributes	tempAtts;
	m_fishBody.GetAttributesCopy(tempAtts);
	tempAtts.SetStomach(tempAtts.GetStomach() + chumConsumed);
	m_fishBody.SetAttributes(tempAtts);

	//play the bite sound too.
	m_fishBody.Bite();
	
	//reset the logic flags
	m_fishBody.m_isEating = false;
	return true;
}

//see header for documentation.
void HFishBody::GetNearestChum(InternalPoint_type& chumPoint)  const throw()
{
	chumPoint.x = chumPoint.y = chumPoint.z = 0;

	if(m_fishBody.m_pTankClient)
	{
		//first, get our position
		InterpPnt_type pos, chumPos;
		m_fishBody.GetPos(pos);
		
		
		//use that position to find the nearest chunk of chum.
		if(!m_fishBody.m_pTankClient->GetClosestChumPosition(pos, chumPos))
		{
///			cout << "No Chum was found..." << endl;
			return;
		}
		
		//translate that into the internalPoint_type format iff we found one.
		chumPoint.x = chumPos[0];
		chumPoint.y = chumPos[1];
		chumPoint.z = chumPos[2];
	}//end if
}

//see header for documentation.
AquariusKey_type HFishBody::GetNearestFish(void)
{
	//check pointer before dereferencing it.
	if(m_fishBody.m_pTankClient) 
	{
		//call the tank simulator client method, return its value.
		return m_fishBody.m_pTankClient->GetKeyForClosestFish(m_fishBody.m_key);
	}

	//if the fish body doesn't have a tank client yet, return 0.
	return 0;
}

//see header for documentation.
bool HFishBody::CheckCollsion(AquariusKey_type key) const throw()
{
	//get a pointer to the other fish
	FishBody* otherFish = m_fishBody.GetFishBodyPtr(key);

	//sanity check
	if(otherFish)
	{
		//get the position of both fish
		InterpPnt_type pntA, pntB;
		otherFish->GetPos(pntB);
		m_fishBody.GetPos(pntA);

		return SphereCollision(pntA, m_fishBody.m_graphicClient.GetRadius(), 
							   pntB, otherFish->m_graphicClient.GetRadius(), 2);

	}
	
	return false;
}

//see header for documentation.
void HFishBody::ClipPoint(InternalPoint_type& pnt) throw()
{
	InterpPnt_type highCords, lowCords;

	if(m_fishBody.m_pTankClient) 
	{
		m_fishBody.m_pTankClient->GetBoundary(lowCords, highCords);


		if(pnt.x < lowCords[0])
			pnt.x = lowCords[0];
		else if(pnt.x > highCords[0])
			pnt.x = highCords[0];


		if(pnt.y < lowCords[1])
			pnt.y = lowCords[1];
		else if(pnt.y > highCords[1])
			pnt.y = highCords[1];


		if(pnt.z < lowCords[2])
			pnt.z = lowCords[2];
		else if(pnt.z > highCords[2])
			pnt.z = highCords[2];
	}
	
}


float HFishBody::GetRadius(void)
{
	return BASE_RADIUS*__max(.09f, m_fishBody.m_attributes.GetSize());
}
