////////////////////////////////////////////////////////////////////////////////
/*!	\file		fishBodyUtils.hpp
**	\brief		Provides some support for the FishBody and its nested classes.
**	\author		Nate Cleveland
**	\date		10/28/2002 9:50:05 AM
**	\par		DESCRIPTION:
**				 Defines the following common elements,
**					- Vector and Point type used by the fishBody.
**					- Datatype for a voxel.
**					- template functions to scale, unitize and compute the
**					magnitude of a vector.
**
**	\par		REMARKS:
**
*/
////////////////////////////////////////////////////////////////////////////////

#include "../math/math.h"
#include "../messageHandler/messageHandler.h"
#include <stdlib.h>
#pragma once
//! Remove this to cease all logging by all FishBody methods except Stats and Attributes.
#define FISH_BODY_LOG

namespace InterpretationModule
{
	//! Typedef for the keys used in this game.
	typedef MsgHandler::Key AquariusKey_type;

	//! Typedef for the vector that is used by the interpretationModule.
	typedef math::linear_alg::Vector3<float> InterpVec_type;

	//! Typedef for the point that is used by the interpretation Module
	typedef math::linear_alg::Vector3<float> InterpPnt_type;


	//! The type of point used by the interpretation module.
	typedef struct InterpPoint{float x,y,z;}InternalPoint_type;


	//! Typedef for the voxels used in the game. At the time of this
	//!comment Voxels have yet to be defined so they are set to int.
	typedef double Voxel_type;

	//! The type of vector used by the interpretation module.
	typedef InternalPoint_type InternalVector_type;

	//! Type for all angles used in interpretation module.
	typedef float InterpAngle_type;

	const InterpAngle_type pi = 3.1415926f;


	//! Gives the caller a random value from 0 to .9999f.
	inline float RandFloat(void)
	{
		int val = rand()%10000;
		return static_cast<float>(val) / 10000.0f;
	}	
}//end namespace InterpretationModule.