#include "frozenFish.hpp"
#include "fishBody.hpp"
#include "../ai_core/AI_System.h"
#include "../tankSimulator/TankSimulator.h"
using namespace InterpretationModule;
using namespace std;


/*
		Attributes m_attributes;		//!<Speaks for itself I think.
		Statistics m_stats;

		//commands
		std::vector<CommandPacket>	m_commands;	//!<Commands yet to be acted upon

		//examples
		std::vector<AI_System::AI_ActionTag> m_examples;
		
		//actions
		std::vector<InterpretationModule::Action>	m_actions;
		
		//state flags
		bool			m_IsMouthStopped;	//!< If true will not allow attacking/eating.
		bool			m_isAttacking;		//!< Set to true if the fish is attacking.
		bool			m_isEating;			//!< Set to true if the fish is eating.
		bool			m_IsFinStopped;		//!< If true the fish may not move itself.
		bool			m_IsLearn;			//!< If set to false fish does not store examples.
		bool			m_IsPathing;		//!< If true the fish will try to pathfind


		//physical stuff
		InterpVec_type	m_currVector;		//!< Current vector fish is moving along.
		InterpVec_type	m_facingVector;		//!< If our direction is zero then we store the facing vector here.

		//sphere data
		InterpVec_type	m_accel;
		InterpVec_type  m_pos;
		InterpVec_type  m_vel;
*/



FrozenFish::FrozenFish(const FishBody& fishBody) throw()
{
	///stats and atts
	m_attributes		= fishBody.m_attributes;
	m_stats				= fishBody.m_stats;
	
	m_key				= fishBody.m_key;

	//state flags
	m_isMouthStopped	= fishBody.m_IsMouthStopped;
	m_isAttacking		= fishBody.m_isAttacking;
	m_isEating			= fishBody.m_isEating;
	m_isFinStopped		= fishBody.m_IsFinStopped;
	m_isLearn			= fishBody.m_IsLearn;
	m_isPathing			= fishBody.m_IsPathing;

	//physical stuff
	m_currVector[0]		= fishBody.m_currVector.x;
	m_currVector[1]		= fishBody.m_currVector.y;
	m_currVector[2]		= fishBody.m_currVector.z;

	m_facingVector[0]	= fishBody.m_facingVector[0];
	m_facingVector[1]	= fishBody.m_facingVector[1];
	m_facingVector[2]	= fishBody.m_facingVector[2];
	

	//sphere data, leave zero if the tank client is null
	if(fishBody.m_pTankClient) 
	{
		InterpVec_type tempVec;

		PhysicsSimulator& phySim		 = fishBody.m_pTankClient->GetPhysicsSimulatorReference();
		PhysicsSimulator::Sphere& sphere = phySim.GetSphereRef(fishBody.m_key);

		m_accel	= sphere.GetAcceleration();
		m_pos	= sphere.GetPosition();
		m_vel	= sphere.GetVelocity();
	}


	//examples
	m_examples = fishBody.m_examples;

	//actions
	//m_actions = fishBody.m_actList;
	std::list<Action*>::const_iterator		actListIT = fishBody.m_actList.begin();
	int size = fishBody.m_actList.size();
	for( ;actListIT != fishBody.m_actList.end() ; actListIT++)
	{
		Action* currAct = *actListIT;
		PsuedoAction newAct;

		newAct.m_bodyID		= currAct->m_cmdPack.GetBodyID();
		newAct.m_cmdID		= currAct->m_cmdPack.GetCommandID();
		newAct.m_senderID	= currAct->m_cmdPack.GetSenderID();
		newAct.m_targetFlag	= currAct->m_cmdPack.GetTargetFlag();
		newAct.m_weight		= currAct->m_cmdPack.GetWeight();
		newAct.m_decisionID	= currAct->m_cmdPack.GetDecisionID();	
		newAct.m_fitness	= currAct->m_cmdPack.GetFitness();		
		

		//set target, key
		if(newAct.m_targetFlag == 1)
		{
			newAct.m_target.m_key = currAct->m_cmdPack.GetTargetFlag();
		}

		//point
		else if(newAct.m_targetFlag == 3)	
		{
			currAct->m_cmdPack.GetTargetPoint(newAct.m_target.m_point);
		}


		//write optional data
		newAct.m_optional = currAct->GiveOptional();


		newAct.m_isDone			= currAct->m_isDone;		
		newAct.m_isComplete		= currAct->m_isComplete;	
		newAct.m_elapsedTime	= currAct->m_elapsedTime;
		m_actions.push_back(newAct);
	}
	

	//commands
	m_commands = fishBody.m_incomingCommands;
}