////////////////////////////////////////////////////////////////////////////////
/*!	\file		frozenFish.hpp
**	\brief		Declares the functionality needed to create a template of a fish's
				 physical charactistics.
**	\author		Nate Cleveland
**	\date		1/16/2003 9:23:21 AM
**	\par		DESCRIPTION:
					Currently preserves the following data
						- Attributes.
**					
**	\par		REMARKS:
					In order for this to fulfill the qualifications of the GDD the
					 brain manager must also save data of the fish.
**					
*/
////////////////////////////////////////////////////////////////////////////////

#include "attributes.hpp"
#include "commandPacket.hpp"
#include "action.hpp"
#include "stats.hpp"
#include <vector>
#include <list>
#pragma once

namespace AI_System 
{
	struct AI_ActionTag;
}


namespace InterpretationModule
{
	class FishBody;
	class Action;

	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		FrozenFish
	**	\brief		Holds all physical information of a fish for use in the fish side
					 bar.
	**	\author		Nate Cleveland
	**	\date		1/16/2003 9:15:56 AM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	struct FrozenFish
	{
		struct PsuedoAction
		{
			unsigned int	m_decisionID;	
			float			m_fitness;	
			unsigned int	m_cmdID;
			unsigned int	m_bodyID;
			unsigned int	m_senderID;
			float			m_weight;
			bool			m_isDone;		//are we done trying?
			bool			m_isComplete;	//did we complete the action succesfully?
			float			m_elapsedTime;	





			int				m_targetFlag;	//0 means no target, 1 means m_key, 3 means
											// m_point. 2 is for depricated functionality
											
			union Target
			{
				unsigned int	m_key;			//target is a key
				InterpPoint		m_point;		//target is a location. InterpPoint is a 
												// point type only used by the Interpretation
												// module. Instead of the typical [] indexing
												// one must use Win32 style .x indexing.
			}m_target;


			float			m_optional;		//some actions need this some don't. Always 
											//specific to the action.
		};
		
		Attributes									m_attributes;		
		Statistics									m_stats;
		AquariusKey_type							m_key;
		std::vector<CommandPacket>					m_commands;	//!<Commands yet to be acted upon
		std::vector<AI_System::AI_ActionTag>		m_examples;
		std::vector<PsuedoAction>					m_actions;
		
		//state flags
		bool			m_isMouthStopped;	//!< If true will not allow attacking/eating.
		bool			m_isAttacking;		//!< Set to true if the fish is attacking.
		bool			m_isEating;			//!< Set to true if the fish is eating.
		bool			m_isFinStopped;		//!< If true the fish may not move itself.
		bool			m_isLearn;			//!< If set to false fish does not store examples.
		bool			m_isPathing;		//!< If true the fish will try to pathfind


		//physical stuff
		InterpVec_type	m_currVector;		//!< Current vector fish is moving along.
		InterpVec_type	m_facingVector;		//!< If our direction is zero then we store the facing vector here.

		//sphere data
		InterpVec_type	m_accel;
		InterpVec_type  m_pos;
		InterpVec_type  m_vel;

		friend class FishBody;

		FrozenFish(void) throw() {}
		FrozenFish(const FishBody& fishBody) throw();

	};
}