#pragma warning(disable:4786)
#include "mutate.hpp"
#include <math.h>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace InterpretationModule;


//see header for documentation.
Mutate::Mutate(const FishBody& mother, const FishBody& father)
:m_mother(NULL), m_father(NULL)
{
	//read in both the mother and the father's attributes.
	m_mother = mother.m_pAttributes;
	m_father = father.m_pAttributes;

}

//see header for documentation.
Mutate::Mutate(const FishBody& mother)
:m_mother(NULL), m_father(NULL)
{
	//read in just the mother's attributes.
	m_mother = 	mother.m_pAttributes;
}

//see header for documentation.
Mutate::Mutate()
:m_mother(NULL), m_father(NULL)
{

}


//see header for documentation.
Attribute_type Mutate::FuseAttributes(Attribute_type a/* = 0*/, Attribute_type b/* = 0*/)
{
	//if we don't have a parent for any one stat, compute a random value
	if(!m_mother) 
		a = RandFloat();
	if(!m_father)
		b = RandFloat();
	
	//randomly choose one of the three possiblities
	switch(rand() % 3)
	{
	//average two parents.
	case 2:
		
		//both or one of these values may be randomized. this will not harm the logic.
		return (a + b) /2;

	//just one parent's value.
	case 1:
		
		//chose father iff we have one, otherwise select mother, which may be a random
		// value. This isn't a big deal since a random value is the only option we have left.
		if(m_father && rand()%2)
			return b;
		return a;

	//random value.
	case 0:
	default:
		return RandFloat();	
	}//end switch
}


//see header for documentation.
void Mutate::operator()(FishBody& child)
{
	//create an object to hold the new attributes for the child.
	Attributes newAtts;
	float		a=0,b=0;


	///////////////Metabolism
	if(m_mother)								//check for a mother fish
		a = m_mother->GetMetabolism();			//if we have a mother fish, record its meta
	if(m_father)								//check for a father fish
		b = m_father->GetMetabolism();			//if we have a fatther, record its meta

	newAtts.SetMetabolism(FuseAttributes(a,b));	//Fuse parent's metabolism into child.


	///////////////Armor
	if(m_mother)
		a = m_mother->GetArmor();
	if(m_father)
		b = m_father->GetArmor();

	newAtts.SetArmor(FuseAttributes(a,b));


	///////////////Eyes
	if(m_mother)
		a = m_mother->GetEyes();
	if(m_father)
		b = m_father->GetEyes();

	newAtts.SetEyes(FuseAttributes(a,b));


	///////////////Teeth_Baleen
	if(m_mother)
		a = m_mother->GetTeeth_Baleen();
	if(m_father)
		b = m_father->GetTeeth_Baleen();

	newAtts.SetTeeth_Baleen(FuseAttributes(a,b));


	///////////////Size
	if(m_mother)
		a = m_mother->GetSize();
	if(m_father)
		b = m_father->GetSize();

	newAtts.SetSize(FuseAttributes(a,b));
	
	//fight the singularity fish.
	newAtts.SetSize(__max(newAtts.GetSize(), .09f));

	//set the reactive atts logically
	newAtts.VerifyValues();
	newAtts.SetHealth_Normalized(1);
	newAtts.SetEnergyLevel_Normalized(1);
	newAtts.SetStomach_Normalized(1);
	newAtts.SetIncline(0);



	//Now, give the child its the attributes created from its parents..
	child.SetAttributes(newAtts);
	

}



