////////////////////////////////////////////////////////////////////////////////
/*!	\file		fishBody.hpp
**	\brief		Defines all the functionality for the mutate class.
**	\author		Nate Cleveland
**	\date		1/13/2003 12:51:20 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
#include "fishBody.hpp"

namespace InterpretationModule
{
	class Mutate
	{
		Attributes const*	m_mother;		//pntr to the mother of the fish (optional).
		Attributes const*	m_father;		//pntr to the father of the fish (optional).

	protected:	

		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Creates a fusion between the two passed in stats.
		**	\param		
		**	\return		none
		**	\exception	none
		**	\par		DESCRIPTION:
		**					Can produce one of the three possible stats fussions.
								- Average between parents two attributes.
								- Totally random
								- Just return one parent's value.
		**	\author		Nate Cleveland
		**	\date		1/14/2003 12:11:37 PM
		**	\warning	none.
		*/
		////////////////////////////////////////////////////////////////////////////////
		virtual Attribute_type FuseAttributes(Attribute_type a = 0, Attribute_type b = 0);
		

	public:
		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		ctor that allows creation of new atts. from two parents.
		**	\param		mother		Reference to one parent.
		**	\param		father		Reference to another parent.
		**	\exception	none
		**	\par		DESCRIPTION:
		**					
		**	\author		Nate Cleveland
		**	\date		1/14/2003 12:07:03 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		Mutate(const FishBody& mother, const FishBody& father) throw();

		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		ctor that allows creation of new atts. from one parent.
		**	\param		mother		Reference to another fish to base stats upon.
		**	\exception	none
		**	\par		DESCRIPTION:
		**					
		**	\author		Nate Cleveland
		**	\date		1/14/2003 12:07:03 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		Mutate(const FishBody& mother) throw();

		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		ctor that allows creation of new atts. from zero parents.
		**	\param		mother		Reference to another fish to base stats upon.
		**	\exception	none
		**	\par		DESCRIPTION:
		**					
		**	\author		Nate Cleveland
		**	\date		1/14/2003 12:07:03 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		Mutate() throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Creates stats for a fish base on the parents specified in ctor.
		**	\param		child		A reference to the child to generator the stats for.
		**	\return		none
		**	\exception	none
		**	\par		DESCRIPTION:
		**					
		**	\author		Nate Cleveland
		**	\date		1/14/2003 12:10:17 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		void operator () (FishBody& child);
	};
}//end namespace InterpretationModule.