////////////////////////////////////////////////////////////////////////////////
/*!	\file		soundLogic.cpp
**	\brief		Defines all the functionality in soundLogic.hpp
**	\author		Nate Cleveland
**	\date		2/10/2003 2:36:32 PM
**	\par		DESCRIPTION:
					See soundLogic.hpp for more details.
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
#include "soundLogic.hpp"
#include "attributes.hpp"
using namespace InterpretationModule;
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif


////Consts globals for varous sound files.
const char*	DEFUALT_BITE_SOUND			= "Resources/Sounds/fish/bite2.mp3";
const char*	DEFUALT_HIT_SOUND			= "Resources/Sounds/fish/wetHit2.mp3";

//think consts
const float SWIMMING_META_BOUNDRY		= .5f;
const char*	DEFUALT_THINK_SOUND_SMALL	= "Resources/Sounds/fish/highLaugh.mp3";
const char*	DEFUALT_THINK_SOUND_LARGE	= "Resources/Sounds/fish/highLaugh.mp3";


const char*	DEFUALT_COLLIDE_SOUND		= NULL;
const char*	DEFUALT_LIFT_SOUND			= NULL;
const char*	DEFUALT_DROP_SOUND			= NULL;

//Death consts
const float DEATH_LIFESPAN_BOUNDRY		= 20;
const char*	DEFUALT_DEATH_SOUND_SMALL	= "Resources/Sounds/fish/squish.mp3";
const char*	DEFUALT_DEATH_SOUND_LARGE	= "Resources/Sounds/fish/devilLaugh.mp3";

//Swim consts
const float SWIMMING_AVERAGE_BOUNDRY	= .5f;
const char*	DEFUALT_SWIMMING_SOUND_SLOW = "Resources/Sounds/fish/bubblingWater2.mp3";
const char*	DEFUALT_SWIMMING_SOUND_FAST = "Resources/Sounds/fish/bubblingWater2.mp3";

//Eat consts
const char*	DEFUALT_EAT_SOUND			= "Resources/Sounds/fish/slurp2_short.mp3";

//See header for documentation.
SoundLoader::SoundLoader(const Attributes& atts) throw()
:m_attributes(atts)
{

}

//See header for documentation.
const char* SoundLoader::Bite() const throw()
{
	return DEFUALT_BITE_SOUND;
}


//See header for documentation.		
const char* SoundLoader::Hit() const throw()
{
	return DEFUALT_HIT_SOUND;
}



//See header for documentation.
const char* SoundLoader::Swim() const throw()
{
	float avg = m_attributes.GetMetabolism() + m_attributes.GetArmor();
	avg /= 2;

	if(avg >= SWIMMING_AVERAGE_BOUNDRY)
		return DEFUALT_SWIMMING_SOUND_FAST;
	return DEFUALT_SWIMMING_SOUND_SLOW;
}



//See header for documentation.
const char* SoundLoader::Think() const throw()
{
	if(m_attributes.GetMetabolism() >= SWIMMING_META_BOUNDRY)
		return DEFUALT_THINK_SOUND_LARGE;
	return DEFUALT_THINK_SOUND_SMALL;
}



//See header for documentation.
const char* SoundLoader::Lift()	 const throw()
{
	return DEFUALT_LIFT_SOUND;
}


//See header for documentation.
const char* SoundLoader::Drop() const throw()
{
	return DEFUALT_DROP_SOUND;
}


//See header for documentation.
const char* SoundLoader::CollideBody() const throw()
{
	return DEFUALT_COLLIDE_SOUND;
}


//See header for documentation.
const char* SoundLoader::CollideTerrain()const throw()
{
	return DEFUALT_COLLIDE_SOUND;
}


//See header for documentation.
const char* SoundLoader::Death() const throw()
{
	if(m_attributes.GetLifeSpan() >= DEATH_LIFESPAN_BOUNDRY)
		return DEFUALT_DEATH_SOUND_LARGE;
	return DEFUALT_DEATH_SOUND_SMALL;
}

//See header for documentation.
const char* SoundLoader::Eat() const throw()
{
	return 	DEFUALT_EAT_SOUND;
}