////////////////////////////////////////////////////////////////////////////////
/*!	\file		soundLogic.hpp
**	\brief		Declares all the functionality for the SoundLoader class.
**	\author		Nate Cleveland
**	\date		2/10/2003 2:33:28 PM
**	\par		DESCRIPTION:
					Sound loader class analizes a given set of attributes to
					 select the proper sound to play for an individual fish 
					 action.
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
namespace InterpretationModule
{
	//the Attributes class is how the SoundLoader class analyzes the fish while
	// picking what sound files it'll use.
	class Attributes;


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		SoundLoader
	**	\brief		Chooses which sounds a fish should use to load itself.
	**	\author		Nate Cleveland
	**	\date		2/10/2003 2:29:12 PM
	**	\warning	none
	*/
	////////////////////////////////////////////////////////////////////////////////
	class SoundLoader
	{
		const Attributes& m_attributes;		//!< Attributes this object looks at.


	public:

		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Loads a reference to the given attributes object for later analysis.
		**	\param		atts		Refernce to reference (PUN!!)
		**	\exception	none					
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:37:33 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		SoundLoader(const Attributes& atts) throw();



		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the bite sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Bite()			const throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Hit sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////		
		const char* Hit()			const throw();



		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Swim sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Swim()			const throw();



		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Think sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Think()			const throw();



		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Lift sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Lift()			const throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Drop sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Drop()			const throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the CollideBody sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* CollideBody()	const throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the CollideTerrain sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* CollideTerrain()const throw();


		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the Death sound.
		**	\return		none
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Death()			const throw();	



		////////////////////////////////////////////////////////////////////////////////
		//	FUNCTION:	
		/*!	\brief		Returns to the caller name of file to play the eat sound.
		**	\return		pntr to a char array of the filename to load.
		**	\exception	none				
		**	\author		Nate Cleveland
		**	\date		2/10/2003 2:32:13 PM
		**	\warning	none
		*/
		////////////////////////////////////////////////////////////////////////////////
		const char* Eat()			const throw();
	};
}