////////////////////////////////////////////////////////////////////////////////
/*!	\file		
**	\brief		
**	\author		Nate Cleveland
**	\date		2/21/2003 9:19:43 AM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
**					
*/
////////////////////////////////////////////////////////////////////////////////
#pragma warning(disable:4786)
#include "worldBoundryTracer.hpp"
#include "../tankSimulator/TankSimulator.h"
#include "fishBodyMath.hpp"
#include <math.h>
using namespace InterpretationModule;
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif





//see header for documentation.
WorldBoundryTracer::WorldBoundryTracer(TankSimulatorClient* pTankClient)
:m_tankClient(pTankClient)
{

}

//see header for documentation.
float WorldBoundryTracer::operator ()(InterpPnt_type& startPnt, float vx, float vy, float vz)
{
	//sanity check
	if(!m_tankClient) 
		return -1;
	

	//Create a ray from the starting point and the given direction.
	Ray3D	ray =	{
						{startPnt[0], startPnt[1], startPnt[2]},
						{vx, vy, vz} 
					};

	//Create an AAB Box, we'll need to get that info from the tankClient.
	InterpVec_type origin, extent;
	m_tankClient->GetBoundary(origin, extent);

	Box3D worldBox =	{
							{origin[0], origin[1], origin[2]},
							{extent[0], extent[1], extent[2]}
						};


	//Next, call RayBoxIntersection and get the intersection points.
	Point3D outPoint[2];
	s8 numOfIntersects = RayBoxIntersection(ray, worldBox, outPoint);


	//Knowing aprioa what we do about the system we should only get on intersection point.
	// still to be safe we'll check to be sure.
	if(numOfIntersects > 0)
		return static_cast<float>(PointDistance3D(outPoint[0], ray.origin));

	return -1;
}



