////////////////////////////////////////////////////////////////////////////////
/*! \file       md2.h
**  \brief      MD2 helper loading and saving functionality.  A private header.
**  \author     M Jared Finder
**  \date       1/23/2003 2:03:15 PM
**  \par        REMARKS:
**                  The structures in MD2.h makes the assumption types sizes,
**                  in particular
**                      - CHAR_BIT      == 8
**                      - sizeof(short) == 2
**                      - sizeof(int)   == 4
**                      - sizeof(float) == 4
**                  It would be nice to have C99 style types (int8_t, int16_t,
**                  etc.)
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
//#include"stdheaders.h"


////////////////////////////////////////////////////////////////////////////////
/*! \struct     MD2Header
**  \brief      MD2 file format header
**  \author     M Jared Finder
**  \date       1/23/2003 6:32:38 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
struct MD2Header
{
    int id;                 //< MD2 ID, must be 'IDP2' == 0x32504449
    enum {CorrectID = 0x32504449};
    int version;            //< MD2 version; must be 8
    enum {CorrectVersion = 8};
    int skinWidth;          //< width of the skin (texture) in pixels
    int skinHeight;         //< height of the skin (texture) in pixels
    int frameSize;          //< size of each frame in bytes
    int numSkins;           //< number of skins made
    int numVertices;        //< number of vertices in each frame
    int numTexCoords;       //< number of texture coordinates
    int numTriangles;       //< number of triangles (slower rendering)
    int numGLCommands;      //< number of ints in the command list (much faster rendering)
    int numFrames;          //< number of frames
    int offsetSkins;        //< location of list of skin names
    int offsetTexCoords;    //< location of list of texture coordinates
    int offsetTriangles;    //< location of list of triangles
    int offsetFrames;       //< location of the frames
    int offsetGLCommands;   //< location of the command list
    int offsetEnd;          //< location of the end (size of the file)
};


////////////////////////////////////////////////////////////////////////////////
/*! \struct     MD2Triangle
**  \brief      Triangles as stored in an MD2 file.
**  \author     M Jared Finder
**  \date       1/23/2003 9:15:43 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
struct MD2Triangle
{
    short vertexIndices[3];     //< triangle vertices' location in frame list
    short textureIndices[3];    //< vertices' texture coordinates location
};


////////////////////////////////////////////////////////////////////////////////
/*! \struct     MD2FrameHeader
**  \brief      Contains information about the frame.
**  \author     M Jared Finder
**  \date       1/24/2003 10:32:28 AM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
struct MD2FrameHeader
{
    float scale[3];         //< scaling amount for each vertex
    float translalte[3];    //< translation amount for each vertex
    char name[16];          //< name of the animation frame
};

////////////////////////////////////////////////////////////////////////////////
/*! \struct     MD2FrameVertex
**  \brief      Each vertex in the MD2 file has one of these.
**  \author     M Jared Finder
**  \date       1/24/2003 10:27:51 AM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
struct MD2FrameVertex
{
    unsigned char   vertex[3];      //< vertex location
    unsigned char   lightNormal;    //< Used to index into table stored by Quake2
};

const MAX_TRIANGLES     = 4096; //< maximum number of triangles
const MAX_VERTICES      = 2048; //< maximum number of vertices
const MAX_TEX_COORDS    = 2048; //< maximum number of texture coordinates
const MAX_FRAMES        =  512; //< maximum number of frames
const MAX_SKINS         =   32; //< maximum number of skins
