////////////////////////////////////////////////////////////////////////////////
/*! \file       matrix2.cpp
**  \brief      Implementation of Matrix2 class
**  \author     M Jared Finder
**  \date       10/4/2002 8:04:02 PM
**  \par        DESCRIPTION:
**                  Implements all Mector2 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Matrix2<type>::Matrix2(type row1col1, type row1col2,
                                         type row2col1, type row2col2)
{
    m_data[0] = row1col1;   m_data[2] = row1col2;
    m_data[1] = row2col1;   m_data[3] = row2col2;
}

template<typename type> inline
Matrix2<type>::Matrix2(type* matrixData)
{
    m_data[0] = matrixData[0];
    m_data[1] = matrixData[1];
    m_data[2] = matrixData[2];
    m_data[3] = matrixData[3];
}

template<typename type> inline
Matrix2<type>::Matrix2(const Matrix2<type>& rhs)
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];
    m_data[3] = rhs.m_data[3];
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::operator=(const Matrix2<type>& rhs)
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];
    m_data[3] = rhs.m_data[3];

    return *this;
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::operator+=(const Matrix2<type>& rhs)
{
    m_data[0] += rhs.m_data[0];
    m_data[1] += rhs.m_data[1];
    m_data[2] += rhs.m_data[2];
    m_data[3] += rhs.m_data[3];

    return *this;
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::operator-=(const Matrix2<type>& rhs)
{
    m_data[0] -= rhs.m_data[0];
    m_data[1] -= rhs.m_data[1];
    m_data[2] -= rhs.m_data[2];
    m_data[3] -= rhs.m_data[3];

    return *this;
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::operator*=(const Matrix2<type>& rhs)
{
    return *this = *this * rhs;
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::operator*=(const type& rhs)
{
    m_data[0] *= rhs;
    m_data[1] *= rhs;
    m_data[2] *= rhs;
    m_data[3] *= rhs;

    return *this;
}


template<typename type> inline
type*
Matrix2<type>::Data(void) const
{
    return const_cast<float*>(m_data);
}

template<typename type> inline
type
Matrix2<type>::Det(void) const
{
    return m_data[0]*m_data[3] - m_data[1]*m_data[2];
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::Transpose(void)
{
    std::swap(m_data[1], m_data[2]);

    return *this;
}

template<typename type> inline
Matrix2<type>&
Matrix2<type>::Invert(void)
{
    //calculate the determinant, and make sure the matrix is invertable
    type determinant = Det();
    if(determinant == 0)
    {
        throw std::domain_error("Non-invertable matrix.");
    }

    //the inverse of data
    type dataInverse[4];

    dataInverse[0] = m_data[3] / determinant;   dataInverse[2] = -m_data[1] / determinant;
    dataInverse[1] = -m_data[2] / determinant;   dataInverse[3] = m_data[0] / determinant;

    //copy back into data
    m_data[0] = dataInverse[0];
    m_data[1] = dataInverse[1];
    m_data[2] = dataInverse[2];
    m_data[3] = dataInverse[3];

    return *this;
}

template<typename type> inline
const_Matrix2Sub<type>
Matrix2<type>::operator[](size_type pos) const
{
    return const_Matrix2Sub<type>(*this, pos);
}

template<typename type> inline
Matrix2Sub<type>
Matrix2<type>::operator[](size_type pos)
{
    return Matrix2Sub<type>(*this, pos);
}


template<typename type> inline
bool
operator ==(const Matrix2<type>& lhs, const Matrix2<type>& rhs) throw()
{
    return lhs.Data()[0]==rhs.Data()[0] && lhs.Data()[1]==rhs.Data()[1]
            && lhs.Data()[2]==rhs.Data()[2] && lhs.Data()[3]==rhs.Data()[3];
}

template<typename type> inline
bool
operator !=(const Matrix2<type>& lhs, const Matrix2<type>& rhs) throw()
{
    return !(lhs == rhs);
}