////////////////////////////////////////////////////////////////////////////////
/*! \file       matrix3.cpp
**  \brief      Implementation of Matrix3 class
**  \author     M Jared Finder
**  \date       10/4/2002 8:04:02 PM
**  \par        DESCRIPTION:
**                  Implements all Mector3 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Matrix3<type>::Matrix3(
        type r1c1, type r1c2, type r1c3,
        type r2c1, type r2c2, type r2c3,
        type r3c1, type r3c2, type r3c3)
{
    m_data[0] = r1c1;   m_data[3] = r1c2;   m_data[6] = r1c3;
    m_data[1] = r2c1;   m_data[4] = r2c2;   m_data[7] = r2c3;
    m_data[2] = r3c1;   m_data[5] = r3c2;   m_data[8] = r3c3;
}

template<typename type> inline
Matrix3<type>::Matrix3(type* matrixData)
{
    m_data[0] = matrixData[0];
    m_data[1] = matrixData[1];
    m_data[2] = matrixData[2];
    m_data[3] = matrixData[3];
    m_data[4] = matrixData[4];
    m_data[5] = matrixData[5];
    m_data[6] = matrixData[6];
    m_data[7] = matrixData[7];
    m_data[8] = matrixData[8];
}

template<typename type> inline
Matrix3<type>::Matrix3(const Matrix3<type>& rhs)
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];
    m_data[3] = rhs.m_data[3];
    m_data[4] = rhs.m_data[4];
    m_data[5] = rhs.m_data[5];
    m_data[6] = rhs.m_data[6];
    m_data[7] = rhs.m_data[7];
    m_data[8] = rhs.m_data[8];
}

template<typename type> inline
Matrix3<type>&
Matrix3<type>::operator=(const Matrix3<type>& rhs)
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];
    m_data[3] = rhs.m_data[3];
    m_data[4] = rhs.m_data[4];
    m_data[5] = rhs.m_data[5];
    m_data[6] = rhs.m_data[6];
    m_data[7] = rhs.m_data[7];
    m_data[8] = rhs.m_data[8];

    return *this;
}

template<typename type> inline
Matrix3<type>&
Matrix3<type>::operator+=(const Matrix3<type>& rhs)
{
    m_data[0] += rhs.m_data[0];
    m_data[1] += rhs.m_data[1];
    m_data[2] += rhs.m_data[2];
    m_data[3] += rhs.m_data[3];
    m_data[4] += rhs.m_data[4];
    m_data[5] += rhs.m_data[5];
    m_data[6] += rhs.m_data[6];
    m_data[7] += rhs.m_data[7];
    m_data[8] += rhs.m_data[8];

    return *this;
}

template<typename type> inline
Matrix3<type>&
Matrix3<type>::operator-=(const Matrix3<type>& rhs)
{
    m_data[0] -= rhs.m_data[0];
    m_data[1] -= rhs.m_data[1];
    m_data[2] -= rhs.m_data[2];
    m_data[3] -= rhs.m_data[3];
    m_data[4] -= rhs.m_data[4];
    m_data[5] -= rhs.m_data[5];
    m_data[6] -= rhs.m_data[6];
    m_data[7] -= rhs.m_data[7];
    m_data[8] -= rhs.m_data[8];

    return *this;
}

template<typename type> inline
Matrix3<type>&
Matrix3<type>::operator*=(const Matrix3<type>& rhs)
{
    return *this = *this * rhs;
}

template<typename type> inline
Matrix3<type>&
Matrix3<type>::operator*=(const type& rhs)
{
    m_data[0] *= rhs;
    m_data[1] *= rhs;
    m_data[2] *= rhs;
    m_data[3] *= rhs;
    m_data[4] *= rhs;
    m_data[5] *= rhs;
    m_data[6] *= rhs;
    m_data[7] *= rhs;
    m_data[8] *= rhs;

    return *this;
}


template<typename type> inline
type*
Matrix3<type>::Data(void) const
{
    return const_cast<float*>(m_data);
}


template<typename type> inline
type
Matrix3<type>::Det(void) const
{
    return m_data[0] *(m_data[4]*m_data[8] - m_data[5]*m_data[7])
           -m_data[1]*(m_data[3]*m_data[8] - m_data[5]*m_data[6])
           +m_data[2]*(m_data[3]*m_data[7] - m_data[4]*m_data[6]);
}


template<typename type> inline
Matrix3<type>&
Matrix3<type>::Transpose(void)
{
    std::swap(m_data[1], m_data[3]);
    std::swap(m_data[2], m_data[6]);
    std::swap(m_data[5], m_data[7]);

    return *this;
}


template<typename type> inline
Matrix3<type>&
Matrix3<type>::Invert(void)
{
    //calculate the determinant, and make sure the matrix is invertable
    type det = Det();
    if(det == 0)
    {
        throw std::domain_error("Non-invertable matrix.");
    }

    //the adjoint of the matrix
    //(really fucking long formula, look it up in a book if you want to understand it)
    Matrix3<type> adjT(m_data[4]*m_data[8] - m_data[5]*m_data[7],
                         m_data[2]*m_data[7] - m_data[1]*m_data[8],
                           m_data[1]*m_data[5] - m_data[2]*m_data[4],
                       m_data[5]*m_data[6] - m_data[3]*m_data[8],
                         m_data[0]*m_data[8] - m_data[2]*m_data[6],
                           m_data[2]*m_data[3] - m_data[0]*m_data[5],
                       m_data[3]*m_data[7] - m_data[4]*m_data[6],
                         m_data[1]*m_data[6] - m_data[0]*m_data[7],
                           m_data[0]*m_data[4] - m_data[1]*m_data[3]);

    //inverse = adj / det
    adjT.Transpose() *= (1/det);

    //copy back into data
    *this = adjT;

    return *this;
}

template<typename type> inline
const_Matrix3Sub<type>
Matrix3<type>::operator[](size_type pos) const
{
    return const_Matrix3Sub<type>(*this, pos);
}

template<typename type> inline
Matrix3Sub<type>
Matrix3<type>::operator[](size_type pos)
{
    return Matrix3Sub<type>(*this, pos);
}



template<typename type> inline
bool
operator ==(const Matrix3<type>& lhs, const Matrix3<type>& rhs) throw()
{
    return lhs.Data()[0]==rhs.Data()[0] && lhs.Data()[1]==rhs.Data()[1]
            && lhs.Data()[2]==rhs.Data()[2] && lhs.Data()[3]==rhs.Data()[3]
            && lhs.Data()[4]==rhs.Data()[4] && lhs.Data()[5]==rhs.Data()[5]
            && lhs.Data()[6]==rhs.Data()[6] && lhs.Data()[7]==rhs.Data()[7]
            && lhs.Data()[8]==rhs.Data()[8];
}

template<typename type> inline
bool
operator !=(const Matrix3<type>& lhs, const Matrix3<type>& rhs) throw()
{
    return !(lhs == rhs);
}