////////////////////////////////////////////////////////////////////////////////
/*! \file       matrix4.cpp
**  \brief      Implementation of Matrix4 class
**  \author     M Jared Finder
**  \date       10/4/2002 8:04:02 PM
**  \par        DESCRIPTION:
**                  Implements all Mector4 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Matrix4<type>::Matrix4(
        type r1c1, type r1c2, type r1c3, type r1c4,
        type r2c1, type r2c2, type r2c3, type r2c4,
        type r3c1, type r3c2, type r3c3, type r3c4,
        type r4c1, type r4c2, type r4c3, type r4c4)
{
    m_data[0] = r1c1;   m_data[4] = r1c2;   m_data[8] = r1c3;   m_data[12] = r1c4;
    m_data[1] = r2c1;   m_data[5] = r2c2;   m_data[9] = r2c3;   m_data[13] = r2c4;
    m_data[2] = r3c1;   m_data[6] = r3c2;   m_data[10] = r3c3;  m_data[14] = r3c4;
    m_data[3] = r4c1;   m_data[7] = r4c2;   m_data[11] = r4c3;  m_data[15] = r4c4;
}

template<typename type> inline
Matrix4<type>::Matrix4(type* matrixData)
{
    m_data[ 0] = matrixData[ 0];
    m_data[ 1] = matrixData[ 1];
    m_data[ 2] = matrixData[ 2];
    m_data[ 3] = matrixData[ 3];
    m_data[ 4] = matrixData[ 4];
    m_data[ 5] = matrixData[ 5];
    m_data[ 6] = matrixData[ 6];
    m_data[ 7] = matrixData[ 7];
    m_data[ 8] = matrixData[ 8];
    m_data[ 9] = matrixData[ 9];
    m_data[10] = matrixData[10];
    m_data[11] = matrixData[11];
    m_data[12] = matrixData[12];
    m_data[13] = matrixData[13];
    m_data[14] = matrixData[14];
    m_data[15] = matrixData[15];
}

template<typename type> inline
Matrix4<type>::Matrix4(const Matrix4<type>& rhs)
{
    m_data[ 0] = rhs.m_data[ 0];
    m_data[ 1] = rhs.m_data[ 1];
    m_data[ 2] = rhs.m_data[ 2];
    m_data[ 3] = rhs.m_data[ 3];
    m_data[ 4] = rhs.m_data[ 4];
    m_data[ 5] = rhs.m_data[ 5];
    m_data[ 6] = rhs.m_data[ 6];
    m_data[ 7] = rhs.m_data[ 7];
    m_data[ 8] = rhs.m_data[ 8];
    m_data[ 9] = rhs.m_data[ 9];
    m_data[10] = rhs.m_data[10];
    m_data[11] = rhs.m_data[11];
    m_data[12] = rhs.m_data[12];
    m_data[13] = rhs.m_data[13];
    m_data[14] = rhs.m_data[14];
    m_data[15] = rhs.m_data[15];
}

template<typename type> inline
Matrix4<type>&
Matrix4<type>::operator=(const Matrix4<type>& rhs)
{
    m_data[ 0] = rhs.m_data[ 0];
    m_data[ 1] = rhs.m_data[ 1];
    m_data[ 2] = rhs.m_data[ 2];
    m_data[ 3] = rhs.m_data[ 3];
    m_data[ 4] = rhs.m_data[ 4];
    m_data[ 5] = rhs.m_data[ 5];
    m_data[ 6] = rhs.m_data[ 6];
    m_data[ 7] = rhs.m_data[ 7];
    m_data[ 8] = rhs.m_data[ 8];
    m_data[ 9] = rhs.m_data[ 9];
    m_data[10] = rhs.m_data[10];
    m_data[11] = rhs.m_data[11];
    m_data[12] = rhs.m_data[12];
    m_data[13] = rhs.m_data[13];
    m_data[14] = rhs.m_data[14];
    m_data[15] = rhs.m_data[15];

    return *this;
}

template<typename type> inline
Matrix4<type>&
Matrix4<type>::operator+=(const Matrix4<type>& rhs)
{
    m_data[ 0] += rhs.m_data[ 0];
    m_data[ 1] += rhs.m_data[ 1];
    m_data[ 2] += rhs.m_data[ 2];
    m_data[ 3] += rhs.m_data[ 3];
    m_data[ 4] += rhs.m_data[ 4];
    m_data[ 5] += rhs.m_data[ 5];
    m_data[ 6] += rhs.m_data[ 6];
    m_data[ 7] += rhs.m_data[ 7];
    m_data[ 8] += rhs.m_data[ 8];
    m_data[ 9] += rhs.m_data[ 9];
    m_data[10] += rhs.m_data[10];
    m_data[11] += rhs.m_data[11];
    m_data[12] += rhs.m_data[12];
    m_data[13] += rhs.m_data[13];
    m_data[14] += rhs.m_data[14];
    m_data[15] += rhs.m_data[15];

    return *this;
}

template<typename type> inline
Matrix4<type>&
Matrix4<type>::operator-=(const Matrix4<type>& rhs)
{
    m_data[ 0] -= rhs.m_data[ 0];
    m_data[ 1] -= rhs.m_data[ 1];
    m_data[ 2] -= rhs.m_data[ 2];
    m_data[ 3] -= rhs.m_data[ 3];
    m_data[ 4] -= rhs.m_data[ 4];
    m_data[ 5] -= rhs.m_data[ 5];
    m_data[ 6] -= rhs.m_data[ 6];
    m_data[ 7] -= rhs.m_data[ 7];
    m_data[ 8] -= rhs.m_data[ 8];
    m_data[ 9] -= rhs.m_data[ 9];
    m_data[10] -= rhs.m_data[10];
    m_data[11] -= rhs.m_data[11];
    m_data[12] -= rhs.m_data[12];
    m_data[13] -= rhs.m_data[13];
    m_data[14] -= rhs.m_data[14];
    m_data[15] -= rhs.m_data[15];

    return *this;
}

template<typename type> inline
Matrix4<type>&
Matrix4<type>::operator*=(const Matrix4<type>& rhs)
{
    return *this = *this * rhs;
}

template<typename type> inline
Matrix4<type>&
Matrix4<type>::operator*=(const type& rhs)
{
    m_data[ 0] *= rhs;
    m_data[ 1] *= rhs;
    m_data[ 2] *= rhs;
    m_data[ 3] *= rhs;
    m_data[ 4] *= rhs;
    m_data[ 5] *= rhs;
    m_data[ 6] *= rhs;
    m_data[ 7] *= rhs;
    m_data[ 8] *= rhs;
    m_data[ 9] *= rhs;
    m_data[10] *= rhs;
    m_data[11] *= rhs;
    m_data[12] *= rhs;
    m_data[13] *= rhs;
    m_data[14] *= rhs;
    m_data[15] *= rhs;

    return *this;
}


template<typename type> inline
type*
Matrix4<type>::Data(void) const
{
    return const_cast<float*>(m_data);
}


template<typename type> inline
type
Matrix4<type>::Det(void) const
{
    type alpha =    m_data[10]*m_data[15] - m_data[11]*m_data[14];
    type beta =     m_data[ 9]*m_data[15] - m_data[11]*m_data[13];
    type gamma =    m_data[ 9]*m_data[14] - m_data[10]*m_data[13];
    type delta =    m_data[ 8]*m_data[15] - m_data[11]*m_data[12];
    type epsilon =  m_data[ 8]*m_data[14] - m_data[10]*m_data[12];
    type zeta =     m_data[ 8]*m_data[13] - m_data[ 9]*m_data[12];

    return +m_data[0]*(m_data[5]*alpha  - m_data[6]*beta    + m_data[7]*gamma)
           -m_data[1]*(m_data[4]*alpha  - m_data[6]*delta   + m_data[7]*epsilon)
           +m_data[2]*(m_data[4]*beta   - m_data[5]*delta   + m_data[7]*zeta)
           -m_data[3]*(m_data[4]*gamma  - m_data[5]*epsilon + m_data[6]*zeta);
}


template<typename type> inline
Matrix4<type>&
Matrix4<type>::Transpose(void)
{
    using std::swap;

    swap(m_data[1], m_data[4]);
    swap(m_data[2], m_data[8]);
    swap(m_data[3], m_data[12]);
    swap(m_data[6], m_data[9]);
    swap(m_data[8], m_data[13]);
    swap(m_data[11], m_data[14]);

    return *this;
}

/////////////////////////////////////////////////////////////////////////////////////////////
template<typename type> inline
Matrix4<type>&
Matrix4<type>::Invert(void)
{
    //calculate the determinant, and make sure the matrix is invertable
    type det = Det();
    if(det == 0)
    {
        throw std::domain_error("Non-invertable matrix.");
    }

    type alpha =    m_data[10]*m_data[15] - m_data[11]*m_data[14];
    type beta =     m_data[ 9]*m_data[15] - m_data[11]*m_data[13];
    type gamma =    m_data[ 9]*m_data[14] - m_data[10]*m_data[13];
    type delta =    m_data[ 8]*m_data[15] - m_data[11]*m_data[12];
    type epsilon =  m_data[ 8]*m_data[14] - m_data[10]*m_data[12];
    type zeta =     m_data[ 8]*m_data[13] - m_data[ 9]*m_data[12];

    type eta =      m_data[ 2]*m_data[ 7] - m_data[ 3]*m_data[ 6];
    type theta =    m_data[ 1]*m_data[ 7] - m_data[ 3]*m_data[ 5];
    type iota =     m_data[ 1]*m_data[ 6] - m_data[ 2]*m_data[ 5];
    type kappa =    m_data[ 0]*m_data[ 7] - m_data[ 3]*m_data[ 4];
    type lambda =   m_data[ 0]*m_data[ 6] - m_data[ 2]*m_data[ 4];
    type mu =       m_data[ 0]*m_data[ 5] - m_data[ 1]*m_data[ 4];

    //the adjoint of the matrix
    //(really fucking long formula, look it up in a book if you want to understand it)
    Matrix4<type> adj(m_data[5]*alpha - m_data[6]*beta + m_data[7]*gamma,
                        -(m_data[4]*alpha - m_data[6]*delta + m_data[7]*epsilon),
                          m_data[4]*beta - m_data[5]*delta + m_data[7]*zeta,
                            -(m_data[4]*gamma - m_data[5]*epsilon + m_data[6]*zeta),
                      -(m_data[1]*alpha - m_data[2]*beta + m_data[3]*gamma),
                        m_data[0]*alpha - m_data[2]*delta + m_data[3]*epsilon,
                          -(m_data[0]*beta - m_data[1]*delta + m_data[3]*zeta),
                            m_data[0]*gamma - m_data[1]*epsilon + m_data[2]*zeta,
                      m_data[13]*eta - m_data[14]*theta + m_data[15]*iota,
                        -(m_data[12]*eta - m_data[14]*kappa + m_data[15]*lambda),
                          m_data[12]*theta - m_data[13]*kappa + m_data[15]*mu,
                            -(m_data[12]*iota - m_data[13]*lambda + m_data[14]*mu),
                      -m_data[ 9]*eta - m_data[10]*theta + m_data[11]*iota,
                        m_data[ 8]*eta - m_data[10]*kappa + m_data[11]*lambda,
                          -(m_data[ 8]*theta - m_data[ 9]*kappa + m_data[11]*mu),
                            m_data[ 8]*iota - m_data[ 9]*lambda + m_data[10]*mu
                      );

    //inverse = adj / det
    adj *= (1/det);

    //copy back into data
    *this = adj;

    return *this;
}
/////////////////////////////////////////////////////////////////////////////////////////////

template<typename type> inline
const_Matrix4Sub<type>
Matrix4<type>::operator[](size_type pos) const
{
    return const_Matrix4Sub<type>(*this, pos);
}

template<typename type> inline
Matrix4Sub<type>
Matrix4<type>::operator[](size_type pos)
{
    return Matrix4Sub<type>(*this, pos);
}



template<typename type> inline
bool
operator ==(const Matrix4<type>& lhs, const Matrix4<type>& rhs) throw()
{
    return     lhs.Data()[ 0]==rhs.Data()[ 0] && lhs.Data()[ 1]==rhs.Data()[ 1]
            && lhs.Data()[ 2]==rhs.Data()[ 2] && lhs.Data()[ 3]==rhs.Data()[ 3]
            && lhs.Data()[ 4]==rhs.Data()[ 4] && lhs.Data()[ 5]==rhs.Data()[ 5]
            && lhs.Data()[ 6]==rhs.Data()[ 6] && lhs.Data()[ 7]==rhs.Data()[ 7]
            && lhs.Data()[ 8]==rhs.Data()[ 8] && lhs.Data()[ 9]==rhs.Data()[ 9]
            && lhs.Data()[10]==rhs.Data()[10] && lhs.Data()[11]==rhs.Data()[11]
            && lhs.Data()[12]==rhs.Data()[12] && lhs.Data()[13]==rhs.Data()[13]
            && lhs.Data()[14]==rhs.Data()[14] && lhs.Data()[15]==rhs.Data()[15];
}

template<typename type> inline
bool
operator !=(const Matrix4<type>& lhs, const Matrix4<type>& rhs) throw()
{
    return !(lhs == rhs);
}
