////////////////////////////////////////////////////////////////////////////////
/*! \file       vector2.cpp
**  \brief      Implementation of Vector2 class
**  \author     M Jared Finder
**  \date       10/4/2002 3:46:35 PM
**  \par        DESCRIPTION:
**                  Implements all vector2 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Vector2<type>::Vector2(type element1, type element2) 
{
    m_data[0] = element1;
    m_data[1] = element2;
}


template<typename type> inline
Vector2<type>::Vector2(type* vectorData) 
{
    m_data[0] = vectorData[0];
    m_data[1] = vectorData[1];
}


template<typename type> inline
Vector2<type>::Vector2(const Vector2<type>& copy) 
{
    m_data[0] = copy.m_data[0];
    m_data[1] = copy.m_data[1];
}


template<typename type> inline
Vector2<type>&
Vector2<type>::operator=(const Vector2<type>& rhs) 
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];

    return *this;
}


template<typename type> inline
Vector2<type>&
Vector2<type>::operator+=(const Vector2<type>& rhs) 
{
    m_data[0] += rhs.m_data[0];
    m_data[1] += rhs.m_data[1];

    return *this;
}


template<typename type> inline
Vector2<type>&
Vector2<type>::operator-=(const Vector2<type>& rhs) 
{
    m_data[0] -= rhs.m_data[0];
    m_data[1] -= rhs.m_data[1];

    return *this;
}


template<typename type> inline
Vector2<type>&
Vector2<type>::operator*=(type rhs)
{
    m_data[0] *= rhs;
    m_data[1] *= rhs;

    return *this;
}


template<typename type> inline
Vector2<type>&
Vector2<type>::operator/=(type rhs) 
{
    m_data[0] /= rhs;
    m_data[1] /= rhs;

    return *this;
}


template<typename type> inline
const type&
Vector2<type>::operator[] (size_type pos) const 
{
    return m_data[pos];
}


template<typename type> inline
type&
Vector2<type>::operator[] (size_type pos) 
{
    return m_data[pos];
}

template<typename type> inline
const type*
Vector2<type>::Data(void) const
{
    return m_data;
}

template<typename type> inline
type
Vector2<type>::Magnitude(void) const 
{
    return type(sqrt(m_data[0]*m_data[0] + m_data[1]*m_data[1]));
}


template<typename type> inline
Vector2<type>&
Vector2<type>::Normalize(void)
{
    operator/=(Magnitude());

    return *this;
}


template<typename type> inline
const Vector2<type>
operator+ (const Vector2<type>& lhs, const Vector2<type>& rhs)
{
    Vector2<type> returnVal(lhs);
    returnVal += rhs;

    return returnVal;
}


template<typename type> inline
const Vector2<type>
operator- (const Vector2<type>& lhs, const Vector2<type>& rhs)
{
    Vector2<type> returnVal(lhs);
    returnVal -= rhs;

    return returnVal;
}


template<typename type> inline
const type
Dot(const Vector2<type>& lhs, const Vector2<type>& rhs)
{
    return lhs[0]*rhs[0] + lhs[1]*rhs[1];
}


template<typename type> inline
bool
operator ==(const Vector2<type>& lhs, const Vector2<type>& rhs) throw()
{
    return lhs[0]==rhs[0] && lhs[1]==rhs[1];
}

template<typename type> inline
bool
operator !=(const Vector2<type>& lhs, const Vector2<type>& rhs) throw()
{
    return !(lhs == rhs);
}