////////////////////////////////////////////////////////////////////////////////
/*! \file       vector2.h
**  \brief      2d vector
**  \author     M Jared Finder
**  \date       10/4/2002 2:42:23 PM
**  \par        DESCRIPTION:
**                  Implementation of a templated 2d vector type.
**  \par        REMARKS:
**                  Because the usage of the functions is obvious, there will
**                  no function headers.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once

////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
namespace math
{
    typedef unsigned int size_type;
}


namespace math { namespace linear_alg {
////////////////////////////////////////////////////////////////////////////////
/*! \class      Vector2<type>
**  \brief      2d Vector, templatized.
**  \author     M Jared Finder
**  \date       10/4/2002 2:42:23 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
template<typename type>
class Vector2
{
public:

    /// Element-wise constructor
    Vector2(type element1=0, type element2=0)   throw();
    /// Data array constructor
    Vector2(type* vectorData)                   throw();
    /// Copy constructor
    Vector2(const Vector2<type>&)               throw();
    /// Assignment operator
    Vector2& operator= (const Vector2<type>&)   throw();

    Vector2& operator+= (const Vector2<type>&)  throw();
    Vector2& operator-= (const Vector2<type>&)  throw();
    Vector2& operator*= (type)                  throw();
    Vector2& operator/= (type)                  throw();

    const type& operator[] (size_type)  const throw();
    type& operator[] (size_type)        throw();
    /// Returns the data
    const type* Data(void)              const throw();

    /// Length of vector
    type Magnitude(void)        const throw();
    /// Normalizes the vector
    Vector2& Normalize(void);

private:
    type    m_data[2];
};

template<typename type>
const Vector2<type>
operator+(const Vector2<type>& lhs, const Vector2<type>& rhs) throw();

template<typename type>
const Vector2<type>
operator-(const Vector2<type>& lhs, const Vector2<type>& rhs) throw();

template<typename type>
const type
Dot(const Vector2<type>& lhs, const Vector2<type>& rhs) throw();

template<typename type>
const Vector2<type>
operator*(const Vector2<type>& lhs, type rhs) throw()
{
    Vector2<type> retVal(lhs);
    retVal *= rhs;
    return retVal;
}

template<typename type>
const Vector2<type>
operator*(type rhs, const Vector2<type>& lhs) throw()
{
    Vector2<type> retVal(lhs);
    retVal *= rhs;
    return retVal;
}


// export workaround
#include "vector2.cpp"


} }
