////////////////////////////////////////////////////////////////////////////////
/*! \file       vector3.cpp
**  \brief      Implementation of Vector3 class
**  \author     M Jared Finder
**  \date       10/4/2002 3:46:35 PM
**  \par        DESCRIPTION:
**                  Implements all vector3 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Vector3<type>::Vector3(type element1, type element2,
                                         type element3)
{
    m_data[0] = element1;
    m_data[1] = element2;
    m_data[2] = element3;
}


template<typename type> inline
Vector3<type>::Vector3(type* vectorData)
{
    m_data[0] = vectorData[0];
    m_data[1] = vectorData[1];
    m_data[2] = vectorData[2];
}


template<typename type> inline
Vector3<type>::Vector3(const Vector3<type>& copy)
{
    m_data[0] = copy.m_data[0];
    m_data[1] = copy.m_data[1];
    m_data[2] = copy.m_data[2];
}


template<typename type> inline
Vector3<type>&
Vector3<type>::operator=(const Vector3<type>& rhs)
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];

    return *this;
}


template<typename type> inline
Vector3<type>&
Vector3<type>::operator+=(const Vector3<type>& rhs)
{
    m_data[0] += rhs.m_data[0];
    m_data[1] += rhs.m_data[1];
    m_data[2] += rhs.m_data[2];

    return *this;
}


template<typename type> inline
Vector3<type>&
Vector3<type>::operator-=(const Vector3<type>& rhs)
{
    m_data[0] -= rhs.m_data[0];
    m_data[1] -= rhs.m_data[1];
    m_data[2] -= rhs.m_data[2];

    return *this;
}


template<typename type> inline
Vector3<type>&
Vector3<type>::operator*=(type rhs)
{
    m_data[0] *= rhs;
    m_data[1] *= rhs;
    m_data[2] *= rhs;

    return *this;
}


template<typename type> inline
Vector3<type>&
Vector3<type>::operator/=(type rhs)
{
    m_data[0] /= rhs;
    m_data[1] /= rhs;
    m_data[2] /= rhs;

    return *this;
}


template<typename type> inline
const type&
Vector3<type>::operator[] (size_type pos) const
{
    return m_data[pos];
}


template<typename type> inline
type&
Vector3<type>::operator[] (size_type pos)
{
    return m_data[pos];
}

template<typename type> inline
const type*
Vector3<type>::Data(void) const
{
    return m_data;
}


template<typename type> inline
type
Vector3<type>::Magnitude(void) const
{
    return type(sqrt(m_data[0]*m_data[0] + m_data[1]*m_data[1] + m_data[2]*m_data[2]));
}


template<typename type> inline
Vector3<type>&
Vector3<type>::Normalize(void)
{
    operator/=(Magnitude());

    return *this;
}


template<typename type> inline
const Vector3<type>
operator+ (const Vector3<type>& lhs, const Vector3<type>& rhs)
{
    Vector3<type> returnVal(lhs);
    returnVal += rhs;

    return returnVal;
}


template<typename type> inline
const Vector3<type>
operator- (const Vector3<type>& lhs, const Vector3<type>& rhs)
{
    Vector3<type> returnVal(lhs);
    returnVal -= rhs;

    return returnVal;
}


template<typename type> inline
const type
Dot(const Vector3<type>& lhs, const Vector3<type>& rhs)
{
    return lhs[0]*rhs[0] + lhs[1]*rhs[1] + lhs[2]*rhs[2];
}


template<typename type> inline
const Vector3<type>
Cross(const Vector3<type>& lhs, const Vector3<type>& rhs)
{
    return Vector3<type>(lhs[1]*rhs[2] - lhs[2]*rhs[1],
                         lhs[2]*rhs[0] - lhs[0]*rhs[2],
                         lhs[0]*rhs[1] - lhs[1]*rhs[0]);
}


template<typename type> inline
bool
operator ==(const Vector3<type>& lhs, const Vector3<type>& rhs) throw()
{
    return lhs[0]==rhs[0] && lhs[1]==rhs[1] && lhs[2]==rhs[2];
}

template<typename type> inline
bool
operator !=(const Vector3<type>& lhs, const Vector3<type>& rhs) throw()
{
    return !(lhs == rhs);
}