////////////////////////////////////////////////////////////////////////////////
/*! \file       vector4.cpp
**  \brief      Implementation of Vector4 class
**  \author     M Jared Finder
**  \date       10/4/2002 3:46:35 PM
**  \par        DESCRIPTION:
**                  Implements all vector4 member functions.
**  \par        REMARKS:
**                  Should not be built because of the export workaround.
*/
////////////////////////////////////////////////////////////////////////////////


template<typename type> inline
Vector4<type>::Vector4(type element1, type element2,
                                         type element3, type element4) 
{
    m_data[0] = element1;
    m_data[1] = element2;
    m_data[2] = element3;
    m_data[3] = element4;
}


template<typename type> inline
Vector4<type>::Vector4(type* vectorData) 
{
    m_data[0] = vectorData[0];
    m_data[1] = vectorData[1];
    m_data[2] = vectorData[2];
    m_data[3] = vectorData[3];
}


template<typename type> inline
Vector4<type>::Vector4(const Vector4<type>& copy) 
{
    m_data[0] = copy.m_data[0];
    m_data[1] = copy.m_data[1];
    m_data[2] = copy.m_data[2];
    m_data[3] = copy.m_data[3];
}


template<typename type> inline
Vector4<type>&
Vector4<type>::operator=(const Vector4<type>& rhs) 
{
    m_data[0] = rhs.m_data[0];
    m_data[1] = rhs.m_data[1];
    m_data[2] = rhs.m_data[2];
    m_data[3] = rhs.m_data[3];

    return *this;
}


template<typename type> inline
Vector4<type>&
Vector4<type>::operator+=(const Vector4<type>& rhs) 
{
    m_data[0] += rhs.m_data[0];
    m_data[1] += rhs.m_data[1];
    m_data[2] += rhs.m_data[2];
    m_data[3] += rhs.m_data[3];

    return *this;
}


template<typename type> inline
Vector4<type>&
Vector4<type>::operator-=(const Vector4<type>& rhs) 
{
    m_data[0] -= rhs.m_data[0];
    m_data[1] -= rhs.m_data[1];
    m_data[2] -= rhs.m_data[2];
    m_data[3] -= rhs.m_data[3];

    return *this;
}


template<typename type> inline
Vector4<type>&
Vector4<type>::operator*=(type rhs)
{
    m_data[0] *= rhs;
    m_data[1] *= rhs;
    m_data[2] *= rhs;
    m_data[3] *= rhs;

    return *this;
}


template<typename type> inline
Vector4<type>&
Vector4<type>::operator/=(type rhs) 
{
    m_data[0] /= rhs;
    m_data[1] /= rhs;
    m_data[2] /= rhs;
    m_data[3] /= rhs;

    return *this;
}


template<typename type> inline
const type&
Vector4<type>::operator[] (size_type pos) const 
{
    return m_data[pos];
}


template<typename type> inline
type&
Vector4<type>::operator[] (size_type pos) 
{
    return m_data[pos];
}

template<typename type> inline
const type*
Vector4<type>::Data(void) const
{
    return m_data;
}


template<typename type> inline
type
Vector4<type>::Magnitude(void) const 
{
    return type(sqrt(m_data[0]*m_data[0] + m_data[1]*m_data[1]
                + m_data[2]*m_data[2] + m_data[3]*m_data[3]));
}


template<typename type> inline
Vector4<type>&
Vector4<type>::Normalize(void)
{
    operator/=(Magnitude());

    return *this;
}


template<typename type> inline
const Vector4<type>
operator+ (const Vector4<type>& lhs, const Vector4<type>& rhs)
{
    Vector4<type> returnVal(lhs);
    returnVal += rhs;

    return returnVal;
}


template<typename type> inline
const Vector4<type>
operator- (const Vector4<type>& lhs, const Vector4<type>& rhs)
{
    Vector4<type> returnVal(lhs);
    returnVal -= rhs;

    return returnVal;
}


template<typename type> inline
const type
Dot(const Vector4<type>& lhs, const Vector4<type>& rhs)
{
    return lhs[0]*rhs[0] + lhs[1]*rhs[1] + lhs[2]*rhs[2] + lhs[3]*rhs[3];
}


template<typename type> inline
bool
operator ==(const Vector4<type>& lhs, const Vector4<type>& rhs) throw()
{
    return lhs[0]==rhs[0] && lhs[1]==rhs[1] && lhs[2]==rhs[2] && lhs[3]==rhs[3];
}

template<typename type> inline
bool
operator !=(const Vector4<type>& lhs, const Vector4<type>& rhs) throw()
{
    return !(lhs == rhs);
}
