////////////////////////////////////////////////////////////////////////////////
/*! \file       vector4.h
**  \brief      4d vector
**  \author     M Jared Finder
**  \date       10/4/2002 2:42:23 PM
**  \par        DESCRIPTION:
**                  Implementation of a templated 4d vector type.
**  \par        REMARKS:
**                  Because the usage of the functions is obvious, there will
**                  no function headers.
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once


////////////////////////////////////////////////////////////////////////////////
// Forward Declarations and needed includes
////////////////////////////////////////////////////////////////////////////////
namespace math
{
    typedef unsigned int size_type;
}


namespace math { namespace linear_alg {
////////////////////////////////////////////////////////////////////////////////
/*! \class      Vector4<type>
**  \brief      4d Vector, templatized.
**  \author     M Jared Finder
**  \date       10/4/2002 2:42:23 PM
**  \warning    none
*/
////////////////////////////////////////////////////////////////////////////////
template<typename type>
class Vector4
{
public:

    /// Element-wise constructor
    Vector4(type element1=0, type element2=0,
            type element3=0, type element4=0)   throw();
    /// Data array constructor
    Vector4(type* vectorData)                   throw();
    /// Copy constructor
    Vector4(const Vector4<type>&)               throw();
    /// Assignment operator
    Vector4& operator= (const Vector4<type>&)   throw();

    Vector4& operator+= (const Vector4<type>&)  throw();
    Vector4& operator-= (const Vector4<type>&)  throw();
    Vector4& operator*= (type)                  throw();
    Vector4& operator/= (type)                  throw();

    const type& operator[] (size_type)  const throw();
    type& operator[] (size_type)        throw();
    /// Returns the data
    const type* Data(void)              const throw();

    /// Length of vector
    type Magnitude(void)        const throw();
    /// Normalizes the vector
    Vector4& Normalize(void);

private:
    type    m_data[4];
};

template<typename type>
const Vector4<type>
operator+(const Vector4<type>& lhs, const Vector4<type>& rhs) throw();

template<typename type>
const Vector4<type>
operator-(const Vector4<type>& lhs, const Vector4<type>& rhs) throw();

template<typename type>
const type
Dot(const Vector4<type>& lhs, const Vector4<type>& rhs) throw();

template<typename type>
const Vector4<type>
operator*(const Vector4<type>& lhs, type rhs) throw()
{
    Vector4<type> retVal(lhs);
    retVal *= rhs;
    return retVal;
}

template<typename type>
const Vector4<type>
operator*(type rhs, const Vector4<type>& lhs) throw()
{
    Vector4<type> retVal(lhs);
    retVal *= rhs;
    return retVal;
}


// export workaround
#include "vector4.cpp"


} }
