////////////////////////////////////////////////////////////////////////////////
/*!	\file		NetMessageID.h
**	\brief		Contains every valid ID that can be sent over the network
**	\author		Nathan Frost
**	\date		2/14/2003 1:21:33 PM
**	\par		DESCRIPTION:
**					
**	\par		REMARKS:
					- the datatype unsigned short is assumed to be two bytes large.  If this is not
					the case, be aware that the ID datafield in a Packet is only two bytes large.
**					
*/
////////////////////////////////////////////////////////////////////////////////
#ifndef NET_MESSAGE_ID_H
#define NET_MESSAGE_ID_H
#pragma once


namespace UDPNet
{
    const unsigned short DEFAULT = 0;
    /* [0] -- should always be ignored */
    const unsigned short HELLO = 1;
    /* [0]  -- message a Client sends to a Server to establish a connection */
    const unsigned short YOU_ARE_ACCEPTED = 2;
    /* [2] -- PortNum -- the port that me, the Client, should send data to when I talk to Server
	   [variable] -- TankSimulator::GenerateHeightMapNetMessage()
	*/
    const unsigned short NOTIFY_SELF = 3;
    /* [0] -- should always be ignored -- used to jar the local machine out of select() */
    const unsigned short BOOTED = 4;
    /* [0] -- the local machine was booted by the sending machine */
    const unsigned short KEEP_ALIVE = 5;
    /* [0] -- a message the Client sends periodically to the Server to maintain contact */
    const unsigned short TEXT = 6;
    /* [variable, NULL terminated]  --  ASCII text string */
    const unsigned short SERVER_UPDATE = 7;
    /* a message the Server sends to all Clients periodically to maintain remote game cohesion
	   [variable] -- TankSimulator::GeneratePartialNetMessage()
	   [variable] -- UserInput::UI_Handler::GenerateCompleteNetMessage()
	*/
    const unsigned short BCB_DELETE_IP = 8;
    /* [0] --   a message used in the BCB testing framework and nowhere else */
	const unsigned short STRESS_TEST_BIG_MESSAGE = 9;
	/* [variable] --   a message used in the StressTest framework and nowhere else */
	const unsigned short SET_FISH_BODY_ACTIONS = 10;
	/* [variable] --	provides the Client with Brains' commands to FishBodies */
	const unsigned short SWIM_MODE_GOTO_POINT = 11;
	/* []			--  informs a remote computer (whether Server or Client) of user's direct 
						commands (ie a user is driving around a Fish) */
	const unsigned short ADD_RANDOM_FISH = 12;
	/* [0]			--	informs the Server that we, the Client, want to Add a Random Fish */
	const unsigned short FISH_SELECT_REQUEST = 13;
	/* []			--	informs the Server that we, the Client, want to Select a Fish */
	const unsigned short FISH_SELECT_LIST_CHANGED = 14;
	/* []			--	informs us that the list of Fish that are selected needs updating */
	const unsigned short FISH_DESELECT_REQUEST = 15;
	/* []			--	informs the Server that we, the Client, want to Deselect a Fish */
	const unsigned short FISH_SUPPRESS_LIST_CHANGED = 16;
	/* []			-- informs us that the list of Fish that are suppressed needs updasting */
	const unsigned short FISH_SUPPRESS_REQUEST = 17;
	/* []			-- informs the Server that we, the Client, want to suppress a Fish */
	const unsigned short FISH_UNSUPPRESS_REQUEST = 18;
	/* []			-- informs the Server that we, the Client, want to unsuppress a Fish */
	const unsigned short ADD_FISH_TO_MATING_LIST = 19;
	/* informs the Server that we, the Client, want to add this Fish to the Mating List
	   [sizeof(Key)]	-- of the Fish we want to add to the Mating List
	*/
	const unsigned short ISSUE_COMMAND = 20;
	/* informs the Server that we, the Client, want to instruct a fish to perform an action
	   [sizeof(Key)]					-- of the Fish we want to instruct
	   [sizeof(AI_System::Action_ID)]	-- of the Action we want the Fish to do
	*/
	const unsigned short NEW_GAME_SPEED = 21;
	/* informs all Clients that a new game speed has been selected
	   [sizeof(double)]	-- the new gameTimeStep
	*/
}

#endif
