#pragma warning(disable:4786)
#include "NodeMgr.h"
#include "UDPNetworking.h"
#include "../nexception/Nexception.h"
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif


#ifdef NULL
#undef NULL
#endif
#define NULL 0
#define GenCast(Node) reinterpret_cast<GenericNode *>(Node)



static NodeManager *nodeManager;



NodeManager *NodeManager::Instance()
{
	if(!nodeManager)
		nodeManager = new NodeManager(UDPNet::MAX_PACKET_SIZE);

	return nodeManager;
}

NodeManager::NodeManager(int NodeSize, int NodesPerPage, int MaxPages):
m_NodeSize(NodeSize), m_NodesPerPage(NodesPerPage), m_MaxPages(MaxPages), 
m_PageSize(m_NodesPerPage*m_NodeSize + sizeof(void *)), m_NumPages(0), m_FreeList(NULL), 
m_PageList(NULL)
{
	AllocateNewPage();
}

NodeManager::~NodeManager()
{
	for(int i=0; i < m_NumPages; ++i)
	{
		GenericNode *temp = m_PageList;
		m_PageList = m_PageList->Next;
		delete [] temp;
	}
}

//should only be called when m_FreeList = NULL, i.e. when there are no more free nodes
bool NodeManager::AllocateNewPage()
{
	if(m_NumPages < m_MaxPages)
	{
		char *pNewPage = new char[m_PageSize];
		if(pNewPage)
		{
			GenCast(pNewPage)->Next = m_PageList;
			m_PageList = GenCast(pNewPage);
			
			//FreeList points to the first free node of this page
			//Note: if the FreeList did not previously point to NULL, then any nodes the FreeList 
			//did point to are now lost (memory leaked)
			GenericNode *pNode = GenCast(pNewPage + sizeof(GenericNode));
			m_FreeList = pNode;
			/* Link all the nodes on the new page together */
			for(int i=0; i < m_NodesPerPage - 1; ++i)
			{
				pNode->Next = GenCast(reinterpret_cast<char *>(pNode) + m_NodeSize);
				pNode = pNode->Next;
			}
			pNode->Next = NULL;
			
			++m_NumPages;
			return true;
		}
	}
	
	return false;
}

void *NodeManager::AllocateNode()
{
	void *ret = NULL;//assume failure
	//if there is either a free node, or, alternately, we successfully create a new page (thereby 
	//freeing m_NumNodesPerPage nodes)
	if(m_FreeList || AllocateNewPage())
	{
		ret = m_FreeList;
		m_FreeList = m_FreeList->Next;
	}
	return ret;
}

bool NodeManager::FreeNode(void *Node)
{
	if(ValidateNode(Node))
	{
		GenCast(Node)->Next = m_FreeList;
		m_FreeList = GenCast(Node);
		return true;
	}
	else
		throw Nexception("INVALID Node", __FILE__, __LINE__, __DATE__, __TIME__);

	return false;
}

bool NodeManager::ValidateNode(void *Node)
{
	if(Node)
	{
		//TODO: Test to make sure that 1) pointer is not already on FreeList, 2) pointer falls on 
		//		correct alignment boundary of page's linked list
		/* Make sure Node is within the valid boundary of all allocated pages */
		bool inBoundary = false;//assume failure
		GenericNode *curPage = m_PageList;
		while(curPage != NULL)
		{
			if(	Node >= GenCast(reinterpret_cast<char *>(curPage) + sizeof(GenericNode)) && 
				Node <= GenCast(reinterpret_cast<char *>(curPage) + m_PageSize - m_NodeSize)		)
			{
				inBoundary = true;
				break;//Node is within a valid page boundary
			}
			curPage = curPage->Next;
		}

		//see if the pointer is already on the FreeList
		curPage = m_FreeList;
		while(curPage != NULL)
		{
			if(Node == curPage)
				return false;//this node is NOT a Valid node to return

			curPage = curPage->Next;
		}

		return inBoundary;
	}

	return false;
}

int NodeManager::FreeNodeCount(void)
{
	int count=0;
	GenericNode *pCurNode = m_FreeList;
	while(pCurNode != NULL)
	{
		++count;
		pCurNode = pCurNode->Next;
	}

	return count;
}

int NodeManager::NodesInUse(void)
{
	return (m_NodesPerPage*m_NumPages - FreeNodeCount());
}

int NodeManager::PagesInUse(void)
{
	return m_NumPages;
}

int NodeManager::PageSize(void)
{
	return m_PageSize;
}
