//---------------------------------------------------------------------------
#ifndef NODEMGR_H
#define NODEMGR_H
//---------------------------------------------------------------------------

// If the client doesn't specify these:
const int DEFAULT_NODES_PER_PAGE = 100;  
const int DEFAULT_MAX_PAGES = 30;

// This allows us to treat all nodes the same
struct GenericNode
{
  GenericNode *Next;
};

class NodeManager
{
  private:
    int m_NodeSize;      // size of each node
    int m_NodesPerPage;  // number of nodes on each page
    int m_PageSize;      // size of a page: m_NodesPerPage * m_NodeSize + sizeof(void*)
    int m_MaxPages;      // maximum number of pages available 
    int m_NumPages;      // number of pages allocated

    GenericNode *m_FreeList;    // the beginning of the list of nodes
    GenericNode *m_PageList;    // the beginning of the list of pages

	//AllocateNewPage should only be called when m_FreeList = NULL, i.e. when there are no more 
	//free nodes
    bool AllocateNewPage(void);    // allocates another page of nodes
    bool ValidateNode(void *Node); // checks that a node is valid
  public:

    // Creates the NodeManager per the specified values
    NodeManager(int NodeSize, int NodesPerPage = DEFAULT_NODES_PER_PAGE, 
				int MaxPages = DEFAULT_MAX_PAGES);
    // Destroys the NodeManager
    ~NodeManager();
	static NodeManager *NodeManager::Instance();

    // Take a node from the free list and give it to the client (simulates new)  
    void *AllocateNode();

    // Returns a node to the free list for the client (simulates delete)
    bool FreeNode(void *Node);

    // Testing/Debugging methods
    int FreeNodeCount(void); // returns the number of nodes on the free list
    int NodesInUse(void);    // returns the number of nodes in use: (total nodes) - (free nodes)
    int PagesInUse(void);    // returns the number of pages allocated
    int PageSize(void);      // returns the size of a page (m_PageSize)
};

#endif

