////////////////////////////////////////////////////////////////////////////////
/*!	\file		UDPNetworking.h
**	\brief		Declares the UDPNetworking class and some associated constants.
**	\author		Nathan Frost
**	\date		10/21/2002 10:56:01 AM
**	\par		DESCRIPTION:
**								-	The UDPNetworking class is a collection of public functions
									that wrap a subset of the WinSock 2.2 API. It provides low
									level Networking functionality.
								-	Singleton
**	\par		REMARKS:
                    - This header file typedef's some datatypes from winsock2.h.  This is to allow
                    us to #include the volatile, badly behaved (read: multiple #include = compiler
                    catastrophe) <winsock2.h> in the .cpp file only.  If these typedef's get out of
                    sync with winsock2.h, this module's behaviour becomes undefined.
**					
*/
////////////////////////////////////////////////////////////////////////////////
#ifndef UDP_NETWORKING_H
#define UDP_NETWORKING_H
#pragma once


#include "WinSockSynthesis.h"


namespace UDPNet
{	
    enum {  ANY_PORT=0,             ///<Pass to CreateSocket() if you don't care what port the
                                    ///<new SOCKET will be bound to
            MAX_PACKET_SIZE=1472	///<in bytes (seems like maximum possible is 65528 on Win2K)
									///<1500 bytes is maximum total payload for Ethernet packet --
									///<20 bytes for IP header, 8 bytes for UDP header
    };
	#define MAX_HOST_NAME_LEN 20///<Maximum length of ASCII representation of host machine name


	////////////////////////////////////////////////////////////////////////////////
	/*!	\class		UDPNetworking
	**	\brief		Provides low level Networking functionality
	**	\author		Nathan Frost
	**	\date		10/21/2002 11:05:23 AM
	**	\warning	Watch out for data stored in Big Endian (it's annotated)
	*/
	////////////////////////////////////////////////////////////////////////////////
	class UDPNetworking
	{
		//Local address information
		char localName[MAX_HOST_NAME_LEN];
		u_long localIP;	///<stored in Big Endian

        UDPNetworking();
	public:
		static UDPNetworking *Instance();
		u_long MyIPAddress();
		void CreateSock(SOCKET &Socket, u_short &Port);
		void DestroySock(SOCKET &Socket);
		void SendPacket(const SOCKET &Socket, const u_long &IPAddress, const u_short &Port,
                        void *Data, const unsigned int SizeOfData);
		int ReceivePacket(	const SOCKET &Socket, void *Buffer, const unsigned int SizeOfBuffer, 
							u_long &SenderIP, u_short &SenderPort);
		~UDPNetworking();


        //BEG_HACK: memory allocation
        int numSocketsAllocated;
        //END_HACK: memory allocation
	};

}
#endif
