////////////////////////////////////////////////////////////////////////////////
/*!	\file		Nexception.cpp
**	\brief		Implementation of class Nexception
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\par		DESCRIPTION:	
								Implementation of class Nexception, its ancillary 
								functions and helper functions that helper me in 
								the implementation
**					
**	\par		REMARKS:	
							It is imperative that no Nexception ever throws an 
							exception.
**					
*/
////////////////////////////////////////////////////////////////////////////////
/*--------------------------------------------Headers--------------------------------------------*/
#include "Nexception.h"
#include <stdlib.h>
#include <assert.h>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace Debugging;	//Update: John Corpening, 10/18/02
/*----------------------------------------Helper Functions---------------------------------------*/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	CStringTruncate()
/*!	\brief		Truncates a C-style string to MaxLength, where MaxLength 
				is smaller than MAX_LENGTH
**	\param		Dest: The C string to place the truncated string into
	\param		Source: The C string to truncate and place into Dest
	\param		MaxLength: The maximum length, in characters, Dest is
				allowed to be (including the 0 terminator)
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:	
								Copies a truncated (down to MaxLength-1 
								characters), 0 terminated Source to Dest
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
void CStringTruncate(char *Dest, const char *Source, int MaxLength)
{
	char copy[MAX_LENGTH];
	strncpy(copy, Source, MaxLength-1);
	copy[MaxLength-1] = 0;
	strcpy(Dest, copy);

	return;
}


/*-----------------------------------Member Function Definitions---------------------------------*/
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Nexception()
/*!	\brief		Constructor
**	\param		Remark: What the thrower has to say for himself
	\param		File: __FILE__ that threw the exception
	\param		Line: __LINE__ that threw the exception
	\param		Date: __DATE__ File was last compiled
	\param		Time: __TIME__ File was last compiled
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:	
								-	Copies all five data fields to (*this) Nexception
								-	All text fields are guaranteed to be 0 terminated
								-	All text fields may be truncated if they exceed their maximum 
									specified length (see header)
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	All data fields other than Remark are intended to be passed the __FILE__, __LINE__,
				__DATE__ and __TIME__ macros in the constructor.  Other usage is not directly 
				supported.
*/
////////////////////////////////////////////////////////////////////////////////
Nexception::Nexception(const char *Remark, const char *File, const unsigned int Line, 
					   const char *Date, const char *Time) throw()
{
	assert(Remark);
	assert(File);
	assert(Date);
	assert(Time);

	/* Copy specified members */
	//if any text field is as long or longer than the specified maximum length then truncate it
	if(strlen(Remark) >= MAX_LENGTH)
		CStringTruncate(remark, Remark, MAX_LENGTH);
	else
		strcpy(remark, Remark);

	if(strlen(File) >= MAX_LENGTH)
		CStringTruncate(file, File, MAX_LENGTH);
	else
		strcpy(file, File);

	line = Line;

	if(strlen(Date) >= DATE_MAX_LENGTH)
		CStringTruncate(date, Date, DATE_MAX_LENGTH);
	else
		strcpy(date, Date);

	if(strlen(Time) >= TIME_MAX_LENGTH)
		CStringTruncate(time, Time, TIME_MAX_LENGTH);
	else
		strcpy(time, Time);

	//Update: John Corpening, 10/18/2002
	string str;
	str << *this;
	Debugger*	debug = Debugger::Instance();
	debug->logException(str.c_str());
	//End Update
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Nexception()
/*!	\brief		Copy Constructor
**	\param		nexception: The nexception to copy from
**	\return		none
**	\exception	none
**	\par		DESCRIPTION:	
					Copies data fields correctly
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
Nexception::Nexception(const Nexception &nexception) throw()
{
	strcpy(remark, nexception.remark);
	strcpy(file, nexception.file);
	line = nexception.line;
	strcpy(date, nexception.date);
	strcpy(time, nexception.time);
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::operator =
/*!	\brief		Assignment Operator
**	\param		nexception:	Right hand side nexception to 
												assign from
**	\return		Nexception &: *this Nexception
**	\exception	none
**	\par		DESCRIPTION:	
					Copies data fields correctly
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
Nexception &Nexception::operator =(const Nexception &nexception) throw()
{
	strcpy(remark, nexception.remark);
	strcpy(file, nexception.file);
	line = nexception.line;
	strcpy(date, nexception.date);
	strcpy(time, nexception.time);

	return *this;
}

////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Accessors	
/*!	\brief		Trivial
**	\param		none
**	\return		Field requested
**	\exception	none
**	\par		DESCRIPTION:	
					Returns the requested data by value
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
const char *Nexception::Remark() const throw()
{
	return remark;
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Accessors	
/*!	\brief		Trivial
**	\param		none
**	\return		Field requested
**	\exception	none
**	\par		DESCRIPTION:	
					Returns the requested data by value
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
const char *Nexception::File() const throw()
{
	return file;
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Accessors	
/*!	\brief		Trivial
**	\param		none
**	\return		Field requested
**	\exception	none
**	\par		DESCRIPTION:	
					Returns the requested data by value
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
const unsigned int Nexception::Line() const throw()
{
	return line;
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	Nexception::Accessors	
/*!	\brief		Trivial
**	\param		none
**	\return		Field requested
**	\exception	none
**	\par		DESCRIPTION:	
					Returns the requested data by value
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
const char *Nexception::Date() const throw()
{
	return date;
}

const char *Nexception::Time() const throw()
{
	return time;
}


/*---------------------------------Ancillary Function Definitions--------------------------------*/
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	operator <<()
/*!	\brief		Output a Nexception to an ostream
**	\param		os: ostream to output to
	\param		nexception: Nexception to output from
**	\return		ostream &: reference to os, for chaining purposes
**	\exception	none
**	\par		DESCRIPTION:	
					Outputs each data field, delineated by newlines, 
					to the specified ostream	
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	none
*/
////////////////////////////////////////////////////////////////////////////////
ostream &operator << (ostream &os, Nexception &nexception)
{
	using std::endl;
	os << nexception.Remark() << endl << nexception.File() << endl << nexception.Line() << endl <<
		nexception.Date() << endl << nexception.Time() << endl;

	return os;
}
////////////////////////////////////////////////////////////////////////////////
//	FUNCTION:	operator<<() 
/*!	\brief		Output a Nexception to a std::string
**	\param		str: string to output to
	\param		nexception: Nexception to output from
**	\return		Reference to str, for chaining purposes
**	\exception	none
**	\par		DESCRIPTION:	
					Outputs each data field, delineated by newlines, 
					to the specified string.
**					
**	\author		Nathan Frost
**	\date		07/04/2002 4:02:00 PM
**	\warning	Relies on NEWLINE, an environment-dependent character token.
*/
////////////////////////////////////////////////////////////////////////////////
string &operator << (string &str, Nexception &nexception)
{
    str = nexception.Remark();
    str += NEWLINE;
    str += nexception.File();
    str += NEWLINE;

    //convert line number from integer representation to ASCII representation
    char lineNum[11];	//there could be up to 10 digits, plus the 0 terminator
    itoa(nexception.Line(), lineNum, 10);
    str += lineNum;
    str += NEWLINE;

    str += nexception.Time();
    str += NEWLINE;
    str += nexception.Date();
    str += NEWLINE;

	return str;
}
