////////////////////////////////////////////////////////////////////////////////
/*! \file           HeightMapFileMemento.h
**  \brief          HeightMapFileMemento object interface file
**  \author         Austin Spafford
**  \date           5/14/2003 6:10:28 PM
**  \par            DESCRIPTION:
**                    The HeightMapFileMemento object encapsulates the
**                    state of a HeightMap object and can read and write
**                    itself to i/o streams.
**  \par            REMARKS:
**                                  
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once
////////////////////////////////////////////////////////////////////////////////
#include "fileIO/memento.h"
#include "math/math.h"
#include <list>
#include <vector>
////////////////////////////////////////////////////////////////////////////////
class HeightMap;
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \class          HeightMapFileMemento
**  \brief          The HeightMapFileMemento object encapsulates the
**                  state of a HeightMap object and can read and write
**                  itself to i/o streams.
**  \author         Austin Spafford
**  \date           10/11/2002 5:38:13 PM
**  \warning        none
*/
////////////////////////////////////////////////////////////////////////////////

class HeightMapFileMemento : public FileMemento
{	
	public:

	HeightMapFileMemento(void) : m_dataPointsBuffer(NULL) {}

	virtual ~HeightMapFileMemento(void) { if (NULL != m_dataPointsBuffer) delete m_dataPointsBuffer; }

	////////////////////////////////////////////////////////////////////////////

	// writes the memento's contents to an ostream in a human readable format
	virtual void Write(std::ostream & inputStream) const;

	// reads the memento's contents from an istream that is using a human readable format
	virtual void Read(std::istream & inputStream);

	////////////////////////////////////////////////////////////////////////////

	protected:

	friend HeightMap;

	math::linear_alg::Vector2<int> m_pointCount;
	math::linear_alg::Vector2<float> m_cellDimensions;
	float *m_dataPointsBuffer;

	math::linear_alg::Vector3<float> m_positionOffset;

	HeightMapFileMemento(const HeightMapFileMemento & fileMementoToCopy); // intentional
	const HeightMapFileMemento & operator=(const HeightMapFileMemento & fileMementoToCopy); // intentional
};
