#include "BodyClient.h"
#include "UIClient.h"
#include <assert.h>
// memory leak detection
#ifdef _DEBUG
    #include"memleak.h"
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

using namespace AcqariusSound;

//See header for documentation.
bool BodyClient::LoadSingleSound(HSound& oldSound, const char* pathName) throw()
{
	HSound newSound	= 0;		//temp var to hold newly loaded m_sound file.

	//Attempt to load a new m_sound.
	if(!m_sound->LoadSound(pathName, newSound,  SOUND_3D | SOUND_SMALL | SOUND_LOOP_OFF)) 
		return false;

	//free the old m_sound iff it isn't zero.
	if(oldSound)
		m_sound->FreeSound(oldSound);

	//set the old m_sound to the new one.
	oldSound = newSound;
	return true;
}


//See header for documentation.
bool BodyClient::PlaySingleSound(HSound& hSound) throw()
{
	//assert that we have a sound client.
#	ifdef _DEBUG
		assert(m_sound);
#	else
		if(!m_sound) return false;
#	endif

	//check to see if sound is already playing.
	if(m_sound->IsPlaying(hSound)) 
		return true;

	//play the sound, check for error
	if(!m_sound->PlaySound(hSound, m_pos)) 
		return false;
	return true;
}

//See header for documentation.
bool BodyClient::LoadSounds(const char* bite, const char* hit, const char* swim,
							const char* think, const char* lift, const char* drop,
							const char* collideBody, const char* collideTerrain, 
							const char* death, const char* eat) throw()
{
	//assert that we have a sound client.
#	ifdef _DEBUG
		assert(m_sound);
#	else
		if(!m_sound) return false;
#	endif		
		

	//load bite sound iff we were given one. check for errors.
	if(bite)
		if(!LoadSingleSound(m_bite, bite))
			return false;

	//load hit sound iff we're given one. check for errors.
	if(hit)
		if(!LoadSingleSound(m_hit, hit))
			return false;

	//load swim sound iff we were given one. check for errors.
	if(swim)
		if(!LoadSingleSound(m_swim, swim))
			return false;

	//load think sound iff we were given one. check for errors.
	if(think)
		if(!LoadSingleSound(m_think, think))
			return false;

	//load drop sound iff we were given one. check for errors.
	if(drop)
		if(!LoadSingleSound(m_drop, drop))
			return false;

	//load fish collide sound iff we were given one. check for errors.
	if(collideBody)
		if(!LoadSingleSound(m_collideBody, collideBody))
			return false;

	//load fish collide sound iff we were given one. check for errors.
	if(collideTerrain)
		if(!LoadSingleSound(m_collideTerrain, collideTerrain))
			return false;

	//load death sound iff we were given one. check for errors.
	if(death)
		if(!LoadSingleSound(m_death, death))
			return false;

	//load eat sound iff we were given one. check for errors.
	if(eat)
		if(!LoadSingleSound(m_eat, death))
			return false;
	return true;
}
