////////////////////////////////////////////////////////////////////////////////
/*! \file           DrawableHeightMap.h
**  \brief          DrawableHeightMap object interface file
**  \author         Austin Spafford
**  \date           11/1/2002 11:32:35 AM
**  \par            DESCRIPTION:
**                    The DrawableHeightMap is derived from the HeightMap object
**                    and adds drawing functionality.
**  \par            REMARKS:
**                    
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once
////////////////////////////////////////////////////////////////////////////////
#include "gfx/renderable.h"
#include "gfx/texture.h"
#include "gfx/vtx_buffer.h"
#include "../physics/HeightMap.h"
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/*! \class          DrawableHeightMap
**  \brief          The DrawableHeightMap is derived from the HeightMap object
**                  and adds drawing functionality.
**  \author         Austin Spafford
**  \date           11/1/2002 11:34:49 AM
**  \warning        none
*/
////////////////////////////////////////////////////////////////////////////////
class DrawableHeightMap : public HeightMap, public gfx::IRenderableWorld
{
	public:

	////////////////////////////////////////////////////////////////////////////

	/// default constructer
	DrawableHeightMap();	

	/// deconstructer
	~DrawableHeightMap();

	////////////////////////////////////////////////////////////////////////////

	/// draws the height map using its member graphics client	
	void Render(void);

    /// tells the world that it should be drawn
    void SetRender(void);


    /// refreshes the data from the height map
    void RefreshData(void);

	////////////////////////////////////////////////////////////////////////////

	private:

    /// texture used for texturing the height map
    gfx::Texture m_tex1;
    gfx::Texture m_tex2;
    gfx::Texture m_texDetail;
    
    /// texture for the walls
    gfx::Texture m_texWall;
    /// texture for the ceiling
    gfx::Texture m_texCeiling;
    
    /// vertices used for the height map
    gfx::VertexBuffer m_vertexArray;
    /// size of the height map array
    int m_vertexDimensionality[2];
    /// an index buffer for the height map
    std::vector<unsigned short> m_indexBuffer;

    /// current position
    float   m_position[3];

    /// current scale
    float   m_scale[3];
};