////////////////////////////////////////////////////////////////////////////////
/*! \file           TankSimulatorFileMemento.h
**  \brief          TankSimulatorFileMemento object interface file
**  \author         Austin Spafford
**  \date           5/14/2003 6:10:28 PM
**  \par            DESCRIPTION:
**                    The TankSimulatorFileMemento object encapsulates the
**                    state of a TankSimulator object and can read and write
**                    itself to i/o streams.
**  \par            REMARKS:
*/
////////////////////////////////////////////////////////////////////////////////
#pragma once
////////////////////////////////////////////////////////////////////////////////
#include "fileIO/memento.h"
#include "physics/HeightMapFileMemento.h"
#include "math/math.h"
#include "interpretationModule/frozenFish.hpp"
#include <list>
#include <vector>
////////////////////////////////////////////////////////////////////////////////
class TankSimulator;
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
/*! \class          TankSimulatorFileMemento
**  \brief          An object that encapsulates the state of a TankSimulator
**                  object and can read and write itself to i/o streams.
**  \author         Austin Spafford
**  \date           5/14/2003 6:10:28 PM
**  \warning        none
*/
////////////////////////////////////////////////////////////////////////////////
class TankSimulatorFileMemento : public FileMemento
{
	public:

	TankSimulatorFileMemento(void) : m_heightMapFileMementoPtr(NULL) {}

	virtual ~TankSimulatorFileMemento(void) { if (NULL != m_heightMapFileMementoPtr) delete m_heightMapFileMementoPtr; }

	////////////////////////////////////////////////////////////////////////////

	// writes the memento's contents to an ostream in a human readable format
	virtual void Write(std::ostream & outputStream) const;

	// reads the memento's contents from an istream that is using a human readable format
	virtual void Read(std::istream & inputStream);

	////////////////////////////////////////////////////////////////////////////

	protected:

	friend TankSimulator;

	math::linear_alg::Vector3<float> m_boundaryMins;
	math::linear_alg::Vector3<float> m_boundaryMaxs;

	std::vector<InterpretationModule::FrozenFish> m_frozenFish;

	HeightMapFileMemento *m_heightMapFileMementoPtr;

	std::list<MsgHandler::Key> m_matingFishList;
	std::list<MsgHandler::Key> m_deleteFishList;

	// CHUM AND PLANKTON ARE EXCLUDED FOR NOW

	private:
	TankSimulatorFileMemento(const TankSimulatorFileMemento & fileMementoToCopy); // intentional
	const TankSimulatorFileMemento & operator=(const TankSimulatorFileMemento & fileMementoToCopy); // intentional
};